/* 2012-01-13: File added and changed by Sony Corporation */
/*
 * arch/arm/include/asm/fiq_debugger.h
 *
 * Copyright (C) 2011 Andrei Warkentin <andrey.warkentin@gmail.com>
 *
 * Copyright (C) 2010 Google, Inc.
 * Author: Colin Cross <ccross@android.com>
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#ifndef _ARCH_ARM_FIQ_DEBUGGER_H_
#define _ARCH_ARM_FIQ_DEBUGGER_H_

#include <linux/serial_core.h>
#include <asm/fiq_glue.h>

#define FIQ_DEBUGGER_NO_CHAR NO_POLL_CHAR
#define FIQ_DEBUGGER_BREAK 0x00ff0100

#define FIQ_DEBUGGER_INT_TX	(1 << 0)
#define FIQ_DEBUGGER_INT_RX	(1 << 1)

struct fiq_debugger_ctxt {
  int (*printf)(void *cookie, const char *fmt, ...);
  void *cookie;
};

struct fiq_debugger_pdata {
	char *console_name;
	int console_major;
	int console_minor;

	int (*uart_init)(struct platform_device *pdev);
	void (*uart_free)(struct platform_device *pdev);
	int (*uart_resume)(struct platform_device *pdev);
	int (*uart_getc)(struct platform_device *pdev);
	void (*uart_putc)(struct platform_device *pdev, unsigned int c);
	void (*uart_flush)(struct platform_device *pdev);
	void (*uart_start_tx)(struct platform_device *pdev);
	void (*uart_stop_tx)(struct platform_device *pdev);
	int (*uart_tx_room)(struct platform_device *pdev);
	int (*uart_status)(struct platform_device *pdev);

	void (*fiq_enable)(struct platform_device *pdev, unsigned int fiq,
			   bool enable);
	void (*fiq_ack)(struct platform_device *pdev, unsigned int fiq);

	void (*force_irq)(struct platform_device *pdev, unsigned int irq);
	void (*force_irq_ack)(struct platform_device *pdev, unsigned int irq);
	struct clk *(*uart_clk)(struct platform_device *pdev);

	int (*fiq_glue_register_handler)(struct platform_device *pdev,
					 struct fiq_glue_handler *handler);

	/*
	 * platform_cmds() is called from IRQ context and should
	 * use the ctxt.printf to write output (do NOT call
	 * printk, do operations not safe from IRQ context, etc).
	 *
	 * ctxt.printf will return -1 if there is not enough
	 * buffer space or if you are being aborted.  In this case
	 * you must return as soon as possible.
	 *
	 * Return non-zero if more data is available -- if buffer
	 * space ran and you had to stop, but could print more,
	 * for example.
	 *
	 * Additional calls where cmd is "more" will be made if
	 * the additional data is desired.
	 *
	 */

	int (*platform_cmds)(struct platform_device *pdev,
			     struct fiq_debugger_ctxt *ctxt,
			     char *cmd);
};

#endif
