/* 2011-10-18: File added by Sony Corporation */
/*
 *  arch/arm/mach-xarina/include/mach/irqs.h
 *
 *  Copyright 2010-2011 Sony Corporation
 *  Copyright (C) 2003 ARM Limited
 *  Copyright (C) 2000 Deep Blue Solutions Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __ASM_ARCH_IRQS_H
#define __ASM_ARCH_IRQS_H

#include <mach/platform.h>

#define NR_VIC_IRQS			(32) /* number of VIC IRQs */
#define NR_PCIE_IRQS		(32) /* number of PCIE IRQs */
#define IRQ_VIC1_START			0
#define IRQ_VIC1_END			(IRQ_VIC1_START + NR_VIC_IRQS - 1)
#define IRQ_VIC2_START			(IRQ_VIC1_END + 1)
#define IRQ_VIC2_END			(IRQ_VIC2_START + NR_VIC_IRQS - 1)

/*
 * interrupt bit assignment Carlsbad 20080930
 */
/* VIC 1 */
#define IRQ_MAILBOX			0
/* 1 reserved */
#define IRQ_SMAC			2
/* 3-5 reserved */
#define IRQ_PCIE0			6
/* 7 reserved */
#define IRQ_A11_DMAC			8
/* 9 reserved */
#define IRQ_PCIE_DMAC			10
#define IRQ_SYSTMR			11
#define IRQ_TIMER2_0			12
#define IRQ_TIMER2_1			13
#define IRQ_TIMER3_0			14
#define IRQ_TIMER3_1			15
/* 16 reserved */
#define IRQ_WDT0			17
#define IRQ_WDT1			18
/* 19 reserved */
#define IRQ_SPI2			20
#define IRQ_GPIOF			21
#define IRQ_I2C1			22
#define IRQ_SMACPMT			23
#define IRQ_UART3			24
#define IRQ_UART0			25
#define IRQ_FEC				26
#define IRQ_CRYPTO			27
#define IRQ_PMU				28
#define IRQ_DDR_VRAM			29
#define IRQ_DDR_SYSRAM			30
#define IRQ_SPI0			31

/* VIC 2 */
#define IRQ_SPI1			32
#define IRQ_GPIOA			33
#define IRQ_UART1			34
#define IRQ_SPI3			35
#define IRQ_H264ENCODER			36
#define IRQ_JPEG			37
#define IRQ_AUDIODMAC			38
#define IRQ_AUDIOIF			39
#define IRQ_POST			40
#define IRQ_UART2			41
#define IRQ_INTELLI			42
#define IRQ_I2C0			43
/* 44-45 reserved */
#define IRQ_NANDIF			46
/* 47-51 reserved */
#define IRQ_SD_CH0			52
#define IRQ_USBH_EHCI			53
#define IRQ_USBH_OHCI			54
#define IRQ_USBH_SYSCTRL		55
#define IRQ_USBD_UDC			56
#define IRQ_USBD_SYSCTRL		57
/* 58-61 reserved */
#define IRQ_COMM_RX			62
#define IRQ_COMM_TX			63

#define IRQ_PCIE0_START			(IRQ_VIC2_END + 1)
#define IRQ_PCIE0_END			(IRQ_PCIE0_START + NR_PCIE_IRQS - 1)

#define NR_IRQS				(IRQ_VIC2_END + NR_PCIE_IRQS + 1)


/*
 * interrupt bit positions
 */
/* return interrupt bit position */
#define VIC_INT_BIT(x)			((x) % NR_VIC_IRQS)

/* bit position */
/* VIC 1 */
#define IRQMASK_MAILBOX			(1 << VIC_INT_BIT(IRQ_MAILBOX))
#define IRQMASK_SMAC			(1 << VIC_INT_BIT(IRQ_SMAC))
#define IRQMASK_PCIE0			(1 << VIC_INT_BIT(IRQ_PCIE0))
#define IRQMASK_A11_DMAC		(1 << VIC_INT_BIT(IRQ_A11_DMAC))
#define IRQMASK_PCIE_DMAC		(1 << VIC_INT_BIT(IRQ_PCIE_DMAC))
#define IRQMASK_SYSTMR			(1 << VIC_INT_BIT(IRQ_SYSTMR))
#define IRQMASK_TIMER2_0		(1 << VIC_INT_BIT(IRQ_TIMER2_0))
#define IRQMASK_TIMER2_1		(1 << VIC_INT_BIT(IRQ_TIMER2_1))
#define IRQMASK_TIMER3_0		(1 << VIC_INT_BIT(IRQ_TIMER3_0))
#define IRQMASK_TIMER3_1		(1 << VIC_INT_BIT(IRQ_TIMER3_1))
#define IRQMASK_WDT0			(1 << VIC_INT_BIT(IRQ_WDT0))
#define IRQMASK_WDT1			(1 << VIC_INT_BIT(IRQ_WDT1))
#define IRQMASK_SPI2			(1 << VIC_INT_BIT(IRQ_SPI2))
#define IRQMASK_GPIOF			(1 << VIC_INT_BIT(IRQ_GPIOF))
#define IRQMASK_I2C1			(1 << VIC_INT_BIT(IRQ_I2C1))
#define IRQMASK_SMACPMT			(1 << VIC_INT_BIT(IRQ_SMACPMT))
#define IRQMASK_UART3			(1 << VIC_INT_BIT(IRQ_UART3))
#define IRQMASK_UART0			(1 << VIC_INT_BIT(IRQ_UART0))
#define IRQMASK_FEC			(1 << VIC_INT_BIT(IRQ_FEC))
#define IRQMASK_CRYPTO			(1 << VIC_INT_BIT(IRQ_CRYPTO))
#define IRQMASK_PMU			(1 << VIC_INT_BIT(IRQ_PMU))
#define IRQMASK_DDR_VRAM		(1 << VIC_INT_BIT(IRQ_DDR_VRAM))
#define IRQMASK_DDR_SYSRAM		(1 << VIC_INT_BIT(IRQ_DDR_SYSRAM))
#define IRQMASK_SPI0			(1 << VIC_INT_BIT(IRQ_SPI0))
/* VIC 2 */
#define IRQMASK_SPI1			(1 << VIC_INT_BIT(IRQ_SPI1))
#define IRQMASK_GPIOA			(1 << VIC_INT_BIT(IRQ_GPIOA))
#define IRQMASK_UART1			(1 << VIC_INT_BIT(IRQ_UART1))
#define IRQMASK_SPI3			(1 << VIC_INT_BIT(IRQ_SPI3))
#define IRQMASK_H264ENCODER		(1 << VIC_INT_BIT(IRQ_H264ENCODER))
#define IRQMASK_JPEG			(1 << VIC_INT_BIT(IRQ_JPEG))
#define IRQMASK_AUDIODMAC		(1 << VIC_INT_BIT(IRQ_AUDIODMAC))
#define IRQMASK_AUDIOIF			(1 << VIC_INT_BIT(IRQ_AUDIOIF))
#define IRQMASK_POST			(1 << VIC_INT_BIT(IRQ_POST))
#define IRQMASK_UART2			(1 << VIC_INT_BIT(IRQ_UART2))
#define IRQMASK_INTELLI			(1 << VIC_INT_BIT(IRQ_INTELLI))
#define IRQMASK_I2C0			(1 << VIC_INT_BIT(IRQ_I2C0))
#define IRQMASK_NANDIF			(1 << VIC_INT_BIT(IRQ_NANDIF))
#define IRQMASK_SD_CH0			(1 << VIC_INT_BIT(IRQ_SD_CH0))
#define IRQMASK_USBH_EHCI		(1 << VIC_INT_BIT(IRQ_USBH_EHCI))
#define IRQMASK_USBH_OHCI		(1 << VIC_INT_BIT(IRQ_USBH_OHCI))
#define IRQMASK_USBH_SYSCTRL		(1 << VIC_INT_BIT(IRQ_USBH_SYSCTRL))
#define IRQMASK_USBD_SYSCTRL		(1 << VIC_INT_BIT(IRQ_USBD_SYSCTRL))
#define IRQMASK_USBD_UDC		(1 << VIC_INT_BIT(IRQ_USBD_UDC))
#define IRQMASK_COMM_RX			(1 << VIC_INT_BIT(IRQ_COMM_RX))
#define IRQMASK_COMM_TX			(1 << VIC_INT_BIT(IRQ_COMM_TX))

/* valid irq mask */
#define PL190_VIC1_VALID_IRQ_MASK	( IRQMASK_MAILBOX	\
					| IRQMASK_SMAC		\
					| IRQMASK_PCIE0		\
					| IRQMASK_A11_DMAC	\
					| IRQMASK_PCIE_DMAC	\
					| IRQMASK_SYSTMR	\
					| IRQMASK_TIMER2_0	\
					| IRQMASK_TIMER2_1	\
					| IRQMASK_TIMER3_0	\
					| IRQMASK_TIMER3_1	\
					| IRQMASK_WDT0		\
					| IRQMASK_WDT1		\
					| IRQMASK_SPI2		\
					| IRQMASK_GPIOF		\
					| IRQMASK_I2C1		\
					| IRQMASK_SMACPMT	\
					| IRQMASK_UART3		\
					| IRQMASK_UART0		\
					| IRQMASK_FEC		\
					| IRQMASK_CRYPTO	\
					| IRQMASK_PMU		\
					| IRQMASK_DDR_VRAM	\
					| IRQMASK_DDR_SYSRAM	\
					| IRQMASK_SPI0		\
					)

#define PL190_VIC2_VALID_IRQ_MASK	( IRQMASK_SPI1		\
					| IRQMASK_GPIOA		\
					| IRQMASK_UART1		\
					| IRQMASK_SPI3		\
					| IRQMASK_H264ENCODER	\
					| IRQMASK_JPEG		\
					| IRQMASK_AUDIODMAC	\
					| IRQMASK_AUDIOIF	\
					| IRQMASK_POST		\
					| IRQMASK_UART2		\
					| IRQMASK_INTELLI	\
					| IRQMASK_I2C0		\
					| IRQMASK_NANDIF	\
					| IRQMASK_SD_CH0	\
					| IRQMASK_USBH_EHCI	\
					| IRQMASK_USBH_OHCI	\
					| IRQMASK_USBH_SYSCTRL	\
					| IRQMASK_USBD_SYSCTRL	\
					| IRQMASK_USBD_UDC	\
					| IRQMASK_COMM_RX	\
					| IRQMASK_COMM_TX	\
					)

#define PCIE_VALID_IRQ_MASK			0xFFFFFFFF

#endif
