/* 2011-10-03: File added and changed by Sony Corporation */
/*
 * arch/arm/mach-xarina/include/mach/platform.h
 *
 * Copyright 2010-2011 Sony Corporation
 * Copyright (c) ARM Limited 2003.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __ASM_ARCH_PLATFORM_H
#define __ASM_ARCH_PLATFORM_H

/*
 *  Memory Map
 */
#define XARINA_NOR_PHYS_BASE			0x20000000
#define XARINA_NOR_SIZE				SZ_64M /* NOR (S29GL512N90 16bitx1) */

/* SMSC LAN9118 */
#define XARINA_SMSC9118_PHYS_BASE		0x28000000
#define XARINA_SMSC9118_SIZE			SZ_128K

/* APB (0x30000000 - 0x3001FFFF) */
#define XARINA_APB_PHYS_BASE			0x30000000 /* physical address */
#define XARINA_APB_PHYS(offset)			(XARINA_APB_PHYS_BASE + (offset))
#define XARINA_APB_SIZE				SZ_128K

/* AHB (0x38000000 - 0x38FFFFFF) */
#define XARINA_AHB_PHYS_BASE			0x38000000 /* physical address */
#define XARINA_AHB_PHYS(offset)			(XARINA_AHB_PHYS_BASE + (offset))
#define XARINA_AHB_SIZE				SZ_16M

/* OCP0 (0x40000000 - 0x40009FFF) */
#define XARINA_OCP0_PHYS_BASE			0x40000000 /* physical address */
#define XARINA_OCP0_PHYS(offset)		(XARINA_OCP0_PHYS_BASE + (offset))
#define XARINA_OCP0_SIZE			SZ_64K

/* OCP1 (0x41000000 - 0x410FFFFF) */
#define XARINA_OCP1_PHYS_BASE			0x41000000 /* physical address */
#define XARINA_OCP1_PHYS(offset)		(XARINA_OCP1_PHYS_BASE + (offset))
#define XARINA_OCP1_SIZE			SZ_1M

/* ARM9 AHB (0x51000000 - 0x51FFFFFF) */
#define XARINA_ARM9_AHB_PHYS_BASE		0x51000000 /* physical address */
#define XARINA_ARM9_AHB_PHYS(offset)		(XARINA_ARM9_AHB_PHYS_BASE + (offset))
#define XARINA_ARM9_AHB_SIZE			SZ_16M

/* ARM9 POST (0x52000000 - 0x520FFFFF) */
#define XARINA_ARM9_POST_PHYS_BASE		0x52000000 /* physical address */
#define XARINA_ARM9_POST_SIZE			SZ_1M

/* ARM9 Audio (0x54000000 - 0x543FFFFF) */
#define XARINA_ARM9_AUDIO_PHYS_BASE		0x54000000 /* physical address */
#define XARINA_ARM9_AUDIO_PHYS(offset)		(XARINA_ARM9_AUDIO_PHYS_BASE + (offset))
#define XARINA_ARM9_AUDIO_SIZE			SZ_4M

/* DDR DRAM (HY5MS7B6BLFP) */
#define XARINA_DDR_PHYS_BASE			0x60000000
#define XARINA_DDR_SIZE				SZ_256M

/* PCI-Express 0 */
#define XARINA_PCIE0_ENDPOINT_PHYS_BASE		0x70000000
#define XARINA_PCIE0_ENDPOINT_SIZE		SZ_128M

/* VRAM */
#define XARINA_VRAM_PHYS_BASE			0xC0000000
#define XARINA_VRAM_SIZE			SZ_256M

/* Intelli */
#define XARINA_INTELLI_PHYS_BASE		0xD0000000
#define XARINA_INTELLI_SIZE			SZ_128M

/*
 * Peripheral Base Address
 */
/* SYSREG1 */
#define XARINA_SYSREG1_OFFSET			0x0000
#define XARINA_SYSREG1_PHYS_BASE		XARINA_APB_PHYS(XARINA_SYSREG1_OFFSET)
#define XARINA_SYSREG1_SIZE			SZ_4K
/* registers */
#define XARINA_SYSREG1_PLL5_CTRL			0x10
#define XARINA_SYSREG1_RST_CTRL				0x030
#define XARINA_SYSREG1_PLL_POWER_567			0x20
 #define XARINA_SYSREG1_PLL_POWER_567_5_PD		(0x1 << 0)
 #define XARINA_SYSREG1_PLL_POWER_567_5_OUT		(0x1 << 1)
#define XARINA_SYSREG1_PLL_READY			0x24
 #define XARINA_SYSREG1_PLL_READY_PLL5			(0x1 << 4)
 #define XARINA_SYSREG1_RST_CTRL_GBE			(0x1 << 1)
 #define XARINA_SYSREG1_RST_CTRL_PCIELINK		(0x1 << 2)
 #define XARINA_SYSREG1_RST_CTRL_PCIE			(0x1 << 3)
#define XARINA_SYSREG1_CLK_CTRL				0x034
 #define XARINA_SYSREG1_CLK_CTRL_GBE_1000		(0x0 << 12)
 #define XARINA_SYSREG1_CLK_CTRL_GBE_100		(0x1 << 12)
 #define XARINA_SYSREG1_CLK_CTRL_GBE_10			(0x2 << 12)
 #define XARINA_SYSREG1_CLK_CTRL_GBE_MII		(0x3 << 12)
#define XARINA_SYSREG1_CLK_EN				0x038
 #define XARINA_SYSREG1_CLK_EN_USBH_CKON		(0x1 << 2)
 #define XARINA_SYSREG1_CLK_EN_GBE_CKON			(0x1 << 4)
 #define XARINA_SYSREG1_CLK_EN_PCIE_CKON		(0x1 << 5)
 #define XARINA_SYSREG1_CLK_EN_SDIF_CKON		(0x1 << 6)
 #define XARINA_SYSREG1_CLK_EN_GBEEXT_EN		(0x1 << 9)
#define XARINA_SYSREG1_MODE_SEL				0x040
 #define XARINA_SYSREG1_MODE_SEL_GBEMODE_RGMII		(0x1 << 0)
 #define XARINA_SYSREG1_MODE_SEL_GBEMODE_MII		(0x0 << 0)
#define XARINA_SYSREG1_FUNC_SEL1			0x048
 #define XARINA_SYSREG1_FUNC_SEL1_USB_DEVSEL		(0x1 << 0)
 #define XARINA_SYSREG1_FUNC_SEL1_XSDCD_MD		(0x1 << 1)	/* SD noise filter */
#define XARINA_SYSREG1_PORT_MON				0x04C
#define XARINA_SYSREG1_PORT_DRV				0x050
#define XARINA_SYSREG1_DEVICE_ID_0704			0x104
#define XARINA_SYSREG1_DEVICE_ID_1108			0x108
#define XARINA_SYSREG1_CAM_GPIOB_SEL			0x200
#define XARINA_SYSREG1_VIDEO_GPIOC_SVI_SEL		0x208
#define XARINA_SYSREG1_GPIO_BE_SEL			0x20C
#define XARINA_SYSREG1_GPIO_G_SEL			0x210
#define XARINA_SYSREG1_GPIO_HI_SEL			0x214
#define XARINA_SYSREG1_NAND_GPIOI_SEL			0x204
#define XARINA_SYSREG1_CAM_PC				0x300
#define XARINA_SYSREG1_ETHER_NAND_PC			0x304
#define XARINA_SYSREG1_GPIO_ABC_PC			0x308
#define XARINA_SYSREG1_GPIO_EFGH_PC			0x30c
#define XARINA_SYSREG1_GPIO_I_PC			0x310
#define XARINA_SYSREG1_USB_ULPI_PC			0x314
#define XARINA_SYSREG1_DEVICE_ID			0x700
#define XARINA_SYSREG1_PCIE_PWC			0x708
 #define XARINA_SYSREG1_PCIE_PWC_INT_EN_PCIE			(0x1 << 1)
#define XARINA_SYSREG1_USB_PWC			0x70c
 #define XARINA_SYSREG1_USB_PWC_INT_EN_EHCI			(0x1 << 1)
 #define XARINA_SYSREG1_USB_PWC_INT_EN_OHCI			(0x1 << 9)


/* I2C1*/
#define XARINA_I2C1_OFFSET			0x3000
#define XARINA_I2C1_PHYS_BASE			XARINA_APB_PHYS(XARINA_I2C1_OFFSET)
#define XARINA_I2C1_SIZE			SZ_4K

/* I2C1 BUFFER_RAM */
#define XARINA_I2C1_BUFFER_RAM_OFFSET		0x4000
#define XARINA_I2C1_BUFFER_RAM_PHYS_BASE	XARINA_APB_PHYS(XARINA_I2C1_BUFFER_RAM_OFFSET)
#define XARINA_I2C1_BUFFER_RAM_SIZE		SZ_4K

/* UART3 */
#define XARINA_UART3_OFFSET			0x7000
#define XARINA_UART3_PHYS_BASE			XARINA_APB_PHYS(XARINA_UART3_OFFSET)
#define XARINA_UART3_SIZE			SZ_4K

/* SPI2 */
#define XARINA_SPI2_OFFSET			0x8000
#define XARINA_SPI2_PHYS_BASE			XARINA_APB_PHYS(XARINA_SPI2_OFFSET)
#define XARINA_SPI2_SIZE			SZ_4K

/* GP_TIMER2 */
#define XARINA_GP_TIMER2_OFFSET			0x9000
#define XARINA_GP_TIMER2_PHYS_BASE		XARINA_APB_PHYS(XARINA_GP_TIMER2_OFFSET)
#define XARINA_GP_TIMER2_SIZE			SZ_4K

/* GP_TIMER3 */
#define XARINA_GP_TIMER3_OFFSET			0xA000
#define XARINA_GP_TIMER3_PHYS_BASE		XARINA_APB_PHYS(XARINA_GP_TIMER3_OFFSET)
#define XARINA_GP_TIMER3_SIZE			SZ_4K

/* SPI3 */
#define XARINA_SPI3_OFFSET			0xC000
#define XARINA_SPI3_PHYS_BASE			XARINA_APB_PHYS(XARINA_SPI3_OFFSET)
#define XARINA_SPI3_SIZE			SZ_4K

/* WDT1 */
#define XARINA_WDT1_OFFSET			0xB000
#define XARINA_WDT1_PHYS_BASE			XARINA_APB_PHYS(XARINA_WDT1_OFFSET)
#define XARINA_WDT1_SIZE			SZ_4K

/* PCIeDMAC */
#define XARINA_PCIE_DMAC_OFFSET			0xD000
#define XARINA_PCIE_DMAC_PHYS_BASE		XARINA_APB_PHYS(XARINA_PCIE_DMAC_OFFSET)
#define XARINA_PCIE_DMAC_SIZE			SZ_4K

/* UART2 */
#define XARINA_UART2_OFFSET			0xE000
#define XARINA_UART2_PHYS_BASE			XARINA_APB_PHYS(XARINA_UART2_OFFSET)
#define XARINA_UART2_SIZE			SZ_4K

/* GPIO5 (port F) */
#define XARINA_GPIOF_OFFSET			0x10000
#define XARINA_GPIOF_PHYS_BASE			XARINA_APB_PHYS(XARINA_GPIOF_OFFSET)
#define XARINA_GPIOF_SIZE			SZ_4K

/* GPIO6 (port G) */
#define XARINA_GPIOG_OFFSET			0x11000
#define XARINA_GPIOG_PHYS_BASE			XARINA_APB_PHYS(XARINA_GPIOG_OFFSET)
#define XARINA_GPIOG_SIZE			SZ_4K

/* GPIO7 (port H) */
#define XARINA_GPIOH_OFFSET			0x12000
#define XARINA_GPIOH_PHYS_BASE			XARINA_APB_PHYS(XARINA_GPIOH_OFFSET)
#define XARINA_GPIOH_SIZE			SZ_4K

/* GPIO8 (port I) */
#define XARINA_GPIOI_OFFSET			0x13000
#define XARINA_GPIOI_PHYS_BASE			XARINA_APB_PHYS(XARINA_GPIOI_OFFSET)
#define XARINA_GPIOI_SIZE			SZ_4K

/* DDR PHY 0 */
#define XARINA_DDRPHY0_OFFSET			0x14000
#define XARINA_DDRPHY0_PHYS_BASE		XARINA_APB_PHYS(XARINA_DDRPHY0_OFFSET)
#define XARINA_DDRPHY0_SIZE			SZ_4K

/* DDR PHY 1 */
#define XARINA_DDRPHY1_OFFSET			0x15000
#define XARINA_DDRPHY1_PHYS_BASE		XARINA_APB_PHYS(XARINA_DDRPHY1_OFFSET)
#define XARINA_DDRPHY1_SIZE			SZ_4K

/* SYSTEM_TIMER */
#define XARINA_SYSTEM_TIMER_OFFSET		0x0000
#define XARINA_SYSTEM_TIMER_PHYS_BASE		XARINA_AHB_PHYS(XARINA_SYSTEM_TIMER_OFFSET)
#define XARINA_SYSTEM_TIMER_SIZE		SZ_4K

/* TICS */
#define XARINA_TICS_OFFSET			0x1000
#define XARINA_TICS_PHYS_BASE			XARINA_AHB_PHYS(XARINA_TICS_OFFSET)
#define XARINA_TICS_SIZE			SZ_4K

/* EMC_SM2 */
#define XARINA_EMC_SM2_OFFSET			0x2000
#define XARINA_EMC_SM2_PHYS_BASE		XARINA_AHB_PHYS(XARINA_EMC_SM2_OFFSET)
#define XARINA_EMC_SM2_SIZE			SZ_4K

/* A11_DMAC */
#define XARINA_A11_DMAC_OFFSET			0x4000
#define XARINA_A11_DMAC_PHYS_BASE		XARINA_AHB_PHYS(XARINA_A11_DMAC_OFFSET)
#define XARINA_A11_DMAC_SIZE			SZ_4K

/* NAND IF */
#define XARINA_NANDIF_OFFSET			0x6000
#define XARINA_NANDIF_PHYS_BASE			XARINA_AHB_PHYS(XARINA_NANDIF_OFFSET)
#define XARINA_NANDIF_SIZE			SZ_4K

/* VIC1 */
#define XARINA_VIC1_OFFSET			0x8000
#define XARINA_VIC1_PHYS_BASE			XARINA_AHB_PHYS(XARINA_VIC1_OFFSET)
#define XARINA_VIC1_SIZE			SZ_4K

/* VIC2 */
#define XARINA_VIC2_OFFSET			0x9000
#define XARINA_VIC2_PHYS_BASE			XARINA_AHB_PHYS(XARINA_VIC2_OFFSET)
#define XARINA_VIC2_SIZE			SZ_4K

/* SDIF0 */
#define XARINA_SDIF0_OFFSET			0xD000
#define XARINA_SDIF0_PHYS_BASE			XARINA_AHB_PHYS(XARINA_SDIF0_OFFSET)
#define XARINA_SDIF0_SIZE			SZ_4K

/* DDR0 */
#define XARINA_DDR0_OFFSET			0x11000
#define XARINA_DDR0_PHYS_BASE			XARINA_AHB_PHYS(XARINA_DDR0_OFFSET)
#define XARINA_DDR0_SIZE			SZ_4K

/* DRM */
#define XARINA_DRM_OFFSET			0x100000
#define XARINA_DRM_PHYS_BASE			XARINA_AHB_PHYS(XARINA_DRM_OFFSET)
#define XARINA_DRM_SIZE				SZ_256K

/* GbE */
#define XARINA_GBE_OFFSET			0x2000
#define XARINA_GBE_PHYS_BASE			XARINA_OCP0_PHYS(XARINA_GBE_OFFSET)
#define XARINA_GBE_SIZE				SZ_8K

/* PCI-Express 0 REG */
#define XARINA_PCIE0_OFFSET			0x4000
#define XARINA_PCIE0_PHYS_BASE			XARINA_OCP0_PHYS(XARINA_PCIE0_OFFSET)
#define XARINA_PCIE0_SIZE			SZ_8K

/* FEC */
#define XARINA_FEC_OFFSET			0x9000
#define XARINA_FEC_PHYS_BASE			XARINA_OCP0_PHYS(XARINA_FEC_OFFSET)
#define XARINA_FEC_SIZE				SZ_4K

/* USB Host OHCI */
#define XARINA_USBH_OHCI_OFFSET			0xA000
#define XARINA_USBH_OHCI_PHYS_BASE		XARINA_OCP0_PHYS(XARINA_USBH_OHCI_OFFSET)
#define XARINA_USBH_OHCI_SIZE			SZ_4K

/* USB Host EHCI */
#define XARINA_USBH_EHCI_OFFSET			0xB000
#define XARINA_USBH_EHCI_PHYS_BASE		XARINA_OCP0_PHYS(XARINA_USBH_EHCI_OFFSET)
#define XARINA_USBH_EHCI_SIZE			SZ_4K

/* USB Host SYSREG */
#define XARINA_USBH_SYSREG_OFFSET		0xC000
#define XARINA_USBH_SYSREG_PHYS_BASE		XARINA_OCP0_PHYS(XARINA_USBH_SYSREG_OFFSET)
#define XARINA_USBH_SYSREG_SIZE			SZ_4K

/* USB Device OHCI/EHCI */
#define XARINA_USBD_HCI_OFFSET			0xD000
#define XARINA_USBD_HCI_PHYS_BASE		XARINA_OCP0_PHYS(XARINA_USBD_HCI_OFFSET)
#define XARINA_USBD_HCI_SIZE			SZ_4K

/* USB Device SYSREG */
#define XARINA_USBD_SYSREG_OFFSET		0xE000
#define XARINA_USBD_SYSREG_PHYS_BASE		XARINA_OCP0_PHYS(XARINA_USBD_SYSREG_OFFSET)
#define XARINA_USBD_SYSREG_SIZE			SZ_4K

/* GPIO 0 (Port A) */
#define XARINA_GPIOA_OFFSET			0x18000
#define XARINA_GPIOA_PHYS_BASE			XARINA_ARM9_AHB_PHYS(XARINA_GPIOA_OFFSET)
#define XARINA_GPIOA_SIZE			SZ_4K

/* GPIO 1 (Port B) */
#define XARINA_GPIOB_OFFSET			0x19000
#define XARINA_GPIOB_PHYS_BASE			XARINA_ARM9_AHB_PHYS(XARINA_GPIOB_OFFSET)
#define XARINA_GPIOB_SIZE			SZ_4K

/* GPIO 2 (Port C) */
#define XARINA_GPIOC_OFFSET			0x1A000
#define XARINA_GPIOC_PHYS_BASE			XARINA_ARM9_AHB_PHYS(XARINA_GPIOC_OFFSET)
#define XARINA_GPIOC_SIZE			SZ_4K

/* GPIO 4 (Port E) */
#define XARINA_GPIOE_OFFSET			0x1C000
#define XARINA_GPIOE_PHYS_BASE			XARINA_ARM9_AHB_PHYS(XARINA_GPIOE_OFFSET)
#define XARINA_GPIOE_SIZE			SZ_4K

/* ARM9 AHB */
/* I2C0 */
#define XARINA_I2C0_OFFSET			0x12000
#define XARINA_I2C0_PHYS_BASE			XARINA_ARM9_AHB_PHYS(XARINA_I2C0_OFFSET)
#define XARINA_I2C0_SIZE			SZ_4K

/* I2C0 BUFFER_RAM */
#define XARINA_I2C0_BUFFER_RAM_OFFSET		0x13000
#define XARINA_I2C0_BUFFER_RAM_PHYS_BASE	XARINA_ARM9_AHB_PHYS(XARINA_I2C0_BUFFER_RAM_OFFSET)
#define XARINA_I2C0_BUFFER_RAM_SIZE		SZ_4K

/* UART0 */
#define XARINA_UART0_OFFSET			0x4000
#define XARINA_UART0_PHYS_BASE			XARINA_ARM9_AHB_PHYS(XARINA_UART0_OFFSET)
#define XARINA_UART0_SIZE			SZ_4K

/* UART1 */
#define XARINA_UART1_OFFSET			0x5000
#define XARINA_UART1_PHYS_BASE			XARINA_ARM9_AHB_PHYS(XARINA_UART1_OFFSET)
#define XARINA_UART1_SIZE			SZ_4K

/*
 * virtual address mapping
 */
#define XARINA_VIRT_BASE			0xF0000000
#define __XARINA_VIRT_BASE(n)			(XARINA_VIRT_BASE + (n * 0x00100000 /* 1MB*/ ))
#define XARINA_SYSREG1_VIRT_BASE		__XARINA_VIRT_BASE(0)
#define XARINA_UART3_VIRT_BASE			__XARINA_VIRT_BASE(1)
#define XARINA_GP_TIMER2_VIRT_BASE		__XARINA_VIRT_BASE(2)
#define XARINA_WDT1_VIRT_BASE			__XARINA_VIRT_BASE(3)
#define XARINA_VIC1_VIRT_BASE			__XARINA_VIRT_BASE(4)
#define XARINA_VIC2_VIRT_BASE			__XARINA_VIRT_BASE(5)

#endif

