/* 2012-04-06: File added by Sony Corporation */
/*
 * linux/arch/arm/mach-xarina/usb/sysreg_usbh.c
 *
 * functions for usb system register.
 *
 * Copyright 2012 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA.
 *
 */
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/spinlock.h>
#include <linux/io.h>
#include <mach/platform.h>
#include <linux/interrupt.h>
#include <linux/module.h>

#include "suh_defs.h"
#include "sysreg_usbh.h"

static spinlock_t usbh_sysreg_lock = SPIN_LOCK_UNLOCKED;

/* _sysreg_readl - read sysreg register value */
static inline u32 _usbh_sysreg_readl(u32 offset, u32 __iomem base)
{
	return __raw_readl(base + offset);
}

/* _usbh_sysreg_writel - write usbh sysreg register value */
static inline void _usbh_sysreg_writel(u32 val, u32 offset, u32 __iomem base)
{
	__raw_writel(val, base + offset);
}

/* write access for usbh sysregs */
void xarina_usbh_sysreg_writel(u32 val, u32 offset, u32 __iomem base)
{
	unsigned long flags = 0;
	int lock_done =0;

	if (!in_interrupt()) {
		spin_lock_irqsave(&usbh_sysreg_lock, flags);
		lock_done = 1;
	} else
		spin_lock(&usbh_sysreg_lock);

	_usbh_sysreg_writel(val, offset, base);

	if (lock_done)
		spin_unlock_irqrestore(&usbh_sysreg_lock, flags);
	else
		spin_unlock(&usbh_sysreg_lock);
}
EXPORT_SYMBOL(xarina_usbh_sysreg_writel);

/* xarina_usbh_sysreg_readl - read usbh sysreg register value */
u32 xarina_usbh_sysreg_readl(u32 offset, u32 __iomem base)
{
	return _usbh_sysreg_readl(offset, base);
}
EXPORT_SYMBOL(xarina_usbh_sysreg_readl);

