/* 2011-10-03: File added and changed by Sony Corporation */
/*
 *  arch/arm/mach-xarina/xarina_evb.c
 *
 *  Copyright 2010-2012 Sony Corporation
 *  Copyright (C) 2004 ARM Limited
 *  Copyright (C) 2000 Deep Blue Solutions Ltd
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/init.h>
#include <linux/device.h>
#include <linux/sysdev.h>
#include <linux/smsc911x.h>
#include <linux/platform_device.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/partitions.h>
#include <linux/mtd/nand.h>
#include <linux/proc_fs.h>

#include <asm/io.h>
#include <asm/irq.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/flash.h>
#include <asm/mach/map.h>

#include <mach/hardware.h>

#include "core.h"
#include "sysreg.h"


/*
 * kernel parameter
 */
/* GPIO dont use ports */
#define XARINA_GPIO_NR		8 /* GPIO A,B,C,E,F,G,H,I */
static int gpio_careports[XARINA_GPIO_NR+1];
enum { GPIO_SIZE, GPIOA, GPIOB, GPIOC, GPIOE, GPIOF, GPIOG, GPIOH, GPIOI };
static int xarina_gpio_setup(char *str)
{
	get_options(str, ARRAY_SIZE(gpio_careports), gpio_careports);
	return 1;
}
__setup("gpio_careports=", xarina_gpio_setup);


/* read_proc_gpio_careports - procfs read handler /proc/xarina/gpio_careports */
static int xarina_read_proc_gpio_careports(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"GPIOA:%02x\n", gpio_careports[GPIOA]);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"GPIOB:%02x\n", gpio_careports[GPIOB]);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"GPIOC:%02x\n", gpio_careports[GPIOC]);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"GPIOE:%02x\n", gpio_careports[GPIOE]);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"GPIOF:%02x\n", gpio_careports[GPIOF]);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"GPIOG:%02x\n", gpio_careports[GPIOG]);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"GPIOH:%02x\n", gpio_careports[GPIOH]);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"GPIOI:%02x\n", gpio_careports[GPIOI]);

	*eof = 1;

	return len;
}


/* read_proc_gpio_initports - procfs read handler /proc/xarina/gpio_initports */
static int xarina_read_proc_gpio_initports(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	const enum xarina_board_t board = xarina_board_type();

	if (board == XARINA_EVB_BOARD) {
		len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
						"GPIOA:ff\n");
		len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
						"GPIOB:ff\n");
		len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
						"GPIOC:ff\n");
		len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
						"GPIOE:f0\n");
		len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
						"GPIOF:00\n");
		len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
						"GPIOG:ff\n");
		len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
						"GPIOH:ff\n");
		len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
						"GPIOI:ff\n");
	} else if (board == XARINA_RD_BOARD) {
		len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
						"GPIOA:ff\n");
		len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
						"GPIOB:ff\n");
		len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
						"GPIOC:ff\n");
		len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
						"GPIOE:f0\n");
		len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
						"GPIOF:00\n");
		len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
						"GPIOG:ff\n");
		len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
						"GPIOH:ff\n");
		len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
						"GPIOI:ff\n");
	}

	*eof = 1;

	return len;
}


/* xarina_read_proc_pinmux - procfs read handler /proc/xarina/pinmux */
static int xarina_read_proc_pinmux(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	struct xarina_pinmux *pinmux = data;

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"SPI2:%d\n", pinmux->spi2);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"SPI3:%d\n", pinmux->spi3);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"UART0:%d\n", pinmux->uart0);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"UART1:%d\n", pinmux->uart1);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"UART2:%d\n", pinmux->uart2);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"UART3:%d\n", pinmux->uart3);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"SDIO:%d\n", pinmux->sd);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PWM0:%d\n", pinmux->pwm0);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PWM1:%d\n", pinmux->pwm1);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"NCE1:%d\n", (pinmux->nand_nr == 1) ? 0 : 1);
#ifdef CONFIG_XARINA_DEBUG
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"GPIOA:%02X\n", pinmux->gpioa);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"GPIOB:%02X\n", pinmux->gpiob);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"GPIOC:%02X\n", pinmux->gpioc);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"GPIOE:%02X\n", pinmux->gpioe);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"GPIOF:%02X\n", pinmux->gpiof);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"GPIOG:%02X\n", pinmux->gpiog);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"GPIOH:%02X\n", pinmux->gpioh);
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"GPIOI:%02X\n", pinmux->gpioi);
#endif

	*eof = 1;

	return len;
}

static int xarina_read_proc_sysreg1_pll5_ctrl(char *page, char **start, off_t off,
											  int count, int *eof, void *data)
{
	int len = 0;
	const u32 val = xarina_sysreg1_readl(XARINA_SYSREG1_PLL5_CTRL);

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PLL5_BYPASS=%02x\n", (val & 0x00000001));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PLL5_CPSEL=%02x\n", ((val & 0x00000006) >> 1));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PLL5_REFDIVR=%02x\n", ((val & 0x00000018) >> 3));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PLL5_HGAIN=%02x\n", ((val & 0x00000020) >> 5));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PLL5_FBDIVR=%02x\n", ((val & 0x0000ff00) >> 8));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PLL5_OUTDIVR=%02x\n", ((val & 0x00ff0000) >> 16));

	*eof = 1;

	return len;
}

static int xarina_read_proc_sysreg1_pll_power_567(char *page, char **start, off_t off,
												  int count, int *eof, void *data)
{
	int len = 0;
	const u32 val = xarina_sysreg1_readl(XARINA_SYSREG1_PLL_POWER_567);

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PLL5_PD_B_N=%02x\n", (val & 0x00000001));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PLL5_OUT_EN=%02x\n", ((val & 0x00000002) >> 1));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PLL6_PD_B_N=%02x\n", ((val & 0x00000100) >> 8));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PLL6_OUT_EN=%02x\n", ((val & 0x00000200) >> 9));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PLL7_PD_B_N=%02x\n", ((val & 0x00010000) >> 16));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PLL7_OUT_END=%02x\n", ((val & 0x00020000) >> 17));

	*eof = 1;

	return len;
}

static int xarina_read_proc_sysreg1_rst_ctrl(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	const u32 val = xarina_sysreg1_readl(XARINA_SYSREG1_RST_CTRL);

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"AVCC_RST_CTL_N=%02x\n", (val & 0x00000001));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"GBE_RST_CTL_N=%02x\n",	((val & 0x00000002)>>1));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PCIELINK_RST_CTL_N=%02x\n", ((val & 0x00000004)>>2));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"RST_PCIE_N=%02x\n", ((val & 0x00000008)>>3));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"DDR1_RST_CTL_N=%02x\n", ((val & 0x00000010)>>4));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"AVCC_ARM9_RST_CTL_N=%02x\n", ((val & 0x00000100)>>8));

	*eof = 1;

	return len;
}


static int xarina_read_proc_sysreg1_clk_ctrl(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	const u32 val = xarina_sysreg1_readl(XARINA_SYSREG1_CLK_CTRL);

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
				   "SYS_CKSEL=%02x\n", (val & 0x00000001));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
				   "SDIF_CKSEL=%02x\n", ((val & 0x00000008)>>3));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
				   "AUDIO_CKSEL=%02x\n", ((val & 0x00000010)>>4));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
				   "VCLK_CKSEL=%02x\n",	((val & 0x000000e0)>>5));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
				   "VCLK_POL=%02x\n", ((val & 0x00000100)>>8));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
				   "GBE_CKSEL=%02x\n", ((val & 0x00003000)>>12));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
				   "DDR0_CKSEL=%02x\n", ((val & 0x00004000)>>14));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
				   "DDR1_CKSEL=%02x\n", ((val & 0x00008000)>>15));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
				   "NFIF_CKSEL=%02x\n",	((val & 0x00070000)>>16));

	*eof = 1;

	return len;
}


static int xarina_read_proc_sysreg1_clk_en(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	const u32 val = xarina_sysreg1_readl(XARINA_SYSREG1_CLK_EN);

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"AVCAMCTL_CKON=%02x\n",	(val & 0x00000001));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"DDR1_CKON=%02x\n", ((val & 0x00000002)>>1));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"USBH_CKON=%02x\n",	((val & 0x00000004)>>2));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"USBD_CKON=%02x\n",	((val & 0x00000008)>>3));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"GBE_CKON=%02x\n", ((val & 0x00000010)>>4));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PCIE_CKON=%02x\n",	((val & 0x00000020)>>5));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"SDIF_CKON=%02x\n",	((val & 0x00000040)>>6));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"DRM_CKON=%02x\n", ((val & 0x00000080)>>7));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"AVA9_CKON=%02x\n",	((val & 0x00000100)>>8));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"GBEEXT_EN=%02x\n",	((val & 0x00000200)>>9));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"DDR0_CKON=%02x\n",	((val & 0x00000400)>>10));

	*eof = 1;

	return len;
}


static int xarina_read_proc_sysreg1_port_mon(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	const u32 val = xarina_sysreg1_readl(XARINA_SYSREG1_PORT_MON);

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CKSEL=%02x\n",	(val & 0x00000001));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"A9JTAGSEL=%02x\n",	((val & 0x0000000e)>>1));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"EVRINIT=%02x\n", ((val & 0x00000010)>>4));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"BOOTSEL=%02x\n", ((val & 0x00000060)>>5));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"RSTCFG_PC7=%02x\n", ((val & 0x00000080)>>7));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"RSTCFG_PA=%02x\n",	((val & 0x0000ff00)>>8));

	*eof = 1;

	return len;
}


static int xarina_read_proc_sysreg1_port_drv(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	const u32 val = xarina_sysreg1_readl(XARINA_SYSREG1_PORT_DRV);

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMHVSYNC_DRV=%02x\n",	(val & 0x00000001));

	*eof = 1;

	return len;
}


static int xarina_read_proc_sysreg1_device_id_0704(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	const u32 val = xarina_sysreg1_readl(XARINA_SYSREG1_DEVICE_ID_0704);

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CID_Q_04=%02x\n", (val & 0x000000ff));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CID_Q_05=%02x\n", ((val & 0x0000ff00)>>8));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CID_Q_06=%02x\n", ((val & 0x00ff0000)>>16));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CID_Q_07=%02x\n", ((val & 0xff000000)>>24));

	*eof = 1;

	return len;
}


static int xarina_read_proc_sysreg1_device_id_1108(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	const u32 val = xarina_sysreg1_readl(XARINA_SYSREG1_DEVICE_ID_1108);

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CID_Q_08=%02x\n", (val & 0x000000ff));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CID_Q_09=%02x\n", ((val & 0x0000ff00)>>8));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CID_Q_10=%02x\n", ((val & 0x00ff0000)>>16));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CID_Q_11=%02x\n", ((val & 0xff000000)>>24));

	*eof = 1;

	return len;
}


static int xarina_read_proc_sysreg1_cam_gpiob_sel(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	const u32 val = xarina_sysreg1_readl(XARINA_SYSREG1_CAM_GPIOB_SEL);

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAM_GPIOB_SEL=%08x\n", val);

	*eof = 1;

	return len;
}


static int xarina_read_proc_sysreg1_nand_gpioi_sel(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	const u32 val = xarina_sysreg1_readl(XARINA_SYSREG1_NAND_GPIOI_SEL);

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PI6_NCE1_SEL=%02x\n", (val & 0x00000001));

	*eof = 1;

	return len;
}


static int xarina_read_proc_sysreg1_video_gpioc_svi_sel(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	const u32 val = xarina_sysreg1_readl(XARINA_SYSREG1_VIDEO_GPIOC_SVI_SEL);

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"VIDEOCLK_SEL=%02x\n", (val & 0x00000003));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PC0_VIDEOD0_SEL=%02x\n", ((val & 0x00000300)>>8));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PC1_VIDEOD1_SEL=%02x\n", ((val & 0x00000c00)>>10));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PC2_VIDEOD2_SEL=%02x\n", ((val & 0x00003000)>>12));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PC3_VIDEOD3_SEL=%02x\n", ((val & 0x0000c000)>>14));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PC4_VIDEOD4_SEL=%02x\n", ((val & 0x00030000)>>16));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PC5_VIDEOD5_SEL=%02x\n", ((val & 0x000c0000)>>18));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PC6_VIDEOD6_SEL=%02x\n", ((val & 0x00300000)>>20));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PC7_VIDEOD7_SEL=%02x\n", ((val & 0x00c00000)>>22));

	*eof = 1;

	return len;
}


static int xarina_read_proc_sysreg1_gpio_be_sel(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	const u32 val = xarina_sysreg1_readl(XARINA_SYSREG1_GPIO_BE_SEL);

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PB3_LRCK_SEL=%02x\n", ((val & 0x00000800)>>11));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PB4_XBCK_SEL=%02x\n", ((val & 0x00001000)>>12));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PB5_DATAIN_SEL=%02x\n", ((val & 0x00002000)>>13));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PB6_DATAOUT_SEL=%02x\n", ((val & 0x00004000)>>14));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PB7_MCLK_SEL=%02x\n", ((val & 0x00008000)>>15));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PE0_SO0_SEL=%02x\n", ((val & 0x00010000)>>16));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PE1_SI0_SEL=%02x\n", ((val & 0x00020000)>>17));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PE2_SCK0N_SEL=%02x\n", ((val & 0x00040000)>>18));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PE3_SFSS0_SEL=%02x\n", ((val & 0x00080000)>>19));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PE4_TXD0_SO1_SEL=%02x\n", ((val & 0x03000000)>>24));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PE5_RXD0_SI1_SEL=%02x\n", ((val & 0x0c000000)>>26));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PE6_UARTCTS0_SCK1N_SEL=%02x\n", ((val & 0x30000000)>>28));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PE7_UARTRTS0_SFSS1N_SEL=%02x\n",
					((val & 0xc0000000)>>30));

	*eof = 1;

	return len;
}


static int xarina_read_proc_sysreg1_gpio_g_sel(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	const u32 val = xarina_sysreg1_readl(XARINA_SYSREG1_GPIO_G_SEL);

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PG0_TXD1_SO2_SEL=%02x\n", ((val & 0x00000300)>>8));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PG1_RXD1_SI2_SEL=%02x\n", ((val & 0x00000c00)>>10));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PG2_UARTCTS1_SCK2N_SEL=%02x\n", ((val & 0x00003000)>>12));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PG3_UARTRTS1_SFSS2N_SEL=%02x\n",
					((val & 0x0000c000)>>14));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PG4_TXD3_SEL=%02x\n", ((val & 0x00010000)>>16));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PG5_RXD3_SEL=%02x\n", ((val & 0x00020000)>>17));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PG6_TXD2_SEL=%02x\n", ((val & 0x000c0000)>>18));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PG7_RXD2_SEL=%02x\n", ((val & 0x00300000)>>20));

	*eof = 1;

	return len;
}


static int xarina_read_proc_sysreg1_gpio_hi_sel(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	const u32 val = xarina_sysreg1_readl(XARINA_SYSREG1_GPIO_HI_SEL);

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PH0_SDCLK0_SEL=%02x\n", (val & 0x00000003));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PH1_SDCMD0_SEL=%02x\n", ((val & 0x0000000c)>>2));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PH2_XSDCD0_SEL=%02x\n", ((val & 0x00000030)>>4));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PH3_SDWP0_SEL=%02x\n", ((val & 0x000000c0)>>6));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PH4_SDDAT00_SEL=%02x\n", ((val & 0x00000300)>>8));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PH5_SDDAT01_SEL=%02x\n", ((val & 0x00000c00)>>10));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PH6_SDDAT02_SEL=%02x\n", ((val & 0x00003000)>>12));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PH7_SDDAT03_SEL=%02x\n", ((val & 0x0000c000)>>14));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PI0_SO3_SEL=%02x\n", ((val & 0x00010000)>>16));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PI1_SI3_SEL=%02x\n", ((val & 0x00040000)>>18));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PI2_SCK3N_SEL=%02x\n", ((val & 0x00100000)>>20));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PI3_SFSS3_SEL=%02x\n", ((val & 0x00400000)>>22));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PI4_PWM0_SEL=%02x\n", ((val & 0x03000000)>>24));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PI5_PWM1_SEL=%02x\n", ((val & 0x0c000000)>>26));

	*eof = 1;

	return len;
}


static int xarina_read_proc_sysreg1_cam_pc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	const u32 val = xarina_sysreg1_readl(XARINA_SYSREG1_CAM_PC);

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMDOUTCLK_PC=%02x\n", (val & 0x00000001));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMHSYNC_PC=%02x\n", ((val & 0x00000002)>>1));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMVSYNC_PC=%02x\n", ((val & 0x00000004)>>2));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMDINCLK_PC=%02x\n", ((val & 0x00000008)>>3));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMDIN0_PC=%02x\n", ((val & 0x00000010)>>4));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMDIN1_PC=%02x\n", ((val & 0x00000020)>>5));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMDIN2_PC=%02x\n", ((val & 0x00000040)>>6));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMDIN3_PC=%02x\n", ((val & 0x00000080)>>7));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMDIN4_PC=%02x\n", ((val & 0x00000100)>>8));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMDIN5_PC=%02x\n", ((val & 0x00000200)>>9));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMDIN6_PC=%02x\n", ((val & 0x00000400)>>10));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMDIN7_PC=%02x\n", ((val & 0x00000800)>>11));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMDIN8_PC=%02x\n", ((val & 0x00001000)>>12));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMDIN9_PC=%02x\n", ((val & 0x00002000)>>13));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMDIN10_PC=%02x\n", ((val & 0x00004000)>>14));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMDIN11_PC=%02x\n", ((val & 0x00008000)>>15));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMDIN12_PC=%02x\n", ((val & 0x00010000)>>16));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMDIN13_PC=%02x\n", ((val & 0x00020000)>>17));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMDIN14_PC=%02x\n", ((val & 0x00040000)>>18));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMDIN15_PC=%02x\n", ((val & 0x00080000)>>19));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMCSTG_PC=%02x\n", ((val & 0x00100000)>>20));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMCSAFE_PC=%02x\n", ((val & 0x00200000)>>21));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMLDEVR_PC=%02x\n", ((val & 0x00400000)>>22));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMSCK_PC=%02x\n", ((val & 0x00800000)>>23));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMSOUT_PC=%02x\n", ((val & 0x01000000)>>24));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMI2CSCL_PC=%02x\n", ((val & 0x02000000)>>25));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMI2CSDA_PC=%02x\n", ((val & 0x04000000)>>26));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMVIHSYNC_PC=%02x\n", ((val & 0x08000000)>>27));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMVIVSYNC_PC=%02x\n", ((val & 0x10000000)>>28));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMVALID_PC=%02x\n", ((val & 0x20000000)>>29));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"CAMXCLR_PC=%02x\n", ((val & 0x40000000)>>30));

	*eof = 1;

	return len;
}


static int xarina_read_proc_sysreg1_ether_nand_pc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	const u32 val = xarina_sysreg1_readl(XARINA_SYSREG1_ETHER_NAND_PC);

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"MDIO_PC=%02x\n", (val & 0x00000001));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"NRBN_PC=%02x\n", ((val & 0x00000100)>>8));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"NAD0_PC=%02x\n", ((val & 0x00000400)>>10));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"NAD1_PC=%02x\n", ((val & 0x00000800)>>11));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"NAD2_PC=%02x\n", ((val & 0x00001000)>>12));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"NAD3_PC=%02x\n", ((val & 0x00002000)>>13));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"NAD4_PC=%02x\n", ((val & 0x00004000)>>14));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"NAD5_PC=%02x\n", ((val & 0x00008000)>>15));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"NAD6_PC=%02x\n", ((val & 0x00010000)>>16));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"NAD7_PC=%02x\n", ((val & 0x00020000)>>17));

	*eof = 1;

	return len;
}


static int xarina_read_proc_sysreg1_gpio_abc_pc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	const u32 val = xarina_sysreg1_readl(XARINA_SYSREG1_GPIO_ABC_PC);

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PA0_PC=%02x\n", (val & 0x00000001));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PA1_PC=%02x\n", ((val & 0x00000002)>>1));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PA2_PC=%02x\n", ((val & 0x00000004)>>2));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PA3_PC=%02x\n", ((val & 0x00000008)>>3));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PA4_PC=%02x\n", ((val & 0x00000010)>>4));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PA5_PC=%02x\n", ((val & 0x00000020)>>5));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PA6_PC=%02x\n", ((val & 0x00000040)>>6));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PA7_PC=%02x\n", ((val & 0x00000080)>>7));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PB3_LRCK_PC=%02x\n", ((val & 0x00000800)>>11));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PB4_XBCK_PC=%02x\n", ((val & 0x00001000)>>12));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PB5_DATAIN_PC=%02x\n", ((val & 0x00002000)>>13));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PB6_DATAOUT_PC=%02x\n", ((val & 0x00004000)>>14));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PB7_MCLK_PC=%02x\n", ((val & 0x00008000)>>15));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PC0_VIDEOD0_PC=%02x\n", ((val & 0x00010000)>>16));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PC1_VIDEOD1_PC=%02x\n", ((val & 0x00020000)>>17));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PC2_VIDEOD2_PC=%02x\n", ((val & 0x00040000)>>18));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PC3_VIDEOD3_PC=%02x\n", ((val & 0x00080000)>>19));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PC4_VIDEOD4_PC=%02x\n", ((val & 0x00100000)>>20));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PC5_VIDEOD5_PC=%02x\n", ((val & 0x00200000)>>21));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PC6_VIDEOD6_PC=%02x\n", ((val & 0x00400000)>>22));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PC7_VIDEOD7_PC=%02x\n", ((val & 0x00800000)>>23));

	*eof = 1;

	return len;
}


static int xarina_read_proc_sysreg1_gpio_efgh_pc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	const u32 val = xarina_sysreg1_readl(XARINA_SYSREG1_GPIO_EFGH_PC);

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PE0_SO0_PC=%02x\n", (val & 0x00000001));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PE1_SI0_PC=%02x\n", ((val & 0x00000002)>>1));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PE2_SCK0N_PC=%02x\n", ((val & 0x00000004)>>2));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PE3_SFSS0_PC=%02x\n", ((val & 0x00000008)>>3));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PE4_TXD0_SO1_PC=%02x\n", ((val & 0x00000010)>>4));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PE5_RXD0_SI1_PC=%02x\n", ((val & 0x00000020)>>5));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PE6_UARTCTS0_SCK1N_PC=%02x\n", ((val & 0x00000040)>>6));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PE7_UARTRTS0_SFSS1_PC=%02x\n", ((val & 0x00000080)>>7));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PF0_PC=%02x\n", ((val & 0x00000100)>>8));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PF1_PC=%02x\n", ((val & 0x00000200)>>9));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PF2_PC=%02x\n", ((val & 0x00000400)>>10));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PF3_PC=%02x\n", ((val & 0x00000800)>>11));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PF4_PC=%02x\n", ((val & 0x00001000)>>12));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PF5_PC=%02x\n", ((val & 0x00002000)>>13));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PF6_PC=%02x\n", ((val & 0x00004000)>>14));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PF7_PC=%02x\n", ((val & 0x00008000)>>15));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PG0_TXD1_SO2_PC=%02x\n", ((val & 0x00010000)>>16));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PG1_RXD1_SI2_PC=%02x\n", ((val & 0x00020000)>>17));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PG2_UARTCTS1_SCK2N_PC=%02x\n", ((val & 0x00040000)>>18));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PG3_UARTRTS1_SFSS2_PC=%02x\n", ((val & 0x00080000)>>19));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PG4_TXD3_PC=%02x\n", ((val & 0x00100000)>>20));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PG5_RXD3_PC=%02x\n", ((val & 0x00200000)>>21));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PG6_TXD2_PC=%02x\n", ((val & 0x00400000)>>22));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PG7_RXD2_PC=%02x\n", ((val & 0x00800000)>>23));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PH0_SDCLK0_PC=%02x\n", ((val & 0x01000000)>>24));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PH1_SDCMD0_PC=%02x\n", ((val & 0x02000000)>>25));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PH2_XSDCD0_PC=%02x\n", ((val & 0x04000000)>>26));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PH3_SDWP0_PC=%02x\n", ((val & 0x08000000)>>27));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PH4_SDDAT00_PC=%02x\n", ((val & 0x10000000)>>28));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PH5_SDDAT01_PC=%02x\n", ((val & 0x20000000)>>29));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PH6_SDDAT02_PC=%02x\n", ((val & 0x40000000)>>30));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PH7_SDDAT03_PC=%02x\n", ((val & 0x80000000)>>31));

	*eof = 1;

	return len;
}


static int xarina_read_proc_sysreg1_gpio_i_pc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	const u32 val = xarina_sysreg1_readl(XARINA_SYSREG1_GPIO_I_PC);

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PI0_SO3_PC=%02x\n", (val & 0x00000001));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PI1_SI3_PC=%02x\n", ((val & 0x00000002)>>1));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PI2_SCK3N_PC=%02x\n", ((val & 0x00000004)>>2));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PI3_SFSS3_PC=%02x\n", ((val & 0x00000008)>>3));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PI4_PWM0_PC=%02x\n", ((val & 0x00000010)>>4));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"PI5_PWM1_PC=%02x\n", ((val & 0x00000020)>>5));

	*eof = 1;

	return len;
}


static int xarina_read_proc_sysreg1_usb_ulpi_pc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	const u32 val = xarina_sysreg1_readl(XARINA_SYSREG1_USB_ULPI_PC);

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"USBMCLK_PC=%02x\n", (val & 0x00000001));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"USBVBSMON_PC=%02x\n", ((val & 0x00000002)>>1));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"USBOVRCUR_PC=%02x\n", ((val & 0x00000004)>>2));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"USBVBUSEN_PC=%02x\n", ((val & 0x00000008)>>3));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"ULPI_CLK_PC=%02x\n", ((val & 0x00000010)>>4));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"ULPI_DIR_PC=%02x\n", ((val & 0x00000020)>>5));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"ULPI_STP_PC=%02x\n", ((val & 0x00000040)>>6));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"ULPI_NXT_PC=%02x\n", ((val & 0x00000080)>>7));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"ULPI_DATA0_PC=%02x\n", ((val & 0x00000100)>>8));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"ULPI_DATA1_PC=%02x\n", ((val & 0x00000200)>>9));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"ULPI_DATA2_PC=%02x\n", ((val & 0x00000400)>>10));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"ULPI_DATA3_PC=%02x\n", ((val & 0x00000800)>>11));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"ULPI_DATA4_PC=%02x\n", ((val & 0x00001000)>>12));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"ULPI_DATA5_PC=%02x\n", ((val & 0x00002000)>>13));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"ULPI_DATA6_PC=%02x\n", ((val & 0x00004000)>>14));
	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"ULPI_DATA7_PC=%02x\n", ((val & 0x00008000)>>15));

	*eof = 1;

	return len;
}


static int xarina_read_proc_sysreg1_device_id(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	const u32 val = xarina_sysreg1_readl(XARINA_SYSREG1_DEVICE_ID);

	len += snprintf(page + len, (((count - len) >= 0) ? (count - len) : 0),
					"DEVICE_ID=%08x\n",	val);

	*eof = 1;

	return len;
}


/* xarina_procfs - create /proc/xarina files */
static void __init xarina_procfs(struct xarina_pinmux* pinmux)
{
	int i;
	struct proc_dir_entry *xarina_dir;

	/* kernel parameter AND original pinmux setting */
	gpio_careports[GPIOA] &= pinmux->gpioa;
	gpio_careports[GPIOB] &= pinmux->gpiob;
	gpio_careports[GPIOC] &= pinmux->gpioc;
	gpio_careports[GPIOE] &= pinmux->gpioe;
	gpio_careports[GPIOF] &= pinmux->gpiof;
	gpio_careports[GPIOG] &= pinmux->gpiog;
	gpio_careports[GPIOH] &= pinmux->gpioh;
	gpio_careports[GPIOI] &= pinmux->gpioi;
	for (i = GPIOA; i <= GPIOI; i++)
		gpio_careports[i] &= 0xFF;


	/* mkdir /proc/xarina */
	xarina_dir = proc_mkdir("xarina", NULL);

	/* /proc/xarina/gpio_careports */
	create_proc_read_entry("gpio_careports", 0, xarina_dir, xarina_read_proc_gpio_careports, NULL);

	/* /proc/xarina/gpio_initports */
	create_proc_read_entry("gpio_initports", 0, xarina_dir, xarina_read_proc_gpio_initports, NULL);

	/* /proc/xarina/pinmux */
	create_proc_read_entry("pinmux", 0, xarina_dir, xarina_read_proc_pinmux, pinmux);

	/* /proc/xarina/sysreg1 */
	{
#define	CREATE_PROC_READ_ENTRY_SYSREG1(name) \
	create_proc_read_entry(#name, 0, sysreg1_dir, xarina_read_proc_sysreg1_ ## name, NULL)

		struct proc_dir_entry *sysreg1_dir = proc_mkdir("sysreg1", xarina_dir);

		CREATE_PROC_READ_ENTRY_SYSREG1(pll5_ctrl);
		CREATE_PROC_READ_ENTRY_SYSREG1(pll_power_567);
		CREATE_PROC_READ_ENTRY_SYSREG1(rst_ctrl);
		CREATE_PROC_READ_ENTRY_SYSREG1(clk_ctrl);
		CREATE_PROC_READ_ENTRY_SYSREG1(clk_en);
		CREATE_PROC_READ_ENTRY_SYSREG1(port_mon);
		CREATE_PROC_READ_ENTRY_SYSREG1(port_drv);
		CREATE_PROC_READ_ENTRY_SYSREG1(device_id_0704);
		CREATE_PROC_READ_ENTRY_SYSREG1(device_id_1108);
		CREATE_PROC_READ_ENTRY_SYSREG1(cam_gpiob_sel);
		CREATE_PROC_READ_ENTRY_SYSREG1(nand_gpioi_sel);
		CREATE_PROC_READ_ENTRY_SYSREG1(video_gpioc_svi_sel);
		CREATE_PROC_READ_ENTRY_SYSREG1(gpio_be_sel);
		CREATE_PROC_READ_ENTRY_SYSREG1(gpio_g_sel);
		CREATE_PROC_READ_ENTRY_SYSREG1(gpio_hi_sel);
		CREATE_PROC_READ_ENTRY_SYSREG1(cam_pc);
		CREATE_PROC_READ_ENTRY_SYSREG1(ether_nand_pc);
		CREATE_PROC_READ_ENTRY_SYSREG1(gpio_abc_pc);
		CREATE_PROC_READ_ENTRY_SYSREG1(gpio_efgh_pc);
		CREATE_PROC_READ_ENTRY_SYSREG1(gpio_i_pc);
		CREATE_PROC_READ_ENTRY_SYSREG1(usb_ulpi_pc);
		CREATE_PROC_READ_ENTRY_SYSREG1(device_id);
	}
}


/*
 * initialize Board (EVB)
 */
static struct xarina_pinmux pinmux = {
	.enable = true,
	.gpioa = 0xFF,
	.gpiob = 0x07,
	.gpioc = 0x00,
	.gpioe = 0x00,
	.gpiof = 0xFF,
	.gpiog = 0x00,
	.gpioh = 0x00,
	.gpioi = 0x80,
	.spi2 = false,
	.spi3 = false,
	.uart0 = false,
	.uart1 = false,
	.uart2 = false,
	.uart3 = false,
	.sd = false,
	.pwm0 = false,
	.pwm1 = false,
	.nand_nr = 1,
};

void __init xarina_init_evb(void)
{
	/* initialize sysreg setting */
	xarina_sysreg_init(&pinmux);

	/* initialize board */
	xarina_init(&pinmux);

	/* create procfs for xarina */
	xarina_procfs(&pinmux);
}


/* initialize machine */
MACHINE_START(XARINA, "\"Xarina\" Design Kit from Sony")
	.phys_io	= XARINA_UART3_PHYS_BASE,
	.io_pg_offst	= ((XARINA_UART3_VIRT_BASE) >> 18) & 0xfffc,
	.boot_params	= CONFIG_XARINA_PHYS_ADDR + 0x100,
	.map_io		= xarina_map_io,
	.init_irq	= xarina_init_irq,
	.timer		= &sp804_sys_timer,
	.init_machine	= xarina_init_evb,
MACHINE_END
