/* 2012-08-07: File added and changed by Sony Corporation */
/*
 *  arch/arm/mach-xarina/xarina_fpga.c
 *
 *  Copyright 2010-2011 Sony Corporation
 *  Copyright (C) 2004 ARM Limited
 *  Copyright (C) 2000 Deep Blue Solutions Ltd
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/init.h>
#include <linux/device.h>
#include <linux/sysdev.h>
#include <linux/smsc911x.h>
#include <linux/platform_device.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/partitions.h>
#include <linux/mtd/nand.h>

#include <asm/io.h>
#include <asm/irq.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/flash.h>
#include <asm/mach/map.h>

#include <mach/hardware.h>

#include "core.h"
#include "sysreg.h"


#ifdef CONFIG_XARINA_FPGA_NOR
/*
 NOR-Flash Partitions
*/
static struct mtd_partition nor_partitions[] = {
	/* name, offset, size */
	MTD_PART("NOR: U-Boot Bin Area",			0,			SZ_256K),
	MTD_PART("NOR: U-Boot Param Save Area",			MTDPART_OFS_APPEND,	SZ_256K),
	MTD_PART("NOR: ARM9 Comm Param Save Area",		MTDPART_OFS_APPEND,	SZ_128K),
	MTD_PART("NOR: Cam FW Set Param Save Area(Bank A)",	MTDPART_OFS_APPEND,	SZ_128K),
	MTD_PART("NOR: Cam FW Set Param Save Area(Bank B)",	MTDPART_OFS_APPEND,	SZ_128K),
	MTD_PART("NOR: Cam FW Defect Revision Data Area",	MTDPART_OFS_APPEND,	SZ_128K),
	MTD_PART("NOR: Reserved Area 1",			MTDPART_OFS_APPEND,	SZ_256K),
	MTD_PART("NOR: ARM9 Comm Param Default Data Area",	MTDPART_OFS_APPEND,	SZ_128K),
	MTD_PART("NOR: Cam FW Set Param Default Data Area",	MTDPART_OFS_APPEND,	SZ_128K),
	MTD_PART("NOR: Initial OSD Color Palette Area",		MTDPART_OFS_APPEND,	SZ_128K),
	MTD_PART("NOR: Initial OSD Data Area",			MTDPART_OFS_APPEND,	SZ_128K),
	MTD_PART("NOR: Initial Mask Data Area",			MTDPART_OFS_APPEND,	SZ_128K),
	MTD_PART("NOR: Reserved Area 2",			MTDPART_OFS_APPEND,	SZ_1M),
	MTD_PART("NOR: ARM9 F/W Bin Area",			MTDPART_OFS_APPEND,	SZ_1M),
	MTD_PART("NOR: Kernel Image Area",			MTDPART_OFS_APPEND,	SZ_4M + SZ_128K),
	MTD_PART("NOR: Userland(jffs2) Area",			MTDPART_OFS_APPEND,	MTDPART_SIZ_FULL),
};

static int xarina_flash_init(void)
{
	return 0;
}

static void xarina_flash_exit(void)
{
	/* empty */
}

static struct flash_platform_data nor_flash_data = {
	.map_name	= "cfi_probe",
	.width		= 2,
	.init		= xarina_flash_init,
	.exit		= xarina_flash_exit,
	.parts		= nor_partitions,
	.nr_parts	= ARRAY_SIZE(nor_partitions),
};

static struct resource nor_flash_resource = {
	.start		= XARINA_NOR_PHYS_BASE,
	.end		= XARINA_NOR_PHYS_BASE + XARINA_NOR_SIZE - 1,
	.flags		= IORESOURCE_MEM,
};

static struct platform_device nor_flash_device = {
	.name		= "xarina_nor_flash",
	.id		= -1,
	.dev		= {
		.platform_data	= &nor_flash_data,
	},
	.num_resources	= 1,
	.resource	= &nor_flash_resource,
};
#endif


/*
 * initialize Board (FPGA)
 */
void __init xarina_init_fpga(void)
{
	struct xarina_pinmux pinmux = {
		.enable = false,
		.gpioa = 0xFF,
		.gpiob = 0xFF,
		.gpioc = 0xFF,
		.gpioe = 0xFF,
		.gpiof = 0xFF,
		.gpiog = 0xFF,
		.gpioh = 0xFF,
		.gpioi = 0xFF,
		.spi2 = true,
		.spi3 = true,
		.uart0 = false,
		.uart1 = false,
		.uart2 = false,
		.uart3 = true,
		.sd = true,
		.nand_nr = 1,
	};

	xarina_sysreg_init(&pinmux);
	xarina_init(&pinmux);

#ifdef CONFIG_XARINA_FPGA_NOR
	xarina_platdev_register(&nor_flash_device, true);
#endif
}


/* initialize machine */
MACHINE_START(XARINA, "\"Xarina\" Design Kit from Sony")
	.phys_io	= XARINA_UART3_PHYS_BASE,
	.io_pg_offst	= ((XARINA_UART3_VIRT_BASE) >> 18) & 0xfffc,
	.boot_params	= CONFIG_XARINA_PHYS_ADDR + 0x100,
	.map_io		= xarina_map_io,
	.init_irq	= xarina_init_irq,
	.timer		= &sp804_sys_timer,
	.init_machine	= xarina_init_fpga,
MACHINE_END
