/* 2011-11-25: File added and changed by Sony Corporation */
/*
 *  Copyright 2007, 2008 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */

#ifndef _ZCOREPCIEROOT_PCI_H_
#define _ZCOREPCIEROOT_PCI_H_
#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#ifdef CONFIG_ZCOREPCIEROOT_DEBUG
#define PCIE_ERR(args...) printk(KERN_ALERT "ZPCIe: " args)
#define PCIE_LOG(args...) printk(KERN_DEBUG "ZPCIe: " args)

extern int zcorepcieroot_test_init( void );
#define ZCOREPCIEROOT_TESTINIT() zcorepcieroot_test_init()

#else

#define PCIE_ERR(args...) printk(KERN_ALERT "ZPCIe: " args)
#define PCIE_LOG(args...)
#define ZCOREPCIEROOT_TESTINIT()

#endif

/* Vender ID */
#define ZCOREPCIEROOT_VENDER_ID                  (0x104D)

/* Device ID */
#define ZCOREPCIEROOT_DEVICE_ID                  (0x908c)

/* number of PCIe core implemented */
#define ZCOREPCIEROOT_CORE_NUM                  (1)
/* maximum number of PCIe core will be implemented */
#define ZCOREPCIEROOT_CORE_NUMMAX               (1)

/* maximum wait time for Linkup (ms) */
#define ZCOREPCIEROOT_LINKUPWAIT                (50)
/* maximum wait time for Retrain (ms) */
#define ZCOREPCIEROOT_RETRAINWAIT               (1000)
/* maximum wait time for resume from bridge access pause (ms) */
#define ZCOREPCIEROOT_BRG_PAUSEWAIT             (1000)

/* MSI IRQ */
#define ZCOREPCIEROOT_MSI_MAX                   (NR_PCIE_IRQS)
/* IRQ */
#define ZCOREPCIEROOT_CORE0_IRQ                 (IRQ_PCIE0)

/* base address of bridge registers */
#define ZCOREPCIEROOT_PCI0_REG_BASE             (u32)(XARINA_OCP0_PHYS_BASE + \
                                                 XARINA_PCIE0_OFFSET)
#define ZCOREPCIEROOT_PCI0_REG_SIZE             (SZ_8K)

/* mem/prefetch mem/io/resource */
#define ZCOREPCIEROOT_PCI0_MEM_BASE0            (u32)(XARINA_PCIE0_ENDPOINT_PHYS_BASE)
#define ZCOREPCIEROOT_PCI0_MEM_BASE0_SIZE       (0x01000000)
#define ZCOREPCIEROOT_PCI0_MEM_BASE1            (u32)(ZCOREPCIEROOT_PCI0_MEM_BASE0 + \
                                                 ZCOREPCIEROOT_PCI0_MEM_BASE0_SIZE)
#define ZCOREPCIEROOT_PCI0_MEM_BASE1_SIZE       (0x04000000)
#define ZCOREPCIEROOT_PCI0_MEM_BASE2            (u32)(ZCOREPCIEROOT_PCI0_MEM_BASE1 + \
                                                 ZCOREPCIEROOT_PCI0_MEM_BASE1_SIZE)
#define ZCOREPCIEROOT_PCI0_MEM_BASE2_SIZE       (0x03000000)

/* register definition */
/************************/
/* OCP-PCIe core bridge */
/************************/
//#define BIT(x)                                  (1 << x)
/* PCIe bridge control register */
#define ZCOREPCIEROOT_BRG_PBRG_ID(x)            (void*)(x + 0x1000)
#define ZCOREPCIEROOT_BRG_PBRG_SET1(x)          (void*)(x + 0x1008)
#define ZCOREPCIEROOT_BRG_PBRG_SET2(x)          (void*)(x + 0x100C)
#define ZCOREPCIEROOT_BRG_PCFG_ADR(x)           (void*)(x + 0x1010)
#define ZCOREPCIEROOT_BRG_PCFG_DATA(x)          (void*)(x + 0x1014)
#define ZCOREPCIEROOT_BRG_PBRG_INT(x)           (void*)(x + 0x1018)
#define ZCOREPCIEROOT_BRG_PBRG_IMASK(x)         (void*)(x + 0x101C)
#define ZCOREPCIEROOT_BRG_PBRG_MSPEADR(x)       (void*)(x + 0x1020)
#define ZCOREPCIEROOT_BRG_PBRG_BEEADR(x)        (void*)(x + 0x1024)
#define ZCOREPCIEROOT_BRG_PBRG_BSTEADR(x)       (void*)(x + 0x1028)
#define ZCOREPCIEROOT_BRG_PBRG_PMSI_BADR(x)     (void*)(x + 0x1030)
#define ZCOREPCIEROOT_BRG_PBRG_PMSI_NUM(x)      (void*)(x + 0x1034)
#define ZCOREPCIEROOT_BRG_PBRG_PMSI_INT(x)      (void*)(x + 0x1038)
#define ZCOREPCIEROOT_BRG_PBRG_PMSI_IMASK(x)    (void*)(x + 0x103C)
/* BRG_PBRG_ID */
//#define ZCOREPCIEROOT_ID_VAL                    (0x12340001) /* -LF3 */
#define ZCOREPCIEROOT_ID_VAL                    (0x00000001) /* LF4- */
#define ZCOREPCIEROOT_ID_IDNUM_MASK             (0xFFFF0000)
#define ZCOREPCIEROOT_ID_REV_MASK               (0x0000FF00)
#define ZCOREPCIEROOT_ID_SUBREV_MASK            (0x000000FF)
/* BRG_PBRG_SET1  */
#define ZCOREPCIEROOT_SET1_MMPLBE_MASK          (0x00000F00)
#define ZCOREPCIEROOT_SET1_MMPWMD               (BIT(2))
#define ZCOREPCIEROOT_SET1_PAUSESTAT            (BIT(1))
#define ZCOREPCIEROOT_SET1_MMPPAUSE             (BIT(0))

/* BRG_PBRG_SET2 */
#define ZCOREPCIEROOT_SET2_CFGRETRYMAX_SHIFT    (24)
#define ZCOREPCIEROOT_SET2_RETRYSRESP           (BIT(16))
#define ZCOREPCIEROOT_SET2_DLSET                (BIT(0))
/* BRG_PCFG_ADR */
#define ZCOREPCIEROOT_ADR_BUS_NUM_SHIFT         (24)
#define ZCOREPCIEROOT_ADR_BUS_NUM_MASK          (0xFF)
#define ZCOREPCIEROOT_ADR_DEV_NUM_SHIFT         (19)
#define ZCOREPCIEROOT_ADR_DEV_NUM_MASK          (0x1F)
#define ZCOREPCIEROOT_ADR_FN_NUM_SHIFT          (16)
#define ZCOREPCIEROOT_ADR_FN_NUM_MASK           (0x7)
#define ZCOREPCIEROOT_ADR_EXREG_ADR_SHIFT       (8)
#define ZCOREPCIEROOT_ADR_REG_ADR_SHIFT         (2)
#define ZCOREPCIEROOT_ADR_REG_ADR_MASK          (0xFFF)
/* BRG_PBRG_INT */
/* BRG_PBRG_IMASK */
#define ZCOREPCIEROOT_PIP_EVSTAT_MASK           (0x00FF0000)
#define ZCOREPCIEROOT_DLM_DL_UP                 (BIT(14))
#define ZCOREPCIEROOT_CRG_INTA                  (BIT(13))
#define ZCOREPCIEROOT_CRG_INTB                  (BIT(12))
#define ZCOREPCIEROOT_CRG_INTC                  (BIT(11))
#define ZCOREPCIEROOT_CRG_INTD                  (BIT(10))
#define ZCOREPCIEROOT_CRG_PME_HP_INT            (BIT(9))
#define ZCOREPCIEROOT_CRG_ERR_INT               (BIT(8))
#define ZCOREPCIEROOT_CFGRETRY                  (BIT(4))
#define ZCOREPCIEROOT_CFGADRERR                 (BIT(3))
#define ZCOREPCIEROOT_BSTERR                    (BIT(2))
#define ZCOREPCIEROOT_BEERR                     (BIT(1))
#define ZCOREPCIEROOT_MSPERR                    (BIT(0))
#define ZCOREPCIEROOT_BRIDGE_MASK               (ZCOREPCIEROOT_MSPERR           |\
                                                 ZCOREPCIEROOT_BEERR            |\
                                                 ZCOREPCIEROOT_BSTERR           |\
                                                 ZCOREPCIEROOT_CFGADRERR        |\
                                                 ZCOREPCIEROOT_CFGRETRY         |\
                                                 ZCOREPCIEROOT_CRG_ERR_INT      |\
                                                 ZCOREPCIEROOT_CRG_PME_HP_INT   |\
                                                 ZCOREPCIEROOT_CRG_INTA         |\
                                                 ZCOREPCIEROOT_CRG_INTB         |\
                                                 ZCOREPCIEROOT_CRG_INTC         |\
                                                 ZCOREPCIEROOT_CRG_INTD         |\
		                                         ZCOREPCIEROOT_DLM_DL_UP        |\
                                                 ZCOREPCIEROOT_PIP_EVSTAT_MASK)
#define ZCOREPCIEROOT_BRIDGE_UNMASK             ((~ZCOREPCIEROOT_BRIDGE_MASK)|\
	                                              ZCOREPCIEROOT_DLM_DL_UP)

/* PCIe IP control register */
#define ZCOREPCIEROOT_BRG_PIP_SET1(x)           (void*)(x + 0x1100)
#define ZCOREPCIEROOT_BRG_PIP_SET2(x)           (void*)(x + 0x1104)
#define ZCOREPCIEROOT_BRG_PIP_STAT(x)           (void*)(x + 0x1110)
#define ZCOREPCIEROOT_BRG_PIP_EVEN(x)           (void*)(x + 0x1120)
#define ZCOREPCIEROOT_BRG_PIP_EVINV(x)          (void*)(x + 0x1124)
#define ZCOREPCIEROOT_BRG_PIP_EVSEL(x)          (void*)(x + 0x1128)
/* BRG_PIP_EVEN */
#define ZCOREPCIEROOT_BRG_PIP_EVEN_MASK         (0x0000000F)
/* BRG_PIP_EVINV */
#define ZCOREPCIEROOT_BRG_PIP_EVINV_MASK        (0x0000000F)
/* BRG_PIP_EVSEL */
#define ZCOREPCIEROOT_BRG_PIP_EVSEL_MASK        (0x0000000F)

/**************************/
/* Configuration Register */
/**************************/
/*
 * PCIe Extended Configuration register addresses are not fixed address.
 * But this code is used only with Inventure Corp PCIe root cores.
 * Note that we use fixed offset definition to access root core's
 * extended configuration register.
 * other than root cores, I use pci_find_capability function to get
 * extended register offset and then access these registers.
 *
 * (please refer to pci_regs.h )
 */

/* PCI Express Capability Register */
#define ZCOREPCIEROOT_PCIE_EXP_PTR(x)           (0x80 + x)
/* Device Capability Register */
#define ZCOREPCIEROOT_PCIE_PAYLOAD_INVALID      (-1)
#define ZCOREPCIEROOT_PCIE_PAYLOAD_128Byte      (0)
#define ZCOREPCIEROOT_PCIE_PAYLOAD_256Byte      (1)
#define ZCOREPCIEROOT_PCIE_PAYLOAD_512Byte      (2)
#define ZCOREPCIEROOT_PCIE_PAYLOAD_1024Byte     (3)
#define ZCOREPCIEROOT_PCIE_PAYLOAD_2048Byte     (4)
#define ZCOREPCIEROOT_PCIE_PAYLOAD_4096Byte     (5)
    /* Role Based Error Reporting */
#define ZCOREPCIEROOT_PCIE_DEVCAP_ROLEERROR     BIT(15)

    /* EndPoint L1 Acceptable Latency (1 <<: convert val for calcuration) */
#define ZCOREPCIEROOT_PCIE_DEVCAP_L1_ACPT(val)  (1 << ((val & (BIT(11)|BIT(10)|BIT(9))) >> 9))
    /* EndPoint L0s Acceptable Latency (1 <<: convert val for calcuration) */
#define ZCOREPCIEROOT_PCIE_DEVCAP_L0S_ACPT(val) (1 << ((val & (BIT(8)|BIT(7)|BIT(6))) >> 6))
    /* EndPoint Lx Acceptable Latency - No limit value */
#define ZCOREPCIEROOT_PCIE_DEVCAP_ASPM_NOLIMIT  (1 << 0x7)

/* Device Control Register */
#define ZCOREPCIEROOT_PCIE_DEVCTL_PAYLOAD_SHIFT (5)
    /* Error send permission(ERR_FATAL/NONFATAL/COR) */
#define ZCOREPCIEROOT_PCIE_DEVCTL_ERROR_MASK    (PCI_EXP_DEVCTL_CERE|PCI_EXP_DEVCTL_NFERE| \
                                                    PCI_EXP_DEVCTL_FERE	)
/* Link Capability Register */

    /* L1 Exit Latency (1 <<: convert val for calcuration) */
#define ZCOREPCIEROOT_PCIE_LNKCAP_L1_EXIT(val)  (1 << ((val & (BIT(17)|BIT(16)|BIT(15))) >> 15))
    /* L0s Exit Latency (1 << : convert val for calcuration) */
#define ZCOREPCIEROOT_PCIE_LNKCAP_L0S_EXIT(val) (1 << ((val & (BIT(14)|BIT(13)|BIT(12))) >> 12))
    /* L0s and L1 supported(b11)/L0s supported(b01) */
#define ZCOREPCIEROOT_PCIE_LNKCAP_ASPM_SUP(val) ((val & (BIT(11)|BIT(10))) >> 10)
#define ZCOREPCIEROOT_PCIE_LNKCAP_ASPM_L1(val)  (val & BIT(11))
#define ZCOREPCIEROOT_PCIE_LNKCAP_ASPM_L0s(val) (val & BIT(10))

/* Link Control Register */
#define ZCOREPCIEROOT_PCIE_LNKCTL_RETRAINLINK   BIT(5)
#define ZCOREPCIEROOT_PCIE_LNKCTL_COMMONCLK     BIT(6)
#define ZCOREPCIEROOT_PCIE_LNKCTL_L1_ENA        BIT(1)
#define ZCOREPCIEROOT_PCIE_LNKCTL_L0S_ENA       BIT(0)
#define ZCOREPCIEROOT_PCIE_LNKCTL_ASPM_MASK     ( ZCOREPCIEROOT_PCIE_LNKCTL_L1_ENA | \
                                                    ZCOREPCIEROOT_PCIE_LNKCTL_L0S_ENA )
/* Link Status Register */
#define ZCOREPCIEROOT_PCIE_LNKSTA_LINKTRAINING  BIT(11)

/* Slot Control Register */
/* Power Controller Control (0b:PowerOn) */
#define ZCOREPCIEROOT_PCIE_SLTCTL_POWERCTRL     BIT(10)
/* Hot-Plug Interrupt Enable */
#define ZCOREPCIEROOT_PCIE_SLTCTL_HPINTENA      BIT(5)
/* Slot Status Register */
#define ZCOREPCIEROOT_PCIE_HOTPLUG_EVT_MASK     (0x1F)
/* PCI Express Extended Configuration Space  */

#define ZCOREPCIEROOT_PCIE_AER_PTR(x)           (0x100 + x)
#define ZCOREPCIEROOT_PCIE_AER_EVT_MASK         (PCI_ERR_ROOT_COR_RCV| \
                                                PCI_ERR_ROOT_MULTI_COR_RCV| \
                                                PCI_ERR_ROOT_UNCOR_RCV| \
                                                PCI_ERR_ROOT_MULTI_UNCOR_RCV| \
                                                PCI_ERR_ROOT_FIRST_FATAL| \
                                                PCI_ERR_ROOT_NONFATAL_RCV| \
                                                PCI_ERR_ROOT_FATAL_RCV)

/* Root Error Command */
/* Correctable/Non-fatal/Fatal  Err Reporting Enable */
#define ZCOREPCIEROOT_PCIE_ROOT_COMMAND_MASK    (PCI_ERR_ROOT_CMD_COR_EN|\
                                                PCI_ERR_ROOT_CMD_NONFATAL_EN|\
                                                PCI_ERR_ROOT_CMD_FATAL_EN)

/* Root Control Register */
#define ZCOREPCIEROOT_PCIE_RTCTL_INTRMASK       (PCI_EXP_RTCTL_SECEE | \
                                                PCI_EXP_RTCTL_SENFEE |\
                                                PCI_EXP_RTCTL_SEFEE | \
                                                PCI_EXP_RTCTL_PMEIE| \
                                                PCI_EXP_RTCTL_CRSSVE)
/* System Error on Correctable/ Non-Fatal / Fatal ,PME Interrupt Enable */
/* "CRS Software Visibility Enable" is Disabled now.. */
#define ZCOREPCIEROOT_PCIE_RTCTL_INTRSET       (PCI_EXP_RTCTL_SECEE | \
                                                PCI_EXP_RTCTL_SENFEE |\
                                                PCI_EXP_RTCTL_SEFEE | \
                                                PCI_EXP_RTCTL_PMEIE)

/* Root Status Register */
#define ZCOREPCIEROOT_PCIE_RTSTA_PMEPENDING     BIT(17)
#define ZCOREPCIEROOT_PCIE_RTSTA_PMESTA         BIT(16)

/******************************/
/* application register block */
/******************************/
#define ZCOREPCIEROOT_PCIE_ARG_PTR              (0x400)
/* Power Control/Status */
#define ZCOREPCIEROOT_PCIE_PWR_CTRL_STATUS      (ZCOREPCIEROOT_PCIE_ARG_PTR + 0x008)
    /* Root/Slot PERST# Control */
#define ZCOREPCIEROOT_PWR_CTRL_STATUS_PERST     BIT(0)
    /* Hot-Plug Auto Power Up/Down Sequence Enable */
#define ZCOREPCIEROOT_PWR_CTRL_STATUS_AUTOHP    BIT(22)
/* Link Status/Control */
#define ZCOREPCIEROOT_PCIE_LINK_STATUS_CTRL     (ZCOREPCIEROOT_PCIE_ARG_PTR + 0x060)
    /* PLM LinkUp */
#define ZCOREPCIEROOT_LINK_STATUS_CTRL_PLM_LINK BIT(28)


struct zcorepcieroot_core_info{
    u32                     brgreg;         /* PCIe-OCP bridge base physical address */
    u32                     brgreg_len;     /* PCIe-OCP bridge register area size */
    int                     irq;            /* IRQ for PCIe */
    char*                   irqstr;         /* String for irq registration */
    struct pci_sys_data*    sys;            /* for core identification process */
    int                     payload_sup;    /* core's payload supported value */
    struct resource*        bridge_rsc;     /* for release_mem_region */
    char*                   bridge_str;     /* String for request_mem_region */
    u32                     bridge_vm;      /* virtual memory of bridge */
    int                     core_down;      /* core doesn't work */
    spinlock_t              lock;           /* spinlock for core */
#ifdef CONFIG_PCI_MSI
    u32                     msi_mng;        /* msi interrupt management */
    u32                     msi_phyadr;     /* msi address area */
    u32                     msi_irqsta;     /* msi irq start */
    u32                     msi_irqend;     /* msi irq end */
    u32                     msi_mem[ZCOREPCIEROOT_MSI_MAX]; /* msi memory */
#endif
};

struct zcorepcieroot_irq_info{
    unsigned int    core_irq;               /* core irq num */
    void __iomem*   base;                   /* irq chip data */
    unsigned int    msi_irq_start;          /* msi irq start num */
    u32             vic_sources;            /* enable bit */
};

extern struct zcorepcieroot_core_info  zcorepcieroot_info[];

/* prototype */
int zcorepcieroot_pci_core_nr(struct pci_sys_data *sysdata);
void __iomem *zcorepcieroot_pci_addr(struct pci_bus *bus,
                                                unsigned int devfn, int offset);
int zcorepcieroot_read_config(struct pci_bus *bus, unsigned int devfn,
                                                int where, int size, u32 *val);
int zcorepcieroot_write_config(struct pci_bus *bus, unsigned int devfn, int where,
                                      int size, u32 val);
int zcorepcieroot_bios_write_config(int nr, int where, int size, u32 val);
int zcorepcieroot_bios_read_config(int nr, int where, int size, u32 *val);
int zcorepcieroot_bios_rmw_config(int nr, int where, int size, u32 mask, u32 set);
int __init zcorepcieroot_pci_setup(int nr, struct pci_sys_data *sys);
struct pci_bus *zcorepcieroot_pci_scan_bus(int nr, struct pci_sys_data *sys);
void __init zcorepcieroot_pci_preinit(void);
int zcorepcieroot_bridge_access_pause(int nr, int pauseon);
int __init zcorepcieroot_bridge_initset(int nr);
int __init zcorepcieroot_core_init(int nr);
void __init zcorepcieroot_pcie_initset(void);
int __init zcorepcieroot_pcie_intrset(int nr);
void zcorepcieroot_pcie_maxpayload_read_size(struct pci_dev* fixdev);
int __init zcorepcieroot_pcie_postinit(void);
irqreturn_t  zcorepcieroot_error_handler(int irq, void* dev_id);
int __init zcorepcieroot_msi_init(int nr);
extern void __init zcorepcieroot_irq_init(struct zcorepcieroot_irq_info* info);

/******************/
/* Macro Function */
/******************/

/* configuration register address of root core */
#define ZCOREPCIEROOT_ROOT_CFGREG(nr, offset) \
                ((u32*)(zcorepcieroot_info[nr].bridge_vm + offset))

/* configuration address setting other than root core */
#define ZCOREPCIEROOT_CFGREG_ADDRSET(nr, bus, devfn, offset)  \
        iowrite32( \
         (((bus->number & ZCOREPCIEROOT_ADR_BUS_NUM_MASK) << ZCOREPCIEROOT_ADR_BUS_NUM_SHIFT)| \
         (PCI_SLOT(devfn)                                 << ZCOREPCIEROOT_ADR_DEV_NUM_SHIFT)| \
         (PCI_FUNC(devfn)                                 << ZCOREPCIEROOT_ADR_FN_NUM_SHIFT)| \
         (offset & ZCOREPCIEROOT_ADR_REG_ADR_MASK)), \
          ZCOREPCIEROOT_BRG_PCFG_ADR(zcorepcieroot_info[nr].bridge_vm))

/* configuration data address other than root core */
#define ZCOREPCIEROOT_CFGREG_DATA(nr)    \
                ((u32*)ZCOREPCIEROOT_BRG_PCFG_DATA(zcorepcieroot_info[nr].bridge_vm))

/* Did we get a Configuration access failed interrupt? */
#define ZCOREPCIEROOT_IS_CONFIGNG       \
        (zcorepcieroot_config_check & (ZCOREPCIEROOT_CFGRETRY|ZCOREPCIEROOT_CFGADRERR))

#define ZCOREPCIEROOT_COREOK          (0)
#define ZCOREPCIEROOT_CORENG          (1)
#define ZCOREPCIEROOT_IS_COREOK       (zcorepcieroot_info[nr].core_down == ZCOREPCIEROOT_COREOK)
#define ZCOREPCIEROOT_IS_CORENG       (zcorepcieroot_info[nr].core_down == ZCOREPCIEROOT_CORENG)

/*******************/
/* spin_lock macro */
/*******************/

/* lock (enable intr) */
#define ZCOREPCIEROOT_LOCK(nr, lock_done) \
                if (!in_interrupt()) { \
                    spin_lock(&zcorepcieroot_info[nr].lock); \
                    lock_done = 1; \
                }\

/* lock (enable_intr & care double lock)*/
#define ZCOREPCIEROOT_LOCK_ATM(nr, lock_done) \
                if (!in_interrupt() && !in_atomic()) { \
                    spin_lock(&zcorepcieroot_info[nr].lock); \
                    lock_done = 1; \
                }

/* unlock (enable intr) */
#define ZCOREPCIEROOT_UNLOCK(nr, lock_done) \
                if (lock_done) { \
                    spin_unlock(&zcorepcieroot_info[nr].lock); \
                    lock_done = 0; \
                }



/* lock (disable intr) */
#define ZCOREPCIEROOT_LOCK_IRQ(nr, flags, lock_done) \
                if (!in_interrupt()) { \
                    spin_lock_irqsave(&zcorepcieroot_info[nr].lock, flags); \
                    lock_done = 1; \
                }

/* lock (disable intr & care double lock) */
#define ZCOREPCIEROOT_LOCK_IRQATM(nr, flags, lock_done) \
                if (!in_interrupt() && !in_atomic()) { \
                    spin_lock_irqsave(&zcorepcieroot_info[nr].lock, flags); \
                    lock_done = 1; \
                }

/* unlock (disable intr)*/
#define ZCOREPCIEROOT_UNLOCK_IRQ(nr, flags, lock_done) \
                if (lock_done) { \
                    spin_unlock_irqrestore(&zcorepcieroot_info[nr].lock, flags); \
                    lock_done = 0; \
                }


#ifdef __cplusplus
}      /* extern "C" { */
#endif /* __cplusplus */
#endif /* _ZCOREPCIEROOT_PCI_H_ */

