/* 2011-11-02: File added and changed by Sony Corporation */
/*
 *  Copyright 2008 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */

#include <asm/uaccess.h>
#include <linux/errno.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <linux/stat.h>

#include <linux/pci.h>
#include <linux/ptrace.h>
#include <linux/slab.h>
#include <linux/ioport.h>
#include <linux/interrupt.h>
#include <linux/spinlock.h>
#include <linux/init.h>
#include <mach/hardware.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/system.h>
#include <asm/mach/pci.h>
#include "zcorepcieroot_pci.h"

#ifdef CONFIG_ZCOREPCIEROOT_DEBUG

extern volatile int zcorepcieroot_pci_master_aborts;
extern volatile int zcorepcieroot_config_check;

/* utility */
int zcorepcieroot_test_get_param(char** str_ptr, unsigned int *val);
int zcorepcieroot_config_write_test(int domain, int bus, int dev, int func, int where,
				    int size, u32 val);
int zcorepcieroot_config_read_test(int domain, int bus, int dev, int func, int where,
				   int size, u32* val);

/*
 * initialize
 */
/*  Parameter Buffer Size */
#define ZCOREPCIEROOT_TEST_PARAMBUF_MAX     128

struct zcorepcieroot_proc_info{
	char * path;
	int (*proc_read)(char* page, char** start, off_t offset, int count, int* eof, void* data);
	int (*proc_write)(struct file* filp, const char* buffer, unsigned long count, void* data);
};


int zcorepcieroot_test_bridge_read(char* page, char** start, off_t offset, int count, int* eof, void* data);
int zcorepcieroot_test_bridge_write(struct file* filp, const char* buffer, unsigned long count, void* data);
int zcorepcieroot_test_mem_read(char* page, char** start, off_t offset, int count, int* eof, void* data);
int zcorepcieroot_test_mem_write(struct file* filp, const char* buffer, unsigned long count, void* data);
int zcorepcieroot_test_bioscfg_read(char* page, char** start, off_t offset, int count, int* eof, void* data);
int zcorepcieroot_test_bioscfg_write(struct file* filp, const char* buffer, unsigned long count, void* data);
int zcorepcieroot_test_cfg_read(char* page, char** start, off_t offset, int count, int* eof, void* data);
int zcorepcieroot_test_cfg_write(struct file* filp, const char* buffer, unsigned long count, void* data);
int zcorepcieroot_test_access_read(char* page, char** start, off_t offset, int count, int* eof, void* data);
int zcorepcieroot_test_access_write(struct file* filp, const char* buffer, unsigned long count, void* data);
int zcorepcieroot_test_setfunc_read(char* page, char** start, off_t offset, int count, int* eof, void* data);
int zcorepcieroot_test_setfunc_write(struct file* filp, const char* buffer, unsigned long count, void* data);
int zcorepcieroot_test_intr_read(char* page, char** start, off_t offset, int count, int* eof, void* data);
int zcorepcieroot_test_intr_write(struct file* filp, const char* buffer, unsigned long count, void* data);


/* proc initialize info table */
struct zcorepcieroot_proc_info  zcorepcieroot_procinfo[] =
{
	/* bridge register access test */
	{
		"bridge",
		zcorepcieroot_test_bridge_read,
		zcorepcieroot_test_bridge_write,
	},
	/* memory space access test */
	{
		"mem",
		zcorepcieroot_test_mem_read,
		zcorepcieroot_test_mem_write,
	},
	/* Configuration Access (BIOS) Test */
	{
		"bioscfg",
		zcorepcieroot_test_bioscfg_read,
		zcorepcieroot_test_bioscfg_write,
	},
	/* Configuration Access (API) Test */
	{
		"cfg",
		zcorepcieroot_test_cfg_read,
		zcorepcieroot_test_cfg_write,
	},
	/* PCI search func Test */
	{
		"access",
		zcorepcieroot_test_access_read,
		zcorepcieroot_test_access_write,
	},
	/* set func Test */
	{
		"setfunc",
		zcorepcieroot_test_setfunc_read,
		zcorepcieroot_test_setfunc_write,
	},
	/* interrupt Test */
	{
		"intr",
		zcorepcieroot_test_intr_read,
		zcorepcieroot_test_intr_write,
	},
	{
		NULL, NULL, NULL,
	}
};

/* semaphore for proc access */
struct semaphore zcorepcieroot_proc_sem;

/************************************/
/*  Bridge Register Access Test     */
/************************************/
/*
 *  executed when you write to /proc/zcorepci/bridge
 *
 *  format:w core where val
 */
static int bridge_par[3] = {0};
int
zcorepcieroot_test_bridge_write( struct file* filp, const char* buffer,
				 unsigned long count, void* data )
{
	int  i;
	char buf[ZCOREPCIEROOT_TEST_PARAMBUF_MAX];
	char *ptr;

	if ( down_interruptible( &zcorepcieroot_proc_sem) ) {
		printk( KERN_INFO "zcorepcieroot_test_bridge_write: down_interruptible failed\n");
		return -ERESTARTSYS;
	}

	if ( count <= 0 || ZCOREPCIEROOT_TEST_PARAMBUF_MAX < count ) {
		up( &zcorepcieroot_proc_sem );
		printk( KERN_INFO "zcorepcieroot_test_bridge_write : count failed\n");
		return -EFAULT;
	}

	if ( copy_from_user( buf, buffer, count ) ) {
		up( &zcorepcieroot_proc_sem );
		printk( KERN_INFO "zcorepcieroot_test_bridge_write : copy_from_user failed\n" );
		return -EFAULT;
	}
	buf[count] = '\0';

	printk("arg:%s\n",buf);

	ptr = buf;
	if(ptr[0] == 'w' || ptr[0] == 'W'){
		ptr++;
		for(i = 0; i < 3; i++){
			if(zcorepcieroot_test_get_param(&ptr, &bridge_par[i]) < 0) {
				up( &zcorepcieroot_proc_sem );
				printk( KERN_INFO "zcorepcieroot_test_bridge_write : parget failed:%d\n", i );
				return -EFAULT;
			}
		}

		iowrite32((u32)bridge_par[2],
			  zcorepcieroot_info[bridge_par[0]].bridge_vm + bridge_par[1]);
	}
	else if(ptr[0] == 'r' || ptr[0] == 'R'){
		ptr++;
		for(i = 0; i < 2; i++){
			/* core where size */
			if(zcorepcieroot_test_get_param(&ptr, &bridge_par[i]) < 0) {
				up( &zcorepcieroot_proc_sem );
				printk( KERN_INFO "zcorepcieroot_test_bridge_write : parget failed:%d\n", i );
				return -EFAULT;
			}
		}
		/* param get only. don't(cannot) read  */
	}
	else{
		up( &zcorepcieroot_proc_sem );
		printk( KERN_INFO "zcorepcieroot_test_bridge_write : r/w get failed\n" );
		return -EFAULT;
	}

	up( &zcorepcieroot_proc_sem );

	return count;
}

/*
 *  executed when you read /proc/zcorepci/bridge
 *
 *  format:r where
 */
int
zcorepcieroot_test_bridge_read( char* page, char** start, off_t offset,
				int count, int* eof, void* data )
{
	int len = 0;
	u32 val = 0;
	unsigned long flags = 0, lock_done = 0;


	if ( down_interruptible( &zcorepcieroot_proc_sem) ) {
		printk( KERN_INFO "zcorepcieroot_test_bridge_read: down_interruptible failed\n");
		return -ERESTARTSYS;
	}

	ZCOREPCIEROOT_LOCK_IRQATM(bridge_par[0], flags, lock_done);
	zcorepcieroot_pci_master_aborts = 0;
	zcorepcieroot_config_check      = 0;
	ZCOREPCIEROOT_UNLOCK_IRQ(bridge_par[0], flags, lock_done);

	val = ioread32(zcorepcieroot_info[bridge_par[0]].bridge_vm + bridge_par[1]);

	ZCOREPCIEROOT_LOCK_IRQATM(bridge_par[0], flags, lock_done);
	if (zcorepcieroot_pci_master_aborts || ZCOREPCIEROOT_IS_CONFIGNG) {
		val = 0xffffffff;
	}
	ZCOREPCIEROOT_UNLOCK_IRQ(bridge_par[0], flags, lock_done);

	len += sprintf( page, "core:%d offset:0x%x\n contents = 0x%x\n",
			bridge_par[0], bridge_par[1], val);

	up( &zcorepcieroot_proc_sem );

	*eof = 1;
	return len;
}
/************************************/
/*     Memory Space Access Test     */
/************************************/
/*
 *  executed when you write to /proc/zcorepci/mem
 *
 *  format:w where size val
 *  format:r where size
 */
static int mem_par[3] = {0};
int
zcorepcieroot_test_mem_write( struct file* filp, const char* buffer,
			      unsigned long count, void* data )
{
	int i;
	char buf[ZCOREPCIEROOT_TEST_PARAMBUF_MAX];
	char *ptr;

	if ( down_interruptible( &zcorepcieroot_proc_sem) ) {
		printk( KERN_INFO "zcorepcieroot_test_mem_write: down_interruptible failed\n");
		return -ERESTARTSYS;
	}

	if ( count <= 0 || ZCOREPCIEROOT_TEST_PARAMBUF_MAX < count ) {
		up( &zcorepcieroot_proc_sem );
		printk( KERN_INFO "zcorepcieroot_test_mem_write : count failed\n");
		return -EFAULT;
	}

	if ( copy_from_user( buf, buffer, count ) ) {
		up( &zcorepcieroot_proc_sem );
		printk( KERN_INFO "zcorepcieroot_test_mem_write : copy_from_user failed\n" );
		return -EFAULT;
	}
	buf[count] = '\0';

	ptr = buf;
	if(ptr[0] == 'w' || ptr[0] == 'W'){
		ptr++;
		for(i = 0; i < 3; i++){
			if(zcorepcieroot_test_get_param(&ptr, &mem_par[i]) < 0) {
				up( &zcorepcieroot_proc_sem );
				printk( KERN_INFO "zcorepcieroot_test_mem_write : parget failed:%d\n", i );
				return -EFAULT;
			}
		}

		/* where size val */
		switch(mem_par[1]){
		case 1:
			iowrite8((u8)mem_par[2], mem_par[0]);
			break;
		case 2:
			iowrite16((u16)mem_par[2], mem_par[0]);
			break;
		case 4:
			iowrite32((u32)mem_par[2], mem_par[0]);
			break;
		default:
			up( &zcorepcieroot_proc_sem );
			printk( KERN_INFO "zcorepcieroot_test_mem_write : func failed\n" );
			return -EFAULT;
		}
	}
	else if(ptr[0] == 'r' || ptr[0] == 'R'){
		ptr++;
		for(i = 0; i < 2; i++){
			/* where size */
			if(zcorepcieroot_test_get_param(&ptr, &mem_par[i]) < 0) {
				up( &zcorepcieroot_proc_sem );
				printk( KERN_INFO "zcorepcieroot_test_mem_write : parget failed:%d\n", i );
				return -EFAULT;
			}
		}
		/* param get only. don't(cannot) read  */
	}
	else{
		up( &zcorepcieroot_proc_sem );
		printk( KERN_INFO "zcorepcieroot_test_mem_write : r/w get failed\n" );
		return -EFAULT;
	}

	up( &zcorepcieroot_proc_sem );

	return count;
}

/*
 *  executed when you read /proc/zcorepci/mem
 *
 */
int
zcorepcieroot_test_mem_read( char* page, char** start, off_t offset,
			     int count, int* eof, void* data )
{
	int len = 0;
	u32 val = 0;

	if ( down_interruptible( &zcorepcieroot_proc_sem) ) {
		printk( KERN_INFO "zcorepcieroot_test_mem_read: down_interruptible failed\n");
		return -ERESTARTSYS;
	}

	switch(mem_par[1]){
        case 1:
		val = ioread8(mem_par[0]);
		break;
        case 2:
		val = ioread16(mem_par[0]);
		break;
        case 4:
		val = ioread32(mem_par[0]);
		break;
        default:
		up( &zcorepcieroot_proc_sem );
		printk( KERN_INFO "zcorepcieroot_test_mem_write : func failed\n" );
		return -EFAULT;
	}

	len += sprintf( page, "contents = 0x%x\n", val);

	up( &zcorepcieroot_proc_sem );

	*eof = 1;
	return len;
}

/************************************/
/* Configuration Access (BIOS) Test */
/************************************/
/*
 *  executed when you write to /proc/zcorepci/bioscfg
 *
 *  format:w core where size val
 *  format:r core where size
 */
static int bioscfg_par[4] = {0};
int
zcorepcieroot_test_bioscfg_write( struct file* filp, const char* buffer,
				  unsigned long count, void* data )
{
	int i;
	char buf[ZCOREPCIEROOT_TEST_PARAMBUF_MAX];
	char *ptr;

	if ( down_interruptible( &zcorepcieroot_proc_sem) ) {
		printk( KERN_INFO "zcorepcieroot_test_bioscfg_write: down_interruptible failed\n");
		return -ERESTARTSYS;
	}

	if ( count <= 0 || ZCOREPCIEROOT_TEST_PARAMBUF_MAX < count ) {
		up( &zcorepcieroot_proc_sem );
		printk( KERN_INFO "zcorepcieroot_test_bioscfg_write : count failed\n");
		return -EFAULT;
	}

	if ( copy_from_user( buf, buffer, count ) ) {
		up( &zcorepcieroot_proc_sem );
		printk( KERN_INFO "zcorepcieroot_test_bioscfg_write : copy_from_user failed\n" );
		return -EFAULT;
	}
	buf[count] = '\0';

	ptr = buf;
	if(ptr[0] == 'w' || ptr[0] == 'W'){
		ptr++;
		for(i = 0; i < 4; i++){
			if(zcorepcieroot_test_get_param(&ptr, &bioscfg_par[i]) < 0) {
				up( &zcorepcieroot_proc_sem );
				printk( KERN_INFO "zcorepcieroot_test_bioscfg_write : parget failed:%d\n", i );
				return -EFAULT;
			}
		}

		/* core where size val */
		if(zcorepcieroot_bios_write_config(bioscfg_par[0], bioscfg_par[1], bioscfg_par[2],
						   (u32)bioscfg_par[3])
		   != PCIBIOS_SUCCESSFUL){
			up( &zcorepcieroot_proc_sem );
			printk( KERN_INFO "zcorepcieroot_test_bioscfg_write : func failed\n" );
			return -EFAULT;
		}
	}
	else if(ptr[0] == 'r' || ptr[0] == 'R'){
		ptr++;
		for(i = 0; i < 3; i++){
			/* core where size */
			if(zcorepcieroot_test_get_param(&ptr, &bioscfg_par[i]) < 0) {
				up( &zcorepcieroot_proc_sem );
				printk( KERN_INFO "zcorepcieroot_test_bioscfg_write : parget failed:%d\n", i );
				return -EFAULT;
			}
		}
		/* param get only. don't(cannot) read  */
	}
	else{
		up( &zcorepcieroot_proc_sem );
		printk( KERN_INFO "zcorepcieroot_test_bioscfg_write : r/w get failed\n" );
		return -EFAULT;
	}

	up( &zcorepcieroot_proc_sem );

	return count;
}


/*
 *  executed when you read /proc/zcorepci/bioscfg
 */
int
zcorepcieroot_test_bioscfg_read( char* page, char** start, off_t offset,
				 int count, int* eof, void* data )
{
	int len = 0;
	u32 val;

	if ( down_interruptible( &zcorepcieroot_proc_sem) ) {
		printk( KERN_INFO "zcorepcieroot_test_bioscfg_read: down_interruptible failed\n");
		return -ERESTARTSYS;
	}
	if(zcorepcieroot_bios_read_config(bioscfg_par[0], bioscfg_par[1], bioscfg_par[2], &val)
	   != PCIBIOS_SUCCESSFUL ){
		printk( KERN_INFO "zcorepcieroot_test_bioscfg_read: func failed\n");
		return -ERESTARTSYS;
	}

	len += sprintf( page, "core:%d offset:0x%x size:%d \ncontents = 0x%x\n",
			bioscfg_par[0], bioscfg_par[1], bioscfg_par[2],val);

	up( &zcorepcieroot_proc_sem );

	*eof = 1;
	return len;
}

/***********************************/
/* Configuration Access (API) Test */
/***********************************/
/*
 *  executed when you write to /proc/zcorepci/cfg
 *
 *  format:w domain(core) bus dev func where size val
 */
static int cfg_par[7] = {0};
int
zcorepcieroot_test_cfg_write( struct file* filp, const char* buffer,
			      unsigned long count, void* data )
{
	int  i;
	char buf[ZCOREPCIEROOT_TEST_PARAMBUF_MAX];
	char *ptr;

	if ( down_interruptible( &zcorepcieroot_proc_sem) ) {
		printk( KERN_INFO "zcorepcieroot_test_cfg_write: down_interruptible failed\n");
		return -ERESTARTSYS;
	}

	if ( count <= 0 || ZCOREPCIEROOT_TEST_PARAMBUF_MAX < count ) {
		up( &zcorepcieroot_proc_sem );
		printk( KERN_INFO "zcorepcieroot_test_cfg_write : count failed\n");
		return -EFAULT;
	}

	if ( copy_from_user( buf, buffer, count ) ) {
		up( &zcorepcieroot_proc_sem );
		printk( KERN_INFO "zcorepcieroot_test_cfg_write : copy_from_user failed\n" );
		return -EFAULT;
	}
	buf[count] = '\0';

	ptr = buf;
	if(ptr[0] == 'w' || ptr[0] == 'W'){
		ptr++;
		for(i = 0; i < 7; i++){
			if(zcorepcieroot_test_get_param(&ptr, &cfg_par[i]) < 0) {
				up( &zcorepcieroot_proc_sem );
				printk( KERN_INFO "zcorepcieroot_test_cfg_write : parget failed:%d\n", i );
				return -EFAULT;
			}
		}

		/* domain(core) bus dev func where size val */
		if(zcorepcieroot_config_write_test(cfg_par[0], cfg_par[1], cfg_par[2], cfg_par[3],
						   cfg_par[4], cfg_par[5], (u32)cfg_par[6]) < 0){
			up( &zcorepcieroot_proc_sem );
			printk( KERN_INFO "zcorepcieroot_test_cfg_write : func failed\n" );
			return -EFAULT;
		}

	}
	else if(ptr[0] == 'r' || ptr[0] == 'R'){
		ptr++;
		for(i = 0; i < 6; i++){
			//domain(core) bus dev func where size
			if(zcorepcieroot_test_get_param(&ptr, &cfg_par[i]) < 0) {
				up( &zcorepcieroot_proc_sem );
				printk( KERN_INFO "zcorepcieroot_test_cfg_write : parget failed:%d\n", i );
				return -EFAULT;
			}
		}
		/* param get only. don't(cannot) read  */
	}
	else{
		up( &zcorepcieroot_proc_sem );
		printk( KERN_INFO "zcorepcieroot_test_cfg_write : r/w get failed\n" );
		return -EFAULT;
	}

	up( &zcorepcieroot_proc_sem );

	return count;
}

/*
 *  executed when you read /proc/zcorepci/cfg
 *
 *  format:r domain(core) bus dev func where size
 */
int
zcorepcieroot_test_cfg_read( char* page, char** start, off_t offset,
			     int count, int* eof, void* data )
{
	int len = 0;
	u32 val;

	if ( down_interruptible( &zcorepcieroot_proc_sem) ) {
		printk( KERN_INFO "zcorepcieroot_test_cfg_read: down_interruptible failed\n");
		return -ERESTARTSYS;
	}

	if(zcorepcieroot_config_read_test(cfg_par[0], cfg_par[1], cfg_par[2],
					  cfg_par[3], cfg_par[4], cfg_par[5], &val) < 0){
		printk( KERN_INFO "zcorepcieroot_test_cfg_read: func failed\n");
		return -ERESTARTSYS;
	}

	len += sprintf( page,
			"core:%d bus:%d dev:%d func:%d offset:0x%x size:%d\ncontents = 0x%x\n",
			cfg_par[0], cfg_par[1], cfg_par[2], cfg_par[3], cfg_par[4], cfg_par[5], val);

	up( &zcorepcieroot_proc_sem );

	*eof = 1;
	return len;
}

int zcorepcieroot_config_write_test(int domain, int bus, int dev, int func, int where,
				    int size, u32 val)
{
	struct pci_bus* pci_bus;
	int             ret;
	unsigned int    devfn = PCI_DEVFN(dev, func);

	pci_bus = pci_find_bus(domain, bus);
	if(!pci_bus){
		printk( KERN_INFO "zcorepcieroot_config_write_test: pci_bus get failed\n");
		return -1;
	}

	ret = zcorepcieroot_write_config(pci_bus, devfn, where, size, val);
	if(ret != PCIBIOS_SUCCESSFUL){
		printk( KERN_INFO "zcorepcieroot_config_write_test: zcorepcieroot_write_config ng\n");
		return -1;
	}

	return 0;
}


int zcorepcieroot_config_read_test(int domain, int bus, int dev, int func, int where,
				   int size, u32* val)
{
	struct pci_bus* pci_bus;
	int             ret;
	unsigned int    devfn = PCI_DEVFN(dev, func);

	pci_bus = pci_find_bus(domain, bus);
	if(!pci_bus){
		return -1;
	}

	ret = zcorepcieroot_read_config(pci_bus, devfn, where, size, val);
	if(ret != PCIBIOS_SUCCESSFUL){
		//error
		return -1;
	}

	return 0;
}

/************************************/
/*          PCI search Test         */
/************************************/
/*
 *  executed when you write to /proc/zcorepci/access
 *
 *  format:v vendorid classid
 *         c class number
 */
int
zcorepcieroot_test_access_write( struct file* filp, const char* buffer,
				 unsigned long count, void* data )
{
	int             i;
	char            buf[ZCOREPCIEROOT_TEST_PARAMBUF_MAX];
	char*           ptr;
	unsigned int    access_par[7] = {0};

	if ( down_interruptible( &zcorepcieroot_proc_sem) ) {
		printk( KERN_INFO "zcorepcieroot_test_access_write: down_interruptible failed\n");
		return -ERESTARTSYS;
	}

	if ( count <= 0 || ZCOREPCIEROOT_TEST_PARAMBUF_MAX < count ) {
		up( &zcorepcieroot_proc_sem );
		printk( KERN_INFO "zcorepcieroot_test_access_write : count failed\n");
		return -EFAULT;
	}

	if ( copy_from_user( buf, buffer, count ) ) {
		up( &zcorepcieroot_proc_sem );
		printk( KERN_INFO "zcorepcieroot_test_access_write : copy_from_user failed\n" );
		return -EFAULT;
	}
	buf[count] = '\0';

	ptr = buf;
	if(ptr[0] == 'v' || ptr[0] == 'V'){
		ptr++;
		for(i = 0; i < 2; i++){
			if(zcorepcieroot_test_get_param(&ptr, &access_par[i]) < 0) {
				up( &zcorepcieroot_proc_sem );
				printk( KERN_INFO "zcorepcieroot_test_access_write : parget failed:%d\n", i );
				return -EFAULT;
			}
		}

		/* get info */
		if(pci_get_device(access_par[0], access_par[1], NULL) != NULL){
			printk( KERN_INFO "zcorepcieroot_test_access_write : vendor(0x%x/0x%x) exist\n",
                                access_par[0], access_par[1]);
		}
		else{
			printk( KERN_INFO "zcorepcieroot_test_access_write : vendor(0x%x/0x%x) "
                                "not exist\n", access_par[0], access_par[1]);
		}
	}
	else if(ptr[0] == 'c' || ptr[0] == 'C'){
		ptr++;
		/* core where size */
		if(zcorepcieroot_test_get_param(&ptr, &access_par[0]) < 0) {
			up( &zcorepcieroot_proc_sem );
			printk( KERN_INFO "zcorepcieroot_test_access_write : parget failed:\n" );
			return -EFAULT;
		}
		/* get info */
		if(pci_get_class(access_par[0], NULL) != NULL){
			printk( KERN_INFO "zcorepcieroot_test_access_write : class(0x%x) exist\n",
				access_par[0]);
		}
		else{
			printk( KERN_INFO "zcorepcieroot_test_access_write : class(0x%x) "
				"not exist\n", access_par[0] );
		}
	}
	else{
		up( &zcorepcieroot_proc_sem );
		printk( KERN_INFO "zcorepcieroot_test_access_write : r/w get failed\n" );
		return -EFAULT;
	}

	up( &zcorepcieroot_proc_sem );

	return count;
}

/*
 *  executed when you read /proc/zcorepci/access
 *
 *  do nothing
 *
 */
int
zcorepcieroot_test_access_read( char* page, char** start, off_t offset,
				int count, int* eof, void* data )
{
	int len = 0;

	*eof = 1;
	return len;
}
/************************************/
/*         set func Test            */
/************************************/
/*
 *  executed when you write to /proc/zcorepci/setfunc
 *
 *  p:pause bridge's Memory Master Port
 *
 *  format:p core pause(1)/resume(0)
 */
int
zcorepcieroot_test_setfunc_write( struct file* filp, const char* buffer,
				  unsigned long count, void* data )
{
	int             i;
	char            buf[ZCOREPCIEROOT_TEST_PARAMBUF_MAX];
	char*           ptr;
	unsigned int    setfunc_par[7] = {0};

	if ( down_interruptible( &zcorepcieroot_proc_sem) ) {
		printk( KERN_INFO "zcorepcieroot_test_setfunc_write: down_interruptible failed\n");
		return -ERESTARTSYS;
	}

	if ( count <= 0 || ZCOREPCIEROOT_TEST_PARAMBUF_MAX < count ) {
		up( &zcorepcieroot_proc_sem );
		printk( KERN_INFO "zcorepcieroot_test_setfunc_write : count failed\n");
		return -EFAULT;
	}

	if ( copy_from_user( buf, buffer, count ) ) {
		up( &zcorepcieroot_proc_sem );
		printk( KERN_INFO "zcorepcieroot_test_setfunc_write : copy_from_user failed\n" );
		return -EFAULT;
	}
	buf[count] = '\0';

	ptr = buf;

	if(ptr[0] == 'p' || ptr[0] == 'P'){
		ptr++;
		for(i = 0; i < 2; i++){
			if(zcorepcieroot_test_get_param(&ptr, &setfunc_par[i]) < 0) {
				up( &zcorepcieroot_proc_sem );
				printk( KERN_INFO "zcorepcieroot_test_setfunc_write : parget failed:%d\n", i );
				return -EFAULT;
			}
		}

		/* setting */
		if(zcorepcieroot_bridge_access_pause(setfunc_par[0], setfunc_par[1]) < 0){
			printk( KERN_INFO "zcorepcieroot_test_setfunc_write : pause error\n");
		}
	}
	else{
		up( &zcorepcieroot_proc_sem );
		printk( KERN_INFO "zcorepcieroot_test_setfunc_write : r/w get failed\n" );
		return -EFAULT;
	}

	up( &zcorepcieroot_proc_sem );

	return count;
}

/*
 *  executed when you read /proc/zcorepci/setfunc
 *
 *  do nothing
 */
int
zcorepcieroot_test_setfunc_read( char* page, char** start, off_t offset,
				 int count, int* eof, void* data )
{
	int len = 0;

	*eof = 1;
	return len;
}
/************************************/
/*        interrupt Test            */
/************************************/
/*
 *  executed when you write to /proc/zcorepci/intr
 *
 *  format:INTA(0)/INTB(1)/INTC(2)/INTD(3)/PME_HP(4)/ERR(5)
 */
int
zcorepcieroot_test_intr_write( struct file* filp, const char* buffer,
			       unsigned long count, void* data )
{
	int             i;
	char            buf[ZCOREPCIEROOT_TEST_PARAMBUF_MAX];
	unsigned int    intr_par[2];
	unsigned int    pin_select, in_data;
	char*           ptr;

	if ( down_interruptible( &zcorepcieroot_proc_sem) ) {
		printk( KERN_INFO "zcorepcieroot_test_intr_write: down_interruptible failed\n");
		return -ERESTARTSYS;
	}

	if ( count <= 0 || ZCOREPCIEROOT_TEST_PARAMBUF_MAX < count ) {
		up( &zcorepcieroot_proc_sem );
		printk( KERN_INFO "zcorepcieroot_test_intr_write : count failed\n");
		return -EFAULT;
	}

	if ( copy_from_user( buf, buffer, count ) ) {
		up( &zcorepcieroot_proc_sem );
		printk( KERN_INFO "zcorepcieroot_test_intr_write : copy_from_user failed\n" );
		return -EFAULT;
	}
	buf[count] = '\0';

	ptr = buf;
	for(i = 0; i < 2; i++){
		if(zcorepcieroot_test_get_param(&ptr, &intr_par[i]) < 0) {
			up( &zcorepcieroot_proc_sem );
			printk( KERN_INFO "zcorepcieroot_test_intr_write : parget failed:%d\n", i );
			return -EFAULT;
		}
	}

	if(5 < intr_par[1]){
		up( &zcorepcieroot_proc_sem );
		printk( KERN_INFO "zcorepcieroot_test_intr_write : param error\n" );
		return -EFAULT;
	}

	pin_select  = (0x1 << intr_par[1]);
	in_data     = (0x100 << intr_par[1]);
	iowrite32(pin_select,
		  ZCOREPCIEROOT_BRG_PIP_SET2(zcorepcieroot_info[intr_par[0]].bridge_vm));
	iowrite32((pin_select|in_data),
		  ZCOREPCIEROOT_BRG_PIP_SET2(zcorepcieroot_info[intr_par[0]].bridge_vm));

	up( &zcorepcieroot_proc_sem );

	return count;
}

/*
 *  executed when you read /proc/zcorepci/intr
 *
 *  do nothing
 */
int
zcorepcieroot_test_intr_read( char* page, char** start, off_t offset,
			      int count, int* eof, void* data )
{
	int len = 0;

	*eof = 1;
	return len;
}


/*******************/
/*  common func    */
/*******************/
/* proc initialize */
int zcorepcieroot_test_init( void )
{
	struct proc_dir_entry*  entry, *parent;
	int                     i;

	parent = create_proc_entry( "zcorepci", S_IFDIR, NULL );
	if(!parent ){
		printk( KERN_INFO "zcorepcieroot_test_init: zcorepci failed\n");
	}


	for(i = 0;zcorepcieroot_procinfo[i].path != NULL; i++){
		entry = create_proc_entry( zcorepcieroot_procinfo[i].path,
					   S_IFREG | S_IRUGO | S_IWUGO, parent );
		if ( entry != NULL ) {
			entry->read_proc  = zcorepcieroot_procinfo[i].proc_read;
			entry->write_proc = zcorepcieroot_procinfo[i].proc_write;
		}
		else {
			printk( KERN_INFO "zcorepcieroot_test_init: %s failed\n",
                                zcorepcieroot_procinfo[i].path);
			return -EBUSY;
		}
	}

	sema_init( &zcorepcieroot_proc_sem, 1 );

	return 0;
}

/*
 *  number Parameter Get from string
 *  we cannot use strtok, etc
 *
 *  add 0x at hexadecimal number
 */
int zcorepcieroot_test_get_param(char** str_ptr, unsigned int *val)
{
	unsigned int    retval  = 0;
	unsigned int    temp;
	char*           str     = *str_ptr;
	int             order   = 10;

	/* ignore space  */
	if(*str == ' '){
		while(*str == ' '){
			str++;
		}
	}

	if(*str == '\0'){
		printk("zcorepcieroot_test_get_param:NULL err\n");
		return -1;
	}

	/* ignore 0x */
	if(*str == '0' && *(str + 1) == 'x'){
		order = 16;
		str += 2;
		if(*str == '\0' || *str == ' '){
			printk("zcorepcieroot_test_get_param:NULL/Space err\n");
			return -1;
		}
	}

	while(1){

		if('a' <= *str && *str <= 'f'){
			if(order != 16){
				printk("zcorepcieroot_test_get_param:hex val err\n");
				return -1;
			}
			temp = (*str - 'a' + 10);
		}
		else if('A' <= *str && *str <= 'F'){
			if(order != 16){
				printk("zcorepcieroot_test_get_param:HEX val err\n");
				return -1;
			}
			temp = (*str - 'A' + 10);
		}
		else if('0' <= *str && *str <= '9'){
			temp = (*str - '0');
		}
		else{
			printk("zcorepcieroot_test_get_param:invalid val err '%c' \n",*str);
			return -1;
		}
		retval += temp;

		str++;
		if(*str == '\0' || *str == ' ' || *str == '\r' || *str == '\n'){
			break;
		}
		else{
			retval *= order;
		}
	}

	*str_ptr = str;
	*val = retval;

	return 0;
}
#endif
