/* 2013-05-29: File added and changed by Sony Corporation */
/*
 * Copyright 2013 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA.
 *
 */

#include "hal_cmd.h"

/*==============================================================
         AES128 Functions
==============================================================*/

/******************************************************
Initialize AES
*******************************************************/
void a_Init_Aes(void)
{
    *p_a128n_INIT_0 = 0x00000000;
}

/******************************************************
Write 128bits of data into normal register
*******************************************************/
void a128_Write_Idata(const void* pInData)
{
    *p_a128n_INPUT_DATA3 = ((uint32_t *)pInData)[0];
    *p_a128n_INPUT_DATA2 = ((uint32_t *)pInData)[1];
    *p_a128n_INPUT_DATA1 = ((uint32_t *)pInData)[2];
    *p_a128n_INPUT_DATA0 = ((uint32_t *)pInData)[3];
}

/******************************************************
Set IV / Set Key for unwrap
*******************************************************/
void a128_Set_Iv(const void* pInData)
{
    *p_a128n_INPUT_DATA3 = ((uint32_t *)pInData)[0];
    *p_a128n_INPUT_DATA2 = ((uint32_t *)pInData)[1];
    *p_a128n_INPUT_DATA1 = ((uint32_t *)pInData)[2];
    *p_a128n_INPUT_DATA0 = ((uint32_t *)pInData)[3];
    *p_a128n_SET_IV = 0x00000000;
}

/******************************************************
Set nonce, IV & counter for AES-CTR
*******************************************************/
void a128_Set_Ctr(const void* pInData)
{
    *p_a128n_INPUT_DATA3 = ((uint32_t *)pInData)[0];
    *p_a128n_INPUT_DATA2 = ((uint32_t *)pInData)[1];
    *p_a128n_INPUT_DATA1 = ((uint32_t *)pInData)[2];
    *p_a128n_INPUT_DATA0 = ((uint32_t *)pInData)[3];
    *p_a128n_SET_CTR = 0x00000000;
}

/******************************************************
IV check
*******************************************************/
#ifdef XARINA_KEY_WRAP
void a128_Check_Iv(const void* pInData)
{
    *p_a128n_INPUT_DATA3 = ((uint32_t *)pInData)[0];
    *p_a128n_INPUT_DATA2 = ((uint32_t *)pInData)[1];
    *p_a128n_INPUT_DATA1 = 0x00000000;
    *p_a128n_INPUT_DATA0 = 0x00000000;
    *p_a128n_CHK_IV = 0x00000000;
}
#endif

/******************************************************
Set Key2 for AES-CBC encrypt or AES-CTR
*******************************************************/
void a_Set_K2_enc(void)
{
    *p_a128n_SET_KE = 0x00000000;
}

/******************************************************
Set Key2 for AES-CBC decrypt
*******************************************************/
void a_Set_K2_dec(void)
{
    *p_a128n_SET_KD = 0x00000000;
}

/******************************************************
AES-CBC encryption with Key2 (output->normal register)
*******************************************************/
void a_CBCenc_K2(void)
{
    *p_a128n_ENC_K2 = 0x00000000;
}

/******************************************************
AES-CBC decryption with Key2 (output->normal register)
*******************************************************/
void a_CBCdec_K2(void)
{
    *p_a128n_DEC_K2 = 0x00000000;
}

/******************************************************
AES-ECB encryption with Key2 (output->normal register)
*******************************************************/
void a_ECBenc_K2(void)
{
    *p_a128n_EECB_K2 = 0x00000000;
}

/******************************************************
AES-ECB decryption with Key2 (output->normal register)
*******************************************************/
void a_ECBdec_K2(void)
{
    *p_a128n_DECB_K2 = 0x00000000;
}

/******************************************************
AES-CTR with Key2 (output->normal register)
*******************************************************/
void a_CTR_K2(void)
{
    *p_a128n_CTR_K2 = 0x00000000;
}

/******************************************************
AES Unwrap Kmac|REK(256bits) with Key2(128bits)
(output->shadow register)
*******************************************************/
#ifdef XARINA_KEY_WRAP
void a128_UWP_K2_KmacRek(const void* pInData)
{
    *p_a128n_INPUT_DATA3 = ((uint32_t *)pInData)[0];
    *p_a128n_INPUT_DATA2 = ((uint32_t *)pInData)[1];
    *p_a128n_INPUT_DATA1 = 0x00000000;
    *p_a128n_INPUT_DATA0 = 0x00000000;
    *p_a128n_UWP_K21 = 0x00000000;
}

/******************************************************
AES Unwrap CEK1(128bits) with Key2(128bits)
(output->shadow register)
*******************************************************/
void a128_UWP_K2_Cek1(const void* pInData)
{
    *p_a128n_INPUT_DATA3 = ((uint32_t *)pInData)[0];
    *p_a128n_INPUT_DATA2 = ((uint32_t *)pInData)[1];
    *p_a128n_INPUT_DATA1 = 0x00000000;
    *p_a128n_INPUT_DATA0 = 0x00000000;
    *p_a128n_UWP_K23 = 0x00000000;
}

/******************************************************
AES Unwrap CEK2(128bits) with Key2(128bits)
(output->shadow register)
*******************************************************/
void a128_UWP_K2_Cek2(const void* pInData)
{
    *p_a128n_INPUT_DATA3 = ((uint32_t *)pInData)[0];
    *p_a128n_INPUT_DATA2 = ((uint32_t *)pInData)[1];
    *p_a128n_INPUT_DATA1 = 0x00000000;
    *p_a128n_INPUT_DATA0 = 0x00000000;
    *p_a128n_UWP_K22 = 0x00000000;
}

/******************************************************
AES Unwrap Kx|Kd(256bits) with K2(128bits)
*******************************************************/
void a128_UWP_K2_Kd(const void* pInData)
{
    *p_a128n_INPUT_DATA3 = ((uint32_t *)pInData)[0];
    *p_a128n_INPUT_DATA2 = ((uint32_t *)pInData)[1];
    *p_a128n_INPUT_DATA1 = 0x00000000;
    *p_a128n_INPUT_DATA0 = 0x00000000;
    *p_a128n_UWP_K24 = 0x00000000;
}
#endif

/******************************************************
Read status of AES (128bits)
*******************************************************/
void a128_Read_Status_aes(void* pOutData)
{
    ((uint32_t *)pOutData)[0] = *p_a128n_STATUS_AES3;
    ((uint32_t *)pOutData)[1] = *p_a128n_STATUS_AES2;
    ((uint32_t *)pOutData)[2] = *p_a128n_STATUS_AES1;
    ((uint32_t *)pOutData)[3] = *p_a128n_STATUS_AES0;
}

