/* 2013-05-29: File added and changed by Sony Corporation */
/*
 * Copyright 2013 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA.
 *
 */

#ifndef _AES128_HAL_H
#define _AES128_HAL_H

/*==============================================================
         AES128 Functions
==============================================================*/

/******************************************************
Initialize AES
<input>  ---
<output> ---
<return> void
*******************************************************/
void a_Init_Aes(void);

/******************************************************
Write 128bits of data into normal register
<input>  Input Data (128bits)
<output> ---
<return> void
*******************************************************/
void a128_Write_Idata(const void* pInData);

/******************************************************
Set IV / Set Key for unwrap
<input>  IV / Key (128bits)
<output> ---
<return> void
*******************************************************/
void a128_Set_Iv(const void* pInData);

/******************************************************
Set nonce, IV & counter for AES-CTR
<input>  ---
<output> ---
<return> void
*******************************************************/
void a_Set_Ctr(void);

/******************************************************
Set nonce, IV & counter for AES-CTR
<input>  Nonce(32bits)|IV(64bits)|Counter(32bits) (128bits)
<output> ---
<return> void
*******************************************************/
void a128_Set_Ctr(const void* pInData);

/******************************************************
IV check
<input>  IV(64bits)
<output> ---
<return> void
*******************************************************/
#ifdef XARINA_KEY_WRAP
void a128_Check_Iv(const void* pInData);
#endif

/******************************************************
Set Key2 for AES-CBC encrypt or AES-CTR
<input>  ---
<output> ---
<return> void
*******************************************************/
void a_Set_K2_enc(void);

/******************************************************
Input and set Key2 for AES-CBC decrypt
<input>  ---
<output> ---
<return> void
*******************************************************/
void a_Set_K2_dec(void);

/******************************************************
AES-CBC encryption with Key2 (output->normal register)
<input>  ---
<output> ---
<return> void
*******************************************************/
void a_CBCenc_K2(void);

/******************************************************
AES-CBC decryption with Key2 (output->normal register)
<input>  ---
<output> ---
<return> void
*******************************************************/
void a_CBCdec_K2(void);

/******************************************************
AES-ECB encryption with Key2 (output->normal register)
<input>  ---
<output> ---
<return> void
*******************************************************/
void a_ECBenc_K2(void);

/******************************************************
AES-ECB decryption with Key2 (output->normal register)
<input>  ---
<output> ---
<return> void
*******************************************************/
void a_ECBdec_K2(void);

/******************************************************
AES-CTR with Key2 (output->normal register)
<input>  ---
<output> ---
<return> void
*******************************************************/
void a_CTR_K2(void);

/******************************************************
AES Unwrap Kmac|REK(256bits) with Key2(128bits)
(output->shadow register)
<input>  counter(64bits)
<output> ---
<return> void
*******************************************************/
#ifdef XARINA_KEY_WRAP
void a128_UWP_K2_KmacRek(const void* pInData);

/******************************************************
AES Unwrap CEK1(128bits) with Key2(128bits)
(output->shadow register)
<input>  counter(64bits)
<output> ---
<return> void
*******************************************************/
void a128_UWP_K2_Cek1(const void* pInData);

/******************************************************
AES Unwrap CEK2(128bits) with Key2(128bits)
(output->shadow register)
<input>  counter(64bits)
<output> ---
<return> void
*******************************************************/
void a128_UWP_K2_Cek2(const void* pInData);

/******************************************************
AES Unwrap Kx|Kd(256bits) with K2(128bits)
<input>  counter(64bits)
<output> ---
<return> void
*******************************************************/
void a128_UWP_K2_Kd(const void* pInData);
#endif

/******************************************************
Read status of AES (128bits)
<input>  ---
<output> status of AES (128bits)
<return> void
*******************************************************/
void a128_Read_Status_aes(void* pOutData);

#endif // _AES128_HAL_H
