/* 2013-05-29: File added by Sony Corporation */
/*
 * Copyright 2013 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA.
 *
 */

#include "hal_cmd.h"

/*==============================================================
         Functions for DMAC
==============================================================*/

/******************************************************
Start DMA
*******************************************************/
void d_Start_Dma(uint32_t inData)
{
    *p_bb_DMA_CTRL = CONVERT_ENDIAN(inData);
}

/******************************************************
Stop DMA
*******************************************************/
void d_Stop_Dma(void)
{
    *p_bb_DMA_CTRL = 0x00000000;
}

/******************************************************
Set number of DMA transfer
*******************************************************/
void d_Set_DmaCount(uint32_t inData)
{
    *p_bb_DMA_CNT = CONVERT_ENDIAN(inData);
}

/******************************************************
Set command to DMAC
*******************************************************/
void d_Set_DmaAutoCommand(uint32_t inData)
{
    uint32_t tmp32;

    tmp32 = (CONVERT_ENDIAN(*p_bb_DMA_CMD) & 0xFFFF0000)
            | (inData & 0x0000FFFF);
    *p_bb_DMA_CMD = CONVERT_ENDIAN(tmp32);
}

/******************************************************
Set DMA command unit
*******************************************************/
void d_Set_DmaCmdUnit(uint32_t inData)
{
    const uint32_t  param[5] = {
        0x00000000,
        0x01000000,
        // original : 0x02010000
        0x02000000, // for Xarina PL080 DMA.
        0x03000000,
        0x04020000
    };

    uint32_t tmp32;

    if (inData < (sizeof(param) / sizeof(uint32_t))) {
        tmp32 = CONVERT_ENDIAN(*p_bb_DMA_CMD) & 0x0000FFFF;
        tmp32 |= param[inData];
        *p_bb_DMA_CMD = CONVERT_ENDIAN(tmp32);
    }
}

/******************************************************
Set destination address for DMA transfer
*******************************************************/
void d_Set_WriteDstAddress(uint32_t inData)
{
    *p_bb_DMA_WADR = CONVERT_ENDIAN(inData & 0xC000FFFF);
}

/******************************************************
Set source address for DMA transfer
*******************************************************/
void d_Set_ReadSrcAddress(uint32_t inData)
{
    *p_bb_DMA_RADR = CONVERT_ENDIAN(inData & 0xC000FFFF);
}

/******************************************************
Write DMA select wait
*******************************************************/
void d_Write_DmaSwait(uint32_t inData)
{
    *p_bb_DMA_SWAIT = CONVERT_ENDIAN(inData & 0x00000003);
}
