/* 2013-05-29: File added by Sony Corporation */
/*
 * Copyright 2013 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA.
 *
 */

#ifndef _DMA_HAL_H
#define _DMA_HAL_H

/*==============================================================
         Functions for DMAC
==============================================================*/

/******************************************************
Start DMA
<input>  Start Select (32bits)
            bit 31- 0 : 0x00000008 Read DMA
                        0x00000004 Write DMA
                        0x00000002 Write/Read DMA without Auto Command
                        0x00000001 Write/Read DMA with Auto Command
<output> ---
<return> void
*******************************************************/
void d_Start_Dma(uint32_t inData);

/******************************************************
Stop DMA
<input>  ---
<output> ---
<return> void
*******************************************************/
void d_Stop_Dma(void);

/******************************************************
Set number of DMA transfer
<input>  DMA Count (32bits)
<output> ---
<return> void
*******************************************************/
void d_Set_DmaCount(uint32_t inData);

/******************************************************
Set command to DMAC
<input>  Auto Command (32bits)
            bit 31-16 : ---
            bit 15- 0 : Auto Command
<output> ---
<return> void
*******************************************************/
void d_Set_DmaAutoCommand(uint32_t inData);

/******************************************************
Set DMA command unit
<input>  DMA Command unit (32bits)
            bit 31- 0 : 0x00000001 : 1 Word
                        0x00000002 : 2 Word
                        0x00000003 : 3 Word
                        0x00000004 : 4 Word
<output> ---
<return> void
*******************************************************/
void d_Set_DmaCmdUnit(uint32_t inData);

/******************************************************
Set destination address for DMA transfer
<input>  Destination Address (32bits)
            bit 31-30 : DMA_WADR_DEC
                         00:same point
                         01:increment
                         10:decryment
                         11:reserved
            bit 29-16 : ---
            bit 15- 0 : DMA_WADR
<output> ---
<return> void
*******************************************************/
void d_Set_WriteDstAddress(uint32_t inData);

/******************************************************
Set source address for DMA transfer
<input>  Source Address (32bits)
            bit 31-30 : DMA_RADR_DEC
                         00:same point
                         01:increment
                         10:decryment
                         11:reserved
            bit 29-16 : ---
            bit 15- 0 : DMA_RADR
<output> ---
<return> void
*******************************************************/
void d_Set_ReadSrcAddress(uint32_t inData);

/******************************************************
Write DMA select wait
<input>  Select Wait (8bits)
            bit 31- 2 : ---
            bit     1 : R_SWAIT (when DMA read)
                         0: don't wait interrupt of Crypto Engine
                         1: wait interrupt of Crypto Engine
            bit     0 : W_SWAIT (when DMA write)
                         0: don't wait interrupt of Crypto Engine
                         1: wait interrupt of Crypto Engine
<output> ---
<return> void
*******************************************************/
void d_Write_DmaSwait(uint32_t inData);


#endif // _DMA_HAL_H
