/* 2013-05-29: File added by Sony Corporation */
/*
 * Copyright 2013 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA.
 *
 */

#include "hal_cmd.h"

/*==============================================================
         General Functions
==============================================================*/

/******************************************************
Read interrupt status
*******************************************************/
uint32_t i_Read_IrqSts(void)
{
    return CONVERT_ENDIAN(*p_bb_IRQ_STS);
}

/******************************************************
Enable normal interrupt of crypto engine
*******************************************************/
void i_Set_IrqenNml(uint32_t inData)
{
    *p_bb_IRQEN_NML = CONVERT_ENDIAN(inData);
}

/******************************************************
Enable error interrupt of crypto engine
*******************************************************/
void i_Set_IrqenErr(uint32_t inData)
{
    *p_bb_IRQEN_ERR = CONVERT_ENDIAN(inData);
}

/******************************************************
Enable DMA interrupt of crypto engine
*******************************************************/
void i_Set_IrqenDma(uint32_t inData)
{
    *p_bb_IRQEN_DMA = CONVERT_ENDIAN(inData & 0x00000001);
}

/******************************************************
Read normal interrupt status
*******************************************************/
uint32_t i_Read_IrqstsNml(void)
{
    return CONVERT_ENDIAN(*p_bb_IRQSTS_NML);
}

/******************************************************
Read error interrupt status
*******************************************************/
uint32_t i_Read_IrqstsErr(void)
{
    return CONVERT_ENDIAN(*p_bb_IRQSTS_ERR);
}

/******************************************************
Read dma interrupt status
*******************************************************/
uint32_t i_Read_IrqstsDma(void)
{
    return CONVERT_ENDIAN(*p_bb_IRQSTS_DMA);
}

/******************************************************
Clear normal interrupt
*******************************************************/
void i_Set_IrqClrNml(uint32_t inData)
{
    *p_bb_IRQCLR_NML = CONVERT_ENDIAN(inData);
}

/******************************************************
Clear error interrupt
*******************************************************/
void i_Set_IrqClrErr(uint32_t inData)
{
    *p_bb_IRQCLR_ERR = CONVERT_ENDIAN(inData);
}

/******************************************************
Clear DMA interrupt
*******************************************************/
void i_Set_IrqClrDma(void)
{
    *p_bb_IRQCLR_DMA = CONVERT_ENDIAN(0x00000001);
}

/******************************************************
Clock control
*******************************************************/
void g_Clk_Ctrl(uint32_t inData)
{
    *p_bb_CLK_CTRL = CONVERT_ENDIAN(inData & 0xFFFFFFFE);
}

/******************************************************
Initialize crypto engine chosen by clear engine map
*******************************************************/
void sc_All_Clr(uint32_t inData)
{
    *p_sc_ALL_CLR = CONVERT_ENDIAN(inData);
}

