/* 2013-05-29: File added by Sony Corporation */
/*
 * Copyright 2013 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA.
 *
 */

#ifndef _GENERAL_HAL_H
#define _GENERAL_HAL_H

/*==============================================================
         General Functions
==============================================================*/

/******************************************************
Read interrupt status
<input>  ---
<output> ---
<return> Interrupt Status (32bits)
            bit 31- 3 : ---
            bit     2 : IRQ_D
            bit     1 : IRQ_E
            bit     0 : IRQ_N
*******************************************************/
uint32_t i_Read_IrqSts(void);

/******************************************************
Enable normal interrupt of crypto engine
<input>  Enable Bit (32bits)
            bit 31- 0 : IEN_NML[31:0]
                         0: disable 1: enable
<output> ---
<return> void
*******************************************************/
void i_Set_IrqenNml(uint32_t inData);

/******************************************************
Enable error interrupt of crypto engine
<input>  Enable Bit (32bits)
            bit 31- 0 : IEN_ERR[31:0]
                         0: disable 1: enable
<output> ---
<return> void
*******************************************************/
void i_Set_IrqenErr(uint32_t inData);

/******************************************************
Enable DMA interrupt of crypto engine
<input>  Enable Bit (32bits)
            bit 31- 1 : ---
            bit     0 : IEN_D
                         0: disable 1: enable
<output> ---
<return> void
*******************************************************/
void i_Set_IrqenDma(uint32_t inData);

/******************************************************
Read normal interrupt status
<input>  ---
<output> ---
<return> Interrupt Requests Bit (32bits)
            bit 31- 0 : IRQ_NML[31:0]
                         0: no interrupt 1: request interrupt
*******************************************************/
uint32_t i_Read_IrqstsNml(void);

/******************************************************
Read error interrupt status
<input>  ---
<output> ---
<return> Interrupt Requests Bit (32bits)
            bit 31- 0 : IRQ_ERR[31:0]
                         0: no interrupt 1: request interrupt
*******************************************************/
uint32_t i_Read_IrqstsErr(void);

/******************************************************
Read error interrupt status
<input>  ---
<output> ---
<return> Interrupt Requests Bit (32bits)
            bit 31- 1 : ---
            bit     0 : IRQ_D
                         0: no interrupt 1: request interrupt
*******************************************************/
uint32_t i_Read_IrqstsDma(void);

/******************************************************
Clear normal interrupt
<input>  Clear Interrupt Bit (32bits)
            bit 31- 0 : ICLR_NML[31:0]
                         0: nothing  1: clear interrupt
<output> ---
<return> void
*******************************************************/
void i_Set_IrqClrNml(uint32_t inData);

/******************************************************
Clear error interrupt
<input>  Clear Interrupt Bit (32bits)
            bit 31- 0 : ICLR_ERR[31:0]
                         0: nothing 1: clear interrupt
<output> ---
<return> void
*******************************************************/
void i_Set_IrqClrErr(uint32_t inData);

/******************************************************
Clear DMA interrupt
<input>  ---
<output> ---
<return> void
*******************************************************/
void i_Set_IrqClrDma(void);

/******************************************************
Clock control
<input>  Crypto Engine map (32bits)
            bit 31- 1 : CLK_EN[31:1]
            bit     0 : ---
                         0: disable 1: enable
<output> ---
<return> void
*******************************************************/
void g_Clk_Ctrl(uint32_t inData);

/******************************************************
Initialize crypto engine chosen by clear engine map
<input>  Clear Engine map (32bits)
<output> ---
<return> void
*******************************************************/
void sc_All_Clr(uint32_t inData);

#endif // _GENERAL_HAL_H
