/* 2013-05-29: File added by Sony Corporation */
/*
 * Copyright 2013 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA.
 *
 */

#include "hal_cmd.h"

/*==============================================================
         Functions for MWC
==============================================================*/

/******************************************************
Clear status
*******************************************************/
void c_Clear_Status(void)
{
    *p_mwcn_IN_COMMAND = CONVERT_ENDIAN(cClearStatus);
}

/******************************************************
Write instruction
*******************************************************/
void c_Write_InstSet(const void* pInData)
{
    int i;
    const uint32_t *inst_addr;

    inst_addr = pInData;
    *p_mwcn_SET_IRAM_PTK10 = 0x00000000;
    for(i = 0; i < 512; i++){
        *p_mwcn_SET_IRAM_PTUP = CONVERT_ENDIAN(*inst_addr);
        inst_addr++;
    }
}

/******************************************************
Set number of words of calculation unit
*******************************************************/
void c_Set_BaseWord6(uint32_t inData)
{
    *p_mwcn_IN_COMMAND = CONVERT_ENDIAN((uint32_t)cSetBaseWord6 + (inData & 0x0000003F));   // input=6bits
}

/******************************************************
Set start address for writing & reading memory in MWC
*******************************************************/
void c_Set_MemPt(uint32_t inData)
{
    *p_mwcn_SET_DRAM_PTK10 = CONVERT_ENDIAN(inData & 0x000003FF);   // input=10bits
}

/******************************************************
Write data into memory in MWC
*******************************************************/
void c_Write_Data_n(uint32_t inData)
{
    *p_mwcn_NOR_SET_PTDN = inData;
}

/******************************************************
Read data from memory in MWC
*******************************************************/
uint32_t c_Read_Data_n(void)
{
    return *p_mwcn_NOR_READ_PTDN;
}

/******************************************************
Initialize for CRT Calculation (output->normal register)
*******************************************************/
void r_Init_CRT_n(void)
{
    // write addresses of keys to DRAM
    *p_mwcn_IN_COMMAND = CONVERT_ENDIAN(cSetDramAdrK10 + (regP_crt << 10) + addrP_crt);
    *p_mwcn_IN_COMMAND = CONVERT_ENDIAN(cSetDramAdrK10 + (regQ_crt << 10) + addrQ_crt);
    *p_mwcn_IN_COMMAND = CONVERT_ENDIAN(cSetDramAdrK10 + (regDP_crt << 10) + addrDP_crt);
    *p_mwcn_IN_COMMAND = CONVERT_ENDIAN(cSetDramAdrK10 + (regDQ_crt << 10) + addrDQ_crt);
    *p_mwcn_IN_COMMAND = CONVERT_ENDIAN(cSetDramAdrK10 + (regQ1_crt << 10) + addrQ1_crt);
    *p_mwcn_IN_COMMAND = CONVERT_ENDIAN(cSetDramAdrK10 + (regQU_crt << 10) + addrQU_crt);
    *p_mwcn_IN_COMMAND = CONVERT_ENDIAN(cSetDramAdrK10 + (regRE_crt << 10) + addrRE_crt);
    *p_mwcn_IN_COMMAND = CONVERT_ENDIAN(cSetDramAdrK10 + (regTM_crt << 10) + addrTM_crt);
    *p_mwcn_IN_COMMAND = CONVERT_ENDIAN(cSetDramAdrK10 + (regTW1_crt << 10) + addrTW1n_crt);
    *p_mwcn_IN_COMMAND = CONVERT_ENDIAN(cSetDramAdrK10 + (regMW_crt << 10) + addrMW_crt);
    *p_mwcn_IN_COMMAND = CONVERT_ENDIAN(cSetDramAdrK10 + (regRW_crt << 10) + addrRW_crt);
    // CRT init
    *p_mwcn_START_PCK10 = CONVERT_ENDIAN(StartCrtInit);
}

/******************************************************
Start CRT calculation
*******************************************************/
void r_Start_CRT(void)
{
    *p_mwcn_START_PCK10 = CONVERT_ENDIAN(StartCrtEnc);
}

/******************************************************
Start RSA calculation
*******************************************************/
void r_Start_RSA(void)
{
    // write addresses of keys to DRAM
    *p_mwcn_IN_COMMAND = CONVERT_ENDIAN(cSetDramAdrK10 + (regN_rsa << 10) + addrN_rsa);
    *p_mwcn_IN_COMMAND = CONVERT_ENDIAN(cSetDramAdrK10 + (regE_rsa << 10) + addrE_rsa);
    *p_mwcn_IN_COMMAND = CONVERT_ENDIAN(cSetDramAdrK10 + (regRE_rsa << 10) + addrRE_rsa);
    *p_mwcn_IN_COMMAND = CONVERT_ENDIAN(cSetDramAdrK10 + (regTM_rsa << 10) + addrTM_rsa);
    *p_mwcn_IN_COMMAND = CONVERT_ENDIAN(cSetDramAdrK10 + (regMW_rsa << 10) + addrMW_rsa);
    *p_mwcn_IN_COMMAND = CONVERT_ENDIAN(cSetDramAdrK10 + (regRW_rsa << 10) + addrRW_rsa);
    // RSA Start
    *p_mwcn_START_PCK10 = CONVERT_ENDIAN(StartRsaEnc);
}

