/* 2013-05-29: File added by Sony Corporation */
/*
 * Copyright 2013 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA.
 *
 */

#ifndef _MWC_HAL_H
#define _MWC_HAL_H

/*==============================================================
         Functions for MWC
==============================================================*/

/******************************************************
Clear status
<input>  ---
<output> ---
<return> void
*******************************************************/
void c_Clear_Status(void);

/******************************************************
Write instruction
<input>  Instruction (32bits)
<output> ---
<return> void
*******************************************************/
void c_Write_InstSet(const void* pInData);

/******************************************************
Set number of words of calculation unit
<input>  Number of Words (32bits)
            bit 31- 6 : ---
            bit  5- 0 : Number of Words
<output> ---
<return> void
*******************************************************/
void c_Set_BaseWord6(uint32_t inData);

/******************************************************
Set start address for writing & reading memory in MWC
<input>  Start Address (32bits)
            bit 31-10 : ---
            bit  9- 0 : Start DRAM Address
<output> ---
<return> void
*******************************************************/
void c_Set_MemPt(uint32_t inData);

/******************************************************
Write data into memory in MWC
<input>  Data (32bits)
<output> ---
<return> void
*******************************************************/
void c_Write_Data_n(uint32_t inData);

/******************************************************
Read data from memory in MWC
<input>   ---
<output> ---
<returnt> Data (32bits)
*******************************************************/
uint32_t c_Read_Data_n(void);

/******************************************************
Initialize for CRT Calculation (output->normal register)
<input>  ---
<output> ---
<return> void
*******************************************************/
void r_Init_CRT_n(void);

/******************************************************
Start CRT calculation
<input>  ---
<output> ---
<return> void
*******************************************************/
void r_Start_CRT(void);

/******************************************************
Start RSA calculation
<input>  ---
<output> ---
<return> void
*******************************************************/
void r_Start_RSA(void);


#endif // _MWC_HAL_H
