/* 2013-05-29: File added by Sony Corporation */
/*
 * Copyright 2013 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA.
 *
 */

#ifndef _XCRYPTO_CMD_H
#define _XCRYPTO_CMD_H

/*==========================================================
       Command [15:0]

       [15:11] : Select Engine           (5bits)
        [11:0] : Command or Register     (3bits)
         [7:0] : Engine internal command (8bits)


       Select Engine [4:0]
          0x00 : Secure Controler
          0x01 : Reserved
          0x02 : Multi Word Calculator (RSA/ECC)
          0x03 : Reserved
          0x04 : AES128
          0x05 : Reserved
          0x06 : Reserved
          0x07 : DES64
          0x08 : Reserved
          0x09 : SHA1
          0x0A : SHA256
          0x0B : Reserved
          0x0C : Reserved
          0x0D : Reserved
          0x0E-1F : Reserved

==========================================================*/


/*---------------------------------------------------
   Bus Bridge
---------------------------------------------------*/
#define BUSBRD_numsel				0x00
#define BUSBRD_Base					(0x0800 * BUSBRD_numsel) // 0x0000
#define	bb_IRQ_STS					(BUSBRD_Base + 0x200)
#define	bb_IRQEN_NML				(BUSBRD_Base + 0x201)
#define	bb_IRQEN_ERR				(BUSBRD_Base + 0x202)
#define	bb_IRQEN_DMA				(BUSBRD_Base + 0x203)
#define	bb_IRQSTS_NML				(BUSBRD_Base + 0x204)
#define	bb_IRQSTS_ERR				(BUSBRD_Base + 0x205)
#define	bb_IRQSTS_DMA				(BUSBRD_Base + 0x206)
#define	bb_IRQCLR_NML				(BUSBRD_Base + 0x207)
#define	bb_IRQCLR_ERR				(BUSBRD_Base + 0x208)
#define	bb_IRQCLR_DMA				(BUSBRD_Base + 0x209)
#define	bb_DMA_CTRL					(BUSBRD_Base + 0x210)
#define	bb_DMA_CNT					(BUSBRD_Base + 0x211)
#define	bb_DMA_CMD					(BUSBRD_Base + 0x212)
#define	bb_DMA_WADR					(BUSBRD_Base + 0x213)
#define	bb_DMA_RADR					(BUSBRD_Base + 0x214)
#define	bb_DMA_SWAIT				(BUSBRD_Base + 0x215)
#define	bb_DMA_DAT					(BUSBRD_Base + 0x218)
#define	bb_CLK_CTRL					(BUSBRD_Base + 0x220)
#define	bb_DSWAP_CTRL				(BUSBRD_Base + 0x230)

/*---------------------------------------------------
   Secure Controller
---------------------------------------------------*/
#define SECCTR_numsel				0x00
#define SECCTR_Base					(0x0800 * SECCTR_numsel) // 0x0000
#define	sc_VERSION					(SECCTR_Base + 0x000)
#define	sc_CONFIG					(SECCTR_Base + 0x001)
#define	sc_ALL_CLR					(SECCTR_Base + 0x002)
#define	sc_DAT_TRN					(SECCTR_Base + 0x003)
#define	sc_DAT_TRN_XOR				(SECCTR_Base + 0x004)
#define	sc_XOR_DATA					(SECCTR_Base + 0x005)
#define	sc_FUL_SET					(SECCTR_Base + 0x0FE)
#define	sc_FUL_CLR					(SECCTR_Base + 0x0FF)

/*---------------------------------------------------
   Multi Word Calculator (RSA / ECC)
---------------------------------------------------*/
#define MWC_numsel					0x02
#define MWC_Base					(0x0800 * MWC_numsel) // 0x1000
// Normal Register
#define	mwcn_READ_VERSION			(MWC_Base + 0x000)
#define	mwcn_IN_COMMAND				(MWC_Base + 0x001)
#define	mwcn_READ_DE_REQ			(MWC_Base + 0x002)
#define	mwcn_READ_DE_ENR			(MWC_Base + 0x003)
#define	mwcn_NOR_READ_PTUP			(MWC_Base + 0x010)
#define	mwcn_NOR_READ_PTDN			(MWC_Base + 0x011)
#define	mwcn_READ_IRAM_PTUP			(MWC_Base + 0x012)
#define	mwcn_READ_IRAM_PTDN			(MWC_Base + 0x013)
#define	mwcn_SET_DE_REQ				(MWC_Base + 0x042)
#define	mwcn_SET_DE_ENR				(MWC_Base + 0x043)
#define	mwcn_SET_DRAM_PTK10			(MWC_Base + 0x050)
#define	mwcn_NOR_SET_PTUP			(MWC_Base + 0x053)
#define	mwcn_NOR_SET_PTDN			(MWC_Base + 0x054)
#define	mwcn_SET_IRAM_PTK10			(MWC_Base + 0x055)
#define	mwcn_SET_IRAM_PTUP			(MWC_Base + 0x056)
#define	mwcn_SET_IRAM_PTDN			(MWC_Base + 0x057)
#define	mwcn_START_PCK10			(MWC_Base + 0x068)
// Secure Register
#define	mwcs_SEC_READ_PTUP			(MWC_Base + 0x410)
#define	mwcs_SEC_READ_PTDN			(MWC_Base + 0x411)
#define	mwcs_SEC_SET_PTUP			(MWC_Base + 0x453)
#define	mwcs_SEC_SET_PTDN			(MWC_Base + 0x454)

/*---------------------------------------------------
   AES128
---------------------------------------------------*/
#define AES128_numsel				0x04
#define AES128_Base					(0x0800 * AES128_numsel) // 0x2000
// Normal Register
#define	a128n_STATUS_AES0			(AES128_Base + 0x018)
#define	a128n_STATUS_AES1			(AES128_Base + 0x019)
#define	a128n_STATUS_AES2			(AES128_Base + 0x01A)
#define	a128n_STATUS_AES3			(AES128_Base + 0x01B)
#define	a128n_INPUT_DATA0			(AES128_Base + 0x01C)
#define	a128n_INPUT_DATA1			(AES128_Base + 0x01D)
#define	a128n_INPUT_DATA2			(AES128_Base + 0x01E)
#define	a128n_INPUT_DATA3			(AES128_Base + 0x01F)
#define	a128n_INIT_0				(AES128_Base + 0x310)
#define	a128n_SET_CTR				(AES128_Base + 0x322)
#define	a128n_CBCD_0				(AES128_Base + 0x342)
#define	a128n_CBCE_0				(AES128_Base + 0x343)
#define	a128n_SET_KE				(AES128_Base + 0x350)
#define	a128n_SET_KD				(AES128_Base + 0x351)
#define	a128n_SET_IV				(AES128_Base + 0x352)
#define	a128n_DEC_K2				(AES128_Base + 0x353)
#define	a128n_ENC_K2				(AES128_Base + 0x354)
#define	a128n_CTR_K2				(AES128_Base + 0x355)
#define	a128n_WRP_K20				(AES128_Base + 0x356)
#define	a128n_WRP_K21				(AES128_Base + 0x357)
#define	a128n_WRP_K22				(AES128_Base + 0x358)
#define	a128n_WRP_K23				(AES128_Base + 0x359)
#define	a128n_UWP_K21				(AES128_Base + 0x35B)
#define	a128n_UWP_K22				(AES128_Base + 0x35C)
#define	a128n_UWP_K23				(AES128_Base + 0x35D)
#define	a128n_UWP_K24				(AES128_Base + 0x35E)
#define	a128n_WRP_K25				(AES128_Base + 0x35F)
#define	a128n_CHK_IV				(AES128_Base + 0x360)
#define	a128n_EECB_K2				(AES128_Base + 0x370)
#define	a128n_DECB_K2				(AES128_Base + 0x371)

/*---------------------------------------------------
   DES
---------------------------------------------------*/
#define DES_numsel					0x07
#define DES_Base					(0x0800 * DES_numsel) // 0x3800
// Normal Register
#define	de64n_STATUS_DES0			(DES_Base + 0x018)
#define	de64n_STATUS_DES1			(DES_Base + 0x019)
#define	de64n_INPUT_DATA0			(DES_Base + 0x01C)
#define	de64n_INPUT_DATA1			(DES_Base + 0x01D)
#define	de64n_INIT_0				(DES_Base + 0x310)
#define	de64n_SET_K2				(DES_Base + 0x350)
#define	de64n_SET_IV				(DES_Base + 0x351)
#define	de64n_ENC_K2				(DES_Base + 0x352)
#define	de64n_DEC_K2				(DES_Base + 0x353)

/*---------------------------------------------------
   SHA1
---------------------------------------------------*/
#define SHA1_numsel					0x09
#define SHA1_Base					(0x0800 * SHA1_numsel) // 0x4800
// Normal Register
#define	s1n_INIT_SHA1				(SHA1_Base + 0x010)
#define	s1n_SEND_DATA				(SHA1_Base + 0x020)
#define	s1n_SEND_LDATA1				(SHA1_Base + 0x021)
#define	s1n_SEND_LDATA2				(SHA1_Base + 0x022)
#define	s1n_SEND_LDATA3				(SHA1_Base + 0x023)
#define	s1n_SEND_LDATA4				(SHA1_Base + 0x024)
#define	s1n_NOR_OUT0				(SHA1_Base + 0x200)
#define	s1n_NOR_OUT1				(SHA1_Base + 0x201)
#define	s1n_NOR_OUT2				(SHA1_Base + 0x202)
#define	s1n_NOR_OUT3				(SHA1_Base + 0x203)
#define	s1n_NOR_OUT4				(SHA1_Base + 0x204)

/*---------------------------------------------------
   SHA256
---------------------------------------------------*/
#define SHA256_numsel				0x0A
#define SHA256_Base					(0x0800 * SHA256_numsel) // 0x5000
// Normal Register
#define	s2n_INIT_SHA256				(SHA256_Base + 0x010)
#define	s2n_SEND_DATA				(SHA256_Base + 0x020)
#define	s2n_SEND_LDATA1				(SHA256_Base + 0x021)
#define	s2n_SEND_LDATA2				(SHA256_Base + 0x022)
#define	s2n_SEND_LDATA3				(SHA256_Base + 0x023)
#define	s2n_SEND_LDATA4				(SHA256_Base + 0x024)
#define	s2n_NOR_OUT0				(SHA256_Base + 0x200)
#define	s2n_NOR_OUT1				(SHA256_Base + 0x201)
#define	s2n_NOR_OUT2				(SHA256_Base + 0x202)
#define	s2n_NOR_OUT3				(SHA256_Base + 0x203)
#define	s2n_NOR_OUT4				(SHA256_Base + 0x204)
#define	s2n_NOR_OUT5				(SHA256_Base + 0x205)
#define	s2n_NOR_OUT6				(SHA256_Base + 0x206)
#define	s2n_NOR_OUT7				(SHA256_Base + 0x207)

#endif // _XCRYPTO_CMD_H
