/* 2013-05-27: File added and changed by Sony Corporation */
/*
 * Copyright 2013 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA.
 *
 */

#ifdef XARINA_DMA
/*-----------------------------------------------------------------------------
 * Include Files
 *---------------------------------------------------------------------------*/
#include <linux/module.h>
#include <linux/kernel.h>      /* printk() */
#include <linux/slab.h>        /* kmalloc() */
#include <linux/errno.h>       /* error codes */
#include <linux/types.h>       /* size_t */
#include <linux/dma-mapping.h> /* dma_map_single() */
#include <asm/system.h>        /* mb() */

#include <linux/amba/pl08x.h>
#include <asm/hardware/pl080.h>

#include "hal/hal_cmd.h"
#include "xcrypto.h"


/*-----------------------------------------------------------------------------
 * Private Macro Definition
 *---------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
 * Private typedef Declaration
 *---------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
 * Private Function Declaration
 *---------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
 * Private Variable Definition
 *---------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
 * Public Function Definition
 *---------------------------------------------------------------------------*/
/*****************************************************************************/
void xcrypto_dma_setparam_dmac_sg(void *dev, int maxburst)
{
	struct scatterlist  *src_sg, *dst_sg;
	int                 src_nents, dst_nents;

	struct xcrypto_device     *dd = (struct xcrypto_device *)dev;
	dma_context_t             *dma_ctx = &dd->dma_ctx;

	struct dma_slave_config	dma_rx_conf;
	struct dma_slave_config dma_tx_conf;
	struct dma_chan *dma_rx, *dma_tx;
	struct dma_async_tx_descriptor *dma_rx_desc;
	struct dma_async_tx_descriptor *dma_tx_desc;
	dma_cookie_t cookie;

	struct pl08x_extra_config extra_config;

	CTX_DMA->nbytes = dd->total;
	CTX_DMA->nwords = dd->total / sizeof(int32_t);
	dma_rx = CTX_DMA->rxch;
	dma_tx = CTX_DMA->txch;

	src_sg = CTX_DMA->src_sg;
	dst_sg = CTX_DMA->dst_sg;
	src_nents = CTX_DMA->src_nents;
	dst_nents = CTX_DMA->dst_nents;

	/*
	 * Configure dma rx, tx channels
	 */
	dma_rx_conf.direction = DMA_TO_DEVICE;
	dma_rx_conf.dst_addr = (dma_addr_t)(XCRYPTO_BASE_ADRS + (bb_DMA_DAT << 2));
	dma_rx_conf.src_addr_width = DMA_SLAVE_BUSWIDTH_4_BYTES;
	dma_rx_conf.src_maxburst = maxburst;
	dma_rx_conf.dst_addr_width = DMA_SLAVE_BUSWIDTH_4_BYTES;
	dma_rx_conf.dst_maxburst = maxburst;

	dma_rx->device->device_control(dma_rx,
			DMA_SLAVE_CONFIG,
			(unsigned long)&dma_rx_conf);


	dma_tx_conf.direction = DMA_FROM_DEVICE;
	dma_tx_conf.src_addr = (dma_addr_t)(XCRYPTO_BASE_ADRS + (bb_DMA_DAT << 2));
	dma_tx_conf.src_addr_width = DMA_SLAVE_BUSWIDTH_4_BYTES;
	dma_tx_conf.src_maxburst = maxburst;
	dma_tx_conf.dst_addr_width = DMA_SLAVE_BUSWIDTH_4_BYTES;
	dma_tx_conf.dst_maxburst = maxburst;

	extra_config.control = 0;
	extra_config.config = 0;
	extra_config.control = PL080_CONTROL_DST_AHB2 | PL080_CONTROL_SRC_AHB2;
	extra_config.config = PL080_CONFIG_DMB | PL080_CONFIG_SMB;
	dma_tx->device->device_control(dma_tx,
			DMA_EXTRA_CONFIG,
			(unsigned long)&extra_config);

	dma_tx->device->device_control(dma_tx,
			DMA_SLAVE_CONFIG,
			(unsigned long)&dma_tx_conf);

	/*
	 * Prepare slave dma scatter gather list
	 */
	dma_rx_desc = dma_rx->device->device_prep_slave_sg(dma_rx,
			src_sg,
			src_nents,
			DMA_TO_DEVICE,
			DMA_CTRL_ACK);
	if (!dma_rx_desc) {
		DPRINTK(1, "dma_rx_desc prepare failed\n");
		return;
	}
	dma_rx_desc->callback = NULL;

	dma_tx_desc = dma_tx->device->device_prep_slave_sg(dma_tx,
			dst_sg,
			dst_nents,
			DMA_FROM_DEVICE,
			DMA_CTRL_ACK);
	if (!dma_tx_desc) {
		DPRINTK(1, "dma_tx_desc prepare failed\n");
		return;
	}
	dma_tx_desc->callback = NULL;

	/*
	 * submit DMA
	 */
	cookie = dma_rx_desc->tx_submit(dma_rx_desc);
	if (dma_submit_error(cookie)) {
		DPRINTK(1, "dma rx submit failed\n");
		return;
	}

	cookie = dma_tx_desc->tx_submit(dma_tx_desc);
	if (dma_submit_error(cookie)) {
		DPRINTK(1, "dma tx submit failed\n");
		return;
	}

	dd->total -= CTX_DMA->nbytes;
}

int xcrypto_dma_setctx(void *dev)
{
	struct xcrypto_device     *dd = (struct xcrypto_device *)dev;
	dma_context_t             *dma_ctx = &dd->dma_ctx;
	int ret;

	DPRINTK(1, "\n");
	if (unlikely(!dd->in_sg || !dd->out_sg)) {
		printk(KERN_ERR "sg is NULL\n");
		ret = -EINVAL;
		goto err1;
	}

	CTX_DMA->src_sg = dd->in_sg;
	CTX_DMA->dst_sg = dd->out_sg;
	CTX_DMA->src_nents = xcrypto_sg_length(dd->req, dd->in_sg);
	CTX_DMA->dst_nents = xcrypto_sg_length(dd->req, dd->out_sg);

	ret = dma_map_sg(NULL, CTX_DMA->src_sg, CTX_DMA->src_nents, DMA_TO_DEVICE);
	if (!ret) {
		printk(KERN_ERR "dma map failed\n");
		ret = -EINVAL;
		goto err1;
	}

	ret = dma_map_sg(NULL, CTX_DMA->dst_sg, CTX_DMA->dst_nents, DMA_FROM_DEVICE);
	if (!ret) {
		printk(KERN_ERR "dma map failed\n");
		ret = -EINVAL;
		goto err2;
	}

	CTX_DMA->rxch = xcrypto_get_rxdma_ch();
	CTX_DMA->txch = xcrypto_get_txdma_ch();

	return 0;

err2:
	dma_unmap_sg(NULL, CTX_DMA->src_sg, CTX_DMA->src_nents, DMA_TO_DEVICE);
err1:
	CTX_DMA->src_sg = NULL;
	CTX_DMA->dst_sg = NULL;

	return ret;
}

void xcrypto_dma_clearctx(void *dev)
{
	struct xcrypto_device     *dd = (struct xcrypto_device *)dev;
	dma_context_t             *dma_ctx = &dd->dma_ctx;

	if (CTX_DMA->src_sg != NULL)
		dma_unmap_sg(NULL, CTX_DMA->src_sg, CTX_DMA->src_nents, DMA_TO_DEVICE);

	if (CTX_DMA->dst_sg != NULL)
		dma_unmap_sg(NULL, CTX_DMA->dst_sg, CTX_DMA->dst_nents, DMA_FROM_DEVICE);

	/* clear dma_ctx except rxch txch */
	memset(dma_ctx, 0, sizeof(dma_context_t));
	CTX_DMA->rxch = xcrypto_get_rxdma_ch();
	CTX_DMA->txch = xcrypto_get_txdma_ch();
}

void issue_dma(struct dma_chan *ch)
{
	ch->device->device_issue_pending(ch);
}

void xcrypto_dma_start_sg(void *dma_ctx, uint32_t dma_ctrl)
{
	d_Start_Dma(dma_ctrl);

	issue_dma(CTX_DMA->rxch);          /* Receive from XCRYPTO */
	issue_dma(CTX_DMA->txch);          /* Transfer to  XCRYPTO */
}

void xcrypto_dma_stop_sg(void *dma_ctx)
{
	struct dma_chan *dma_rx, *dma_tx;
	d_Stop_Dma();

	dma_rx = CTX_DMA->rxch;
	dma_tx = CTX_DMA->txch;

	dma_rx->device->device_control(dma_rx, DMA_TERMINATE_ALL, 0);
	dma_tx->device->device_control(dma_tx, DMA_TERMINATE_ALL, 0);
}
/*-----------------------------------------------------------------------------
 * Private Function Definition
 *---------------------------------------------------------------------------*/

#endif  /* #ifdef XARINA_DMA */
