/* 2013-05-27: File added and changed by Sony Corporation */
/*
 * Copyright 2013 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA.
 *
 */

#ifdef XARINA_DMA

#ifndef XCRYPTO_DMA_H
#define XCRYPTO_DMA_H

/*-----------------------------------------------------------------------------
 * Include Files
 *---------------------------------------------------------------------------*/
#include <asm/dma.h>

/*-----------------------------------------------------------------------------
 * Definition
 *---------------------------------------------------------------------------*/
#define XCRYPTO_DMA_START_AUTOCMD               (0x00000001UL)  // d_Start_Dma
#define XCRYPTO_DMA_WRITE_DMA                   (0x00000004UL)  // d_Start_Dma
#define XCRYPTO_DMA_WRAP_1                      (0x00000001UL)  // d_Set_DmaCmdUnit
#define XCRYPTO_DMA_WRAP_2                      (0x00000002UL)  // d_Set_DmaCmdUnit
#define XCRYPTO_DMA_WRAP_4                      (0x00000004UL)  // d_Set_DmaCmdUnit
#define XCRYPTO_DMA_ADR_KEEP                    (0x00000000UL)  // d_Set_WriteDstAddress & d_Set_ReadSrcAddress
#define XCRYPTO_DMA_ADR_INC                     (0x40000000UL)  // d_Set_WriteDstAddress & d_Set_ReadSrcAddress
#define XCRYPTO_DMA_ADR_DEC                     (0x80000000UL)  // d_Set_WriteDstAddress & d_Set_ReadSrcAddress
#define XCRYPTO_DMA_ADR_NOUSE                   (0xC0000000UL)  // d_Set_WriteDstAddress & d_Set_ReadSrcAddress
#define XCRYPTO_DMA_NO_WAIT                     (0x00000000UL)  // d_Write_DmaSwait & d_Read_DmaSwait
#define XCRYPTO_DMA_W_WAIT                      (0x00000001UL)  // d_Write_DmaSwait & d_Read_DmaSwait
#define XCRYPTO_DMA_R_WAIT                      (0x00000002UL)  // d_Write_DmaSwait & d_Read_DmaSwait

#define XCRYPTO_DMA_MAX_TRANSFER_SIZE           (0x00000FF0UL)  // split dma size

#define XCRYPTO_DMA_CONT_B1                     (1)             // burst size is 1
#define XCRYPTO_DMA_CONT_B4                     (4)             // burst size is 4

#define CTX_DMA                                 ((dma_context_t *)dma_ctx)

/*-----------------------------------------------------------------------------
 * Type Declaration
 *---------------------------------------------------------------------------*/
typedef struct {
    dma_addr_t              bus_src_addr;
    dma_addr_t              bus_dst_addr;
    uint32_t                nbytes;
    uint32_t                nwords;

    struct scatterlist      *src_sg;
    struct scatterlist      *dst_sg;
    int                     src_nents;
    int                     dst_nents;

    struct dma_chan         *rxch;
    struct dma_chan         *txch;

} dma_context_t;

/*-----------------------------------------------------------------------------
 * Public Function Declaration
 *---------------------------------------------------------------------------*/
extern uint32_t xcrypto_dma_setparam_dmac(void *dma_ctx, uint8_t *src, uint8_t *dst, uint32_t nbytes, uint32_t ctrl_burst);
extern void xcrypto_dma_start(void *dma_ctx, uint32_t dma_ctrl);
extern void     xcrypto_dma_stop(void *dma_ctx);
extern void xcrypto_dma_setparam_dmac_sg(void *dev, int maxburst);
extern void xcrypto_dma_start_sg(void *dma_ctx, uint32_t dma_ctrl);
extern void xcrypto_dma_stop_sg(void *dma_ctx);
extern int xcrypto_dma_setctx(void *dev);
extern void xcrypto_dma_clearctx(void *dev);

#endif  /* XCRYPTO_DMA_H */
#endif  /* XARINA_DMA */
