/* 2011-11-02: File added and changed by Sony Corporation */
/*
 *  Copyright 2008-2010 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */
#ifndef I2C_I2C00_H
#define I2C_I2C00_H 1

/********************************************************/
/* User configurations                                  */
/********************************************************/
#define I2C00_CLK		(CONFIG_I2C00_CLOCK)
#define PSCLK_NSEC		(1000000000/I2C00_CLK)

/* Platform infomation */
#define I2C00_RAM_OFFSET        0x1000  /* Offset from base address */
#define I2C00_MAP_SIZE          0x2000  /* Register+RAM map total region size */


/********************************************************/
/* Offset address of Registers                          */
/********************************************************/
#define ICDA            (0x000)         /* data shift register */
#define ICAR            (0x000)         /* address register */
#define ICCS            (0x004)         /* control status register */
#define ICCM            (0x008)         /* mode register */
#define ICCCL           (0x00c)         /* clock control register low */
#define ICCCH           (0x010)         /* clock control register high */
#define ICATMC          (0x014)         /* auto transfer control register */
#define ICTCR           (0x018)         /* auto transfer byte register */
#define ICATMS          (0x01c)         /* auto transfer status register */
#define ICDS            (0x020)         /* data setup control register */
#define ICINT           (0x024)         /* interrupt status register */
#define ICINTC          (0x028)         /* interrupt clear register */
#define ICBE            (0x02c)         /* dummy-i/o register */
#define ICAL            (0x030)         /* arbitration status register */
#define ICID            (0x0fc)         /* ID register */

#define ICRAM           (I2C00_RAM_OFFSET) /* RAM top offset address */
#define ICRAMSIZE       (64)            /* RAM buffer byte size */

/********************************************************/
/* Bit assign                                           */
/********************************************************/
/* ICCS */
#define MST             (0x80)
#define TRX             (0x40)
#define BB              (0x20)
#define PIN             (0x10)
#define ES0             (0x08)
#define AAS             (0x04)
#define AD0             (0x02)
#define LRB             (0x01)
#define BC2             (0x04)
#define BC1             (0x02)
#define BC0             (0x01)

/* ICCM */
#define ACK             (0x80)
#define ALMSK           (0x40)
#define EXADR           (0x08)
#define ATM             (0x04)
#define SDA             (0x02)
#define SCL             (0x01)

/* ICATMC */
#define IRST            (0x80)
#define ERRST           (0x40)
#define EXT             (0x10)
#define APC             (0x04)
#define AT1             (0x02)
#define AT0             (0x01)

/* ICATMS */
#define AS2             (0x04)
#define AS1             (0x02)
#define AS0             (0x01)

/* ICINT */
#define I2CSEN          (0x40)
#define ATMEN           (0x20)
#define I2CEN           (0x10)
#define INTI2CS         (0x04)
#define INTATM          (0x02)
#define INTI2C          (0x01)

/* ICINTC */
#define INTI2CSC        (0x04)
#define INTATMC         (0x02)
#define INTI2CC         (0x01)

/* ICBE */
#define TSDAIE          (0x80)
#define TSCLIE          (0x40)
#define TSDAOE          (0x20)
#define TSCLOE          (0x10)
#define TSDAI           (0x08)
#define TSCLI           (0x04)
#define TSDAO           (0x02)
#define TSCLO           (0x01)

/* ICAL */
#define AL              (0x01)

/********************************************************/
/* Macro                                                */
/********************************************************/
/* Transfer rate */
#define I2C00_100KBPS           (0)             /* Standard mode (100kbps) */
#define I2C00_400KBPS           (1)             /* Fast mode (400kbps) */

#define I2C00_SEMI              (AT0)           /* Semi auto (ST/Sr + data) */
#define I2C00_SP                (AT1)           /* Stop condition (SP) */
#define I2C00_FULL              (AT1 | AT0)     /* Full auto (ST/Sr + data + SP) */

/* Slave address */
#define SLAVE_ADDR_MAX          (0x7F)          /* Max value of slave address */
#define SLAVE_ADDR_10BIT_MAX    (0x3FF)         /* Max value of slave address (10bit) */

/* Timing parameter,  STANDARD-MODE(100kHz) */
#define STD_TR_MUL1000          (0)                              /* SCL up-edge time(ns) * 1000 */
#define STD_TF_MUL1000          (0)                              /* SCL down-edge time(ns) * 1000 */
#define STD_THIGH_MUL1000       (4300000 - STD_TF_MUL1000)       /* SCL clock high, hold time(ns) * 1000 */
#define STD_TLOW_MUL1000        (5700000 - STD_TR_MUL1000)       /* SCL clock low, hold time(ns) * 1000 */
#define STD_VIHMIN_MUL1000      (700)                            /* VIHmin(vdd) * 1000 */
#define STD_VILMAX_MUL1000      (300)                            /* VILmin(vdd) * 1000 */
#define STD_TSUDAT_MUL1000      (250000)                         /* data setup time(ns) * 1000 */
#define STD_LCNT_MUL1000        (STD_TLOW_MUL1000 -			\
				 ((STD_TR_MUL1000 * (STD_VILMAX_MUL1000 - 0)) / \
				  (STD_VIHMIN_MUL1000 - STD_VILMAX_MUL1000)))     /* Low count time * 1000 */
#define STD_HCNT_MUL1000        (STD_THIGH_MUL1000 -			\
				 ((STD_TF_MUL1000 * (1000 - STD_VIHMIN_MUL1000)) / \
				  (STD_VIHMIN_MUL1000 - STD_VILMAX_MUL1000)))     /* High count time * 1000 */

#define STD_LCNT                (STD_LCNT_MUL1000 / 1000)
#define STD_HCNT                (STD_HCNT_MUL1000 / 1000)
#define STD_TR_TSUDAT           ((STD_TR_MUL1000 + STD_TSUDAT_MUL1000) / 1000)

/* Timing parameter,  FAST-MODE(400kHz) */
#define FAST_TR_MUL1000         (0)                              /* SCL up-edge time(ns) * 1000 */
#define FAST_TF_MUL1000         (0)                              /* SCL down-edge time(ns) * 1000 */
#define FAST_THIGH_MUL1000      (900000  - FAST_TF_MUL1000)      /* SCL clock high, hold time(ns) * 1000 */
#define FAST_TLOW_MUL1000       (1600000 - FAST_TR_MUL1000)      /* SCL clock low, hold time(ns) * 1000 */
#define FAST_VIHMIN_MUL1000     (700)                            /* VIHmin(vdd) * 1000 */
#define FAST_VILMAX_MUL1000     (300)                            /* VILmin(vdd) * 1000 */
#define FAST_TSUDAT_MUL1000     (100000)                         /* data setup time(ns) * 1000 */
#define FAST_LCNT_MUL1000       (FAST_TLOW_MUL1000 -			\
				 ((FAST_TR_MUL1000 * (FAST_VILMAX_MUL1000 - 0)) / \
				  (FAST_VIHMIN_MUL1000 - FAST_VILMAX_MUL1000)))   /* Low count time * 1000 */
#define FAST_HCNT_MUL1000       (FAST_THIGH_MUL1000 -			\
				 ((FAST_TF_MUL1000 * (1000 - FAST_VIHMIN_MUL1000)) / \
				  (FAST_VIHMIN_MUL1000 - FAST_VILMAX_MUL1000)))   /* High count time * 1000 */
#define FAST_LCNT               (FAST_LCNT_MUL1000 / 1000)
#define FAST_HCNT               (FAST_HCNT_MUL1000 / 1000)
#define FAST_TR_TSUDAT          ((FAST_TR_MUL1000 + FAST_TSUDAT_MUL1000) / 1000)

/* Configuration Macro */
#define I2C00_CFG_ICDH          (((PSCLK_NSEC *  7) >= 300) ? 0 :	\
                                 ((PSCLK_NSEC * 15) >= 300) ? 1 :	\
                                 ((PSCLK_NSEC * 23) >= 300) ? 2 : 3)

#define ADDJUST                 1  /* 0:down round 1:up round */

#define GET_ICCCL(rate)         ({					\
			unsigned long _lval =				\
				(((rate) == I2C00_400KBPS) ?		\
				 (unsigned long)(FAST_LCNT / PSCLK_NSEC) : \
				 (unsigned long)(STD_LCNT  / PSCLK_NSEC)) + ADDJUST; \
			((_lval < 6) ? 6 : _lval);			\
		})

#define GET_ICCCH(rate)         ({					\
			unsigned long _lval =				\
				(((rate) == I2C00_400KBPS) ?		\
				 (unsigned long)(FAST_HCNT / PSCLK_NSEC) : \
				 (unsigned long)(STD_HCNT  / PSCLK_NSEC)) + ADDJUST; \
			((_lval < 6) ? 6 : _lval);			\
		})

#define GET_ICDS(rate)          ({					\
			unsigned long _lval =				\
				(((rate) == I2C00_400KBPS) ?		\
				 (unsigned long)(FAST_TR_TSUDAT / PSCLK_NSEC) : \
				 (unsigned long)(STD_TR_TSUDAT  / PSCLK_NSEC)) + ADDJUST; \
			((_lval == 0) ? 1 : _lval);			\
		})

#endif  /* I2C_I2C00_H */
