/* 2013-07-05: File added by Sony Corporation */
/*
 *  drivers/mmc/card/max_discard.c
 *
 *  Copyright 2013 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */

#include <linux/blkdev.h>

#include <linux/mmc/card.h>

#include "max_discard.h"

#define LINE_MAX_LEN	16

ssize_t mmc_max_discard_sectors_show(struct device *dev,
				     struct device_attribute *attr, char *buf)
{
	struct mmc_card *card = container_of(dev, struct mmc_card, dev);

	return snprintf(buf, LINE_MAX_LEN, "%u\n",
			card->queue->limits.max_discard_sectors);
}

ssize_t mmc_max_discard_sectors_store(struct device *dev,
				      struct device_attribute *attr,
				      const char *buf, size_t len)
{
	struct mmc_card *card = container_of(dev, struct mmc_card, dev);
	int ret;
	unsigned long max_discard;

	ret = strict_strtoul(buf,  10, &max_discard);
	if (ret) {
		printk(KERN_ERR "strict_strtol() failed (ret=%d)\n", ret);
		return ret;
	}

	if (max_discard == 0) {
		card->queue->limits.max_discard_sectors =
			MMC_DEFAULT_MAX_DISCARD_SECTORS;
	} else {
		card->queue->limits.max_discard_sectors = max_discard;
	}

	return len;
}
