/* 2013-07-02: File added by Sony Corporation */
/*
 *  drivers/mmc/core/trim_priority.c
 *
 *  Copyright 2013 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */

#include <linux/mmc/card.h>

#include "trim_priority.h"

#define LINE_MAX_LEN	16

void mmc_get_blk_discard_rq_cmd(struct mmc_card *card,
				unsigned int *arg)
{
	if (card->trim_has_priority && mmc_can_trim(card)) {
		*arg = MMC_TRIM_ARG;
	} else if (mmc_can_discard(card)) {
		*arg = MMC_DISCARD_ARG;
	} else if (mmc_can_trim(card)) {
		*arg = MMC_TRIM_ARG;
	} else {
		*arg = MMC_ERASE_ARG;
	}
}

ssize_t mmc_trim_has_priority_show(struct device *dev,
				   struct device_attribute *attr, char *buf)
{
	struct mmc_card *card = container_of(dev, struct mmc_card, dev);

	return snprintf(buf, LINE_MAX_LEN, "%d\n", card->trim_has_priority);
}

ssize_t mmc_trim_has_priority_store(struct device *dev,
				    struct device_attribute *attr,
				    const char *buf, size_t len)
{
	struct mmc_card *card = container_of(dev, struct mmc_card, dev);
	int ret;
	long trim_prio;

	ret = strict_strtol(buf,  10, &trim_prio);
	if (ret) {
		printk(KERN_ERR "strict_strtol() failed (ret=%d)\n", ret);
		return ret;
	}

	if ((trim_prio != 0) && (trim_prio != 1)) {
		printk(KERN_ERR "Invalid value specified (%ld)\n", trim_prio);
		return -EINVAL;
	}

	card->trim_has_priority = (int)trim_prio;

	return len;
}

ssize_t mmc_blk_discard_rq_cmd_show(struct device *dev,
				    struct device_attribute *attr,
				    char *buf)
{
	struct mmc_card *card = container_of(dev, struct mmc_card, dev);
	char *cmd;
	unsigned int arg;

	mmc_get_blk_discard_rq_cmd(card, &arg);
	switch (arg) {
	case MMC_DISCARD_ARG:
		cmd = "Discard";
		break;
	case MMC_TRIM_ARG:
		cmd = "TRIM";
		break;
	case MMC_ERASE_ARG:
		cmd = "Erase";
		break;
	default:
		printk(KERN_ERR "Unknown command (%u)\n", arg);
		return -EINVAL;
	}

	return snprintf(buf,  LINE_MAX_LEN, "%s\n", cmd);
}
