/* 2011-10-03: File added and changed by Sony Corporation */
/*
 *  SDIF SDHC controller driver for the platform bus.
 *
 *  Copyright 2010, 2011 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */

#include <linux/device.h>
#include <linux/mmc/host.h>
#include <linux/sdhci-pltfm.h>
#include "sdhci.h"
#include "sdhci-pltfm.h"
#include <mach/sdif.h>
#include <mach/xarina_sysreg1.h>

#define SDIF_SD_AMBA_CTL0			(0x0100)
#define SDIF_SD_AMBA_CTL1			(0x0104)
#define SDIF_SD_CAP_CTL				(0x0200)
#define SDIF_SD_MAXC_CTL			(0x0208)
#define SDIF_SD_CEATA_CAP_CTL			(0x0210)
#define SDIF_SD_CLK_DLY_CTL			(0x0214)


/* sdif_sdhci_get_ro -
 *  return 0 if read/write card
 *  return 1 else if read-only card
 */
static unsigned int sdif_sdhci_get_ro(struct sdhci_host *host)
{
	if (xarina_board_is_rd()) {
		/* workaround for Xarina RD board */
		pr_debug("sdhci-sdif: disable write protection mode\n");
		return 0;
	}

	return !(sdhci_readl(host, SDHCI_PRESENT_STATE) & SDHCI_WRITE_PROTECT);
}


static struct sdhci_ops sdif_sdhci_ops = {
	.get_ro	= sdif_sdhci_get_ro,
};


/* sdif_pltfm_init - initialize SDIF Host Controller */
static int sdif_pltfm_init(struct sdhci_host *host)
{
        /* Vendor Specific Register */
	sdhci_writew(host, SDIF_SD_AMBA_CTL0_VAL,	SDIF_SD_AMBA_CTL0);
	sdhci_writew(host, SDIF_SD_AMBA_CTL1_VAL,	SDIF_SD_AMBA_CTL1);
	sdhci_writel(host, SDIF_SD_CAP_CTL_VAL,		SDIF_SD_CAP_CTL);
	sdhci_writel(host, SDIF_SD_MAXC_CTL_VAL,	SDIF_SD_MAXC_CTL);
	sdhci_writew(host, SDIF_SD_CEATA_CAP_CTL_VAL,	SDIF_SD_CEATA_CAP_CTL);
	sdhci_writew(host, SDIF_SD_CLK_DLY_CTL_VAL,	SDIF_SD_CLK_DLY_CTL);

	pr_info("sdhci-sdif: Capabilities Register: 0x%08X\n",
		sdhci_readl(host, SDHCI_CAPABILITIES));

	return 0;
}

struct sdhci_pltfm_data sdhci_sdif_pdata = {
	.ops	= &sdif_sdhci_ops,
	.init	= sdif_pltfm_init,
	.quirks	= (
		   SDHCI_QUIRK_BROKEN_TIMEOUT_VAL |
		   SDHCI_QUIRK_NO_ENDATTR_IN_NOPDESC |
		   SDHCI_QUIRK_32BIT_DMA_ADDR |
		   SDHCI_QUIRK_32BIT_DMA_SIZE |
		   SDHCI_QUIRK_32BIT_ADMA_SIZE |
		   SDHCI_QUIRK_FORCE_BLK_SZ_2048 |
#if 0	/*  for debug: disable DMA */
		   SDHCI_QUIRK_BROKEN_ADMA | SDHCI_QUIRK_BROKEN_DMA |
#endif
#if 0	/*  for debug: disable Hi-speed */
		   SDHCI_QUIRK_NO_HISPD_BIT |
#endif
		   SDHCI_QUIRK_MULTIBLOCK_READ_ACMD12
		   ),
};
