/* 2012-03-12: File added by Sony Corporation */
/*
 * nand_tmbuf.h - a very simple temporary buffer designed to avoid
 * memory fragment with kmalloc
 */

#ifndef __NAND_TMPBUF_H__
#define __NAND_TMPBUF_H__

#include <linux/kernel.h>
#include <linux/mempool.h>

struct tmp_buf_s {
	void *addr; //buffer location
	int priv; //to save page order
};

static inline int init_tmpbuf(struct tmp_buf_s *tmpbuf, int size)
{
	int order;

	if (unlikely(NULL == tmpbuf || 0 == size)) {
		printk(KERN_WARNING "TMPBUF: Invalid parameter\n");
		return -EINVAL;
	}

	order = get_order((unsigned long)size);

	tmpbuf->addr = (void *)__get_free_pages(GFP_KERNEL, order);
	if (NULL == tmpbuf->addr) {
		printk(KERN_ERR "TMPBUF: Failed to get free pages\n");
		return -ENOMEM;
	}

	tmpbuf->priv = order;

	return 0;
}

static inline void release_tmpbuf(struct tmp_buf_s *tmpbuf)
{
	if (unlikely(NULL != tmpbuf && NULL != tmpbuf->addr)) {
		free_pages((unsigned long)tmpbuf->addr, tmpbuf->priv);
		tmpbuf->addr = NULL;
		tmpbuf->priv = 0;
	}
}

static inline void* alloc_tmpbuf(struct tmp_buf_s *tmpbuf, int size)
{
	int order;

	if (unlikely(NULL == tmpbuf || 0 == size)) {
		printk(KERN_WARNING "TMPBUF: Invalid parameter\n");
		return NULL;
	}

	order = get_order((unsigned long)size);
	if (order > tmpbuf->priv) {
		printk(KERN_ERR "TMPBUF: Invalid buffer size\n");
		return NULL;
	}

	return tmpbuf->addr;
}

static inline void free_tmpbuf(struct tmp_buf_s *tmpbuf)
{
}

#endif
