/* 2011-10-03: File added and changed by Sony Corporation */
/*
 *  Copyright 2011 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */

#include <linux/init.h>
#include <linux/io.h>
#include <linux/delay.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/platform_device.h>
#include <linux/types.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/nand.h>
#include <linux/mtd/partitions.h>
#include <linux/slab.h>
#include <mach/platform.h>
#ifdef CONFIG_MTD_XARINA_NAND_DMAC
#include <linux/dmaengine.h>
#include <linux/dma-mapping.h>
#include <linux/scatterlist.h>
#include <linux/amba/pl08x.h>
#endif

#include "xarina_nand1.h"

#define XARINA_NAND1_MODULE_TEST

static int debug=5;
#define DBG1(format, arg...)	if (debug>=1) printk(KERN_ALERT "XARINA_NAND1: " format, ## arg)
#define DBG2(format, arg...)	if (debug>=2) printk(KERN_ALERT "XARINA_NAND1: " format, ## arg)
#define DBG3(format, arg...)	if (debug>=3) printk(KERN_ALERT "XARINA_NAND1: " format, ## arg)
#define DBG4(format, arg...)	if (debug>=4) printk(KERN_ALERT "XARINA_NAND1: " format, ## arg)
#define DBG5(format, arg...)	if (debug>=5) printk(KERN_ALERT "XARINA_NAND1: " format, ## arg)
#define DBG6(format, arg...)	if (debug>=6) printk(KERN_ALERT "XARINA_NAND1: " format, ## arg)
#define DBG7(format, arg...)	if (debug>=7) printk(KERN_ALERT "XARINA_NAND1: " format, ## arg)
#define DBG8(format, arg...)	if (debug>=8) printk(KERN_ALERT "XARINA_NAND1: " format, ## arg)

#ifdef CONFIG_MTD_PARTITIONS
static const char *part_probes[] = { "cmdlinepart", NULL };
#endif

/* XARINA_NAND1 ECCLAYOUT : ecc(4+4) * 4 for (512+8) * 4 bytes */
static struct nand_ecclayout nand_oob_32 = {/* for 1GB */
	.eccbytes = XARINA_NAND1_ECC_TOTAL,
	.eccpos = { 32, 33, 34, 35, 36, 37, 38, 39,
				40, 41, 42, 43, 44, 45, 46, 47,
				48, 49, 50, 51, 52, 53, 54, 55,
				56, 57, 58, 59, 60, 61, 62, 63},
	.oobfree = { {0, XARINA_NAND1_OOBSIZE - XARINA_NAND1_ECC_TOTAL}, },
};

/* for bbt */
static uint8_t bbt_pattern[] = {'B', 'b', 't', '0' };
static uint8_t mirror_pattern[] = {'1', 't', 'b', 'B' };

/* main discriptor */
static struct nand_bbt_descr xarina_bbt_main_descr = {
	.options = NAND_BBT_LASTBLOCK | NAND_BBT_CREATE |
			NAND_BBT_WRITE | NAND_BBT_2BIT |
			NAND_BBT_VERSION | NAND_BBT_PERCHIP |
			NAND_BBT_SCAN2NDPAGE |NAND_NO_SUBPAGE_WRITE,
	.offs = 1,		/* BBT pattern offset */
	.len = 4,		/* BBT pattern length */
	.veroffs = 0,		/* version location */
	.maxblocks = 4,		/* btt location max block */
	.pattern = bbt_pattern	/* bbt pattern */
};

/* mirror discriptor */
static struct nand_bbt_descr xarina_bbt_mirror_descr = {
	.options = NAND_BBT_LASTBLOCK | NAND_BBT_CREATE |
			NAND_BBT_WRITE | NAND_BBT_2BIT |
			NAND_BBT_VERSION | NAND_BBT_PERCHIP |
			NAND_BBT_SCAN2NDPAGE |NAND_NO_SUBPAGE_WRITE,
	.offs =	1,		/* BBT pattern offset */
	.len = 4,		/* BBT pattern length */
	.veroffs = 0,		/* version location */
	.maxblocks = 4,		/* btt location max block */
	.pattern = mirror_pattern	/* bbt pattern */
};

static uint8_t scan_ff_pattern[] = { 0xff };

static struct nand_bbt_descr xarina_largepage_flashbased = {
	.options = NAND_BBT_SCAN2NDPAGE | NAND_BBT_SCANEMPTY,
	.offs = 0,		/* bad block mark offset */
	.len = 1,		/* bad block mark length */
	.pattern = scan_ff_pattern	/* good block mark pattern */
};

struct xarina_nand1_info {
	struct platform_nand_chip *pdata;
#ifdef CONFIG_MTD_PARTITIONS
	struct mtd_partition	*parts;
#endif
	struct mtd_info		mtd;
	struct nand_chip	nand;

	u_char	cmp_buf[XARINA_NAND1_PAGESIZE];
	u_char	status_id_read_cnt;
	u_char	read_byte_reg[6];
	u_short	ecc_task;

	/* hw ecc calculation result */
	u_short	check_ecc_buf;

#ifdef CONFIG_MTD_XARINA_NAND_DMAC
	struct dma_chan                 *dma_channel;
	struct sg_table                 sgt;
	void *dma_buf;
	bool f_use_own_buf;
	struct completion cmp;
#endif

};

#ifdef CONFIG_MTD_XARINA_NAND_DMAC
#include <mach/platform.h>
static void dump_dmareg_print(void)
{
	void *dmac_base;

	dmac_base = ioremap_nocache(XARINA_A11_DMAC_PHYS_BASE,
								XARINA_A11_DMAC_SIZE);
	if (!dmac_base)
		return;

	printk(KERN_EMERG "DMACIntStatus 0x%08x\n", ioread32(dmac_base + 0x0000));
	printk(KERN_EMERG "DMACIntTCStatus 0x%08x\n", ioread32(dmac_base + 0x0004));
	printk(KERN_EMERG "DMACIntErrorStatus 0x%08x\n", ioread32(dmac_base + 0x000c));
	printk(KERN_EMERG "DMACRawIntTCStatus 0x%08x\n", ioread32(dmac_base + 0x0014));
	printk(KERN_EMERG "DMACRawIntErrorStatus 0x%08x\n", ioread32(dmac_base + 0x0018));
	printk(KERN_EMERG "DMAEnabledChns 0x%08x\n", ioread32(dmac_base + 0x0001c));
	printk(KERN_EMERG "DMACSoftBReq 0x%08x\n", ioread32(dmac_base + 0x0020));
	printk(KERN_EMERG "DMACSoftSReq 0x%08x\n", ioread32(dmac_base + 0x0024));
	printk(KERN_EMERG "DMACSoftLBReq 0x%08x\n", ioread32(dmac_base + 0x0028));
	printk(KERN_EMERG "DMACSoftLSReq 0x%08x\n", ioread32(dmac_base + 0x002c));
	printk(KERN_EMERG "DMACConfiguration 0x%08x\n", ioread32(dmac_base + 0x0030));
	printk(KERN_EMERG "DMACSync 0x%08x\n", ioread32(dmac_base + 0x0034));
	printk(KERN_EMERG "DMACSReqMask 0x%08x\n", ioread32(dmac_base + 0x0038));
	printk(KERN_EMERG "DMACSrcAddr_0 0x%08x\n", ioread32(dmac_base + 0x0100));
	printk(KERN_EMERG "DMACDestAddr_0 0x%08x\n", ioread32(dmac_base + 0x0104));
	printk(KERN_EMERG "DMACLLI_0 0x%08x\n", ioread32(dmac_base + 0x0108));
	printk(KERN_EMERG "DMACDefControl_0 0x%08x\n", ioread32(dmac_base + 0x010c));
	printk(KERN_EMERG "DMACConfiguration_0 0x%08x\n", ioread32(dmac_base + 0x0110));
	printk(KERN_EMERG "DMACDefLLI_0 0x%08x\n", ioread32(dmac_base + 0x0114));

	iounmap(dmac_base);
}

static void dump_nandreg_print(void)
{
	void *nand_base;

	nand_base = ioremap_nocache(XARINA_NANDIF_PHYS_BASE,
								XARINA_NANDIF_SIZE);
	if (!nand_base)
		return;

	printk(KERN_EMERG "NF_MODE 0x%04x\n", ioread16(nand_base + 0x0000));
	printk(KERN_EMERG "NF_DSIZE 0x%04x\n", ioread16(nand_base + 0x0004));
	printk(KERN_EMERG "NF_TASK 0x%04x\n", ioread16(nand_base + 0x00008));
	printk(KERN_EMERG "NF_CTLSTS 0x%04x\n", ioread16(nand_base + 0x000c));
	printk(KERN_EMERG "NF_CMD1 0x%04x\n", ioread16(nand_base + 0x0020));
	printk(KERN_EMERG "NF_CMD2 0x%04x\n", ioread16(nand_base + 0x0030));
	printk(KERN_EMERG "NF_CMD3 0x%04x\n", ioread16(nand_base + 0x0034));
	printk(KERN_EMERG "NF_CADR01 0x%04x\n", ioread16(nand_base + 0x0024));
	printk(KERN_EMERG "NF_PADR01 0x%04x\n", ioread16(nand_base + 0x0028));
	printk(KERN_EMERG "NF_PADR02 0x%04x\n", ioread16(nand_base + 0x002c));
	printk(KERN_EMERG "NF_ECCLR 0x%04x\n", ioread16(nand_base + 0x0040));
	printk(KERN_EMERG "NF_ECCHK 0x%04x\n", ioread16(nand_base + 0x0044));
	printk(KERN_EMERG "NF_ECRSL_LM 0x%04x\n", ioread16(nand_base + 0x0048));
	printk(KERN_EMERG "NF_ECRSL_H_STSD 0x%04x\n", ioread16(nand_base + 0x004c));
	printk(KERN_EMERG "NF_ECERBIT 0x%04x\n", ioread16(nand_base + 0x0050));
	printk(KERN_EMERG "NF_ECERADR 0x%04x\n", ioread16(nand_base + 0x0058));
	printk(KERN_EMERG "NF_NANDMODE 0x%04x\n", ioread16(nand_base + 0x0060));
	printk(KERN_EMERG "NF_AUTOCTL 0x%04x\n", ioread16(nand_base + 0x0100));
	printk(KERN_EMERG "NF_STOP 0x%04x\n", ioread16(nand_base + 0x0104));
	printk(KERN_EMERG "NF_RIS 0x%04x\n", ioread16(nand_base + 0x0110));
	printk(KERN_EMERG "NF_IS 0x%04x\n", ioread16(nand_base + 0x0114));
	printk(KERN_EMERG "NF_IEN 0x%04x\n", ioread16(nand_base + 0x0118));

	iounmap(nand_base);
}

static bool xarina_nand1_dma_filter(struct dma_chan *chan, void *fn_param){

	struct pl08x_dma_chan *plchan = container_of(chan, struct pl08x_dma_chan, chan);

	DBG8("xarina_nand1_dma_filter(plchan): %s\n",plchan->name);

	/* check channel name */
	if(strcmp(plchan->name, fn_param)==0)
		return true;
	else
		return false;

}
#endif
/**
 * xarina_nand1_write_protect
 *
 * set write_protect on or off
 * protect :  ON or OFF
*/
static void xarina_nand1_write_protect(struct mtd_info *mtd, int protect)
{
	struct nand_chip *chip = mtd->priv;

	if(protect == XARINA_NAND1_WP_OFF){
		DBG8("XARINA_NAND1 XARINA_NAND1_WP_OFF\n");
		/* write protect off*/
		WRITE16(READ16(chip->IO_ADDR_R + XARINA_NAND1_STATUS_CONTROL_REG) | XARINA_NAND1_BIT_FWPN, chip->IO_ADDR_W + XARINA_NAND1_STATUS_CONTROL_REG);
	}else if(protect == XARINA_NAND1_WP_ON){
		DBG8("XARINA_NAND1 XARINA_NAND1_WP_ON\n");
	/* write protect on*/
		WRITE16(READ16(chip->IO_ADDR_R + XARINA_NAND1_STATUS_CONTROL_REG) & ~XARINA_NAND1_BIT_FWPN, chip->IO_ADDR_W + XARINA_NAND1_STATUS_CONTROL_REG);
	}else{
		DBG1("invalid value for write_protect\n");
	}
	DBG8("XARINA_NAND1 CONTROL_REG : 0x%04x\n",READ16(chip->IO_ADDR_W + XARINA_NAND1_STATUS_CONTROL_REG));

	return;
}

/**
 * xarina_nand1_check_task
 *
 * check if task finished
 * task :  task_bits
 *
 * ret=0 indicates task_finished
*/
static int xarina_nand1_check_task(struct mtd_info *mtd, u_short task)
{
	int ret;
	struct nand_chip *chip = mtd->priv;

	ret = READ16(chip->IO_ADDR_W + XARINA_NAND1_TASK_REG) & task;

	return ret;
}

/**
 * xarina_nand1_set_task
 *
 * write task function
 * task :  task_bits
*/
static void xarina_nand1_set_task(struct mtd_info *mtd, u_short task)
{

	struct nand_chip *chip = mtd->priv;
	u_short tmp_task;

	tmp_task = READ16(chip->IO_ADDR_W + XARINA_NAND1_TASK_REG);

	tmp_task |= task;

	WRITE16(tmp_task, chip->IO_ADDR_W + XARINA_NAND1_TASK_REG);

	DBG8("TASK_REG:0x%04x\n",READ16(chip->IO_ADDR_W + XARINA_NAND1_TASK_REG));

	return;
}

/**
 * xarina_nand1_write_cmd
 *
 * write command function
 * cmd1,2,3 : -1 indicates invalid
*/
static void xarina_nand1_write_cmd(struct mtd_info *mtd, unsigned cmd1, unsigned cmd2, unsigned cmd3)
{
	u_short tmp_cmd;
	struct nand_chip *chip = mtd->priv;
	struct xarina_nand1_info *info = container_of(mtd, struct xarina_nand1_info, mtd);

	/* cmd2 & cmd3 are optional */
	if(cmd3!=XARINA_NAND1_CMD_INVALID)
		DBG1("Error: xarina_nand1_write_cmd()\n");

	if(cmd1 != XARINA_NAND1_CMD_INVALID){
		tmp_cmd = cmd1 & XARINA_NAND1_MASK_8BIT;
		/* set command1 */
		WRITE16(tmp_cmd, chip->IO_ADDR_W + XARINA_NAND1_COMMAND1_REG);

	}

	if(cmd2 != XARINA_NAND1_CMD_INVALID){
		tmp_cmd = cmd2 & XARINA_NAND1_MASK_8BIT;
		/* set command2 */
		WRITE16(tmp_cmd, chip->IO_ADDR_W + XARINA_NAND1_COMMAND2_REG);
	}

	if(cmd3 != XARINA_NAND1_CMD_INVALID){
		tmp_cmd = cmd3 & XARINA_NAND1_MASK_8BIT;
		/* set command3 */
		WRITE16(tmp_cmd, chip->IO_ADDR_W + XARINA_NAND1_COMMAND3_REG);
	}

	if(cmd1==NAND_CMD_READID){/* check only cmd1 */
		tmp_cmd = READ16(chip->IO_ADDR_W + XARINA_NAND1_MODE_REG);
		tmp_cmd &= ~XARINA_NAND1_BIT_REG_NUM;
		tmp_cmd |= XARINA_NAND1_BIT_READ_ID;
		WRITE16(tmp_cmd, chip->IO_ADDR_W + XARINA_NAND1_MODE_REG);
		DBG8("Set RegNum for NAND_CMD_READID ...0x%04x\n",XARINA_NAND1_BIT_READ_ID);
		info->status_id_read_cnt = 0;
	}

	if(cmd1==NAND_CMD_STATUS){/* check only cmd1 */
		tmp_cmd = READ16(chip->IO_ADDR_W + XARINA_NAND1_MODE_REG);
		tmp_cmd &= ~XARINA_NAND1_BIT_REG_NUM;
		tmp_cmd |= XARINA_NAND1_BIT_READ_STATUS;
		WRITE16(tmp_cmd, chip->IO_ADDR_W + XARINA_NAND1_MODE_REG);
		DBG8("Set RegNum for NAND_CMD_STATUS ...0x%04x\n",XARINA_NAND1_BIT_READ_STATUS);
		info->status_id_read_cnt = 0;
	}

	return;
}

/**
 * xarina_nand1_write_addr
 *
 * write address function
 * column, page_addr : -1 indicates invalid
*/
static void xarina_nand1_write_addr(struct mtd_info *mtd, int column, int page_addr)
{

	u_short tmp_addr;
	struct nand_chip *chip = mtd->priv;

	DBG8("write_addr col:%04x page:%04x\n",column,page_addr);

	if(page_addr != XARINA_NAND1_ADDR_INVALID){ /* page_addr must be valid */

		tmp_addr = (u_short)(page_addr & XARINA_NAND1_MASK_16BIT);

		/* set page0, page1 */
		WRITE16(tmp_addr, chip->IO_ADDR_W + XARINA_NAND1_PAGE_ADDR0_REG);

		tmp_addr = (u_short)((page_addr >> XARINA_NAND1_SHIFT_16)&XARINA_NAND1_MASK_8BIT);
		/* set page2 */
		WRITE16(tmp_addr, chip->IO_ADDR_W + XARINA_NAND1_PAGE_ADDR2_REG);

	}

	if (column != XARINA_NAND1_ADDR_INVALID) {

		tmp_addr = (u_short)(column & XARINA_NAND1_MASK_16BIT);

		/* set col0, col1 */
		WRITE16(tmp_addr, chip->IO_ADDR_W + XARINA_NAND1_COL_ADDR0_REG);

	}

	return;
}

/**
 * xarina_nand1_command
 * @mtd:        MTD device structure
 * @command:    nand_command
 * @column:     offset inside page
 * @page_addr:  page offset
 *
 */
static void xarina_nand1_command(struct mtd_info *mtd, unsigned command, int column, int page_addr){

	register struct nand_chip *chip = mtd->priv;
	u_short task;
	unsigned long deadline;

	DBG8("xarina_nand1_command cmd:0x%x page:0x%x col:0x%x\n",command,page_addr,column);

	switch (command) {
	/* Emulate NAND_CMD_READOOB */
	case NAND_CMD_READOOB:
	case NAND_CMD_READ0:
		DBG8("READ:%08x\n",page_addr);

		/* command setting */
		xarina_nand1_write_cmd(mtd, NAND_CMD_READ0, NAND_CMD_READSTART, XARINA_NAND1_CMD_INVALID);
		/* address setting */
		xarina_nand1_write_addr(mtd,column, page_addr);

		task = XARINA_NAND1_BIT_TASK_COMMAND1
				| XARINA_NAND1_BIT_TASK_COMMAND2
				| XARINA_NAND1_BIT_TASK_COL_ADR0
				| XARINA_NAND1_BIT_TASK_COL_ADR1
				| XARINA_NAND1_BIT_TASK_PAGE_ADR0
				| XARINA_NAND1_BIT_TASK_PAGE_ADR1
				| XARINA_NAND1_BIT_TASK_PAGE_ADR2
				| XARINA_NAND1_BIT_TASK_WAIT_READY1;

		xarina_nand1_set_task(mtd,task);

		/* ready waiting */
		deadline = jiffies + XARINA_CMD_TIMEOUT_MSECS;
		while (!time_after_eq(jiffies, deadline)) {
			if (chip->dev_ready(mtd)) {
				break;
			}
			schedule();
		}

		break;

	case NAND_CMD_READID:
		DBG8("NAND_CMD_READID Received\n");
		/* command setting */
		xarina_nand1_write_cmd(mtd, command,XARINA_NAND1_CMD_INVALID,XARINA_NAND1_CMD_INVALID);

		/* address setting */
		xarina_nand1_write_addr(mtd, XARINA_NAND1_ADDR_INVALID, XARINA_READID_ADDR);
		task = XARINA_NAND1_BIT_TASK_COMMAND1
				| XARINA_NAND1_BIT_TASK_PAGE_ADR0
				| XARINA_NAND1_BIT_TASK_RECV2REG;

		xarina_nand1_set_task(mtd,task);

		break;

	case NAND_CMD_STATUS:
		xarina_nand1_write_protect(mtd, XARINA_NAND1_WP_OFF);
DBG8("NAND_CMD_STATUS\n");
		/* command setting */
		xarina_nand1_write_cmd(mtd, command,XARINA_NAND1_CMD_INVALID,XARINA_NAND1_CMD_INVALID);

		task = XARINA_NAND1_BIT_TASK_COMMAND1
				| XARINA_NAND1_BIT_TASK_RECV2REG;

		xarina_nand1_set_task(mtd,task);

		xarina_nand1_write_protect(mtd, XARINA_NAND1_WP_ON);
		break;

	case NAND_CMD_SEQIN:
DBG8("nand_write_cmd col:0x%x page:0x%x\n",column,page_addr);

		xarina_nand1_write_protect(mtd, XARINA_NAND1_WP_OFF);

		/* command setting */
		xarina_nand1_write_cmd(mtd, command,XARINA_NAND1_CMD_INVALID,XARINA_NAND1_CMD_INVALID);

		/* address setting */
		xarina_nand1_write_addr(mtd,column, page_addr);

		task = XARINA_NAND1_BIT_TASK_COMMAND1
				| XARINA_NAND1_BIT_TASK_COL_ADR0
				| XARINA_NAND1_BIT_TASK_COL_ADR1
				| XARINA_NAND1_BIT_TASK_PAGE_ADR0
				| XARINA_NAND1_BIT_TASK_PAGE_ADR1
				| XARINA_NAND1_BIT_TASK_PAGE_ADR2
				| XARINA_NAND1_BIT_TASK_WAIT_READY1;

		xarina_nand1_set_task(mtd,task);

		break;

	case NAND_CMD_PAGEPROG:

		/* command setting */
		xarina_nand1_write_cmd(mtd, command,XARINA_NAND1_CMD_INVALID,XARINA_NAND1_CMD_INVALID);

		task = XARINA_NAND1_BIT_TASK_COMMAND1
				| XARINA_NAND1_BIT_TASK_WAIT_READY1;
		xarina_nand1_set_task(mtd,task);

		/* ready waiting */
		deadline = jiffies + XARINA_CMD_TIMEOUT_MSECS;
		while (!time_after_eq(jiffies, deadline)) {
			if (chip->dev_ready(mtd)) {
				break;
			}
			schedule();
		}
		xarina_nand1_write_protect(mtd, XARINA_NAND1_WP_ON);

		break;

	case NAND_CMD_ERASE1:

		xarina_nand1_write_protect(mtd, XARINA_NAND1_WP_OFF);

		/* command setting */
		xarina_nand1_write_cmd(mtd, command,XARINA_NAND1_CMD_INVALID,XARINA_NAND1_CMD_INVALID);

DBG8("NAND_CMD_ERASE1\n");
		/* address setting */
		xarina_nand1_write_addr(mtd,XARINA_NAND1_ADDR_INVALID, page_addr);

		task = XARINA_NAND1_BIT_TASK_COMMAND1
				| XARINA_NAND1_BIT_TASK_PAGE_ADR0
				| XARINA_NAND1_BIT_TASK_PAGE_ADR1
				| XARINA_NAND1_BIT_TASK_PAGE_ADR2
				| XARINA_NAND1_BIT_TASK_WAIT_READY1;

		xarina_nand1_set_task(mtd,task);

		break;

	case NAND_CMD_ERASE2:
DBG8("NAND_CMD_ERASE2\n");

		/* wait for all task finished */
		deadline = jiffies + XARINA_CMD_TIMEOUT_MSECS;
		while (!time_after_eq(jiffies, deadline)) {
			if(xarina_nand1_check_task(mtd, XARINA_NAND1_BIT_TASK_ALL)==0)
				break;
			schedule();
		}

		/* command setting */
		xarina_nand1_write_cmd(mtd, command,XARINA_NAND1_CMD_INVALID,XARINA_NAND1_CMD_INVALID);

		task = XARINA_NAND1_BIT_TASK_COMMAND1
				| XARINA_NAND1_BIT_TASK_WAIT_READY1;

		xarina_nand1_set_task(mtd,task);

		/* ready waiting */
		deadline = jiffies + XARINA_CMD_TIMEOUT_MSECS;
		while (!time_after_eq(jiffies, deadline)) {
			if (chip->dev_ready(mtd)) {
				break;
			}
			schedule();
		}

		xarina_nand1_write_protect(mtd, XARINA_NAND1_WP_ON);

		break;
/* NAND_CMD_ERASE2 -> NAND_CMD_RESET sequence */

	case NAND_CMD_RESET:
DBG8("NAND_CMD_RESET\n");
		xarina_nand1_write_protect(mtd, XARINA_NAND1_WP_OFF);
		/* command setting */
		xarina_nand1_write_cmd(mtd, NAND_CMD_RESET,XARINA_NAND1_CMD_INVALID,XARINA_NAND1_CMD_INVALID);

		task = XARINA_NAND1_BIT_TASK_COMMAND1
				| XARINA_NAND1_BIT_TASK_WAIT_READY1;
		xarina_nand1_set_task(mtd,task);

		/* ready waiting */
		deadline = jiffies + XARINA_CMD_TIMEOUT_MSECS;
		while (!time_after_eq(jiffies, deadline)) {
			if (chip->dev_ready(mtd)) {
				break;
			}
			schedule();
		}
		xarina_nand1_write_protect(mtd, XARINA_NAND1_WP_ON);
		break;

	default:
		DBG1("CMD:0x%x not supported.\n",command);
		break;
	}

	nand_wait_ready(mtd);

	return;
}


/*
 *	hardware specific access to control-lines
 *	NOTE:  boards may use different bits for these!!
 *
 *	ctrl:
 *	NAND_NCE: bit 0 -  ~ChipEnable
 *	NAND_CLE: bit 1 -> bit 1  (0x0002)
 *	NAND_ALE: bit 2 -> bit 2  (0x0004)
 *
 *	NAND_CTRL_CHANGE: bit 7 -> bit 7  (0x0080)
 */

static void xarina_cmd_ctrl(struct mtd_info *mtd, int cmd, unsigned int ctrl)
{
	/* dummy funciton */
	DBG8("Dummy Function xarina_cmd_ctrl() should not be called\n");

	return;
}

/**
 * xarina_dev_ready - chip status
 * @mtd:    MTD device structure
 *
 * Chip Status Return
*/
static int xarina_nand1_dev_ready(struct mtd_info *mtd)
{
	struct nand_chip *chip = mtd->priv;
	int chip_status;

	/* Ready Status Read */
	chip_status = READ16(chip->IO_ADDR_R + XARINA_NAND1_STATUS_CONTROL_REG) & XARINA_NAND1_BIT_FBSYN;

	if (chip_status) {
		return XARINA_NAND1_DEVICE_READY;	/* Ready */
	} else {
		return XARINA_NAND1_DEVICE_BUSY;	/* busy */
	}

}


/*****************************************************************************/
/*****************************************************************************/
/*  READ OPERATION                                                           */
/*****************************************************************************/
/*****************************************************************************/
static uint8_t xarina_read_byte(struct mtd_info *mtd)
{
	struct nand_chip *chip = mtd->priv;
	struct xarina_nand1_info *info = container_of(mtd, struct xarina_nand1_info, mtd);
	uint8_t ret;
	uint16_t tmp_buf;

	if(info->status_id_read_cnt >= XARINA_READ_BYTE_CNT_MAX){
		return 0;
	}

	if(info->status_id_read_cnt==0){
		tmp_buf = READ16(chip->IO_ADDR_R + XARINA_NAND1_STATUS_ID_READ12_REG);
		info->read_byte_reg[0] = (uint8_t)(tmp_buf&XARINA_NAND1_MASK_8BIT);
		info->read_byte_reg[1] = (uint8_t)((tmp_buf>>8)&XARINA_NAND1_MASK_8BIT);

		tmp_buf = READ16(chip->IO_ADDR_R + XARINA_NAND1_STATUS_ID_READ34_REG);
		info->read_byte_reg[2] = (uint8_t)(tmp_buf&XARINA_NAND1_MASK_8BIT);
		info->read_byte_reg[3] = (uint8_t)((tmp_buf>>8)&XARINA_NAND1_MASK_8BIT);

		tmp_buf = READ16(chip->IO_ADDR_R + XARINA_NAND1_STATUS_ID_READ56_REG);
		info->read_byte_reg[4] = (uint8_t)(tmp_buf&XARINA_NAND1_MASK_8BIT);
		info->read_byte_reg[5] = (uint8_t)((tmp_buf>>8)&XARINA_NAND1_MASK_8BIT);

#ifdef CONFIG_MACH_XARINA_FPGA
		if(info->read_byte_reg[1]==0xd5) /* NAND 2GiB */
			info->read_byte_reg[1] = 0xd3; /* NAND 1GiB */
		if(info->read_byte_reg[3]==0xa6) /* pagesize:4k, blocksize:256k oob16for512*/
			info->read_byte_reg[3] = 0x15; /* pagesize:2k, blocksize:128k oob16for512*/
#endif
	}

	ret = info->read_byte_reg[info->status_id_read_cnt];

	DBG8("read_byte - cnt:%d value:0x%x\n",info->status_id_read_cnt,ret);

	info->status_id_read_cnt++;

	return ret;
}

static u16 xarina_read_word(struct mtd_info *mtd)
{
	printk("nand_read_word not supported\n");
	return 0;
}

#ifdef CONFIG_MTD_XARINA_NAND_DMAC
/**
 * nf_dma_callback
 * @data:	mtd info structure
 *
 */
static void nf_dma_callback( void *data )
{
	struct xarina_nand1_info *info = data;
	struct mtd_info *mtd = &info->mtd;
	struct nand_chip *chip = mtd->priv;

	/* stop dma */
	writel(0,chip->IO_ADDR_W + XARINA_NAND1_NF_AUTOCTL);

	/* clear interrupt */
	writel((readl(chip->IO_ADDR_R + XARINA_NAND1_NF_ICR) | 0x00000001 ),chip->IO_ADDR_W + XARINA_NAND1_NF_ICR);

	complete(&info->cmp);

	return;
}


static void xarina_nand1_dma_transfer(struct mtd_info *mtd, u_char *buf, int len, int direction)
{

	int i;
	u_short autoctl = 0;
	void *bufp = buf;
	struct scatterlist *sg;
	struct nand_chip *chip = mtd->priv;
	struct xarina_nand1_info *info = container_of(mtd, struct xarina_nand1_info, mtd);
	struct dma_async_tx_descriptor *dma_desc;
	struct dma_slave_config dma_conf;

	struct dma_chan *dma_channel = info->dma_channel;
	dma_cookie_t cookie;
	unsigned long deadline;

	info->f_use_own_buf = false;

	/* First, check if the data NOT cross over the PAGE-BOUNDARY */
	/* and also check word alignment */
	if ((((size_t)buf & PAGE_MASK) != ((size_t)(buf + len - 1) & PAGE_MASK))
		|| (((size_t)buf & XARINA_WORD_MASK)!=0)){
		bufp = info->dma_buf;
		info->f_use_own_buf = true;
		if(direction == DMA_TO_DEVICE){
			memcpy(bufp, buf, len);
		}
	}
	/* Second, convert vmalloc to page */
	else if ((void *)buf >= high_memory) {

		struct page *p1;

		p1 = vmalloc_to_page(buf);
		if (!p1){
			bufp = info->dma_buf;
			info->f_use_own_buf = true;
			if(direction == DMA_TO_DEVICE){
				memcpy(bufp, buf, len);
			}

		}else{
			bufp = page_address(p1) + ((size_t)buf & ~PAGE_MASK);
		}
	}

	/* generate parameter for auto control register */
	/* autoctl[0]:read/write, autoctl[1]:start_transfer, autoctl[2]:mem2mem/peri2mem */
	dma_conf.direction = direction;
	if(direction == DMA_FROM_DEVICE){
		dma_conf.src_addr = XARINA_NANDIF_PHYS_BASE + XARINA_NAND1_NF_FIFO;
		dma_conf.src_addr_width = DMA_SLAVE_BUSWIDTH_4_BYTES;
		dma_conf.src_maxburst = 4;
		autoctl = XARINA_NAND1_NF_BIT_DM_TYPE|XARINA_NAND1_NF_BIT_AUTO_EN;
	}else if(direction == DMA_TO_DEVICE){
		dma_conf.dst_addr = XARINA_NANDIF_PHYS_BASE + XARINA_NAND1_NF_FIFO;
		dma_conf.dst_addr_width = DMA_SLAVE_BUSWIDTH_4_BYTES;
		dma_conf.dst_maxburst = 4;
		autoctl = XARINA_NAND1_NF_BIT_DM_TYPE|XARINA_NAND1_NF_BIT_AUTO_EN|XARINA_NAND1_NF_BIT_WXR;
	}else{
		DBG1("DMA Direction Invalid\n");
	}

	/* set dma config */
	dma_channel->device->device_control(dma_channel, DMA_SLAVE_CONFIG, (unsigned long)&dma_conf);

#ifndef CONFIG_SLUB /* sg_alloc_table test */
	/* alloc 4k */
	if(sg_alloc_table(&info->sgt, 1, GFP_KERNEL)){
		DBG1("sg_alloc_table error\n");
		return;
	}
#endif

	/* setup sg */
	for_each_sg(info->sgt.sgl, sg, info->sgt.nents, i) {

		sg_set_page(sg, virt_to_page(bufp), len, offset_in_page(bufp));

	}

	dma_map_sg(&mtd->dev, info->sgt.sgl, info->sgt.nents, direction);

	/* dma channel config */
	dma_desc = dma_channel->device->device_prep_slave_sg(dma_channel,
					info->sgt.sgl,
					info->sgt.nents,
					direction,
					DMA_CTRL_ACK | DMA_PREP_INTERRUPT);
	if(!dma_desc){
		DBG1("device_prep_slave_sg() failed\n");
		xarina_nand1_set_task(mtd,XARINA_NAND1_BIT_TASK_CLR);
		return;
	}

	dma_desc->callback = nf_dma_callback;
	dma_desc->callback_param = info;
	init_completion(&info->cmp);

	cookie = dma_desc->tx_submit(dma_desc);
	if (dma_submit_error(cookie)){
		DBG1("tx_submit() failed\n");
		xarina_nand1_set_task(mtd,XARINA_NAND1_BIT_TASK_CLR);
		return;
	}

	/* Set dma control register */
	writel(autoctl ,chip->IO_ADDR_W + XARINA_NAND1_NF_AUTOCTL);

	dma_channel->device->device_issue_pending(dma_channel);

	/* wait for dma transfer complete */
	if(!wait_for_completion_timeout(&info->cmp,msecs_to_jiffies(XARINA_TRANS_TIMEOUT_MSECS))){
		dump_dmareg_print();
		dump_nandreg_print();
		/* clear task */
		xarina_nand1_set_task(mtd,XARINA_NAND1_BIT_TASK_CLR);

		DBG1("dma timeout counter expired\n");

		/* stop dma */
		writel(0,chip->IO_ADDR_W + XARINA_NAND1_NF_AUTOCTL);
		/* clear interrupt */
		writel((readl(chip->IO_ADDR_R + XARINA_NAND1_NF_ICR) | 0x00000001 ),chip->IO_ADDR_W + XARINA_NAND1_NF_ICR);
	}

	/* check if all the task finished */
	deadline = jiffies + XARINA_TRANS_TIMEOUT_MSECS;
	while (!time_after_eq(jiffies, deadline)) {
		if(xarina_nand1_check_task(mtd, XARINA_NAND1_BIT_TASK_ALL)==0)
			break;
		schedule();
	}

	dma_unmap_sg(&mtd->dev, info->sgt.sgl, info->sgt.nents, direction);
#ifndef CONFIG_SLUB /*sg_alloc_table test*/
	sg_free_table(&info->sgt);
#endif

	if((info->f_use_own_buf==true)&&(direction == DMA_FROM_DEVICE)){
		memcpy(buf, info->dma_buf, len);
	}

	return;
}
#endif
/**
 * xarina_read_buf - read chip data into buffer
 * @mtd:	MTD device structure
 * @buf:	buffer to store date
 * @len:	number of bytes to read
 *
 * Default read function for 8bit buswith
 */
static void xarina_read_buf(struct mtd_info *mtd, u_char *buf, int len)
{
	struct nand_chip *chip = mtd->priv;
	struct xarina_nand1_info *info = container_of(mtd, struct xarina_nand1_info, mtd);
	u_short task;
	int i;
	unsigned long deadline;

	DBG8("ReadBuf adr:%p len:%d\n",buf,len);

	/* ecc clear */
	WRITE16(0xffff, chip->IO_ADDR_W + XARINA_NAND1_ECC_CLEAR_REG);

	/* set flash size = len */
	WRITE16((u_short)len, chip->IO_ADDR_W + XARINA_NAND1_FLASH_DATA_SIZE_REG);

	/* set task RecvData/ChkECC */
	task = XARINA_NAND1_BIT_TASK_RECV_DATA | info->ecc_task;

	xarina_nand1_set_task(mtd,task);

#ifdef CONFIG_MTD_XARINA_NAND_DMAC
	if(len==XARINA_NAND1_ECC_SIZE){

		xarina_nand1_dma_transfer(mtd, buf, len, DMA_FROM_DEVICE);

	}else{
#endif

		/* Read */
		for ( i = 0; i < len / sizeof(u_short); i++){
			/* FIFO NOT Empty Check */
			deadline = jiffies + XARINA_TRANS_TIMEOUT_MSECS;
			while (!time_after_eq(jiffies, deadline)) {

				if ((READ16(chip->IO_ADDR_R + XARINA_NAND1_STATUS_CONTROL_REG) & XARINA_NAND1_BIT_FIFO_EMPTY) == 0) {
					/* not empty */
					break;
				}
				DBG8("ReadBufEmptyWait\n");
				schedule();

			}

			*((u_short *)buf + i) = READ16(chip->IO_ADDR_R + XARINA_NAND1_DATA_REG);

		}

		/* wait for all the task finished */
		deadline = jiffies + XARINA_TRANS_TIMEOUT_MSECS;
		while (!time_after_eq(jiffies, deadline)) {
			if(xarina_nand1_check_task(mtd, XARINA_NAND1_BIT_TASK_ALL)==0)
				break;
			schedule();
		}

#ifdef CONFIG_MTD_XARINA_NAND_DMAC
	}
#endif

	return;
}
/**
 * xarina_nand1_read_page_raw
 * @mtd:	mtd info structure
 * @chip:	nand chip info structure
 * @buf:	buffer to store read data
 */
static int xarina_nand1_read_page_raw(struct mtd_info *mtd, struct nand_chip *chip, uint8_t *buf, int page)
{

	int i;
	uint8_t *oob_offset;
	uint8_t *ecc_offset;

/*  | <------------------------------ 1 step -------------------------------> |             */
/*  ,-------------------------------------------------+------+---------+------+------------ */
/*  |            data(512)                            |ecc(4)|  oob(8) |ecc(4)|             */
/*  `-------------------------------------------------+------+---------+------+-----------  */
/*
*   1. read data 512 byte to ( buf + i * 512 )
*   2. read ecc 4 byte to (oob_pointer + 32 + i * 8 )
*   3. read oob 8 byte to (oob_pointer + i * 8 )
*   4. read ecc 4 byte to (oob_pointer + 32 + i * 8 + 4 )
*
*    Buffer for OOB
*      OOB: oob_pointer +  0 ~ 31
*      ECC: oob_pointer + 32 ~ 63
*/

	for(i=0;i<chip->ecc.steps;i++){

		/* ecc.size = 512 byte */
		/* read data (512 byte) */
		chip->read_buf(mtd, buf+(i*chip->ecc.size), chip->ecc.size);

		/* oobsize=64, ecc.steps=4 ecc.bytes=4+4 */
		ecc_offset = (uint8_t *)(chip->oob_poi + mtd->oobsize/2 + i * (mtd->oobsize / chip->ecc.steps - chip->ecc.bytes));
		/* read ecc (4 byte) */
		chip->read_buf(mtd, ecc_offset, chip->ecc.bytes / 2);

		/* oobsize=64, ecc.steps=4 ecc.bytes=4+4 */
  		oob_offset = (uint8_t *)(chip->oob_poi + i * ( mtd->oobsize / chip->ecc.steps - chip->ecc.bytes ));
		/* read oob (8 byte) */
		chip->read_buf(mtd, oob_offset, mtd->oobsize / chip->ecc.steps - chip->ecc.bytes);

		/* oobsize=64, ecc.steps=4 ecc.bytes=4+4 */
		ecc_offset = (uint8_t *)(chip->oob_poi + mtd->oobsize / 2 + i * ( mtd->oobsize / chip->ecc.steps - chip->ecc.bytes) + chip->ecc.bytes / 2);
		/* read ecc (4 byte) */
		chip->read_buf(mtd, ecc_offset, chip->ecc.bytes / 2);

	}

	return 0;

}

/**
 * xarina_nand1_read_page_hwecc
 * @mtd:	mtd info structure
 * @chip:	nand chip info structure
 * @buf:	buffer to store read data
 *
 * Not for syndrome calculating ecc controllers which need a special oob layout
 */
static int xarina_nand1_read_page_hwecc(struct mtd_info *mtd, struct nand_chip *chip, uint8_t *buf, int page)
{

	struct xarina_nand1_info *info = container_of(mtd, struct xarina_nand1_info, mtd);
	int i;
	uint8_t *oob_offset;
	uint8_t *data_offset;
	int stat;

	DBG8("read_page_hwecc()\n");

/*  | <------------------------------ 1 step -------------------------------> |             */
/*  ,-------------------------------------------------+------+---------+------+------------ */
/*  |            data(512)                            |ecc(4)|  oob(8) |ecc(4)|             */
/*  `-------------------------------------------------+------+---------+------+-----------  */
/*
*   1. read data 512 byte to ( buf + i * 512 )
*   2. error correction for data area
*   3. read oob 8 byte to (oob_pointer + i * 8 )
*   4. error correction for oob area
*
*    Buffer for OOB
*      OOB: oob_pointer +  0 ~ 31
*/

	/* ecc_switch on */
	chip->ecc.hwctl(mtd, NAND_ECC_READ);

	for(i=0;i<chip->ecc.steps;i++){

		/* read data (512 byte) */
		data_offset = buf+(i*chip->ecc.size);
		chip->read_buf(mtd, data_offset, chip->ecc.size);

		/* store ecc check reg */
		info->check_ecc_buf =  READ16( chip->IO_ADDR_W + XARINA_NAND1_ECC_CHECK_REG );

		if(info->check_ecc_buf){
			stat = chip->ecc.correct(mtd, data_offset, NULL, NULL);

			if (stat < 0)
				mtd->ecc_stats.failed++;
 			else
 				mtd->ecc_stats.corrected += stat;
		}

		/* oobsize=64, ecc.steps=4 ecc.bytes=4+4 */
		oob_offset = (uint8_t *)( chip->oob_poi + i * ( mtd->oobsize / chip->ecc.steps - chip->ecc.bytes ));
		/* read oob */
		chip->read_buf(mtd, oob_offset, mtd->oobsize/chip->ecc.steps-chip->ecc.bytes);

		/* store ecc check reg */
		info->check_ecc_buf =  READ16( chip->IO_ADDR_W + XARINA_NAND1_ECC_CHECK_REG );

		if(info->check_ecc_buf){
			stat = chip->ecc.correct(mtd, oob_offset, NULL, NULL);

			if (stat < 0)
				mtd->ecc_stats.failed++;
			else
				mtd->ecc_stats.corrected += stat;
		}

	}

	/* ecc_switch off */
	info->ecc_task = 0;

	return 0;
}

/**
 * xarina_nand1_read_oob_std
 * @mtd:	mtd info structure
 * @chip:	nand chip info structure
 * @page:	page number to read
 * @sndcmd:	flag whether to issue read command or not
 */
static int xarina_nand1_read_oob_std(struct mtd_info *mtd, struct nand_chip *chip, int page, int sndcmd)
{

	DBG6("read_oob_std page[0x%06x]\n",page);

	if(sndcmd){
		chip->cmdfunc(mtd, NAND_CMD_READ0, 0, page);
	}

	chip->ecc.read_page(mtd, chip, chip->buffers->databuf, page);

	return 0;
}


/*****************************************************************************/
/*****************************************************************************/
/*  WRITE OPERATION                                                          */
/*****************************************************************************/
/*****************************************************************************/
/**
 * xarina_write_buf - write buffer to chip
 * @mtd:	MTD device structure
 * @buf:	data buffer
 * @len:	number of bytes to write
 *
 * Default write function for 8bit buswith
 */
static void xarina_write_buf(struct mtd_info *mtd, const u_char *buf, int len)
{
	struct nand_chip *chip = mtd->priv;
	struct xarina_nand1_info *info = container_of(mtd, struct xarina_nand1_info, mtd);
	u_short task;
	int i;
	unsigned long deadline;

	/* ecc clear */
	WRITE16(0xffff, chip->IO_ADDR_W + XARINA_NAND1_ECC_CLEAR_REG);

	/* set flash size = len */
	WRITE16((u_short)len, chip->IO_ADDR_W + XARINA_NAND1_FLASH_DATA_SIZE_REG);

	/* set data_status flag */
	WRITE16(READ16(chip->IO_ADDR_R + XARINA_NAND1_ECC_RES_HIGH_STATUS_REG)|XARINA_NAND1_STATUS_FLAG , chip->IO_ADDR_W + XARINA_NAND1_ECC_RES_HIGH_STATUS_REG);

	/* check control/status reg : FBSYn == 1 */
	while(chip->dev_ready(mtd)==XARINA_NAND1_DEVICE_BUSY);

	/* set task SendData/SendECC */
	task = XARINA_NAND1_BIT_TASK_SEND_DATA | info->ecc_task;
	xarina_nand1_set_task(mtd,task);

#ifdef CONFIG_MTD_XARINA_NAND_DMAC
	if(len==XARINA_NAND1_ECC_SIZE){

		xarina_nand1_dma_transfer(mtd, (u_char *)buf, len, DMA_TO_DEVICE);

	}else{

#endif
		/* Write */
		for (i = 0; i < len / sizeof(u_short); i++) {
			/* FIFO full check */
			deadline = jiffies + XARINA_TRANS_TIMEOUT_MSECS;
			while (!time_after_eq(jiffies, deadline)) {
				if ((READ16(chip->IO_ADDR_R + XARINA_NAND1_STATUS_CONTROL_REG) & XARINA_NAND1_BIT_FIFO_FULL) == 0) {
					/* FIFO NOT FULL */
					break;
				}
				schedule();
			}

			WRITE16(*((u_short *)buf + i), chip->IO_ADDR_W + XARINA_NAND1_DATA_REG);

		}

		/* wait for all the task finished */
		deadline = jiffies + XARINA_TRANS_TIMEOUT_MSECS;
		while (!time_after_eq(jiffies, deadline)) {
			if(xarina_nand1_check_task(mtd, XARINA_NAND1_BIT_TASK_ALL)==0)
				break;
			schedule();
		}

#ifdef CONFIG_MTD_XARINA_NAND_DMAC
	}
#endif

	return;
}

/**
 * xarina_nand1_write_page_raw
 * @mtd:	mtd info structure
 * @chip:	nand chip info structure
 * @buf:	data buffer
 */
static void xarina_nand1_write_page_raw(struct mtd_info *mtd, struct nand_chip *chip, const uint8_t *buf)
{

	int i;
	uint8_t *ecc_offset;
	uint8_t *oob_offset;

	DBG8("write_page_raw()\n");

	for(i=0;i<chip->ecc.steps;i++){

		/* ecc.size = 512 byte */
		/* write data ( 512 byte ) */
		chip->write_buf(mtd, buf+(i*chip->ecc.size), chip->ecc.size);

		/* oobsize=64, ecc.steps=4 ecc.bytes=4+4 */
		ecc_offset = (uint8_t *)(chip->oob_poi + mtd->oobsize / 2 + i * (mtd->oobsize / chip->ecc.steps - chip->ecc.bytes));
		/* write oob to ecc area ( 4 byte ) */
		chip->write_buf(mtd, ecc_offset, chip->ecc.bytes/2);

		/* oobsize=64, ecc.steps=4 ecc.bytes=4+4 */
  		oob_offset = (uint8_t *)(chip->oob_poi + i * ( mtd->oobsize / chip->ecc.steps - chip->ecc.bytes ));
		/* write oob ( 8 byte ) */
		chip->write_buf(mtd, oob_offset, mtd->oobsize/chip->ecc.steps - chip->ecc.bytes);

		/* oobsize=64, ecc.steps=4 ecc.bytes=4+4 */
		ecc_offset = (uint8_t *)(chip->oob_poi + mtd->oobsize / 2 + i * ( mtd->oobsize / chip->ecc.steps - chip->ecc.bytes) + chip->ecc.bytes / 2);
		/* write oob to ecc area ( 4 byte ) */
		chip->write_buf(mtd, ecc_offset, chip->ecc.bytes/2);

	}

}

/**
 * xarina_nand1_write_page_hwecc
 * @mtd:	mtd info structure
 * @chip:	nand chip info structure
 * @buf:	data buffer
 */
static void xarina_nand1_write_page_hwecc(struct mtd_info *mtd, struct nand_chip *chip, const uint8_t *buf)
{

	int i;
	struct xarina_nand1_info *info = container_of(mtd, struct xarina_nand1_info, mtd);
	uint8_t *oob_offset;

	DBG6("write_page_hwecc()\n");

	/* ecc_switch on */
	chip->ecc.hwctl(mtd, NAND_ECC_WRITE);

	for(i=0;i<chip->ecc.steps;i++){

		/* ecc.size = 512 byte */
		/* write data ( 512 byte ) */
		chip->write_buf(mtd, buf+(i*chip->ecc.size), chip->ecc.size);

		/* oobsize=64, ecc.steps=4 ecc.bytes=4+4 */
  		oob_offset = (uint8_t *)(chip->oob_poi + i * ( mtd->oobsize / chip->ecc.steps - chip->ecc.bytes ));
		/* write_oob (8byte) */
		chip->write_buf(mtd, oob_offset, mtd->oobsize/chip->ecc.steps - chip->ecc.bytes);

	}

	/* ecc_switch off */
	info->ecc_task = 0;

}

/**
 * xarina_nand1_write_oob_std
 * @mtd:	mtd info structure
 * @chip:	nand chip info structure
 * @page:	page number to write
 */
static int xarina_nand1_write_oob_std(struct mtd_info *mtd, struct nand_chip *chip, int page)
{
	int status = 0;
	struct xarina_nand1_info *info = container_of(mtd, struct xarina_nand1_info, mtd);

	int col_offset=0;
	int i;
	uint8_t *oob_offset;
	DBG6("write_oob_std page[0x%06x]\n",page);

	for(i=0;i<chip->ecc.steps;i++){/* steps = 4 */

		col_offset += chip->ecc.size + chip->ecc.bytes/2;/* ecc bytes = 8, col_offset += (512+4+)*i */

		chip->cmdfunc(mtd, NAND_CMD_SEQIN, col_offset, page);
		/* ecc_switch on */
		chip->ecc.hwctl(mtd, NAND_ECC_WRITE);

		/* write oob operation */
		/* oobsize=64, ecc.steps=4 ecc.bytes=4+4 */
  		oob_offset = (uint8_t *)(chip->oob_poi + i * ( mtd->oobsize / chip->ecc.steps - chip->ecc.bytes ));
		/* write oob ( 8 byte ) */
		chip->write_buf(mtd, oob_offset, mtd->oobsize/chip->ecc.steps - chip->ecc.bytes);

		/* advance 8 + 4 for oob */
		col_offset += (XARINA_NAND1_OOBSIZE-XARINA_NAND1_ECC_TOTAL)/chip->ecc.steps + chip->ecc.bytes/2;

		/* ecc_switch off */
		info->ecc_task = 0;
		/* finish write cmd */
		chip->cmdfunc(mtd, NAND_CMD_PAGEPROG, -1, -1);

		status = chip->waitfunc(mtd, chip);
		if(status & NAND_STATUS_FAIL)
			return -EIO;

	}

	return 0;

}

/**
 * xarina_nand1_verify_buf - [DEFAULT] Verify chip data against buffer
 * @mtd:	MTD device structure
 * @buf:	buffer containing the data to compare
 * @len:	number of bytes to compare
 */
static int xarina_nand1_verify_buf(struct mtd_info *mtd, const uint8_t *buf, int len)
{

	int i;
	struct nand_chip *chip = mtd->priv;
	struct xarina_nand1_info *info = container_of(mtd, struct xarina_nand1_info, mtd);
    struct mtd_ecc_stats stats = mtd->ecc_stats;

	/* read data to cmp_buf */
	chip->ecc.read_page(mtd, chip, info->cmp_buf, 0);

    if (mtd->ecc_stats.failed - stats.failed) {
        return -EBADMSG;
    }

	/* Compare Data */
	DBG8("verify_buf[compare start] length:%d\n",len);
	for (i = 0; i < len; i++){
		if(buf[i]!=info->cmp_buf[i]){

			DBG8("compare buf:0x%02x cmp_buf0x%02x\n",buf[i],info->cmp_buf[i]);

			DBG8("for UBI, dummy_write(0xff) on header(512byte)\n");
			if(buf[i]!=0xff){ /* unless dummy_write */
				DBG1("Failed verify_buf PAGE buf[%d]\n",i);
				return -EFAULT;
			}
		}
	}

	return 0;
}

/*****************************************************************************/
/*  Chip Select for xarina nand                                              */
/*****************************************************************************/
static void xarina_select_chip(struct mtd_info *mtd, int chip)
{
	struct nand_chip *nand_chip = mtd->priv;
	u_short data;

	/* select chip enable/disable */
	DBG8("chip_select No.%d\n",chip);

	data = READ16(nand_chip->IO_ADDR_R + XARINA_NAND1_STATUS_CONTROL_REG ) | XARINA_NAND1_BIT_FCDISABLE;

	/* chip select */
	if (chip == 0) {
		/* enable chip0 */
		WRITE16(data & ~XARINA_NAND1_BIT_FCE0N, nand_chip->IO_ADDR_W + XARINA_NAND1_STATUS_CONTROL_REG);
	}else if (chip == 1) {
		/* enable chip1 */
		WRITE16(data & ~XARINA_NAND1_BIT_FCE1N, nand_chip->IO_ADDR_W + XARINA_NAND1_STATUS_CONTROL_REG);
	} else {  /* chip deselect */
		/* disable all */
		WRITE16(data, nand_chip->IO_ADDR_W + XARINA_NAND1_STATUS_CONTROL_REG);
		/* write protect on*/
		xarina_nand1_write_protect(mtd, XARINA_NAND1_WP_ON);
	}

	DBG8("XARINA_NAND1 CONTROL_REG : 0x%04x\n",READ16(nand_chip->IO_ADDR_W + XARINA_NAND1_STATUS_CONTROL_REG));

	return;
}

/*
 *  xarina_nand1_calculate_ecc
 *  Dummy Function
 */
static int xarina_nand1_calculate_ecc(struct mtd_info *mtd, const u_char *dat, u_char *ecc_code)
{
	/* do nothing */
	/* HW auto calculation */
	DBG8("xarina_nand1_calculate_ecc\n");
	return 0;
}

/**
 * xarina_nand1_correct_data
 * @mtd:        MTD device structure
 * @dat:        target data part
 * @oob:        NULL(don't use)
 * @ecc_data:   NULL(don't use)
 *
 * Correct Data of ECC
 */
static int xarina_nand1_correct_data(struct mtd_info *mtd, u_char *dat, u_char *oob, u_char *ecc_data)
{
	int err_correct_f;
	struct nand_chip *chip = mtd->priv;
	struct xarina_nand1_info *info = container_of(mtd, struct xarina_nand1_info, mtd);
	u_short *e_addr;
	u_short e_bits;

	err_correct_f = 0;

	if(info->check_ecc_buf&(XARINA_NAND1_BIT_UCERR|XARINA_NAND1_BIT_CBERR)){
DBG6("uncorrectable error ecc_check=0x%08x\n",info->check_ecc_buf);
			return -1;
	} else if(info->check_ecc_buf&XARINA_NAND1_BIT_CERR){
			err_correct_f = 1;
	}

	/* No Error */
	if(err_correct_f==0)
		return 0;

DBG6("data correcting\n");

	e_addr = (u_short *)(dat) + READ16(chip->IO_ADDR_R + XARINA_NAND1_ECC_ERR_ADDR_REG);
DBG7("e_addr = %p\n",e_addr);
	e_bits = READ16(chip->IO_ADDR_R + XARINA_NAND1_ECC_ERR_BIT_REG );
DBG7("e_bits = %08x\n",e_bits);
DBG7("raw_data = %08x\n",*(e_addr));
	*(e_addr) ^= e_bits;
DBG7("corrected_data = %08x\n",*(e_addr));

	return err_correct_f;
}

/**
 * xarina_nand1_enable_hwecc
 * @mtd:    MTD device structure
 * mode:   Read / Write Mode
 *
 * Set Task for SendECC or ChkECC
 */
static void xarina_nand1_enable_hwecc(struct mtd_info *mtd, int mode)
{

	/* ecc task switch */

	struct xarina_nand1_info *info = container_of(mtd, struct xarina_nand1_info, mtd);

	DBG8("ecc_enable mode:0x%x\n",mode);

	switch (mode) {
	case NAND_ECC_READ:     /* read mode */

		/* Set Check ECC Task */
		info->ecc_task = XARINA_NAND1_BIT_TASK_CHK_ECC;

		break;

	case NAND_ECC_WRITE:    /* write mode */

		/* Set Send ECC Task */
		info->ecc_task = XARINA_NAND1_BIT_TASK_SEND_ECC;

		break;

	default:
		/* Clear ECC Task */
		info->ecc_task = 0;
		break;
	}
	return;
}

/**
 * xarina_nand1_probe - initialize platform_device
 * @pdev:	platform_device structure
 *
 *
 *
 */
static int __devinit xarina_nand1_probe(struct platform_device *pdev)
{
	struct xarina_nand1_info *info;
	struct platform_nand_chip *pdata = pdev->dev.platform_data;
	int err = 0;
	struct resource *res;
	void *clk_sel; /* for sysreg */
	u_long clk_param;
	u_short clk_rate=0;
#ifdef CONFIG_MTD_XARINA_NAND_DMAC
	dma_cap_mask_t dma_mask;
	char *dma_channel_name = "nand_dmac";
#endif

	DBG5("NAND Probe Start\n");

	/* Get Clockrate */
	clk_sel = ioremap_nocache(XARINA_SYSREG_CLK_CTRL_PADDR, sizeof(u_long));
	if(!clk_sel){
		return -ENOMEM;
	}
	clk_param = ( readl(clk_sel) & XARINA_SYSREG_NFIF_CKSEL_MASK >> XARINA_NAND1_SHIFT_16);
	iounmap(clk_sel);

	switch(clk_param){
	case 0:
		DBG5("NFIF_CLKSEL 33.3MHz\n");
		clk_rate = XARINA_NAND1_CLK_RATE_2;
		DBG5("Select CLK_RATE 1/2\n");
		break;
	case 1:
		DBG5("NFIF_CLKSEL 40MHz\n");
		clk_rate = XARINA_NAND1_CLK_RATE_2;
		DBG5("Select CLK_RATE 1/2\n");
		break;
	case 2:
		DBG5("NFIF_CLKSEL 50MHz\n");
		clk_rate = XARINA_NAND1_CLK_RATE_3;
		DBG5("Select CLK_RATE 1/3\n");
		break;
	case 3:
		DBG5("NFIF_CLKSEL 66.6MHz\n");
		clk_rate = XARINA_NAND1_CLK_RATE_4;
		DBG5("Select CLK_RATE 1/4\n");
		break;
	case 4:
		DBG5("NFIF_CLKSEL 80MHz\n");
		clk_rate = XARINA_NAND1_CLK_RATE_4;
		DBG5("Select CLK_RATE 1/4\n");
		break;
	case 5:
		DBG5("NFIF_CLKSEL 100MHz(OVER SPEC)\n");
		clk_rate = XARINA_NAND1_CLK_RATE_4;
		DBG5("Select CLK_RATE 1/4\n");
		break;
	default:
		DBG1("CLK_SEL: FatalError\n");
		break;
	}

	info = kzalloc(sizeof(struct xarina_nand1_info), GFP_KERNEL);
	if (!info)
		return -ENOMEM;

	res = platform_get_resource(pdev, IORESOURCE_MEM, 0);

	if(!request_mem_region(res->start, res->end - res->start + 1, DRIVER_NAME)){
		err = -ENOMEM;
		goto out_free_mem;
	}

	info->nand.IO_ADDR_R = ioremap_nocache(res->start, res->end - res->start + 1);
	if (!info->nand.IO_ADDR_R) {
		err = -ENOMEM;
		goto out_release_mem;
	}
	info->nand.IO_ADDR_W = info->nand.IO_ADDR_R;

	DBG8("ioremap phys_addr:0x%08x size:0x%x\n",res->start, res->end - res->start + 1);

	info->pdata = pdata;

	info->nand.options = pdata->options;
	info->mtd.name = pdev->name;
	info->mtd.priv = &info->nand;
	info->nand.chip_delay = XARINA_NAND1_CHIP_DELAY;

	info->nand.cmd_ctrl = xarina_cmd_ctrl;
	info->nand.cmdfunc = xarina_nand1_command;
	info->nand.read_byte = xarina_read_byte;
	info->nand.read_word = xarina_read_word;
	info->nand.select_chip = xarina_select_chip;
	info->nand.dev_ready = xarina_nand1_dev_ready;
	info->nand.verify_buf = xarina_nand1_verify_buf;
	info->nand.read_buf = xarina_read_buf;
	info->nand.write_buf = xarina_write_buf;
	info->nand.bbt_td = &xarina_bbt_main_descr;
	info->nand.bbt_md = &xarina_bbt_mirror_descr;
	info->nand.badblock_pattern = &xarina_largepage_flashbased;

	info->nand.ecc.mode = NAND_ECC_HW;
	info->nand.ecc.layout = &nand_oob_32;
	info->nand.ecc.calculate = xarina_nand1_calculate_ecc;
	info->nand.ecc.correct = xarina_nand1_correct_data;
	info->nand.ecc.hwctl = xarina_nand1_enable_hwecc;
	info->nand.ecc.size = XARINA_NAND1_ECC_SIZE;
	info->nand.ecc.steps = XARINA_NAND1_ECC_STEPS;
	info->nand.ecc.bytes = XARINA_NAND1_ECC_BYTES;
	info->nand.ecc.total = XARINA_NAND1_ECC_TOTAL;
	info->nand.ecc.read_oob = xarina_nand1_read_oob_std;
	info->nand.ecc.write_oob = xarina_nand1_write_oob_std;
	info->nand.ecc.read_page_raw = xarina_nand1_read_page_raw;
	info->nand.ecc.write_page_raw = xarina_nand1_write_page_raw;
	info->nand.ecc.read_page = xarina_nand1_read_page_hwecc;
	info->nand.ecc.write_page = xarina_nand1_write_page_hwecc;
	info->mtd.owner = THIS_MODULE;

	/* Set ClockRate(bit1, bit0), Endian(bit2=0) */
	WRITE16((READ16(info->nand.IO_ADDR_W + XARINA_NAND1_MODE_REG)&0xfff8) | clk_rate, info->nand.IO_ADDR_W + XARINA_NAND1_MODE_REG);

#ifdef CONFIG_MTD_XARINA_NAND_DMAC
	/* dma initialize */
	dma_cap_zero(dma_mask);
	dma_cap_set(DMA_SLAVE, dma_mask);
	info->dma_buf = kzalloc(XARINA_NAND1_ECC_SIZE, GFP_KERNEL);
	if(!info->dma_buf){
		err=-ENOMEM;
		goto out_free_for_dma;
	}

	/* dma channel setting */
	info->dma_channel = dma_request_channel(dma_mask, xarina_nand1_dma_filter,dma_channel_name);
	DBG7("dma_request_channel:%p \n",info->dma_channel);
	if(!info->dma_channel){
		err=-ENOMEM;
		goto out_free_for_dma;
	}
#ifdef CONFIG_SLUB/* sg_alloc_table test */
	/* alloc 4k */
	if(sg_alloc_table(&info->sgt, 1, GFP_KERNEL)){
		DBG1("sg_alloc_table error\n");
		err=-ENOMEM;
		goto release_dma_ch;
	}
#endif

	/* Enable WAIT_ACT for NAND DMAC */
	WRITE16(READ16(info->nand.IO_ADDR_W + XARINA_NAND1_MODE_REG) | XARINA_NAND1_BIT_WAIT_ACT, info->nand.IO_ADDR_W + XARINA_NAND1_MODE_REG);

	/* enable interrupt */
	writel(1,info->nand.IO_ADDR_W + XARINA_NAND1_NF_IEN);
#else
	/* Disable WAIT_ACT for NAND DMAC */
	WRITE16(READ16(info->nand.IO_ADDR_W + XARINA_NAND1_MODE_REG) & ~XARINA_NAND1_BIT_WAIT_ACT, info->nand.IO_ADDR_W + XARINA_NAND1_MODE_REG);
#endif

	/* set read_id count 0 */
	info->status_id_read_cnt = 0;
	/* initialize read id/status buffer */
	memset(info->read_byte_reg, 0, XARINA_READ_BYTE_CNT_MAX);
	/* set ecc_task 0 */
	info->ecc_task = 0;

	/* task all clear */
	xarina_nand1_set_task(&info->mtd,XARINA_NAND1_BIT_TASK_CLR);

	/* write protect Off */
	xarina_nand1_write_protect(&info->mtd, XARINA_NAND1_WP_OFF);


	/* scan xarina_nand id */
	if (nand_scan(&info->mtd, pdata->nr_chips)) {
		err = -ENXIO;
		/* write protect on [failed to find chip] */
		xarina_nand1_write_protect(&info->mtd, XARINA_NAND1_WP_ON);
		goto out_iounmap;
	}

	/* 1. parse cmd_line mtd_partition */
	/* 2. if cmd_line partition exists, add_mtd_partition(cmd_line partition) */
	/* 3. else if cmd_line partition not exists && pdata->partition exists , add_mtd_partition(pdata->partition) */
	/* 4. else add_mtd_devices() */
#ifdef CONFIG_MTD_PARTITIONS
	err = parse_mtd_partitions(&info->mtd, part_probes, &info->parts, 0);
	if (err > 0)
		add_mtd_partitions(&info->mtd, info->parts, err);
	else if (err <= 0 && pdata->partitions)
		add_mtd_partitions(&info->mtd, pdata->partitions, pdata->nr_partitions);
	else
#endif
		add_mtd_device(&info->mtd);

	/* write protect on*/
	xarina_nand1_write_protect(&info->mtd, XARINA_NAND1_WP_ON);

	platform_set_drvdata(pdev, info);

	return 0;

out_iounmap:
#ifdef CONFIG_MTD_XARINA_NAND_DMAC
#ifdef CONFIG_SLUB /*sg_alloc_table test*/
	/* disable interrupt */
	writel(0,info->nand.IO_ADDR_W + XARINA_NAND1_NF_IEN);
	sg_free_table(&info->sgt);
release_dma_ch:
#endif
	dma_release_channel(info->dma_channel);
	kfree(info->dma_buf);
out_free_for_dma:
#endif
	iounmap(info->nand.IO_ADDR_R);
out_release_mem:
	release_mem_region(res->start, res->end - res->start + 1);

out_free_mem:
	kfree(info);

	return err;

}

#ifdef XARINA_NAND1_MODULE_TEST
static int xarina_nand1_remove(struct platform_device *pdev)
{
	struct xarina_nand1_info *info = platform_get_drvdata(pdev);
	struct mtd_info *mtd = &info->mtd;
	struct resource *res;

	res = platform_get_resource(pdev, IORESOURCE_MEM, 0);

	/* Release NAND device, its internal structures and partitions */
	nand_release(mtd);
#ifdef CONFIG_MTD_XARINA_NAND_DMAC
#ifdef CONFIG_SLUB /*sg_alloc_table test*/
	/* disable interrupt */
	writel(0,info->nand.IO_ADDR_W + XARINA_NAND1_NF_IEN);
	sg_free_table(&info->sgt);
#endif
	dma_release_channel(info->dma_channel);
	kfree(info->dma_buf);
#endif
	iounmap(info->nand.IO_ADDR_R);
	release_mem_region(res->start, res->end - res->start + 1);
	kfree(info);
	return 0;
}
#endif
/* platform_device structure */
static struct platform_driver xarina_nand1_driver = {
	.probe		= xarina_nand1_probe,
#ifdef XARINA_NAND1_MODULE_TEST
	.remove		= xarina_nand1_remove,
#endif
	.driver		= {
		.name	= DRIVER_NAME,
	},
};
MODULE_ALIAS(DRIVER_NAME);

/**
 *
 * xarina_nand1_exit
 * module not supported
 *  -- no exit funciton
 *
 */
#ifdef XARINA_NAND1_MODULE_TEST
static void __exit xarina_nand1_exit(void)
{
	platform_driver_unregister(&xarina_nand1_driver);
}
module_exit(xarina_nand1_exit);
#endif

/**
 *
 * xarina_nand1_init - module_init
 *
 */
static int __init xarina_nand1_init(void)
{
	return platform_driver_register(&xarina_nand1_driver);
}

module_init(xarina_nand1_init);

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("NAND Controller driver for Xarina-NAND1");

