/* 2012-03-12: File added and changed by Sony Corporation */
/*
 *  Copyright 2011 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */

#ifndef _XARINA_NAND2_H
#define _XARINA_NAND2_H

#define	DRIVER_NAME	"xarina_nand2_flash"
#define	XARINA_NAND2_DMA_CHANNEL_NAME	"nand_dmac"

/* 16bit access macro for xarina */
#define WRITE16	writew
#define READ16	readw

#define XARINA_TRANS_TIMEOUT_MSECS	100
#define XARINA_CMD_TIMEOUT_MSECS 100

#define XARINA_NAND2_PAGESIZE		4096
#define XARINA_NAND2_OOBSIZE		128

#define XARINA_NAND2_MAX_NUM_CHIPS	8

#define XARINA_NAND2_ECC_START_OFFSET	28
#define XARINA_NAND2_ECC_SIZE		512
#define XARINA_NAND2_ECC_STEPS		XARINA_NAND2_PAGESIZE / XARINA_NAND2_ECC_SIZE
#define XARINA_NAND2_ECC_BYTES		10
#define XARINA_NAND2_ECC_TOTAL		XARINA_NAND2_ECC_BYTES * XARINA_NAND2_ECC_STEPS
#define XARINA_NAND2_OOB_UNIT		8
#define XARINA_NAND2_OOB_ECC_TOTAL	20
#define XARINA_NAND2_RESERVED_BYTES	12

#define XARINA_READID_ADDR	0
#define XARINA_READ_BYTE_CNT_MAX	6
#define XARINA_WORD_MASK		0x00000003

#define XARINA_SYSREG_CLK_CTRL_PADDR	0x30000034
#define XARINA_SYSREG_NFIF_CKSEL_MASK	0x00070000
#define XARINA_NAND2_CLK_RATE_1	0x1 /* 1/1  ..... - 20MHz */
#define XARINA_NAND2_CLK_RATE_2	0x2 /* 1/2  20MHz - 40MHz */
#define XARINA_NAND2_CLK_RATE_3	0x3 /* 1/3  40MHz - 60MHz */
#define XARINA_NAND2_CLK_RATE_4	0x0 /* 1/4  60MHz - 80MHz */

#define XARINA_NAND2_WP_ON	1
#define XARINA_NAND2_WP_OFF	0

/* chip delay */
#define XARINA_NAND2_CHIP_DELAY		(5) /* dummy parameter */

#define XARINA_NAND2_SHIFT_8			8
#define XARINA_NAND2_SHIFT_16		16
#define XARINA_NAND2_MASK_8BIT		0x000000ff
#define XARINA_NAND2_MASK_16BIT		0x0000ffff
#define XARINA_NAND2_ADDR_INVALID	(-1)
#define XARINA_NAND2_CMD_INVALID		(-1)

#define XARINA_NAND2_DEVICE_READY	1
#define XARINA_NAND2_DEVICE_BUSY		0

#define XARINA_NAND2_STATUS_FLAG		0xff00

/* XARINA_NAND2 16bit Registers */
#define XARINA_NAND2_MODE_REG				0x00
#define XARINA_NAND2_FLASH_DATA_SIZE_REG	0x04
#define XARINA_NAND2_TASK_REG				0x08
#define XARINA_NAND2_STATUS_REG				0x0c
#define XARINA_NAND2_STATUS_ID_READ12_REG	0x10
#define XARINA_NAND2_STATUS_ID_READ34_REG	0x14
#define XARINA_NAND2_STATUS_ID_READ56_REG	0x18
#define XARINA_NAND2_CONTROL_REG			0x1c
#define XARINA_NAND2_COMMAND1_REG			0x20
#define XARINA_NAND2_COL_ADDR_REG			0x24
#define XARINA_NAND2_PAGE_ADDR01_REG		0x28
#define XARINA_NAND2_PAGE_ADDR2_REG			0x2c
#define XARINA_NAND2_COMMAND2_REG			0x30
#define XARINA_NAND2_COMMAND3_REG			0x34
#define XARINA_NAND2_DATA_REG				0x38
#define XARINA_NAND2_MIRROR_DATA_REG		0x3c

#define XARINA_NAND2_ECC_CHECK_REG			0x40/* Read:  ECC Check */
#define XARINA_NAND2_ECC_CLEAR_REG			0x40/* Write: ECC Clear */
#define XARINA_NAND2_ECC_CALC_START_REG		0x44
#define XARINA_NAND2_ECC_RESULT_REG			0x48

#define XARINA_NAND2_ECC_ERR_ADDR_BASE_REG	0x60
#define XARINA_NAND2_ECC_ERR_BIT_BASE_REG	0x64

#define XARINA_NAND2_ECC_ERR_ADDR1_REG		0x60
#define XARINA_NAND2_ECC_ERR_BIT1_REG		0x64
#define XARINA_NAND2_ECC_ERR_ADDR2_REG		0x68
#define XARINA_NAND2_ECC_ERR_BIT2_REG		0x6c
#define XARINA_NAND2_ECC_ERR_ADDR3_REG		0x70
#define XARINA_NAND2_ECC_ERR_BIT3_REG		0x74
#define XARINA_NAND2_ECC_ERR_ADDR4_REG		0x78
#define XARINA_NAND2_ECC_ERR_BIT4_REG		0x7c
#define XARINA_NAND2_COMMAND_RATE_REG		0x80
#define XARINA_NAND2_RW_RATE_REG			0x84
#define XARINA_NAND2_BUSY_WAIT1_REG			0x88
#define XARINA_NAND2_BUSY_WAIT2_REG			0x8c

#define XARINA_NAND2_BOOT_STATUS_REG		0xa0

/* for nand dma if registers */
#define XARINA_NAND2_NF_AUTOCTL			(0x0100)	/* DMA Setting */
#define XARINA_NAND2_NF_STOP			(0x0104)	/* Control DMA Transfer autostop */
#define XARINA_NAND2_NF_DATASIZE		(0x0108)	/* Transfer Data Size (word align)*/
#define XARINA_NAND2_NF_RIS				(0x0110)	/* DMA RAW Interrupt Status */
#define XARINA_NAND2_NF_IS				(0x0114)	/* DMA Interrupt Status */
#define XARINA_NAND2_NF_IEN				(0x0118)	/* DMA Interrupt Enable for Sector autotransfer */
#define XARINA_NAND2_NF_ICR				(0x011C)	/* DMA Interrupt Clear */

#define XARINA_NAND2_NF_FIFO			(0x1000)	/* DMA FIFO for Burst Transfer (32bit reg) */

#define XARINA_NAND2_NF_BIT_DM_TYPE		0x0004 /* transfer type m2m/p2m */
#define XARINA_NAND2_NF_BIT_AUTO_EN		0x0002 /* transfer start */
#define XARINA_NAND2_NF_BIT_WXR			0x0001 /* Read/Write 1:write */


/* Mode Register */
#define XARINA_NAND2_BIT_WAIT_ACT			0x0080
#define XARINA_NAND2_BIT_REG_NUM			0x0070
#define XARINA_NAND2_BIT_READ_ID			0x0050
#define XARINA_NAND2_BIT_READ_STATUS		0x0010
#define XARINA_NAND2_BIT_ENDIAN				0x0004

/* Task Register */
#define XARINA_NAND2_BIT_TASK_ALL				0xffff
#define XARINA_NAND2_BIT_TASK_RECV2REG			0x8000
#define XARINA_NAND2_BIT_TASK_WAIT_READY2		0x4000
#define XARINA_NAND2_BIT_TASK_COMMAND3			0x2000
#define XARINA_NAND2_BIT_TASK_SEND_ECC			0x1000
#define XARINA_NAND2_BIT_TASK_SEND_DATA			0x0800
#define XARINA_NAND2_BIT_TASK_CHK_ECC			0x0400
#define XARINA_NAND2_BIT_TASK_RECV_DATA			0x0200
#define XARINA_NAND2_BIT_TASK_WAIT_READY1		0x0100
#define XARINA_NAND2_BIT_TASK_COMMAND2			0x0080
#define XARINA_NAND2_BIT_TASK_PAGE_ADR2			0x0040
#define XARINA_NAND2_BIT_TASK_PAGE_ADR1			0x0020
#define XARINA_NAND2_BIT_TASK_PAGE_ADR0			0x0010
#define XARINA_NAND2_BIT_TASK_COL_ADR1			0x0008
#define XARINA_NAND2_BIT_TASK_COL_ADR0			0x0004
#define XARINA_NAND2_BIT_TASK_COMMAND1			0x0002
#define XARINA_NAND2_BIT_TASK_CLR				0x0001

/* Status Register */
#define XARINA_NAND2_BIT_FIFO_FULL			0x0008
#define XARINA_NAND2_BIT_FIFO_EMPTY			0x0004
#define XARINA_NAND2_BIT_FBSYN				0x0001

/* Control Register */
#define XARINA_NAND2_BIT_FCE1N				0x0200
#define XARINA_NAND2_BIT_FCE0N				0x0100
#define XARINA_NAND2_BIT_FCDISABLE			0xff00
#define XARINA_NAND2_BIT_FWPN				0x0002
#define XARINA_NAND2_BIT_FOUT				0x0001

/* ECC Check Register */
#define XARINA_NAND2_BIT_ECCCHK					0x0001 /* ecc code error */

/* ECC Calculation Start Register */
#define XARINA_NAND2_BIT_ECC_CALC_START			0x0001 /* ecc calculation start */

/* ECC Result Register */
#define XARINA_NAND2_BIT_ECC_CORRECT_MASK		0x0007 /* ecc correct number */
#define XARINA_NAND2_BIT_ECC_NO_ERR			0x0
#define XARINA_NAND2_BIT_ECC_5_ERR			0x1
#define XARINA_NAND2_BIT_ECC_UC_ERR			0x2
#define XARINA_NAND2_BIT_ECC_C_ERR			0x3
#define XARINA_NAND2_ECC_RESULT_SHIFT		4
// no error, uncorrectable, over 5 error, correctable

#define XARINA_NAND2_ECC_ERROR_REG_OFFSET	8

// for timing control registers
#define XARINA_NAND2_PARAM_CMD_RATE_80	0x0010
#define XARINA_NAND2_PARAM_RW_RATE_80	0x1312
#define XARINA_NAND2_PARAM_BUSYWAIT1_80	0x0490
#define XARINA_NAND2_PARAM_BUSYWAIT2_80	0x0012
#define XARINA_NAND2_PARAM_CMD_RATE_66	0x0000
#define XARINA_NAND2_PARAM_RW_RATE_66	0x1212
#define XARINA_NAND2_PARAM_BUSYWAIT1_66	0x0160
#define XARINA_NAND2_PARAM_BUSYWAIT2_66	0x0008
#define XARINA_NAND2_PARAM_CMD_RATE_50	0x0000
#define XARINA_NAND2_PARAM_RW_RATE_50	0x0101
#define XARINA_NAND2_PARAM_BUSYWAIT1_50	0x0140
#define XARINA_NAND2_PARAM_BUSYWAIT2_50	0x0006
#define XARINA_NAND2_PARAM_CMD_RATE_40	0x0000
#define XARINA_NAND2_PARAM_RW_RATE_40	0x0100
#define XARINA_NAND2_PARAM_BUSYWAIT1_40	0x0130
#define XARINA_NAND2_PARAM_BUSYWAIT2_40	0x0005
#define XARINA_NAND2_PARAM_CMD_RATE_33	0x0000
#define XARINA_NAND2_PARAM_RW_RATE_33	0x0100
#define XARINA_NAND2_PARAM_BUSYWAIT1_33	0x0130
#define XARINA_NAND2_PARAM_BUSYWAIT2_33	0x0004
#define XARINA_NAND2_PARAM_CMD_RATE_6	0x0000
#define XARINA_NAND2_PARAM_RW_RATE_6	0x0000
#define XARINA_NAND2_PARAM_BUSYWAIT1_6	0x0130
#define XARINA_NAND2_PARAM_BUSYWAIT2_6	0x0002

/* for nand dma if registers */
#define XARINA_NAND2_NF_BIT_DMA_TYPE				0x0004
#define XARINA_NAND2_NF_BIT_DMA_AUTO_EN				0x0002
#define XARINA_NAND2_NF_BIT_DMA_WXR					0x0001

#define XARINA_NAND2_NF_BIT_DMA_STOP				0x0001

#define XARINA_NAND2_NF_BIT_DMA_RIS					0x0001
#define XARINA_NAND2_NF_BIT_DMA_IS					0x0001
#define XARINA_NAND2_NF_BIT_DMA_IEN					0x0001
#define XARINA_NAND2_NF_BIT_DMA_ICR					0x0001

/* safe correct data count */
#define XARINA_NAND2_SAFE_CORRECT_ERROR_MAX			2

#endif  /* _XARINA_NAND2_H */

