/* 2011-10-03: File added and changed by Sony Corporation */
/*
 *
 * Copyright 2008-2011 Sony Corporation.
 *
 * Derived from Intel e1000 driver
 * Copyright(c) 1999 - 2006 Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA.
 *
 */

/* smac_ethtool.c - ethtool support */

#include "smac_local.h"
#include "smac_phy_vendor.h"

/* unit test */
#ifdef CONFIG_SMAC_UNIT_TEST
extern enum smac_latency_range smac_update_itnr(enum smac_latency_range current_itnr, int packets, int bytes);
static int test_smac_update_itnr(struct smac_adapter *adapter, uint64_t *data);
extern void smac_set_itnr(struct smac_adapter *adapter);
static int test_smac_set_itnr(struct smac_adapter *adapter, uint64_t *data);
#endif

enum {NETDEV_STATS, SMAC_STATS};

/* for statistics */
struct smac_stats {
	char stat_string[ETH_GSTRING_LEN];
	int type;
	int sizeof_stat;
	int stat_offset;
};

#define SMAC_STAT(m)		SMAC_STATS, \
				sizeof(((struct smac_adapter *)0)->m), \
		      		offsetof(struct smac_adapter, m)
#define SMAC_NETDEV_STAT(m)	NETDEV_STATS, \
				sizeof(((struct net_device *)0)->m), \
				offsetof(struct net_device, m)

static const struct smac_stats smac_gstrings_stats[] = {
#ifdef CONFIG_SMAC_DEBUG
	{ "intr_stat", SMAC_STAT(stats.intr_stat) },
	{ "intr_enable", SMAC_STAT(stats.intr_enable) },
	{ "operation", SMAC_STAT(stats.operation) },
	{ "tx_db_base", SMAC_STAT(stats.tx_db_base) },
	{ "tx_db_length_minus1", SMAC_STAT(stats.tx_db_length_minus1) },
	{ "tx_db_head", SMAC_STAT(stats.tx_db_head) },
	{ "tx_db_tail", SMAC_STAT(stats.tx_db_tail) },
	{ "rx_db_base", SMAC_STAT(stats.rx_db_base) },
	{ "rx_db_length_minus1", SMAC_STAT(stats.rx_db_length_minus1) },
	{ "rx_db_head", SMAC_STAT(stats.rx_db_head) },
	{ "rx_db_tail", SMAC_STAT(stats.rx_db_tail) },
	{ "bus_mode", SMAC_STAT(stats.bus_mode) },
	{ "rx_max_frm_length", SMAC_STAT(stats.rx_max_frm_length) },
	{ "rx_frame_count_limit", SMAC_STAT(stats.rx_frame_count_limit) },
#ifdef CONFIG_SMAC_ENABLE_RX_DELAY_TIMER
	{ "rx_delay_timer", SMAC_STAT(stats.rx_delay_timer) },
	{ "rx_absolute_timer", SMAC_STAT(stats.rx_absolute_timer) },
#endif
	{ "rx_desc_limit", SMAC_STAT(stats.rx_desc_limit) },
	{ "rxfifo_frm_cnt", SMAC_STAT(stats.rxfifo_frm_cnt) },
	{ "phy_polling", SMAC_STAT(stats.phy_polling) },
	{ "phy_polled_data", SMAC_STAT(stats.phy_polled_data) },
	{ "tx_desc_addr", SMAC_STAT(stats.tx_desc_addr) },
	{ "rx_desc_addr", SMAC_STAT(stats.rx_desc_addr) },
	{ "tx_desc0", SMAC_STAT(stats.tx_desc0) },
	{ "tx_desc1", SMAC_STAT(stats.tx_desc1) },
	{ "tx_desc2", SMAC_STAT(stats.tx_desc2) },
	{ "tx_desc3", SMAC_STAT(stats.tx_desc3) },
	{ "rx_desc0", SMAC_STAT(stats.rx_desc0) },
	{ "rx_desc1", SMAC_STAT(stats.rx_desc1) },
	{ "rx_desc2", SMAC_STAT(stats.rx_desc2) },
	{ "rx_desc3", SMAC_STAT(stats.rx_desc3) },
	{ "state_tx_rd", SMAC_STAT(stats.state_tx_rd) },
	{ "state_tx_gd", SMAC_STAT(stats.state_tx_gd) },
	{ "state_tx_gb", SMAC_STAT(stats.state_tx_gb) },
	{ "state_tx_wr", SMAC_STAT(stats.state_tx_wr) },
	{ "state_tx_wb", SMAC_STAT(stats.state_tx_wb) },
	{ "state_rx_rd", SMAC_STAT(stats.state_rx_rd) },
	{ "state_rx_gd", SMAC_STAT(stats.state_rx_gd) },
	{ "state_rx_wr", SMAC_STAT(stats.state_rx_wr) },
	{ "state_rx_wb", SMAC_STAT(stats.state_rx_wb) },
	{ "state_swr", SMAC_STAT(stats.state_swr) },
	{ "state_ra", SMAC_STAT(stats.state_ra) },
	{ "state_ra_pl", SMAC_STAT(stats.state_ra_pl) },
	{ "hw_version", SMAC_STAT(stats.hw_version) },
	{ "revise", SMAC_STAT(stats.revise) },
	{ "mac_config", SMAC_STAT(stats.mac_config) },
	{ "mac_frame_filter", SMAC_STAT(stats.mac_frame_filter) },
	{ "phy_addr", SMAC_STAT(stats.phy_addr) },
	{ "phy_data", SMAC_STAT(stats.phy_data) },
	{ "control_frm", SMAC_STAT(stats.control_frm) },
	{ "vlan_tag", SMAC_STAT(stats.vlan_tag) },
	{ "mac_version", SMAC_STAT(stats.mac_version) },
	{ "power_management", SMAC_STAT(stats.power_management) },
	{ "mac_intr_stat", SMAC_STAT(stats.mac_intr_stat) },
	{ "mac_intr_mask", SMAC_STAT(stats.mac_intr_mask) },
	{ "mac_addr_0_high", SMAC_STAT(stats.mac_addr_0_high) },
	{ "mac_addr_0_low", SMAC_STAT(stats.mac_addr_0_low) },
	{ "rgmii_stat", SMAC_STAT(stats.rgmii_stat) },
	{ "mmc_control", SMAC_STAT(stats.mmc_control) },
	{ "mmc_intr_rx", SMAC_STAT(stats.mmc_intr_rx) },
	{ "mmc_intr_tx", SMAC_STAT(stats.mmc_intr_tx) },
	{ "mmc_intr_mask_rx", SMAC_STAT(stats.mmc_intr_mask_rx) },
	{ "mmc_intr_mask_tx", SMAC_STAT(stats.mmc_intr_mask_tx) },
#endif

	/* HW Statistics */
	{ "txoctetcount_gb", SMAC_STAT(stats.txoctetcount_gb) },
	{ "txframecount_gb", SMAC_STAT(stats.txframecount_gb) },
	{ "txbroadcastframes_g", SMAC_STAT(stats.txbroadcastframes_g) },
	{ "txmulticastframes_g", SMAC_STAT(stats.txmulticastframes_g) },
	{ "tx64octets_gb", SMAC_STAT(stats.tx64octets_gb) },
	{ "tx65to127octets_gb", SMAC_STAT(stats.tx65to127octets_gb) },
	{ "tx128to255octets_gb", SMAC_STAT(stats.tx128to255octets_gb) },
	{ "tx256to511octets_gb", SMAC_STAT(stats.tx256to511octets_gb) },
	{ "tx512to1023octets_gb", SMAC_STAT(stats.tx512to1023octets_gb) },
	{ "tx1024tomaxoctets_gb", SMAC_STAT(stats.tx1024tomaxoctets_gb) },
	{ "txunicastframes_gb", SMAC_STAT(stats.txunicastframes_gb) },
	{ "txmulticastframes_gb", SMAC_STAT(stats.txmulticastframes_gb) },
	{ "txbroadcastframes_gb", SMAC_STAT(stats.txbroadcastframes_gb) },
	{ "txunderflowerror", SMAC_STAT(stats.txunderflowerror) },
	{ "txsinglecol_g", SMAC_STAT(stats.txsinglecol_g) },
	{ "txmulticol_g", SMAC_STAT(stats.txmulticol_g) },
	{ "txdeferred", SMAC_STAT(stats.txdeferred) },
	{ "txlatecol", SMAC_STAT(stats.txlatecol) },
	{ "txexesscol", SMAC_STAT(stats.txexesscol) },
	{ "txcarriererror", SMAC_STAT(stats.txcarriererror) },
	{ "txoctetcount_g", SMAC_STAT(stats.txoctetcount_g) },
	{ "txframecount_g", SMAC_STAT(stats.txframecount_g) },
	{ "txexcessdef", SMAC_STAT(stats.txexcessdef) },
	{ "txpauseframes", SMAC_STAT(stats.txpauseframes) },
	{ "txvlanframes_g", SMAC_STAT(stats.txvlanframes_g) },
	{ "rxframecount_gb", SMAC_STAT(stats.rxframecount_gb) },
	{ "rxoctetcount_gb", SMAC_STAT(stats.rxoctetcount_gb) },
	{ "rxoctetcount_g", SMAC_STAT(stats.rxoctetcount_g) },
	{ "rxbroadcastframes_g", SMAC_STAT(stats.rxbroadcastframes_g) },
	{ "rxmulticastframes_g", SMAC_STAT(stats.rxmulticastframes_g) },
	{ "rxcrcerror", SMAC_STAT(stats.rxcrcerror) },
	{ "rxalignmenterror", SMAC_STAT(stats.rxalignmenterror) },
	{ "rxrunterror", SMAC_STAT(stats.rxrunterror) },
	{ "rxjabbererror", SMAC_STAT(stats.rxjabbererror) },
	{ "rxundersize_g", SMAC_STAT(stats.rxundersize_g) },
	{ "rxoversize_g", SMAC_STAT(stats.rxoversize_g) },
	{ "rx64octets_gb", SMAC_STAT(stats.rx64octets_gb) },
	{ "rx65to127octets_gb", SMAC_STAT(stats.rx65to127octets_gb) },
	{ "rx128to255octets_gb", SMAC_STAT(stats.rx128to255octets_gb) },
	{ "rx256to511octets_gb", SMAC_STAT(stats.rx256to511octets_gb) },
	{ "rx512to1023octets_gb", SMAC_STAT(stats.rx512to1023octets_gb) },
	{ "rx1024tomaxoctets_gb", SMAC_STAT(stats.rx1024tomaxoctets_gb) },
	{ "rxunicastframes_g", SMAC_STAT(stats.rxunicastframes_g) },
	{ "rxlengtherror", SMAC_STAT(stats.rxlengtherror) },
	{ "rxoutofrangetype", SMAC_STAT(stats.rxoutofrangetype) },
	{ "rxpauseframes", SMAC_STAT(stats.rxpauseframes) },
	{ "rxfifooverflow", SMAC_STAT(stats.rxfifooverflow) },
	{ "rxvlanframes_gb", SMAC_STAT(stats.rxvlanframes_gb) },
	{ "rxwatchdogerror", SMAC_STAT(stats.rxwatchdogerror) },
	{ "rxipv4_gd_frms", SMAC_STAT(stats.rxipv4_gd_frms) },
	{ "rxipv4_hdrerr_frms", SMAC_STAT(stats.rxipv4_hdrerr_frms) },
	{ "rxipv4_nopay_frms", SMAC_STAT(stats.rxipv4_nopay_frms) },
	{ "rxipv4_frag_frms", SMAC_STAT(stats.rxipv4_frag_frms) },
	{ "rxipv4_udsbl_frms", SMAC_STAT(stats.rxipv4_udsbl_frms) },
	{ "rxipv6_gd_frms", SMAC_STAT(stats.rxipv6_gd_frms) },
	{ "rxipv6_hdrerr_frms", SMAC_STAT(stats.rxipv6_hdrerr_frms) },
	{ "rxipv6_nopay_frms", SMAC_STAT(stats.rxipv6_nopay_frms) },
	{ "rxudp_gd_frms", SMAC_STAT(stats.rxudp_gd_frms) },
	{ "rxudp_err_frms", SMAC_STAT(stats.rxudp_err_frms) },
	{ "rxtcp_gd_frms", SMAC_STAT(stats.rxtcp_gd_frms) },
	{ "rxtcp_err_frms", SMAC_STAT(stats.rxtcp_err_frms) },
	{ "rxicmp_gd_frms", SMAC_STAT(stats.rxicmp_gd_frms) },
	{ "rxicmp_err_frms", SMAC_STAT(stats.rxicmp_err_frms) },
	{ "rxipv4_gd_octets", SMAC_STAT(stats.rxipv4_gd_octets) },
	{ "rxipv4_hdrerr_octets", SMAC_STAT(stats.rxipv4_hdrerr_octets) },
	{ "rxipv4_nopay_octets", SMAC_STAT(stats.rxipv4_nopay_octets) },
	{ "rxipv4_frag_octets", SMAC_STAT(stats.rxipv4_frag_octets) },
	{ "rxipv4_udsel_octets", SMAC_STAT(stats.rxipv4_udsel_octets) },
	{ "rxipv6_gd_octets", SMAC_STAT(stats.rxipv6_gd_octets) },
	{ "rxipv6_hdrerr_octets", SMAC_STAT(stats.rxipv6_hdrerr_octets) },
	{ "rxipv6_nopay_octets", SMAC_STAT(stats.rxipv6_nopay_octets) },
	{ "rxudp_gd_octets", SMAC_STAT(stats.rxudp_gd_octets) },
	{ "rxudp_err_octets", SMAC_STAT(stats.rxudp_err_octets) },
	{ "rxtcp_gd_octets", SMAC_STAT(stats.rxtcp_gd_octets) },
	{ "rxtcp_err_octets", SMAC_STAT(stats.rxtcp_err_octets) },
	{ "rxicmp_gd_octets", SMAC_STAT(stats.rxicmp_gd_octets) },
	{ "rxicmp_err_octets", SMAC_STAT(stats.rxicmp_err_octets) },
};

#define SMAC_QUEUE_STATS_LEN 0
#define SMAC_GLOBAL_STATS_LEN ARRAY_SIZE(smac_gstrings_stats)
#define SMAC_STATS_LEN (SMAC_GLOBAL_STATS_LEN + SMAC_QUEUE_STATS_LEN)

/* for ethtool -t */
static const char smac_gstrings_test[][ETH_GSTRING_LEN] = {
#ifdef CONFIG_SMAC_UNIT_TEST
	"smac_update_itnr (UnitTest)",
	"smac_set_itnr (UnitTest)",
#else
	/* normal test */
#endif
};
#define SMAC_TEST_LEN	ARRAY_SIZE(smac_gstrings_test)

#ifdef CONFIG_SMAC_UNIT_TEST
enum unit_test { TEST_SMAC_UPDATE_ITNR, TEST_SMAC_SET_ITNR };
#endif


/*----------------------------------------------------------------------------*/
/* set struct ethtool_cmd member */
static int
smac_get_settings(struct net_device *netdev, struct ethtool_cmd *ecmd)
{
	struct smac_adapter *adapter = netdev_priv(netdev);

	/* set supported */
	ecmd->supported = (SUPPORTED_10baseT_Half |
				SUPPORTED_10baseT_Full |
				SUPPORTED_100baseT_Half |
				SUPPORTED_100baseT_Full |
				SUPPORTED_Autoneg |
				SUPPORTED_TP);	/* Twisted Pair */

	if (smac_phy_is_gmii(adapter))
		ecmd->supported |= SUPPORTED_1000baseT_Full;

	/* set advertising */
	ecmd->advertising = ADVERTISED_TP;
	if (adapter->autoneg) {
		ecmd->advertising |= ADVERTISED_Autoneg;
		/* the smac autoneg seems to match ethtool nicely */
		ecmd->advertising |= adapter->autoneg_advertised;
	}

	/* set port */
	ecmd->port = PORT_TP;	/* Twisted Pair */

	/* set phy_address */
	ecmd->phy_address = adapter->phy_addr;	/* PHY address */

	/* set transceiver */
	ecmd->transceiver = XCVR_INTERNAL;

	/* set speed, duplex */
	if (smac_link_is_up(adapter)) {
		smac_get_speed_and_duplex(adapter,
					  &adapter->link_speed,
					  &adapter->link_duplex);
		ecmd->speed = adapter->link_speed;
		if (adapter->link_duplex == FULL_DUPLEX)
			ecmd->duplex = DUPLEX_FULL;
		else
			ecmd->duplex = DUPLEX_HALF;

		if (smac_phy_is_mdix(adapter))
			ecmd->eth_tp_mdix = ETH_TP_MDI_X;
		else
			ecmd->eth_tp_mdix = ETH_TP_MDI;
	} else {
		ecmd->speed = -1;
		ecmd->duplex = -1;
		ecmd->eth_tp_mdix = ETH_TP_MDI_INVALID;
	}

	/* set autoneg */
	ecmd->autoneg = adapter->autoneg ? AUTONEG_ENABLE : AUTONEG_DISABLE;

	return 0;
}

/*----------------------------------------------------------------------------*/
/* set speed and duplex */
static int
smac_set_spd_dplx(struct smac_adapter *adapter, uint16_t spddplx)
{
	int ret = 0;

	adapter->autoneg = 0;

	switch (spddplx) {
	case SPEED_10 + DUPLEX_HALF:
		adapter->forced_speed_duplex = smac_10_half;
		break;
	case SPEED_10 + DUPLEX_FULL:
		adapter->forced_speed_duplex = smac_10_full;
		break;
	case SPEED_100 + DUPLEX_HALF:
		adapter->forced_speed_duplex = smac_100_half;
		break;
	case SPEED_100 + DUPLEX_FULL:
		adapter->forced_speed_duplex = smac_100_full;
		break;
	case SPEED_1000 + DUPLEX_FULL:
		if (smac_phy_is_mii(adapter) || !smac_phy_autoneg_ability(adapter)) {
			SMAC_ERR_OUT("Not support 1000base\n");
			ret = -EINVAL;
		} else {
			SMAC_DPRINTK(PROBE, INFO, "Using Autonegotiation at 1000 Mbps Full Duplex only\n");
			adapter->autoneg = 1;
			adapter->autoneg_advertised = ADVERTISE_1000_FULL;
		}
		break;
	case SPEED_1000 + DUPLEX_HALF: /* not supported */
	default:
		SMAC_DPRINTK(PROBE, ERR, "Unsupported Speed/Duplex configuration\n");
		ret = -EINVAL;
		break;
	}

	return ret;
}


/*----------------------------------------------------------------------------*/
/* set speed and duplex (ethtool -s) */
static int
smac_set_settings(struct net_device *netdev, struct ethtool_cmd *ecmd)
{
	struct smac_adapter *adapter = netdev_priv(netdev);
	int ret = 0;

	if (!smac_set_resetting_bit(adapter))
		return -EBUSY;

	if (ecmd->autoneg == AUTONEG_ENABLE) {
		if (ecmd->advertising & ADVERTISED_1000baseT_Half) {
			SMAC_ERR_OUT("Not Support 1000BASE/Half\n");
			ret = -EINVAL;
			goto END_SET_SETTINGS;
		}

		if (smac_phy_is_mii(adapter)) {
			if (ecmd->advertising & ADVERTISED_1000baseT_Full) {
				SMAC_ERR_OUT("Not Support 1000BASE\n");
				ret = -EINVAL;
				goto END_SET_SETTINGS;
			}
		}

		if (!smac_phy_autoneg_ability(adapter)) {
			SMAC_ERR_OUT("PHY do not support Auto-Negotiation\n");
			ret = -EINVAL;
			goto END_SET_SETTINGS;
		}

		/* work-around for Micrel KSZ8051/31/21 Rev-A2 Errata Sheet-(10-11-11) #2 */
		if (smac_mach_phy_vendor(adapter, SMAC_MICREL_KSZ8051MNL)) {
			ecmd->advertising |= (ADVERTISED_10baseT_Half
						| ADVERTISED_10baseT_Full
						| ADVERTISED_100baseT_Half
						| ADVERTISED_100baseT_Full);
			SMAC_DPRINTK(PROBE, INFO, "Set AutoNego Advertisement to 0x0F (Micrel KSZ8051MNL erratum)\n");
		}

		adapter->autoneg = 1;
		adapter->autoneg_advertised = ecmd->advertising | ADVERTISED_TP | ADVERTISED_Autoneg;
		ecmd->advertising = adapter->autoneg_advertised;

		/* recovery fc setting */
		adapter->fc = adapter->original_fc;
	} else {
		if (smac_set_spd_dplx(adapter, ecmd->speed + ecmd->duplex)) {
			ret = -EINVAL;
			goto END_SET_SETTINGS;
		}
	}

	switch (ecmd->eth_tp_mdix) {
	case ETH_TP_MDI:
		adapter->mdix = SMAC_MDIX_OFF;
		break;
	case ETH_TP_MDI_X:
		adapter->mdix = SMAC_MDIX_ON;
		break;
	case ETH_TP_MDI_INVALID:
	default:
		adapter->mdix = SMAC_MDIX_AUTO;
		break;
	}

	/* reset the link */

	if (netif_running(adapter->netdev)) {
		smac_down(adapter);
		smac_up(adapter);
	} else {
		smac_reset(adapter);
	}

END_SET_SETTINGS:
	smac_clear_resetting_bit(adapter);
	return ret;
}



/*----------------------------------------------------------------------------*/
/* get flow control parameter */
static void
smac_get_pauseparam(struct net_device *netdev, struct ethtool_pauseparam *pause)
{
	struct smac_adapter *adapter = netdev_priv(netdev);

	pause->autoneg = (adapter->fc_autoneg ? AUTONEG_ENABLE : AUTONEG_DISABLE);

	if (adapter->fc == SMAC_FC_RX_PAUSE) {
		pause->rx_pause = 1;
	} else if (adapter->fc == SMAC_FC_TX_PAUSE) {
		pause->tx_pause = 1;
	} else if (adapter->fc == SMAC_FC_FULL) {
		pause->rx_pause = 1;
		pause->tx_pause = 1;
	}
}


/*----------------------------------------------------------------------------*/
/* set flow control parameter */
static int
smac_set_pauseparam(struct net_device *netdev, struct ethtool_pauseparam *pause)
{
	struct smac_adapter *adapter = netdev_priv(netdev);

	if (!smac_set_resetting_bit(adapter))
		return -EBUSY;

	adapter->fc_autoneg = pause->autoneg;

	if (pause->rx_pause && pause->tx_pause)
		adapter->fc = SMAC_FC_FULL;
	else if (pause->rx_pause && !pause->tx_pause)
		adapter->fc = SMAC_FC_RX_PAUSE;
	else if (!pause->rx_pause && pause->tx_pause)
		adapter->fc = SMAC_FC_TX_PAUSE;
	else if (!pause->rx_pause && !pause->tx_pause)
		adapter->fc = SMAC_FC_NONE;

	/* escape fc setting */
	adapter->original_fc = adapter->fc;

	if (adapter->fc_autoneg == AUTONEG_ENABLE) {
		if (netif_running(adapter->netdev)) {
			smac_down(adapter);
			smac_up(adapter);
		}
	} else {
		smac_force_mac_fc(adapter);
	}

	smac_clear_resetting_bit(adapter);

	return 0;
}


static int smac_get_sset_count(struct net_device *netdev, int sset)
{
	switch (sset) {
	case ETH_SS_TEST:
		return SMAC_TEST_LEN;
	case ETH_SS_STATS:
		return SMAC_STATS_LEN;
	default:
		return -EOPNOTSUPP;
	}
}


/*----------------------------------------------------------------------------*/
/* return driver infomation */
#define DRV_NAME_LENGTH	32
static void smac_get_drvinfo(struct net_device *netdev,
			     struct ethtool_drvinfo *drvinfo)
{
	struct smac_adapter *adapter = netdev_priv(netdev);
	char firmware_version[DRV_NAME_LENGTH];
	uint32_t hw_version = smac_readreg(SMAC_HW_VERSION);
	uint8_t* h = (uint8_t*) &hw_version;

	/* driver name and version */
	strncpy(drvinfo->driver, adapter->driver_name, DRV_NAME_LENGTH);
	strncpy(drvinfo->version, adapter->driver_version, DRV_NAME_LENGTH);

	/* hw version */
	snprintf(firmware_version, DRV_NAME_LENGTH,
		"%d.%d.%d.%d", h[3], h[2], h[1], h[0]);
	strncpy(drvinfo->fw_version, firmware_version, DRV_NAME_LENGTH);
}


/*----------------------------------------------------------------------------*/
static void
smac_diag_test(struct net_device *netdev,
		   struct ethtool_test *eth_test, uint64_t *data)
{
	struct smac_adapter *adapter = netdev_priv(netdev);

	set_bit(__SMAC_TESTING, &adapter->flags);

	if (eth_test->flags == ETH_TEST_FL_OFFLINE) {
		bool if_running = netif_running(netdev);

		/* Offline tests */
		SMAC_DPRINTK(HW, INFO, "offline testing starting\n");

		if (if_running)
			/* indicate we're in test mode */
			dev_close(netdev);
		else
			smac_reset(adapter);

#ifdef CONFIG_SMAC_UNIT_TEST
		if (!test_smac_update_itnr(adapter, &data[TEST_SMAC_UPDATE_ITNR]))
			eth_test->flags |= ETH_TEST_FL_FAILED;

		if (!test_smac_set_itnr(adapter, &data[TEST_SMAC_SET_ITNR]))
			eth_test->flags |= ETH_TEST_FL_FAILED;
#endif

		clear_bit(__SMAC_TESTING, &adapter->flags);
		if (if_running)
			dev_open(netdev);
	} else {
		SMAC_DPRINTK(HW, INFO, "online testing starting\n");
		clear_bit(__SMAC_TESTING, &adapter->flags);
	}
	msleep_interruptible(4 * 1000);
}


/*----------------------------------------------------------------------------*/
/* restart auto-negtiation */
static int smac_nway_reset(struct net_device *netdev)
{
	struct smac_adapter *adapter = netdev_priv(netdev);

	if (netif_running(netdev))
		smac_reinit_locked(adapter);

	return 0;
}


/*----------------------------------------------------------------------------*/
/* set statistics value */
static void
smac_get_ethtool_stats(struct net_device *netdev,
		struct ethtool_stats *stats, uint64_t *data)
{
	struct smac_adapter *adapter = netdev_priv(netdev);
	int i;

	smac_update_stats(adapter);
	for (i = 0; i < SMAC_GLOBAL_STATS_LEN; i++) {
		char *p = (char *)adapter + smac_gstrings_stats[i].stat_offset;

		data[i] = (smac_gstrings_stats[i].sizeof_stat == sizeof(uint64_t))
		? *(uint64_t *)p : *(uint32_t *)p;
	}

	SMAC_ASSERT(i == SMAC_STATS_LEN);
}


/*----------------------------------------------------------------------------*/
/* set statistics string */
static void
smac_get_strings(struct net_device *netdev, uint32_t stringset, uint8_t *data)
{
	uint8_t *p = data;
	int i;

	switch (stringset) {
	case ETH_SS_TEST:
		memcpy(data, *smac_gstrings_test,
			sizeof(smac_gstrings_test));
		break;

	case ETH_SS_STATS:	/* for statistics: option -S */
		for (i = 0; i < SMAC_GLOBAL_STATS_LEN; i++) {
			memcpy(p, smac_gstrings_stats[i].stat_string,
			       ETH_GSTRING_LEN);
			p += ETH_GSTRING_LEN;
		}

		SMAC_ASSERT(p - data == SMAC_STATS_LEN * ETH_GSTRING_LEN);
		break;
	}
}


/*----------------------------------------------------------------------------*/
/* ethtool operations */
static struct ethtool_ops smac_ethtool_ops = {
	.get_settings		= smac_get_settings,
	.set_settings		= smac_set_settings,
	.get_drvinfo		= smac_get_drvinfo,
	.nway_reset		= smac_nway_reset,
	.get_link		= ethtool_op_get_link,
	.get_pauseparam         = smac_get_pauseparam,
	.set_pauseparam         = smac_set_pauseparam,
	.self_test		= smac_diag_test,
	.get_strings		= smac_get_strings,
	.get_ethtool_stats	= smac_get_ethtool_stats,
	.get_sset_count		= smac_get_sset_count,
};


/*----------------------------------------------------------------------------*/
void smac_set_ethtool_ops(struct net_device *netdev)
{
	SET_ETHTOOL_OPS(netdev, &smac_ethtool_ops);
}



#ifdef CONFIG_SMAC_UNIT_TEST
/*----------------------------------------------------------------------------*/
/* test smac_update_itnr() */
static int
test_smac_update_itnr(struct smac_adapter *adapter, uint64_t *data)
{
	int ret = 1;

	/* lowest_latency_range */
	if (smac_update_itnr(lowest_latency_range, 1, 8001) != bulk_latency_range) {
		ret = -lowest_latency_range;
		goto eof;
	}
	if (smac_update_itnr(lowest_latency_range, 4, 513) != low_latency_range) {
		ret = -lowest_latency_range;
		goto eof;
	}
	if (smac_update_itnr(lowest_latency_range, 5, 513) != lowest_latency_range) {
		ret = -lowest_latency_range;
		goto eof;
	}

	/* low_latency_range */
	if (smac_update_itnr(low_latency_range, 1, 10001) != bulk_latency_range) {
		ret = -low_latency_range;
		goto eof;
	}
	if (smac_update_itnr(low_latency_range, 2, 10001) != bulk_latency_range) {
		ret = -low_latency_range;
		goto eof;
	}
	if (smac_update_itnr(low_latency_range, 10, 12010) != bulk_latency_range) {
		ret = -low_latency_range;
		goto eof;
	}
	if (smac_update_itnr(low_latency_range, 36, 10001) != lowest_latency_range) {
		ret = -low_latency_range;
		goto eof;
	}
	if (smac_update_itnr(low_latency_range, 35, 10001) != low_latency_range) {
		ret = -low_latency_range;
		goto eof;
	}
	if (smac_update_itnr(low_latency_range, 2, 4002) != bulk_latency_range) {
		ret = -low_latency_range;
		goto eof;
	}
	if (smac_update_itnr(low_latency_range, 2, 511) != lowest_latency_range) {
		ret = -low_latency_range;
		goto eof;
	}
	if (smac_update_itnr(low_latency_range, 2, 512) != low_latency_range) {
		ret = -low_latency_range;
		goto eof;
	}

	/* bulk_latency_range */
	if (smac_update_itnr(bulk_latency_range, 36, 25001) != low_latency_range) {
		ret = -bulk_latency_range;
		goto eof;
	}
	if (smac_update_itnr(bulk_latency_range, 35, 25001) != bulk_latency_range) {
		ret = -bulk_latency_range;
		goto eof;
	}
	if (smac_update_itnr(bulk_latency_range, 1, 5999) != low_latency_range) {
		ret = -bulk_latency_range;
		goto eof;
	}
	if (smac_update_itnr(bulk_latency_range, 1, 6000) != bulk_latency_range) {
		ret = -bulk_latency_range;
		goto eof;
	}

eof:
	switch (ret) {
	case -lowest_latency_range:
		*data = 1;
		SMAC_ERR_OUT("lowest_latency_range\n");
	case -low_latency_range:
		*data = 2;
		SMAC_ERR_OUT("low_latency_range\n");
	case -bulk_latency_range:
		*data = 3;
		SMAC_ERR_OUT("bulk_latency_range\n");
	default:
		/* success */
		*data = 0;
	}

	return (*data) ? false : true;
}


/*----------------------------------------------------------------------------*/
/* test smac_set_itnr() */
static int
test_smac_set_itnr(struct smac_adapter *adapter, uint64_t *data)
{
	*data = 0; /* success */

	/* new_itnr == adapter->itnr */
	adapter->link_speed = SPEED_1000;
	adapter->itnr_setting = SMAC_ITNR_DYNAMIC;
	adapter->current_itnr = bulk_latency_range;
	adapter->itnr = SMAC_BULK_LATENCY;
	adapter->total_rx_packets = 35;
	adapter->total_rx_bytes = 25001;
	smac_set_itnr(adapter);
	if (adapter->itnr != SMAC_BULK_LATENCY) {
		*data = 1;
		SMAC_ERR_OUT("new_itnr == adapter->itnr\n");
		return false;
	}

	/* new_itnr > adapter->itnr */
	adapter->link_speed = SPEED_1000;
	adapter->itnr_setting = SMAC_ITNR_DYNAMIC;
	adapter->current_itnr = lowest_latency_range;
	adapter->itnr = SMAC_LOWEST_LATENCY;
	adapter->total_rx_packets = 1;
	adapter->total_rx_bytes = 8001;
	smac_set_itnr(adapter);
	if (adapter->itnr != SMAC_BULK_LATENCY) {
		*data = 2;
		SMAC_ERR_OUT("new_itnr > adapter->itnr\n");
		return false;
	}

	/* new_itnr < adapter->itnr */
	adapter->link_speed = SPEED_1000;
	adapter->itnr_setting = SMAC_ITNR_DYNAMIC;
	adapter->current_itnr = low_latency_range;
	adapter->itnr = SMAC_LOW_LATENCY;
	adapter->total_rx_packets = 36;
	adapter->total_rx_bytes = 10001;
	smac_set_itnr(adapter);
	if (adapter->itnr != (SMAC_LOW_LATENCY - SMAC_LOWEST_LATENCY * 2)) {
		*data = 3;
		SMAC_ERR_OUT("new_itnr < adapter->itnr\n");
		return false;
	}

	/* test dynamic conservative */
	adapter->link_speed = SPEED_1000;
	adapter->itnr_setting = SMAC_ITNR_DYNAMIC_CONSERVATIVE;
	adapter->current_itnr = low_latency_range;
	adapter->itnr = SMAC_LOW_LATENCY;
	adapter->total_rx_packets = 36;
	adapter->total_rx_bytes = 10001;
	smac_set_itnr(adapter);
	if (adapter->itnr != SMAC_LOW_LATENCY) {
		*data = 4;
		SMAC_ERR_OUT("dynamic conservative\n");
		return false;
	}

	/* test 100base */
	adapter->link_speed = SPEED_100;
	adapter->itnr_setting = SMAC_ITNR_DYNAMIC;
	adapter->current_itnr = lowest_latency_range;
	adapter->itnr = SMAC_LOWEST_LATENCY;
	adapter->total_rx_packets = 1;
	adapter->total_rx_bytes = 8001;
	smac_set_itnr(adapter);
	if (adapter->itnr != SMAC_LOW_LATENCY) {
		*data = 5;
		SMAC_ERR_OUT("100Base\n");
		return false;
	}

	return true;
}

#endif /* CONFIG_SMAC_UNIT_TEST */
