/* 2011-11-02: File added and changed by Sony Corporation */
/*
 *  Copyright 2008-2011 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */

/* smac_hw.h - SMAC HW definitions  */

#ifndef __SMAC_HW_H__
#define __SMAC_HW_H__

#include <mach/irqs.h>
#include <mach/hardware.h>
#include "smac_local.h"

/* ------------------------------ */
/* system dependent */
/* ------------------------------ */
/* Descriptor */
/* SMAC Tx desc size: Max 256*16byte */
#define SMAC_TX_DESC_SIZE		(SMAC_MAX_TXD * sizeof(struct smac_desc))
/* SMAC Rx desc size: Max 256*16byte */
#define SMAC_RX_DESC_SIZE		(SMAC_MAX_RXD * sizeof(struct smac_desc))
/* SMAC total desc size */
#define SMAC_DESC_SIZE			(SMAC_TX_DESC_SIZE + SMAC_RX_DESC_SIZE)
#define SMAC_TX_DESC_OFFSET		0x00000000
#define SMAC_RX_DESC_OFFSET		(SMAC_TX_DESC_SIZE)

/* ------------------------------ */
/* SMAC Register Map */
#define SMAC_INTR_STAT			(0x0000)
#define SMAC_INTR_ENABLE		(0x0004)
#define SMAC_INTR_DISABLE		(0x0008)
#define SMAC_INTR_CLEAR			(0x000c)
#define SMAC_INTR_TEST			(0x0010)
#define SMAC_INTR_RFC			(1 << 29)
#define SMAC_INTR_RDTA			(1 << 27)
#define SMAC_INTR_RDT			(1 << 26)
#define SMAC_INTR_RDL			(1 << 25)
#define SMAC_INTR_MIS			(1 << 20)
#define SMAC_INTR_SWRD			(1 << 18)
#define SMAC_INTR_PSUD			(1 << 17)
#define SMAC_INTR_RU			(1 << 7)
#define SMAC_INTR_RI			(1 << 6)
#define SMAC_INTR_OVF			(1 << 4)
#define SMAC_INTR_TU			(1 << 2)
#define SMAC_INTR_TI			(1 << 0)

#define SMAC_OPERATION			(0x0100)
#define SMAC_OP_RSF			(1 << 25) /* Recieve Store and Forward */
/* Threshold for flow control
   000 : Full - 1KB
   001 : Full - 2KB
   010 : Full - 3KB
   011 : Full - 4KB
   100 : Full - 5KB
   101 : Full - 6KB
   110 : Full - 7KB */
#define SMAC_OP_RF_MSB(val)		((((val) - 1) & 0x4) >> 2)
#define SMAC_OP_RF_LSB(val)		(((val) - 1) & 0x3)
 /* Threshold for activating (bit23,10-9) */
#define SMAC_OP_RFA			((SMAC_OP_RF_MSB(CONFIG_SMAC_RX_FIFO_ACTIVE) << 23) \
						| (SMAC_OP_RF_LSB(CONFIG_SMAC_RX_FIFO_ACTIVE) << 9))
/* Threshold for deactivating (bit22,12-11) */
#define SMAC_OP_RFD			((SMAC_OP_RF_MSB(CONFIG_SMAC_RX_FIFO_DEACTIVE) << 22) \
						| (SMAC_OP_RF_LSB(CONFIG_SMAC_RX_FIFO_DEACTIVE) << 11))
#define SMAC_OP_TSF			(1 << 21) /* Transmit Store and Forward */
#define SMAC_OP_EFC			(1 << 8) /* Enable HW Flow Control */
#define SMAC_OP_STORE_AND_FORWARD	(SMAC_OP_TSF | SMAC_OP_RSF)

#define SMAC_TX_DB_BASE			(0x010c)
#define SMAC_TX_DB_LENGTH_MINUS1	(0x0110)
#define SMAC_TX_DB_HEAD			(0x0114)
#define SMAC_TX_DB_TAIL			(0x0118)
#define SMAC_RX_DB_BASE			(0x011c)
#define SMAC_RX_DB_LENGTH_MINUS1	(0x0120)
#define SMAC_RX_DB_HEAD			(0x0124)
#define SMAC_RX_DB_TAIL			(0x0128)

#define SMAC_BUS_MODE			(0x012c)
#define SMAC_BM_RMW_ENA			(1 << 16)
#define SMAC_BM_BLE_BIG			(1 << 7)
#define SMAC_BM_BLE_LITTLE		(0 << 7)
#define SMAC_BM_PBL(len)		((len) << 8) /* Programmable Burst Length */
#define SMAC_BM_SWR			(1 << 0) /* sw reset */

#define SMAC_BUS_ARB			(0x0130)
#define SMAC_RX_MAX_FRM_LENGTH		(0x0134)
#define SMAC_RX_FRAME_COUNT_LIMIT	(0x0140)

#ifdef CONFIG_SMAC_ENABLE_RX_DELAY_TIMER
#define SMAC_RX_DELAY_TIMER		(0x0148)
#define SMAC_RX_ABSOLUTE_TIMER		(0x014c)
#endif /* CONFIG_SMAC_ENABLE_RX_DELAY_TIMER */

#define SMAC_RX_DESC_LIMIT		(0x0150)
#define SMAC_RXFIFO_FRM_CNT		(0x0154)

#define SMAC_PHY_POLLING		(0x0440)
#define SMAC_PP_M_PA			0xF800
#define SMAC_PP_S_PA			11
#define SMAC_PP_M_GR			0x07C0
#define SMAC_PP_S_GR			6
#define SMAC_PP_PE			(1 << 5)

#define SMAC_PHY_POLLED_DATA		(0x0444)
#define SMAC_TX_DESC_ADDR		(0x0530)
#define SMAC_RX_DESC_ADDR		(0x0534)
#define SMAC_TX_DESC0			(0x0538)
#define SMAC_TX_DESC1			(0x053c)
#define SMAC_TX_DESC2			(0x0540)
#define SMAC_TX_DESC3			(0x0544)
#define SMAC_RX_DESC0			(0x0548)
#define SMAC_RX_DESC1			(0x054c)
#define SMAC_RX_DESC2			(0x0550)
#define SMAC_RX_DESC3			(0x0554)

#define SMAC_STATE_TX_RD		(0x0D00)
#define SMAC_STATE_TX_RD_IDLE		(0x0)
#define SMAC_STATE_TX_RD_WAIT		(0x1)
#define SMAC_STATE_TX_RD_DESC		(0x2)
#define SMAC_STATE_TX_RD_BUF1		(0x3)
#define SMAC_STATE_TX_RD_BUF2		(0x4)
#define SMAC_STATE_TX_RD_LEN		(0x5)
#define SMAC_STATE_TX_RD_CLR		(0x6)

#define SMAC_STATE_TX_GD		(0x0D04)
#define SMAC_STATE_TX_GB		(0x0D08)
#define SMAC_STATE_TX_WR		(0x0D0C)
#define SMAC_STATE_TX_WB		(0x0D10)
#define SMAC_STATE_RX_RD		(0x0D20)
#define SMAC_STATE_RX_GD		(0x0D24)
#define SMAC_STATE_RX_WR		(0x0D2C)
#define SMAC_STATE_RX_WB		(0x0D30)
#define SMAC_STATE_SWR			(0x0D40)
#define SMAC_STATE_RA			(0x0D60)
#define SMAC_STATE_RA_PL		(0x0D64)

#define SMAC_HW_VERSION			(0x0F00)
#define SMAC_HV_S_MAV			24
#define SMAC_HV_M_MAV			0xff000000
#define SMAC_HV_S_MIV0			16
#define SMAC_HV_M_MIV0			0x00ff0000
#define SMAC_HV_S_MIV1			8
#define SMAC_HV_M_MIV1			0x0000ff00
#define SMAC_HV_S_MIV2			0
#define SMAC_HV_M_MIV2			0x000000ff

#define SMAC_REVISE			(0x0F04)
#define SMAC_RE_SRV			(1 << 0)

#define SMAC_MAC_CONFIG			(0x1000)
#define SMAC_MC_WD			(1 << 23)
#define SMAC_MC_JD			(1 << 22)
#define SMAC_MC_JE			(1 << 20)	/* jumbo frame enable */
#define SMAC_MC_M_IFG			(7 << 17)
#define SMAC_MC_IFG_FULL		(0 << 17)	/* inter frame gap for full duplex, 0=96bit */
#define SMAC_MC_IFG_HALF		(4 << 17)	/* inter frame gap for half duplex, 4=64bit */
#define SMAC_MC_PS_MII			(1 << 15)	/* MII */
#define SMAC_MC_FES_100			(1 << 14)	/* 100Mbps */
#define SMAC_MC_DM_FULL			(1 << 11)	/* full duplex */
#define SMAC_MC_RCS			(1 << 10)	/* to enable Rx checksum offload */
#define SMAC_MC_DFS_ENA			(0 << 7)	/* enable FCS stripping */
#define SMAC_MC_DFS_DIS			(1 << 7)
#define SMAC_MC_TE			(1 << 3)
#define SMAC_MC_RE			(1 << 2)

#define SMAC_MAC_FRAME_FILTER		(0x1004)
#define SMAC_MFF_PM			(1 << 4)	/* enable pass all multicast */
#define SMAC_MFF_PR			(1 << 0)	/* enable promiscuous mode */

#define SMAC_PHY_ADDR			(0x1010)
#define SMAC_PA_M_PA			0xf800
#define SMAC_PA_S_PA			11
#define SMAC_PA_M_GR			0x07c0
#define SMAC_PA_S_GR			6
#define SMAC_PA_M_MCR			0x001c
#define SMAC_PA_S_MCR			2
#define SMAC_PA_M_GW			0x0002
#define SMAC_PA_READ			0x0000
#define SMAC_PA_WRITE			0x0002
#define SMAC_PA_M_GB			0x0001
#define SMAC_PA_KICK			0x0001
#define SMAC_PA_BUSY			0x0001
#define SMAC_PA_FREE			0x0000

#define SMAC_PHY_DATA			(0x1014)

#define SMAC_CONTROL_FRM		(0x1018)
#define SMAC_CF_PT			((CONFIG_SMAC_FC_PAUSE_TIME & 0xFFFF) << 16) /* Pause Time */
/* PLT: Pause Low Threshold
   00:   4 slot times
   01:  28 slot times
   10: 144 slot times
   11: 256 slot times */
#define SMAC_CF_PLT			(1 << 4) /* Pause Low Threshold */
#define SMAC_CF_UP			(1 << 3) /* Unicast Pause Frame Detect */
#define SMAC_CF_RFE			(1 << 2) /* Receive Flow Control Enable */
#define SMAC_CF_TFE			(1 << 1) /* Transmit Flow Control Enable */
#define SMAC_CF_FCB			(1 << 0) /* Flow Control Busy/Backpressure Active */

#define SMAC_VLAN_TAG			(0x101c)
#define SMAC_MAC_VERSION		(0x1020)
#define SMAC_POWER_MANAGEMENT		(0x102c)
#define SMAC_POWER_MANAGEMENT		(0x102c)
#define SMAC_MAC_INTR_STAT		(0x1038)
#define SMAC_MAC_INTR_MASK		(0x103C)

#define SMAC_MAC_ADDR_HIGH(n)		(0x1040 + n * 8)
#define SMAC_MAC_ADDR_LOW(n)		(0x1044 + n * 8)
#define SMAC_MAC_ADDR_0_HIGH		SMAC_MAC_ADDR_HIGH(0)
#define SMAC_MAC_ADDR_0_LOW		SMAC_MAC_ADDR_LOW(0)
#define SMAC_MAC_AE			0x80000000

#define SMAC_RGMII_STAT			(0x10D8)

#define SMAC_MMC_CONTROL		(0x1100)
#define SMAC_MMC_CR			(1 << 0)
#define SMAC_MMC_CSR			(1 << 1)
#define SMAC_MMC_RR			(1 << 2)
#define SMAC_MMC_CF			(1 << 3)

#define SMAC_MMC_INTR_RX		(0x1104)
#define SMAC_MMC_INTR_TX		(0x1108)
#define SMAC_MMC_INTR_MASK_RX		(0x110C)
#define SMAC_MMC_INTR_MASK_TX		(0x1110)

/* HW Statistics */
#define SMAC_TXOCTETCOUNT_GB		(0x1114)
#define SMAC_TXFRAMECOUNT_GB		(0x1118)
#define SMAC_TXBROADCASTFRAMES_G	(0x111C)
#define SMAC_TXMULTICASTFRAMES_G	(0x1120)
#define SMAC_TX64OCTETS_GB		(0x1124)
#define SMAC_TX65TO127OCTETS_GB		(0x1128)
#define SMAC_TX128TO255OCTETS_GB	(0x112C)
#define SMAC_TX256TO511OCTETS_GB	(0x1130)
#define SMAC_TX512TO1023OCTETS_GB	(0x1134)
#define SMAC_TX1024TOMAXOCTETS_GB	(0x1138)
#define SMAC_TXUNICASTFRAMES_GB		(0x113C)
#define SMAC_TXMULTICASTFRAMES_GB	(0x1140)
#define SMAC_TXBROADCASTFRAMES_GB	(0x1144)
#define SMAC_TXUNDERFLOWERROR		(0x1148)
#define SMAC_TXSINGLECOL_G		(0x114C)
#define SMAC_TXMULTICOL_G		(0x1150)
#define SMAC_TXDEFERRED			(0x1154)
#define SMAC_TXLATECOL			(0x1158)
#define SMAC_TXEXESSCOL			(0x115C)
#define SMAC_TXCARRIERERROR		(0x1160)
#define SMAC_TXOCTETCOUNT_G		(0x1164)
#define SMAC_TXFRAMECOUNT_G		(0x1168)
#define SMAC_TXEXCESSDEF		(0x116C)
#define SMAC_TXPAUSEFRAMES		(0x1170)
#define SMAC_TXVLANFRAMES_G		(0x1174)
#define SMAC_RXFRAMECOUNT_GB		(0x1180)
#define SMAC_RXOCTETCOUNT_GB		(0x1184)
#define SMAC_RXOCTETCOUNT_G		(0x1188)
#define SMAC_RXBROADCASTFRAMES_G	(0x118C)
#define SMAC_RXMULTICASTFRAMES_G	(0x1190)
#define SMAC_RXCRCERROR			(0x1194)
#define SMAC_RXALIGNMENTERROR		(0x1198)
#define SMAC_RXRUNTERROR		(0x119C)
#define SMAC_RXJABBERERROR		(0x11A0)
#define SMAC_RXUNDERSIZE_G		(0x11A4)
#define SMAC_RXOVERSIZE_G		(0x11A8)
#define SMAC_RX64OCTETS_GB		(0x11AC)
#define SMAC_RX65TO127OCTETS_GB		(0x11B0)
#define SMAC_RX128TO255OCTETS_GB	(0x11B4)
#define SMAC_RX256TO511OCTETS_GB	(0x11B8)
#define SMAC_RX512TO1023OCTETS_GB	(0x11BC)
#define SMAC_RX1024TOMAXOCTETS_GB	(0x11C0)
#define SMAC_RXUNICASTFRAMES_G		(0x11C4)
#define SMAC_RXLENGTHERROR		(0x11C8)
#define SMAC_RXOUTOFRANGETYPE		(0x11CC)
#define SMAC_RXPAUSEFRAMES		(0x11D0)
#define SMAC_RXFIFOOVERFLOW		(0x11D4)
#define SMAC_RXVLANFRAMES_GB		(0x11D8)
#define SMAC_RXWATCHDOGERROR		(0x11DC)
#define SMAC_RXIPV4_GD_FRMS		(0x1210)
#define SMAC_RXIPV4_HDRERR_FRMS		(0x1214)
#define SMAC_RXIPV4_NOPAY_FRMS		(0x1218)
#define SMAC_RXIPV4_FRAG_FRMS		(0x121C)
#define SMAC_RXIPV4_UDSBL_FRMS		(0x1220)
#define SMAC_RXIPV6_GD_FRMS		(0x1224)
#define SMAC_RXIPV6_HDRERR_FRMS		(0x1228)
#define SMAC_RXIPV6_NOPAY_FRMS		(0x122C)
#define SMAC_RXUDP_GD_FRMS		(0x1230)
#define SMAC_RXUDP_ERR_FRMS		(0x1234)
#define SMAC_RXTCP_GD_FRMS		(0x1238)
#define SMAC_RXTCP_ERR_FRMS		(0x123C)
#define SMAC_RXICMP_GD_FRMS		(0x1240)
#define SMAC_RXICMP_ERR_FRMS		(0x1244)
#define SMAC_RXIPV4_GD_OCTETS		(0x1250)
#define SMAC_RXIPV4_HDRERR_OCTETS	(0x1254)
#define SMAC_RXIPV4_NOPAY_OCTETS	(0x1258)
#define SMAC_RXIPV4_FRAG_OCTETS		(0x125C)
#define SMAC_RXIPV4_UDSEL_OCTETS	(0x1260)
#define SMAC_RXIPV6_GD_OCTETS		(0x1264)
#define SMAC_RXIPV6_HDRERR_OCTETS	(0x1268)
#define SMAC_RXIPV6_NOPAY_OCTETS	(0x126C)
#define SMAC_RXUDP_GD_OCTETS		(0x1270)
#define SMAC_RXUDP_ERR_OCTETS		(0x1274)
#define SMAC_RXTCP_GD_OCTETS		(0x1278)
#define SMAC_RXTCP_ERR_OCTETS		(0x127C)
#define SMAC_RXICMP_GD_OCTETS		(0x1280)
#define SMAC_RXICMP_ERR_OCTETS		(0x1284)

/* ------------------------------ */
/* descriptor */
struct smac_desc {
	volatile uint32_t desc0;
	uint32_t desc1;
	uint32_t desc2;
	uint32_t desc3;
};

/* TX descriptor */
#define SMAC_STDD_M_WB			(1 << 31)
#define SMAC_STDD_M_IC			(1 << 28)
#define SMAC_STDD_M_ICS			(1 << 25)
#define SMAC_STDD_M_TCS			(1 << 24)
#define SMAC_STDD_M_PHCS		(1 << 23)
#define SMAC_STDD_M_LS			(1 << 27)
#define SMAC_STDD_M_FS			(1 << 26)
#define SMAC_STDD_M_DFCS_ENA		(1 << 22)
#define SMAC_STDD_M_DFCS_DIS		(0 << 22)
#define SMAC_STDD_M_DPAD_ENA		(1 << 21)
#define SMAC_STDD_M_DPAD_DIS		(0 << 21)
#define SMAC_STDD_M_LSE			(1 << 19)
#define SMAC_STDD_M_BUF_LEN2		0x3fff0000
#define SMAC_STDD_S_BUF_LEN2		16
#define SMAC_STDD_M_BUF_LEN1		0x000000ff
#define SMAC_STDD_S_BUF_LEN1		0

/* RX descriptor */
/* RDES0 */
#define SMAC_SRD_M_WB			(1 << 31)
#define SMAC_SRD_M_FRAME_LEN		0x3fff0000
#define SMAC_SRD_S_FRAME_LEN		16
#define SMAC_SRD_M_TCS			(1 << 14)
#define SMAC_SRD_M_SAF			(1 << 13)
#define SMAC_SRD_M_LE			(1 << 12)
#define SMAC_SRD_M_VL			(1 << 10)
#define SMAC_SRD_M_ICSE			(1 << 7)
#define SMAC_SRD_M_LC			(1 << 6)
#define SMAC_SRD_M_ICS			(1 << 5)
#define SMAC_SRD_M_RE			(1 << 3)
#define SMAC_SRD_M_DE			(1 << 2)
#define SMAC_SRD_M_CE			(1 << 1)
#define SMAC_SRD_M_TCSE			(1 << 0)
#define SMAC_SRD_M_ERROR		(SMAC_SRD_M_LE | SMAC_SRD_M_LC | SMAC_SRD_M_RE | SMAC_SRD_M_DE | SMAC_SRD_M_CE)

/* RDES1 */
#define SMAC_SRD1_M_FT			(1 << 31)
#ifdef CONFIG_SMAC_ENABLE_RX_DELAY_TIMER
#define SMAC_SRD1_M_DT			(1 << 15)
#define SMAC_SRD1_M_ADT			(1 << 14)
#endif
#define SMAC_SRD1_M_FCL			(1 << 13)
#define SMAC_SRD1_M_FBL			(1 << 12)


/* ------------------------------ */
/* PHY */
/* ------------------------------ */
/* MDC Clock Range */
#define SMAC_PHY_CLK(mcr)			((mcr) << SMAC_PA_S_MCR)

/* PHY 1000 MII Register/Bit Definitions */
/* PHY Registers defined by IEEE */
#define SMAC_PHY_CTRL			0 /* Control Register */
#define SMAC_PHY_STATUS			1 /* Status Regiser */
#define SMAC_PHY_ID1			2 /* Phy Id Reg (word 1) */
#define SMAC_PHY_ID2			3 /* Phy Id Reg (word 2) */
#define SMAC_PHY_AUTONEG_ADV		4 /* Autoneg Advertisement */
#define SMAC_PHY_LP_ABILITY		5 /* Link Partner Ability (Base Page) */
#define SMAC_PHY_AUTONEG_EXP		6 /* Autoneg Expansion Reg */
#define SMAC_PHY_NEXT_PAGE_TX		7 /* Next Page TX */
#define SMAC_PHY_LP_NEXT_PAGE		8 /* Link Partner Next Page */
#define SMAC_PHY_1000T_CTRL		9 /* 1000Base-T Control Reg */
#define SMAC_PHY_1000T_STATUS		10 /* 1000Base-T Status Reg */

/* Control Register bit definitions (#0) */
#define MII_CR_SPEED_SELECT_MSB		0x0040  /* bits 6,13: 10=1000, 01=100, 00=10 */
#define MII_CR_COLL_TEST_ENABLE		0x0080  /* Collision test enable */
#define MII_CR_FULL_DUPLEX		0x0100  /* full-duplex=1, half-duplex=0 */
#define MII_CR_RESTART_AUTO_NEG		0x0200  /* Restart auto negotiation */
#define MII_CR_ISOLATE			0x0400  /* Isolate PHY from MII */
#define MII_CR_POWER_DOWN		0x0800  /* Power down */
#define MII_CR_AUTO_NEG_EN		0x1000  /* Auto Neg Enable */
#define MII_CR_SPEED_SELECT_LSB		0x2000  /* bits 6,13: 10=1000, 01=100, 00=10 */
#define MII_CR_LOOPBACK			0x4000  /* 0 = normal, 1 = loopback */
#define MII_CR_RESET			0x8000  /* 0 = normal, 1 = PHY reset */

#define MII_CR_SPEED_10			((0 << 13) | (0 << 6))
#define MII_CR_SPEED_100		((1 << 13) | (0 << 6))
#define MII_CR_SPEED_1000		((0 << 13) | (1 << 6))
#define MII_CR_M_RESET			(1 << 15)

/* PHY Status Register (#1) */
#define MII_SR_EXTENDED_CAPS		0x0001 /* Extended register capabilities */
#define MII_SR_JABBER_DETECT		0x0002 /* Jabber Detected */
#define MII_SR_LINK_STATUS		0x0004 /* Link Status 1 = link */
#define MII_SR_AUTONEG_CAPS		0x0008 /* Auto Neg Capable */
#define MII_SR_REMOTE_FAULT		0x0010 /* Remote Fault Detect */
#define MII_SR_AUTONEG_COMPLETE		0x0020 /* Auto Neg Complete */
#define MII_SR_PREAMBLE_SUPPRESS	0x0040 /* Preamble may be suppressed */
#define MII_SR_EXTENDED_STATUS		0x0100 /* Ext. status info in Reg 0x0F */
#define MII_SR_100T2_HD_CAPS		0x0200 /* 100T2 Half Duplex Capable */
#define MII_SR_100T2_FD_CAPS		0x0400 /* 100T2 Full Duplex Capable */
#define MII_SR_10T_HD_CAPS		0x0800 /* 10T   Half Duplex Capable */
#define MII_SR_10T_FD_CAPS		0x1000 /* 10T   Full Duplex Capable */
#define MII_SR_100X_HD_CAPS		0x2000 /* 100X  Half Duplex Capable */
#define MII_SR_100X_FD_CAPS		0x4000 /* 100X  Full Duplex Capable */
#define MII_SR_100T4_CAPS		0x8000 /* 100T4 Capable */

/* Auto-Negotiation advertisement register (#4) */
/* Technology Ability Field bit assignments */
/* see. IEEE Std 802.3-2005 Table 28B-1 */
#define NWAY_AR_SELECTOR_FIELD		0x0001 /* indicates IEEE 802.3 CSMA/CD */
#define NWAY_AR_10T_HD_CAPS		0x0020 /* 10T   Half Duplex Capable */
#define NWAY_AR_10T_FD_CAPS		0x0040 /* 10T   Full Duplex Capable */
#define NWAY_AR_100TX_HD_CAPS		0x0080 /* 100TX Half Duplex Capable */
#define NWAY_AR_100TX_FD_CAPS		0x0100 /* 100TX Full Duplex Capable */
#define NWAY_AR_100T4_CAPS		0x0200 /* 100T4 Capable */
#define NWAY_AR_PAUSE			0x0400 /* Pause operation desired */
#define NWAY_AR_ASM_DIR			0x0800 /* Asymmetric Pause Direction bit */
#define NWAY_AR_REMOTE_FAULT		0x2000 /* Remote Fault detected */
#define NWAY_AR_NEXT_PAGE		0x8000 /* Next Page ability supported */

/* Link Partner Ability Register (#5) */
#define NWAY_LPAR_SELECTOR_FIELD	0x0000 /* LP protocol selector field */
#define NWAY_LPAR_10T_HD_CAPS		0x0020 /* LP is 10T   Half Duplex Capable */
#define NWAY_LPAR_10T_FD_CAPS		0x0040 /* LP is 10T   Full Duplex Capable */
#define NWAY_LPAR_100TX_HD_CAPS		0x0080 /* LP is 100TX Half Duplex Capable */
#define NWAY_LPAR_100TX_FD_CAPS		0x0100 /* LP is 100TX Full Duplex Capable */
#define NWAY_LPAR_100T4_CAPS		0x0200 /* LP is 100T4 Capable */
#define NWAY_LPAR_PAUSE			0x0400 /* LP Pause operation desired */
#define NWAY_LPAR_ASM_DIR		0x0800 /* LP Asymmetric Pause Direction bit */
#define NWAY_LPAR_REMOTE_FAULT		0x2000 /* LP has detected Remote Fault */
#define NWAY_LPAR_ACKNOWLEDGE		0x4000 /* LP has rx'd link code word */
#define NWAY_LPAR_NEXT_PAGE		0x8000 /* Next Page ability supported */

/* MASTER-SLAVE (1000BASE-T) contorol register (#9) */
/* see. IEEE Std 802.3-2005 Table 40-3 */
#define CR_1000T_HD_CAPS		0x0100 /* Advertise 1000T HD capability */
#define CR_1000T_FD_CAPS		0x0200 /* Advertise 1000T FD capability  */

#endif	/* __SMAC_HW_H__ */

