/* 2011-10-03: File added and changed by Sony Corporation */
/*
 *
 * Copyright 2008-2011 Sony Corporation.
 *
 * Derived from Intel e1000 driver
 * Copyright(c) 1999 - 2006 Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA.
 *
 */

/* smac_local.h - local data */

#ifndef __SMAC_LOCAL_H__
#define __SMAC_LOCAL_H__

/*------------------------------------------------------------------------------
  Include Files
  ------------------------------------------------------------------------------*/
#include <linux/version.h>
#include <linux/ioport.h>
#include <linux/slab.h>
#include <linux/list.h>
#include <linux/sched.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>
#include <linux/delay.h>
#include <linux/timer.h>
#include <linux/vmalloc.h>
#include <linux/interrupt.h>
#include <linux/string.h>
#include <linux/pagemap.h>
#include <linux/bitops.h>
#include <linux/capability.h>
#include <linux/in.h>
#include <linux/ip.h>
#include <linux/tcp.h>
#include <linux/udp.h>
#include <linux/reboot.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/time.h>
#include <linux/stddef.h>
#include <linux/poll.h>
#include <linux/dma-mapping.h>
#include <linux/semaphore.h>
#include <linux/ipv6.h>
#include <linux/platform_device.h>
#include <linux/smac-xarina_platform.h>
#include <linux/phy.h>

#include <net/pkt_sched.h>
#include <net/ipv6.h>
#include <net/ip.h>

#include <asm/byteorder.h>
#include <asm/io.h>
#include <asm/irq.h>

#ifdef SIOCGMIIPHY
#include <linux/mii.h>
#endif
#ifdef SIOCETHTOOL
#include <linux/ethtool.h>
#endif
#ifdef NETIF_F_HW_VLAN_TX
#include <linux/if_vlan.h>
#endif

#include "smac_hw.h"		/* smac hw register map */


/*------------------------------------------------------------------------------
  Debug Print
  ------------------------------------------------------------------------------*/
#define SMAC_PFX "smac: "

#ifdef CONFIG_SMAC_TRACE_FUNCTION
#define SMAC_DEBUG_FUNC() printk(KERN_DEBUG SMAC_PFX "%s()\n", __func__)
#else
#define SMAC_DEBUG_FUNC() do {} while (0)
#endif

#ifdef CONFIG_SMAC_DEBUG
#define SMAC_ASSERT(expr) \
	if (!(expr)) { \
		printk(KERN_DEBUG SMAC_PFX "assertion failed! %s,%s,%s,line=%d\n", \
		#expr, __FILE__, __func__, __LINE__); \
	}
#define SMAC_DEBUG_OUT(fmt, args...) printk(KERN_DEBUG SMAC_PFX "debug: " "%s: " fmt, __func__, ## args)
#else
#define SMAC_ASSERT(expr) do {} while (0)
#define SMAC_DEBUG_OUT(fmt, args...) do {} while (0)
#endif

#define SMAC_ERR_OUT(fmt, args...) printk(KERN_ERR SMAC_PFX "error: " "%s: " fmt, __func__, ## args)
#define SMAC_DPRINTK(nlevel, klevel, fmt, args...)			\
do {									\
	if (NETIF_MSG_##nlevel & adapter->msg_enable)			\
		printk(KERN_##klevel SMAC_PFX "%s: " fmt,		\
		       adapter->netdev->name, ##args);	\
} while (0)

#if 0	/* for debug */
#define SMAC_VERBOSE(fmt, args...) printk(KERN_DEBUG SMAC_PFX "debug: " "%s(): " fmt, __func__, ## args)
#else
#define SMAC_VERBOSE(fmt, args...) do {} while (0)
#endif


/*------------------------------------------------------------------------------
  Macros
  ------------------------------------------------------------------------------*/
/* platform dependent */
/* swap 32bit for endian */
#define SWAP32(x)  __swab32(x)

#ifdef CONFIG_CPU_BIG_ENDIAN
#define SMAC_CPU_DESC_BE
#else
#define SMAC_CPU_DESC_LE
#endif

#if defined(SMAC_CPU_DESC_LE)
#define cpu_to_s32 cpu_to_le32
#define cpu_to_s16 cpu_to_le16
#elif defined(SMAC_CPU_DESC_BE)
#define cpu_to_s32 cpu_to_be32
#define cpu_to_s16 cpu_to_be16
#endif

/* advertised speed & duplex */
#define ADVERTISE_10_HALF		0x0001
#define ADVERTISE_10_FULL		0x0002
#define ADVERTISE_100_HALF		0x0004
#define ADVERTISE_100_FULL		0x0008
#define ADVERTISE_1000_HALF		0x0010
#define ADVERTISE_1000_FULL		0x0020

/* TX flags */
#define SMAC_TX_FLAGS_CSUM		0x00000001	/* IP checksum only */
#define SMAC_TX_FLAGS_TSO		0x00000004
#define SMAC_TX_FLAGS_TCSUM		0x00000010	/* TCP/UDP checksum + pseudo IP header checksum */

/* tx/rx descriptor defines */
#define SMAC_MAX_TXD			(CONFIG_SMAC_NR_TX_DESCRIPTORS)
#define SMAC_MAX_RXD			(CONFIG_SMAC_NR_RX_DESCRIPTORS)
#define SMAC_DEFAULT_TXD		(SMAC_MAX_TXD)
#define SMAC_DEFAULT_RXD		(SMAC_MAX_RXD)
#define SMAC_RX_BUFFER_WRITE		16

/* link speed */
#define SPEED_10			10
#define SPEED_100			100
#define SPEED_1000			1000

/* duplex */
#define HALF_DUPLEX			1
#define FULL_DUPLEX			2

/* frame size */
#define MIN_ETHERNET_VLAN_SIZE		(VLAN_HLEN + ETH_FCS_LEN + ETH_ZLEN)
#define MAX_ETHERNET_VLAN_SIZE		(VLAN_HLEN + ETH_FCS_LEN + ETH_FRAME_LEN)
#define SMAC_MIN_FRAME_SIZE		MIN_ETHERNET_VLAN_SIZE
#ifdef	CONFIG_SMAC_ENABLE_JUMBOFRAME
#define SMAC_MAX_FRAME_SIZE		4096
#else
#define SMAC_MAX_FRAME_SIZE		MAX_ETHERNET_VLAN_SIZE
#endif

/* # of storable MAC address */
#define SMAC_MULTICAST_ADDR_NUM		16

/* InterruptThrottleNumber latency */
/* SMAC_LOWEST_LATENCY: # of thinning. set it so that interrupt period = 15us */
#define SMAC_LOWEST_LATENCY		(CONFIG_SMAC_LOWEST_LATENCY)
#define SMAC_LOW_LATENCY		(SMAC_LOWEST_LATENCY * 4)
#define SMAC_BULK_LATENCY		(SMAC_LOW_LATENCY * 5)
/* parameter of InterruptThrottleNum */
#define SMAC_ITNR_OFF			0
#define SMAC_ITNR_DYNAMIC		1
#define SMAC_ITNR_DYNAMIC_CONSERVATIVE	3

/* for clean tx desc */
#define SMAC_CLEAN_TX_DESC_DELAY(t)	(msecs_to_jiffies((t)))

/* # of frames on TxFIFO */
#define SMAC_TX_FIFO_SIZE		(8)

/* size of dummy skb for hw reset */
#define SMAC_DUMMY_SKB_SIZE		(64)
#define SMAC_DUMMY_DESC_NR		(SMAC_TX_FIFO_SIZE + 2)

/* check link status cycle */
#define SMAC_LINK_HEARTBEAT		(1 * HZ)
/* net_device.watchdog_timeo */
#define SMAC_WATCHDOG_TIMEO		(10 * HZ)
/* net_device.weight */
#define SMAC_NAPI_WEIGHT		(64)

/* wait setting done */
#define SMAC_INIT_WAITTIMEOUT(x)	int x
#define SMAC_WAITTIMEOUT(x, cond, timesec) \
	do {  \
		for (x = (timesec) * 100; x > 0; x--) { \
			if ((cond)) \
				break; /* success */ \
			mdelay(10); \
		} \
	} while (0)
#define SMAC_OVER_WAITTIMEOUT(x)	(x == 0)


/*------------------------------------------------------------------------------
  Enums
  ------------------------------------------------------------------------------*/
/* speed & duplex settings */
enum smac_speed_duplex_type {
	smac_10_half = 0,
	smac_10_full = 1,
	smac_100_half = 2,
	smac_100_full = 3
};

/* flow control settings */
enum smac_fc_type {
	SMAC_FC_NONE = 0,
	SMAC_FC_RX_PAUSE = 1,
	SMAC_FC_TX_PAUSE = 2,
	SMAC_FC_FULL = 3
};

enum smac_mdix_type {
	SMAC_MDIX_AUTO = 0,
	SMAC_MDIX_OFF  = 1,
	SMAC_MDIX_ON   = 2
};

/* InterruptThrottleNum latency range  */
enum smac_latency_range {
	lowest_latency_range = 0,	/* 15 usec/interrupt */
	low_latency_range = 1,		/* 50 usec/interrupt */
	bulk_latency_range = 2,		/* 250 usec/interrupt */
};

/* for smac_adapter.flags */
enum smac_state_t {
	__SMAC_TESTING,
	__SMAC_RESETTING,
	__SMAC_DOWN
};


/*------------------------------------------------------------------------------
  Structs
  ------------------------------------------------------------------------------*/
struct smac_adapter;

/* struct for sk_buff */
struct smac_buffer {
	struct sk_buff *skb;		/* skb base address (virtual) */
	dma_addr_t dma;			/* skb base address (physical) */
	unsigned long time_stamp;	/* jiffies */
	uint16_t length;		/* length of skb */
	uint16_t next_to_watch;		/* index of descriptor (set up skb infomation) */
	unsigned int segs;
	unsigned int bytecount;
	u16 mapped_as_page;
};

/* struct for tx/rx descriptor */
/* descriptor is controled as ring buffer */
/* tx (rx) have SMAC_MAX_TXD (SMAC_MAX_RXD) buffer_info */
/* tx descriptor ring */
struct smac_tx_ring {
	void *desc;				/* descriptor base address (virtual) */
	dma_addr_t dma;				/* descriptor base address (physical) */
	unsigned int size;			/* descriptor size (byte) */
	unsigned int count;			/* number of descriptor */
	unsigned int next_to_use;		/* index of descriptor (next use) */
	unsigned int next_to_clean;		/* index of descriptor (next clean up) */
	struct smac_buffer *buffer_info;	/* skb information */
};

/* rx descriptor ring */
struct smac_rx_ring {
	void *desc;
	dma_addr_t dma;
	unsigned int size;
	unsigned int count;
	unsigned int next_to_use;
	unsigned int next_to_clean;
	struct smac_buffer *buffer_info;
};

/* descriptor operation macro */
#define SMAC_DESC_UNUSED(R) \
    ((((R)->next_to_clean > (R)->next_to_use) ? 0 : (R)->count) + (R)->next_to_clean - (R)->next_to_use - 1)

#define SMAC_GET_DESC(R, i, type)    (&(((struct type *)((R).desc))[i]))
#define SMAC_RX_DESC(R, i)	  SMAC_GET_DESC(R, i, smac_desc)
#define SMAC_TX_DESC(R, i)	  SMAC_GET_DESC(R, i, smac_desc)


/* Statistics counters collected by the MAC */
struct smac_hw_stats {
#ifdef CONFIG_SMAC_DEBUG
	uint64_t intr_stat;
	uint64_t intr_enable;
	uint64_t operation;
	uint64_t tx_db_base;
	uint64_t tx_db_length_minus1;
	uint64_t tx_db_head;
	uint64_t tx_db_tail;
	uint64_t rx_db_base;
	uint64_t rx_db_length_minus1;
	uint64_t rx_db_head;
	uint64_t rx_db_tail;
	uint64_t bus_mode;
	uint64_t rx_max_frm_length;
	uint64_t rx_frame_count_limit;
#ifdef CONFIG_SMAC_ENABLE_RX_DELAY_TIMER
	uint64_t rx_delay_timer;
	uint64_t rx_absolute_timer;
#endif
	uint64_t rx_desc_limit;
	uint64_t rxfifo_frm_cnt;
	uint64_t phy_polling;
	uint64_t phy_polled_data;
	uint64_t tx_desc_addr;
	uint64_t rx_desc_addr;
	uint64_t tx_desc0;
	uint64_t tx_desc1;
	uint64_t tx_desc2;
	uint64_t tx_desc3;
	uint64_t rx_desc0;
	uint64_t rx_desc1;
	uint64_t rx_desc2;
	uint64_t rx_desc3;
	uint64_t state_tx_rd;
	uint64_t state_tx_gd;
	uint64_t state_tx_gb;
	uint64_t state_tx_wr;
	uint64_t state_tx_wb;
	uint64_t state_rx_rd;
	uint64_t state_rx_gd;
	uint64_t state_rx_wr;
	uint64_t state_rx_wb;
	uint64_t state_swr;
	uint64_t state_ra;
	uint64_t state_ra_pl;
	uint64_t hw_version;
	uint64_t revise;
	uint64_t mac_config;
	uint64_t mac_frame_filter;
	uint64_t phy_addr;
	uint64_t phy_data;
	uint64_t control_frm;
	uint64_t vlan_tag;
	uint64_t mac_version;
	uint64_t power_management;
	uint64_t mac_intr_stat;
	uint64_t mac_intr_mask;
	uint64_t mac_addr_0_high;
	uint64_t mac_addr_0_low;
	uint64_t rgmii_stat;
	uint64_t mmc_control;
	uint64_t mmc_intr_rx;
	uint64_t mmc_intr_tx;
	uint64_t mmc_intr_mask_rx;
	uint64_t mmc_intr_mask_tx;
#endif

	/* HW Statistics */
	uint64_t txoctetcount_gb;
	uint64_t txframecount_gb;
	uint64_t txbroadcastframes_g;
	uint64_t txmulticastframes_g;
	uint64_t tx64octets_gb;
	uint64_t tx65to127octets_gb;
	uint64_t tx128to255octets_gb;
	uint64_t tx256to511octets_gb;
	uint64_t tx512to1023octets_gb;
	uint64_t tx1024tomaxoctets_gb;
	uint64_t txunicastframes_gb;
	uint64_t txmulticastframes_gb;
	uint64_t txbroadcastframes_gb;
	uint64_t txunderflowerror;
	uint64_t txsinglecol_g;
	uint64_t txmulticol_g;
	uint64_t txdeferred;
	uint64_t txlatecol;
	uint64_t txexesscol;
	uint64_t txcarriererror;
	uint64_t txoctetcount_g;
	uint64_t txframecount_g;
	uint64_t txexcessdef;
	uint64_t txpauseframes;
	uint64_t txvlanframes_g;
	uint64_t rxframecount_gb;
	uint64_t rxoctetcount_gb;
	uint64_t rxoctetcount_g;
	uint64_t rxbroadcastframes_g;
	uint64_t rxmulticastframes_g;
	uint64_t rxcrcerror;
	uint64_t rxalignmenterror;
	uint64_t rxrunterror;
	uint64_t rxjabbererror;
	uint64_t rxundersize_g;
	uint64_t rxoversize_g;
	uint64_t rx64octets_gb;
	uint64_t rx65to127octets_gb;
	uint64_t rx128to255octets_gb;
	uint64_t rx256to511octets_gb;
	uint64_t rx512to1023octets_gb;
	uint64_t rx1024tomaxoctets_gb;
	uint64_t rxunicastframes_g;
	uint64_t rxlengtherror;
	uint64_t rxoutofrangetype;
	uint64_t rxpauseframes;
	uint64_t rxfifooverflow;
	uint64_t rxvlanframes_gb;
	uint64_t rxwatchdogerror;
	uint64_t rxipv4_gd_frms;
	uint64_t rxipv4_hdrerr_frms;
	uint64_t rxipv4_nopay_frms;
	uint64_t rxipv4_frag_frms;
	uint64_t rxipv4_udsbl_frms;
	uint64_t rxipv6_gd_frms;
	uint64_t rxipv6_hdrerr_frms;
	uint64_t rxipv6_nopay_frms;
	uint64_t rxudp_gd_frms;
	uint64_t rxudp_err_frms;
	uint64_t rxtcp_gd_frms;
	uint64_t rxtcp_err_frms;
	uint64_t rxicmp_gd_frms;
	uint64_t rxicmp_err_frms;
	uint64_t rxipv4_gd_octets;
	uint64_t rxipv4_hdrerr_octets;
	uint64_t rxipv4_nopay_octets;
	uint64_t rxipv4_frag_octets;
	uint64_t rxipv4_udsel_octets;
	uint64_t rxipv6_gd_octets;
	uint64_t rxipv6_hdrerr_octets;
	uint64_t rxipv6_nopay_octets;
	uint64_t rxudp_gd_octets;
	uint64_t rxudp_err_octets;
	uint64_t rxtcp_gd_octets;
	uint64_t rxtcp_err_octets;
	uint64_t rxicmp_gd_octets;
	uint64_t rxicmp_err_octets;
};


/* interface for phy vendor specific functions */
struct smac_phy_ops {
	int (*phy_vendor_init)(struct smac_adapter *adapter);
	void (*get_speed_and_duplex)(struct smac_adapter *adapter, u16 *speed, u16 *duplex);
	bool (*link_is_up)(struct smac_adapter *adapter);
	void (*assert_crs)(struct smac_adapter *adapter);
	bool (*restart_autoneg)(struct smac_adapter *adapter);
	bool (*force_speed_duplex)(struct smac_adapter *adapter);
	bool (*is_mdix)(struct smac_adapter *adapter);
	bool (*set_mdix)(struct smac_adapter *adapter);
};


/* private data structure */
struct smac_adapter {
	const char* driver_name;		/* driver short name */
	const char* driver_version;		/* driver version string */

	struct timer_list watchdog_timer;	/* timer for check link status */
	uint32_t rx_buffer_len;			/* length of rx buffer */
	uint16_t link_speed;			/* transfer speed */
	uint16_t link_duplex;			/* duplex */
	unsigned int irq;			/* IRQ No. */
	spinlock_t stats_lock;			/* spinlock for net_stats */
	spinlock_t phy_lock;			/* spinlock for PHY access */

	/* tx */
	struct smac_tx_ring *tx_ring;		/* tx descriptor */
	unsigned int restart_queue;		/* # of call netif_wake_queue() */
	uint32_t tx_timeout_count;		/* # of reset tx */
	struct timer_list clean_tx_timer;	/* timer for cleanning tx desc */

	/* rx */
	struct smac_rx_ring *rx_ring;		/* rx descriptor */
	bool rx_csum;				/* checksum offload enable/disable */

	/* NAPI */
	struct napi_struct napi;

	/* statistics */
	struct smac_hw_stats stats;		/* SMAC H/W register's value */
	unsigned int total_tx_bytes;		/* transfered bytes */
	unsigned int total_tx_packets;		/* transfered packets */
	unsigned int total_rx_bytes;		/* received bytes */
	unsigned int total_rx_packets;		/* received packets */
	struct work_struct reset_task;		/* work queue for reset */
	uint32_t hw_csum_err;			/* # of checksum error */
	uint32_t hw_csum_good;			/* # of checksum good */
	uint32_t alloc_rx_buff_failed;		/* # of failed allocate rx buffer */

#ifdef CONFIG_SMAC_ENABLE_RX_DELAY_TIMER
	/* Receive Interrupt Delay */
	uint32_t rx_int_delay;			/* Receive Interrupt Delay */
	uint32_t rx_abs_int_delay;		/* Receive Absolute Interrupt Delay */
#endif

	/* Interrupt Throttle Number */
	uint32_t itnr;				/* value of setting SMAC_RX_FRAME_COUNT_LIMIT */
	uint32_t itnr_setting;			/* InterruptThrottleNumbeR parameter */
	enum smac_latency_range current_itnr;	/* current latency range */

	/* SMAC HW */
	uint8_t *hw_addr;			/* SMAC register base virtual address */
	unsigned long hw_addr_phys;		/* SMAC register base physical address */
	resource_size_t hw_size;		/* size of SMAC register */

	/* descriptor */
	uint8_t *tx_desc_addr;			/* tx descriptor base virtual address */
	uint8_t *rx_desc_addr;			/* rx descriptor base virtual address */
	dma_addr_t tx_desc_addr_phys;		/* tx descriptor base physical address */
	dma_addr_t rx_desc_addr_phys;		/* rx descriptor base physical address */
	u32 tx_desc_size;			/* tx descriptor size (16byte * number of descriptors) */
	u32 rx_desc_size;			/* rx descriptor size (16byte * number of descriptors) */

	uint8_t mac_addr[6];			/* MAC address */

	/* auto-nego */
	uint8_t forced_speed_duplex;		/* force speed and duplex */
	bool autoneg;				/* auto-nego enable/disable */
	uint16_t autoneg_advertised;		/* auto-neg advertised */

	/* Auto MDI/MDI-X */
	uint8_t mdix;				/* Auto MDI/MDI-X */

	/* flow control */
	enum smac_fc_type fc;			/* flow control */
	enum smac_fc_type original_fc;		/* flow control (backup) */
	bool fc_set_done;			/* done flow control setting (true=set register, false=set phy only) */
	bool fc_autoneg;			/* flow control auto-nego enable/disable */

	/* OS defined structs */
	struct net_device *netdev;		/* struct netdev */

	/* smac_platform_data */
	struct smac_platform_data *platform_data;	/* platform_device.dev.platform_data */
	u8 phy_addr;				/* PHY Address */
	phy_interface_t phy_if;			/* PHY interface type */
	u8 mcr;					/* MDC Clock Range */
	u8 burst_length;			/* burst length for AHB bus */
	enum smac_phy_vendor phy_vendor;	/* phy vendor */

	/* for PHY dependent */
	struct smac_phy_ops phy_ops;		/* interface for phy vendor specific functions */

	unsigned long dummy_skb_virt;		/* dummy skb for hw reset */
	int msg_enable;				/* network interface message level */
	unsigned long flags;			/* status flag: assign smac_state_t bit */
};


/*------------------------------------------------------------------------------
  Inline Function
  ------------------------------------------------------------------------------*/
/* return true if MII or RMII */
static inline bool smac_phy_is_mii(struct smac_adapter *adapter)
{
	return (adapter->phy_if == PHY_INTERFACE_MODE_MII || adapter->phy_if == PHY_INTERFACE_MODE_RMII);
}


/* return true if GMII or RGMII */
static inline bool smac_phy_is_gmii(struct smac_adapter *adapter)
{
	return (adapter->phy_if == PHY_INTERFACE_MODE_GMII || adapter->phy_if == PHY_INTERFACE_MODE_RGMII);
}


/* return true if mach PHY vendor */
static inline bool smac_mach_phy_vendor(struct smac_adapter *adapter,
					enum smac_phy_vendor vendor)
{
	return adapter->phy_vendor == vendor;
}


/* return true if get __SMAC_RESETTING bit */
static inline bool smac_set_resetting_bit(struct smac_adapter *adapter)
{
	int i;

	/* try 5sec. */
	for (i = 5000; i > 0; i--) {
		if (!test_and_set_bit(__SMAC_RESETTING, &adapter->flags))
			return true;
		msleep(1);
	}

	WARN(1, SMAC_PFX "Timeout Resetting!!\n");
	return false;
}


/* clear __SMAC_RESETTING bit */
static inline void smac_clear_resetting_bit(struct smac_adapter *adapter)
{
	clear_bit(__SMAC_RESETTING, &adapter->flags);
}


/*------------------------------------------------------------------------------
  Function Prototype
  ------------------------------------------------------------------------------*/
/* smac_ethtool.c */
void smac_set_ethtool_ops(struct net_device *netdev);

/* smac_io.c */
int smac_map_base_address(struct smac_adapter *adapter);
void smac_unmap_base_address(struct smac_adapter *adapter);
bool smac_wait_phy_reset(struct smac_adapter *adapter, u32 timesec);
bool smac_wait_phy_autoneg(struct smac_adapter *adapter, u32 timesec);
bool smac_phy_autoneg_ability(struct smac_adapter *adapter);
void smac_set_phy_speed_and_duplex(struct smac_adapter *adapter, u16* control);
void smac_read_mac_addr(unsigned char* mac_addr);
void smac_write_mac_addr(u8 *addr, u32 index);
void smac_force_mac_fc(struct smac_adapter *adapter);
bool smac_config_fc_after_link_up(struct smac_adapter *adapter);
bool smac_setup_link(struct smac_adapter *adapter);
void smac_update_hw_stats(struct smac_adapter *adapter);
uint32_t smac_readreg(unsigned long adr);
void smac_writereg(unsigned long adr, uint32_t data);
uint16_t smac_readphy(struct smac_adapter *adapter, uint16_t reg);
void smac_writephy(struct smac_adapter *adapter, uint16_t reg, uint16_t data);
uint32_t smac_read_desc(void* addr);
void smac_write_desc(uint32_t val, void* addr);

/* smac_param.c */
void __devinit smac_check_options(struct smac_adapter *adapter);

/* smac_main.c */
void smac_reinit_locked(struct smac_adapter *adapter);
int smac_up(struct smac_adapter *adapter);
void smac_down(struct smac_adapter *adapter);
void smac_reset(struct smac_adapter *adapter);
void smac_update_stats(struct smac_adapter *adapter);

#endif /* __SMAC_LOCAL_H__ */
