/* 2011-11-02: File added and changed by Sony Corporation */
/*
 *  Copyright 2008-2011 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */

/* smac_phy_vendor.c - functions for PHY vendor specific */

#include "smac_local.h"
#include "smac_phy_vendor.h"

/*----------------------------------------------------------------------------*/
/* smac_phy_vendor_init - initialize phy vendor specific register */
int smac_phy_vendor_init(struct smac_adapter *adapter)
{
	struct smac_phy_ops *phy_ops = &adapter->phy_ops;
	int ret = 0;

	if (adapter->phy_vendor == SMAC_MARVELL_88E1111)
		smac_marvell_phy_setup(phy_ops);
	else if (adapter->phy_vendor == SMAC_MICREL_KSZ8051MNL)
		smac_micrel_phy_setup(phy_ops);

	if (phy_ops->phy_vendor_init)
		ret = phy_ops->phy_vendor_init(adapter);

	return ret;
}


/*----------------------------------------------------------------------------*/
/* smac_get_speed_and_duplex - get speed and duplex from PHY */
void smac_get_speed_and_duplex(struct smac_adapter *adapter,
				uint16_t *speed, uint16_t *duplex)
{
	struct smac_phy_ops *phy_ops = &adapter->phy_ops;

	if (phy_ops->get_speed_and_duplex)
		phy_ops->get_speed_and_duplex(adapter, speed, duplex);
}


/*----------------------------------------------------------------------------*/
/* smac_link_is_up - check link up */
bool smac_link_is_up(struct smac_adapter *adapter)
{
	struct smac_phy_ops *phy_ops = &adapter->phy_ops;
	bool ret = false;

	if (phy_ops->link_is_up)
		ret = phy_ops->link_is_up(adapter);

	return ret;
}


/*----------------------------------------------------------------------------*/
/* smac_phy_is_mdix - check MDI/MDI-X */
bool smac_phy_is_mdix(struct smac_adapter *adapter)
{
	struct smac_phy_ops *phy_ops = &adapter->phy_ops;
	bool ret = false;

	if (phy_ops->is_mdix)
		ret = phy_ops->is_mdix(adapter);

	return ret;
}


/*----------------------------------------------------------------------------*/
/* smac_assert_crs - assert carrier sense on transmit if half duplex */
void smac_assert_crs(struct smac_adapter *adapter)
{
	struct smac_phy_ops *phy_ops = &adapter->phy_ops;

	if (phy_ops->assert_crs)
		phy_ops->assert_crs(adapter);
}


/*----------------------------------------------------------------------------*/
/* smac_restart_autoneg - restart phy auto-neg */
bool smac_restart_autoneg(struct smac_adapter *adapter)
{
	struct smac_phy_ops *phy_ops = &adapter->phy_ops;
	bool ret = false;

	if (phy_ops->restart_autoneg)
		ret = phy_ops->restart_autoneg(adapter);

	return ret;
}


/*----------------------------------------------------------------------------*/
/* smac_set_force_speed_duplex - set phy speed and duplex (disable auto-neg) */
bool smac_set_force_speed_duplex(struct smac_adapter *adapter)
{
	struct smac_phy_ops *phy_ops = &adapter->phy_ops;
	bool ret = false;

	if (phy_ops->force_speed_duplex)
		ret = phy_ops->force_speed_duplex(adapter);

	return ret;
}

/*----------------------------------------------------------------------------*/
/* smac_phy_set_mdix - set MDI-X Auto/on/off */
bool smac_phy_set_mdix(struct smac_adapter *adapter)
{
	struct smac_phy_ops *phy_ops = &adapter->phy_ops;
	bool ret = false;

	if (phy_ops->set_mdix)
		ret = phy_ops->set_mdix(adapter);

	return ret;
}
