/* 2011-11-02: File added and changed by Sony Corporation */
/*
 *  Copyright 2008-2011 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */

/* smac_sysdep.c - system independent part */

#include <asm/sizes.h>
#include <asm/io.h>
#include <linux/spinlock.h>

#include <mach/xarina_sysreg1.h>

#include "smac_local.h"
#include "smac_sysdep.h"

#define SMAC_RST_CTL			0x30
#define SMAC_RESET_OFF			(1 << 1)

#define SMAC_CLK_CTL			0x34
#define SMAC_CLK_CTL_1000		(0x0 << 12)
#define SMAC_CLK_CTL_100		(0x1 << 12)
#define SMAC_CLK_CTL_10			(0x2 << 12)
#define SMAC_CLK_CTL_RGMII		SMAC_CLK_CTL_1000
#define SMAC_CLK_CTL_MII		(0x3 << 12)
#define SMAC_CLK_CTL_ALL		(0x3 << 12)

#define SMAC_CLK_EN			0x38
#define SMAC_GBE_CKON			(0x1 << 4)
#define SMAC_GBEEXT_EN			(0x1 << 9)
#define SMAC_CLK_EN_RGMII		(SMAC_GBE_CKON | SMAC_GBEEXT_EN)
#define SMAC_CLK_EN_MII			(SMAC_GBE_CKON)

#define SMAC_MODE_SEL			0x40
#define SMAC_GBEMODE_RGMII		(0x1 << 0)
#define SMAC_GBEMODE_MII		(0x0 << 0)


/*----------------------------------------------------------------------------*/
/* please implement system initialize */
int smac_system_init(struct smac_adapter *adapter)
{
	u32 mode_sel;
	u32 clk_ctl;
	u32 clk_en;

	SMAC_DEBUG_FUNC();

	if (smac_phy_is_mii(adapter)) {
		mode_sel	= SMAC_GBEMODE_MII;
		clk_ctl		= SMAC_CLK_CTL_MII;
		clk_en		= SMAC_CLK_EN_MII;
	} else {
		mode_sel	= SMAC_GBEMODE_RGMII;
		clk_ctl		= SMAC_CLK_CTL_RGMII;
		clk_en		= SMAC_CLK_EN_RGMII;
	}

	/* reset GbE */
	xarina_sysreg1_rmwl(SMAC_RST_CTL, 0, SMAC_RESET_OFF);

	udelay(4);

	/* select mode */
	xarina_sysreg1_writel(mode_sel, SMAC_MODE_SEL);

	/* set clock */
	xarina_sysreg1_rmwl(SMAC_CLK_CTL, clk_ctl, SMAC_CLK_CTL_ALL);

	/* enable clock */
	xarina_sysreg1_rmwl(SMAC_CLK_EN, clk_en, (SMAC_GBE_CKON | SMAC_GBEEXT_EN));

	/* clear reset GbE */
	xarina_sysreg1_rmwl(SMAC_RST_CTL, SMAC_RESET_OFF, 0);

	udelay(4);

	return 0;
}


/*----------------------------------------------------------------------------*/
/* please implement system exit */
int smac_system_exit(struct smac_adapter *adapter)
{
	SMAC_DEBUG_FUNC();

	/* nothing */

	return 0;
}


/*----------------------------------------------------------------------------*/
/* please implement hw reset (reset to SMAC HW Block) */
int smac_system_hw_reset(struct smac_adapter *adapter)
{
	SMAC_DEBUG_FUNC();

	/* reset GbE */
	xarina_sysreg1_rmwl(SMAC_RST_CTL, 0, SMAC_RESET_OFF);
	udelay(4);

	/* clear reset GbE */
	xarina_sysreg1_rmwl(SMAC_RST_CTL, SMAC_RESET_OFF, 0);
	udelay(4);

	return 0;
}


/*----------------------------------------------------------------------------*/
/* please implement system clock setting */
int smac_system_set_clock(struct smac_adapter *adapter)
{
	u32 val;

	SMAC_DEBUG_FUNC();

	if (smac_phy_is_gmii(adapter)) {
		/* GMII */
		switch (adapter->link_speed) {
		case SPEED_10:
			val = SMAC_CLK_CTL_10;
			break;
		case SPEED_100:
			val = SMAC_CLK_CTL_100;
			break;
		case SPEED_1000:
		default:
			val = SMAC_CLK_CTL_1000;
			break;
		}
	} else {
		/* MII */
		val = SMAC_CLK_CTL_MII;
	}

	/* set clock */
	xarina_sysreg1_rmwl(SMAC_CLK_CTL, val, SMAC_CLK_CTL_ALL);

	return 0;
}

