#ifndef LINUX_MMC_IOCTL_H
#define LINUX_MMC_IOCTL_H

struct mmc_ioc_cmd {
    unsigned int is_acmd;          /* Application-specific command.  true = precede with CMD55 */
    unsigned int write_flag;       /* implies direction of data.  true = write, false = read */
    unsigned int opcode;
    unsigned int arg;
    unsigned int flags;
    unsigned int blksz;
    unsigned int blocks;

    /*
     * Sleep postsleep_us useconds *after* issuing command.  Needed for
     * some read commands for which cards have no other way of indicating
     * they're ready for the next command (i.e. there is no equivalent of
     * a "busy" indicator for read operations).
     */
    unsigned int postsleep_us;

    /*
     * Override driver-computed timeouts.  Note the difference in units!
     */
    unsigned int data_timeout_ns;
    unsigned int cmd_timeout_ms;

    unsigned int response[4];  /* CMD response */
    unsigned char *data;       /* DAT buffer */
};

#define CMD_ARRAY_SIZE_MAX (50)
struct mmc_ioc_cmd_sequence {
    unsigned int cmd_count;
    struct mmc_ioc_cmd *cmd_array;
};

#define MMC_IOC_CMD  (0xc040b300)  /* this magic number is generated by _IOWR(MMC_BLOCK_MAJOR, 0, struct mmc_ioc_cmd) */
/*
 * Since this ioctl is only meant to enhance (and not replace) normal access
 * to the mmc bus device, an upper data transfer limit of MMC_IOC_MAX_BYTES
 * is enforced per ioctl call.  For larger data transfers, use the normal
 * block device operations.
 */
#define MMC_IOC_MAX_BYTES  (512L * 256)

#endif  /* LINUX_MMC_IOCTL_H */
