#!/usr/local/bin/python

######################################################################
#
#
# Copyright 2012,2013 Sony Corporation.
#
#
######################################################################

"""LCTracer viewer"""

import os
import sys
import re
import svgwrite
import struct

SCALE = 100
SVG_WIDTH = 800
SVG_HEIGHT = 1200
BASE_HEIGHT = 40
IOWAIT_HEIGHT = 220
ROW_HEIGHT = 20
NAME_WIDTH = 160
TGID_WIDTH = 40
PID_WIDTH = 40
PRIO_WIDTH = 40
CPU_RATIO_WIDTH = 100
BASE_WIDTH = NAME_WIDTH + TGID_WIDTH + PID_WIDTH +\
    PRIO_WIDTH + CPU_RATIO_WIDTH + 10

START_TIME = 0
END_TIME = 0
RATE = 1
THREADS_LIST = {}
IRQS_LIST = {}
USERINFO_LIST = {}
TOTAL = 0
UNIT = 100
BINARY_DATA_LEN = 0
DATA_COUNT = 0

TIME_UNIT = 1000000
IRQ_DATA_UNIT = 1000000

THREAD_INFO = 0
IRQ_INFO = 1
USER_INFO = 2
DATA_FMAT_TIME = 0
DATA_FMAT_DATA = 1
DATA_FMAT_OFFSET = 2
DATA_FMAT_TGID = 3
DATA_FMAT_PPID = 4
DATA_FMAT_NPID = 5
DATA_FMAT_NAME = 6
DATA_FMAT_TNAME = 7

IOWAIT_TIME_DATA = {}

PROCESS_DATA_SIZE = 64
BINARY_DATA = []

SHOW_TIME_INFO_FLAG = 0
SHOW_TID_IRQ_INFO_FLAG = 0
SHOW_FUNC_INFO_FLAG = 0
OUTPUT_FILE_FLAG = -1

PID_TGID_LIST = {}
PID_TID_FUNC_LIST = {}
PID_PRIO_LIST = {}
PRIO_PID_LIST = []
TID_FUNC_INDEX = 2

SHOW_ALL_TID_MODE = 0
SHOW_ALL_IRQ_MODE = 0
SHOW_ALL_TGID_MODE = 0

IRQ_INPUT_LIST = []
TID_INPUT_LIST = []
TGID_INPUT_LIST = []

VERSION = ''
HZ = 0
MS_PER_SEC = 1000
MS_PER_JIFFIES = 0
ENDIAN_INFO = ''
TITLE_NAME = ''
SYSTEM_UNAME = ''
SYSTEM_CPU = ''
BUFFER_SIZE = ''
THRESHOLD = ''
AUTOSTART = ''
STORE_TO_FILESYSTEM = ''
IOWAIT_SAMPLE_INTERVAL = ''
SAVE_PATH = ''
BUFFER_ADDRESS = ''

TASK_STATE_BITMASK = {
    0: 'TASK_RUNNING', 1: 'TASK_INTERRUPTIBLE', 2: 'TASK_UNINTERRUPTIBLE'}

EARNING_MSG = '''[-b t1] [-e t2] [-i interval] [-s tid,irq,tgid]
     [-o svg,SVG,txt] [-t tid,irq,all] [-w] input output
'''

BEGINNING_TIME_USAGE = '''
-b t1: Specify the beginning time shown in chart.
'''

END_TIME_USAGE = '''
-e t2: Specify the end time shown in chart.
'''

INTERVAL_TIME_USAGE = '''
-i: Specify the interval time(unit is millisecond) of time line shown in chart.
'''

SHOW_IRQ_TID_USAGE = '''
-s tid,irq,tgid: It means show thread or/and irq information.
'''

SHOW_IIME_USAGE = '''
-t: It means show time information for specified thread or/and irq by
"-s" option.
'''
SHOW_WCHAN_USAGE = '''
-w: It means show wchan information for each thread in chart, including
    function name, address and offset, which the thread went to sleep.
'''
INPUT_USAGE = '''
input: Input is a directory name, which should include
    - lct_lctracer.dat:        measurement data file for thread and irq.
    - lct_iowait.log:          system iowait information.
    - lct_thread_table.log:    thread name and id table for application.
    - lct_measurement_env.log: kernel environment setting's information.
'''
OUTPUT_USAGE = '''
output: Specify the name of output, whose format is decided by "-o" option.
'''

OUTPUT_FILE_USAGE = '''
-o: Specify the output file type
    -o svg, output is svg format as Output_horizontal.svg
    -o SVG, output is svg format as Output_in_2_lines.svg
    -o txt, output is txt format
    If this option is not specified, it will be identical with "-o svg"
'''

CSS = '''
.Title {
    font-family: sans-serif;
    font-size: 14pt;
    font-weight: bold;
    fill: #000000;
}
.Title_stat {
    font-family: sans-serif;
    font-size: 10pt;
    font-weight: bold;
    fill: #000000;
}
.ProcessLabel {
    font-family: sans-serif;
    font-size: 9pt;
    fill: #000000;
}
.LegendTextBold {
    font-family: sans-serif;
    font-size: 9pt;
    font-weight: bold;
    fill: #000000;
}
.LegendText {
    font-family: sans-serif;
    font-size: 9pt;
    fill: #000000;
}
.Back {
    fill: white;
}
.Tick {
    stroke: #e6e6e6;
    stroke-width: 1px;
}
.AxisLabel {
    font-family: sans-serif;
    font-size: 9pt;
    fill: #000000;
    opacity: 0.7;
}
.Bold {
    stroke: #d0d0d0;
}

.CPU {
    fill-opacity: 0.25;
    fill: #668cb2;
    stroke: #668cb2;
}

.IO {
    fill-opacity: 0.25;
    fill: #ffff00;
    stroke: #ffff00;
}
.THRD1 {
    fill-opacity: 0.25;
    fill:   #6db266;
    stroke: #6db266;
}
.IRQ2 {
    fill-opacity: 0.5;
    fill:   #66b2b0;
    stroke: #66b2b0;
}
.THREAD{
    fill-opacity: 0.5;
    fill:   #ff0000;
    stroke: #ff0000;
}
.IRQ {
    fill-opacity: 0.5;
    fill:   #ff0000;
    stroke: #ff0000;
}
.IRQ1 {
    fill-opacity: 0.5;
    fill:   #00fa00;
    stroke: #00fa00;
}
.THRD {
    fill-opacity: 0.5;
    fill:   #8b008b;
    stroke: #8b008b;
}
.USER{
    fill-opacity: 0.5;
    fill:   #0000fb;
    stroke: #0000fb;
}
.IDLE{
    fill-opacity: 0.5;
    fill:   #ffff00;
    stroke: #66b2b0;
}
.Process {
    stroke: #668cb2;
    fill: none;
}
.ProcessStatus {
    stroke: none;
}
.Sleeping {
    fill: #fff200;
    fill-opacity: 0.25;
}
.Running {
    fill: #668cb2;
    fill-opacity: 0.25;
}
'''


def get_time(string_usage, strtime):
    """This function is used to change string to time"""

    time = 0
    if re.search(r'ms', strtime):
        beginning_time = re.sub(r'ms', '', strtime)
        try:
            time = float(beginning_time)
        except ValueError:
            print ValueError
            print 'Error: the %s time is invalid!' % string_usage
            sys.exit(ValueError)
    elif re.search(r's', strtime):
        beginning_time = re.sub(r's', '', strtime)
        try:
            time = float(beginning_time) * 1000
        except ValueError:
            print ValueError
            print 'Error: the %s time is invalid!' % string_usage
            sys.exit(ValueError)
    else:
        print 'Error: the %s time is invalid!' % string_usage
        sys.exit(ValueError)

    return time


def parse_parameter(arg_num, arg_value):
    """This function is used to parse parameter"""

    global START_TIME
    global END_TIME
    global UNIT
    global OUTPUT_FILE_FLAG
    global SHOW_TIME_INFO_FLAG
    global SHOW_TID_IRQ_INFO_FLAG
    global SHOW_FUNC_INFO_FLAG
    global TID_INPUT_LIST
    global TGID_INPUT_LIST
    global IRQ_INPUT_LIST
    global SHOW_ALL_TID_MODE
    global SHOW_ALL_IRQ_MODE
    global SHOW_ALL_TGID_MODE

    i = 0
    for i in range(i, arg_num):
        if arg_value[i] == '-b':
            START_TIME = get_time("beginning", arg_value[i + 1])
            if START_TIME < 0:
                print 'Error: beginning time is invalid!'
                sys.exit(ValueError)
        elif arg_value[i] == '-e':
            END_TIME = get_time("end", arg_value[i + 1])
            if END_TIME <= 0:
                print 'Error: end time is invalid!'
                sys.exit(ValueError)

        elif arg_value[i] == '-i':
            try:
                UNIT = float(arg_value[i + 1])
            except ValueError:
                print ValueError
                print 'Error: specify the interval time(UNIT is millisecond)!'
                print 'Usage:\n%s' % INTERVAL_TIME_USAGE
                sys.exit(ValueError)
            if UNIT < 0:
                print 'Error: specify the interval time(UNIT is millisecond)!'
                sys.exit(ValueError)

        elif arg_value[i] == '-s':
            tmp_list = arg_value[i + 1].split(',')
            error_parameter = 0
            if re.search(r'tid', arg_value[i + 1]):
                error_parameter = 1
                TID_INPUT_LIST = get_tid_irq_tgid_list('tid', tmp_list)
                if len(TID_INPUT_LIST) > 0:
                    SHOW_ALL_TID_MODE = 2
            if re.search(r'irq', arg_value[i + 1]):
                error_parameter = 1
                IRQ_INPUT_LIST = get_tid_irq_tgid_list('irq', tmp_list)
                if len(IRQ_INPUT_LIST) > 0:
                    SHOW_ALL_IRQ_MODE = 2
            if re.search(r'tgid', arg_value[i + 1]):
                error_parameter = 1
                TGID_INPUT_LIST = get_tid_irq_tgid_list('tgid', tmp_list)
                if len(TGID_INPUT_LIST) > 0:
                    SHOW_ALL_TGID_MODE = 2
            if error_parameter == 0:
                print 'Error: -s option is invalid!'
                sys.exit(ValueError)
            SHOW_TID_IRQ_INFO_FLAG = 1
        elif arg_value[i] == '-t':
            SHOW_TIME_INFO_FLAG = 1
        elif arg_value[i] == '-w':
            SHOW_FUNC_INFO_FLAG = 1
        elif arg_value[i] == '-o':
            if 'txt' == arg_value[i + 1]:
                OUTPUT_FILE_FLAG = 0
            elif 'svg' == arg_value[i + 1]:
                OUTPUT_FILE_FLAG = 1
            elif 'SVG' == arg_value[i + 1]:
                OUTPUT_FILE_FLAG = 2
            else:
                print 'Error: unknown type of output!'
                print 'Usage:\n %s' % OUTPUT_FILE_USAGE
                sys.exit(ValueError)
        elif arg_value[i] == '-h' or arg_value[i] == '--help' or arg_num < 3:
            print 'Usage: %s %s%s%s%s%s%s%s%s%s%s' % (
                arg_value[0], EARNING_MSG, BEGINNING_TIME_USAGE,
                END_TIME_USAGE, SHOW_IRQ_TID_USAGE,
                INTERVAL_TIME_USAGE, SHOW_IIME_USAGE,
                OUTPUT_FILE_USAGE, SHOW_WCHAN_USAGE, OUTPUT_USAGE,
                INPUT_USAGE)
            sys.exit(0)

        i = i + 1
    if OUTPUT_FILE_FLAG < 0:
        OUTPUT_FILE_FLAG = 1

    if START_TIME != 0 and END_TIME != 0 and START_TIME >= END_TIME:
        print 'Error: the beginning time is later than end time.'
        sys.exit(ValueError)


def get_tid_irq_tgid_list(string, input_list):
    """This function is used to get tig, tgid, irq through -s option"""

    tid_irq_list = []
    tgid_flag = 0
    global SHOW_ALL_TID_MODE
    global SHOW_ALL_IRQ_MODE
    global SHOW_ALL_TGID_MODE

    for item in input_list:
        if tgid_flag == 1:
            if re.search(r'[a-z]', item):
                break
            tid_irq_list.append(int(item))

        if re.search(string, item):
            tgid_flag = 1
            tmpid = item.split('=')
            if len(tmpid) == 1:
                if string == 'irq':
                    SHOW_ALL_IRQ_MODE = 1
                elif string == 'tid':
                    SHOW_ALL_TID_MODE = 1
                else:
                    SHOW_ALL_TGID_MODE = 1
                break
            else:
                try:
                    input_item = int(tmpid[1])
                    if input_item < 0:
                        print 'Error: -s option is invalid!'
                        print '%s is negative' % string
                        sys.exit(ValueError)
                    tid_irq_list.append(int(tmpid[1]))
                except ValueError:
                    print ValueError
                    print 'Error: -s option is invalid!'
                    print SHOW_IRQ_TID_USAGE
                    sys.exit(ValueError)
    return tid_irq_list


def get_iowait_data(iowait_log):
    """This function is used to get iowait information through iowait log"""

    if not os.path.isfile(iowait_log):
        print 'Warning: File %s does not exist!' % iowait_log
        return

    file_obj = open(iowait_log, 'r')

    time = 0
    lines = file_obj.readlines()
    while len(lines) > 0:
        line = lines.pop(0)
        if re.search(r'^\[', line):
            try:
                time = float(line.split()[1])
            except ValueError:
                print ValueError
            line = lines.pop(0)
            data = line.split()
            # The fifth value is iowait
            if len(data) > 5:
                IOWAIT_TIME_DATA.update({time: data[5]})
            else:
                print 'Warning: no iowait information at %s Sec!' % time

    file_obj.close()

    return IOWAIT_TIME_DATA


def get_binary_data(lct_lctracer_dat):
    """This function is used get binary through lct_lctracer.dat"""

    if not os.path.isfile(lct_lctracer_dat):
        print 'Error: File %s does not exist!' % lct_lctracer_dat
        sys.exit(IOError)

    global BINARY_DATA
    global BINARY_DATA_LEN
    global THREADS_LIST
    global IRQS_LIST
    global USERINFO_LIST

    irqs = []
    threads = []
    userinfo = []
    BINARY_DATA = []

    file_desc = open(lct_lctracer_dat, 'rb')
    while file_desc is not None:
        tmpdata = file_desc.read(PROCESS_DATA_SIZE)
        if len(tmpdata) < PROCESS_DATA_SIZE:
            break
        #struct trace_entry {
        #    u64 time;
        #    u32 data;
        #    u32 user:1;
        #    u32 policy:3;
        #    u32 offset:12;
        #    u32 prio:8;
        #    u32 state:8;
        #    s16 tgid;
        #    s16 ppid;
        #    s16 npid;
        #    char wchan[26];
        #    char tname[16];
        #} __attribute__((__packed__));
        if ENDIAN_INFO == 'little endian':
            tmp_data = struct.unpack('<1Q2I3h26s16s', tmpdata)
        else:
            tmp_data = struct.unpack('>1Q2I3h26s16s', tmpdata)
        data = list(tmp_data)
        if ((data[DATA_FMAT_OFFSET] >> 30) & 0x3) == IRQ_INFO:
            irqs.append(data)
        elif ((data[DATA_FMAT_OFFSET] >> 30) & 0x3) == THREAD_INFO:
            threads.append(data)
        else:
            userinfo.append(data)
        BINARY_DATA_LEN += 1
        BINARY_DATA.append(data)
    file_desc.close()

    THREADS_LIST = get_thread_list(threads)

    IRQS_LIST = get_irqs_list(irqs)

    USERINFO_LIST = get_useinfo_list(userinfo)

    parse_binary_data()

    return (THREADS_LIST, IRQS_LIST)


def get_useinfo_list(userinfo):
    """This function is used to get user entry information"""

    userlenth = len(userinfo)
    if userlenth <= 0:
        return

    userlist = {}
    i = 0
    for i in range(i, userlenth):
        ppid = userinfo[i][DATA_FMAT_PPID]
        ppid_user_list = []
        if ppid in userlist.keys():
            ppid_user_list = userlist[ppid]
        ppid_user_list.append(userinfo[i])
        userlist.update({ppid: ppid_user_list})

    return userlist


def get_header_info(header_file_name):
    """This function is used to get header information"""

    if not os.path.isfile(header_file_name):
        print 'Error: File %s does not exist!' % header_file_name
        sys.exit(IOError)

    header_fd = open(header_file_name, 'r')

    global VERSION
    global HZ
    global MS_PER_JIFFIES
    global ENDIAN_INFO
    global TITLE_NAME
    global SYSTEM_UNAME
    global SYSTEM_CPU
    global BUFFER_SIZE
    global THRESHOLD
    global AUTOSTART
    global STORE_TO_FILESYSTEM
    global IOWAIT_SAMPLE_INTERVAL
    global SAVE_PATH
    global BUFFER_ADDRESS

    for line in header_fd.readlines():
        if re.search(r'Linux', line):
            SYSTEM_UNAME = line.split('\n')[0]
        elif re.search(r'version', line):
            VERSION = (line.split(':')[1]).strip().split('.')
        elif re.search(r'Endian', line):
            ENDIAN_INFO = (line.split(':')[1]).strip()
        elif re.search(r'HZ', line):
            HZ = int((line.split(':')[1]).strip())
            MS_PER_JIFFIES = MS_PER_SEC / HZ
        elif re.search(r'system type', line) or re.search(r'Hardware', line):
            TITLE_NAME = line.split(':')[1]
        elif re.search(r'cpu model', line) or re.search(r'Processor', line):
            SYSTEM_CPU = line.split(':')[1]
        elif re.search(r'buffer size', line):
            BUFFER_SIZE = line.split(':')[1]
        elif re.search(r'threshold', line):
            THRESHOLD = line.split(':')[1]
        elif re.search(r'autostart', line):
            AUTOSTART = line.split(':')[1]
        elif re.search(r'storing to filesystem', line):
            STORE_TO_FILESYSTEM = line.split(':')[1]
        elif re.search(r'iowait', line):
            IOWAIT_SAMPLE_INTERVAL = line.split(':')[1]
        elif re.search(r'saving path', line):
            SAVE_PATH = line.split(':')[1]
        elif re.search(r'buffer address', line):
            BUFFER_ADDRESS = line.split(':')[1]

    header_fd.close()

    return


def get_thread_list(threads):
    """This function is used to get thread information"""

    thread_lenth = len(threads)
    if thread_lenth <= 0:
        print 'Warning: thread data is NULL!'
        return

    global DATA_COUNT
    threadslist = {}
    i = 0
    for i in range(i, thread_lenth):
        ppid = threads[i][DATA_FMAT_PPID]
        ppid_list = []
        if ppid not in threadslist.keys():
            DATA_COUNT += 1
        else:
            ppid_list = threadslist[ppid]
        ppid_list.append(threads[i])
        threadslist.update({ppid: ppid_list})

        npid = threads[i][DATA_FMAT_NPID]
        npid_list = []
        if npid not in threadslist.keys():
            DATA_COUNT += 1
        else:
            npid_list = threadslist[npid]
        npid_list.append(threads[i])
        threadslist.update({npid: npid_list})

    return threadslist


def get_irqs_list(irqs):
    """this function is used to get irq information"""

    irqs_len = len(irqs)
    if irqs_len <= 0:
        print 'Warning: IRQ data is NULL!'
        return

    i = 0
    irqslist = {}
    global DATA_COUNT

    for i in range(i, irqs_len):
        irq_id = irqs[i][DATA_FMAT_DATA] & 0xffff
        tmp_list = []
        if irq_id in irqslist.keys():
            tmp_list = irqslist[irq_id]
        else:
            DATA_COUNT += 1

        tmp_list.append(irqs[i])
        irqslist.update({irq_id: tmp_list})

    return irqslist


def parse_binary_data():
    """This function is used to parse binary data,
    get the svg width and heigh and so on"""

    if DATA_COUNT <= 0:
        print 'Warning: binary data is NULL!'
        return

    global START_TIME
    global END_TIME
    global RATE
    global TOTAL
    global SVG_WIDTH
    global SVG_HEIGHT

    # If does not specify the beginning time, set the first BINARY data time
    if START_TIME == 0:
        START_TIME = float(BINARY_DATA[0][DATA_FMAT_TIME]) / TIME_UNIT
    elif START_TIME < float(BINARY_DATA[0][DATA_FMAT_TIME]) / TIME_UNIT:
        print '''
            Warning: the beginning time is earlier than
            the first data time. Use the first data time.'''
        START_TIME = float(BINARY_DATA[0][DATA_FMAT_TIME]) / TIME_UNIT

    # If does not specify the end time, set the last BINARY data time
    if END_TIME == 0:
        END_TIME = float(
            BINARY_DATA[BINARY_DATA_LEN - 1][DATA_FMAT_TIME]) / TIME_UNIT
    elif END_TIME > float(
            BINARY_DATA[BINARY_DATA_LEN - 1][DATA_FMAT_TIME]) / TIME_UNIT:
        print '''
            Warning: the end time is later than
            the last data time. Use the last data time.'''
        END_TIME = float(
            BINARY_DATA[BINARY_DATA_LEN - 1][DATA_FMAT_TIME]) / TIME_UNIT

    if END_TIME <= START_TIME:
        print 'Error: the end time is earlier than beginning time!'
        sys.exit(ValueError)

    # If TOTAL <= 0: How to set TOTAL?
    TOTAL = int(float(END_TIME - START_TIME) / UNIT)
    if TOTAL <= 0:
        TOTAL = 1

    RATE = SCALE / UNIT

    if OUTPUT_FILE_FLAG == 1:
        SVG_WIDTH = BASE_WIDTH + RATE * (END_TIME - START_TIME) - 8
        SVG_HEIGHT = DATA_COUNT * ROW_HEIGHT + BASE_HEIGHT + IOWAIT_HEIGHT
        TOTAL = int((SVG_WIDTH - BASE_WIDTH) / SCALE)
    elif OUTPUT_FILE_FLAG == 2:
        SVG_WIDTH = RATE * (END_TIME - START_TIME) + NAME_WIDTH
        SVG_HEIGHT = 3 * ROW_HEIGHT + BASE_HEIGHT + IOWAIT_HEIGHT
        TOTAL = int((SVG_WIDTH - NAME_WIDTH) / SCALE)
    return


def parse_log(input_dir):
    """This function is read input directory"""

    if not os.path.exists(input_dir):
        print 'Error: file path %s does not exist!' % input_dir
        sys.exit(IOError)

    get_header_info(input_dir + '/' + 'lct_measurement_env.log')
    get_binary_data(input_dir + '/' + 'lct_lctracer.dat')
    get_iowait_data(input_dir + '/' + 'lct_iowait.log')
    get_thread_name(input_dir + '/' + 'lct_thread_table.log')

    return


def draw_header(dwg, table):
    """This function is used to draw header information to svg"""

    global BASE_HEIGHT

    table_group = table.add(dwg.g(transform='translate(0, -30)'))
    header_row_height = 16
    header_max_width = 160
    table_group.add(dwg.text(
        'LCTracer chart(V%s.%s):%s' % (VERSION[0], VERSION[1], TITLE_NAME),
        dx=[2], dy=[5], class_='Title'))
    table_group.add(dwg.text(
        'system uname: ', dx=[2], dy=[5 + header_row_height],
        class_='LegendText'))
    table_group.add(dwg.text(
        '%s' % SYSTEM_UNAME, dx=[header_max_width],
        dy=[5 + header_row_height], class_='LegendText'))
    table_group.add(dwg.text(
        'cpu: ', dx=[2], dy=[5 + 2 * header_row_height],
        class_='LegendText'))
    table_group.add(dwg.text(
        '%s' % SYSTEM_CPU, dx=[header_max_width],
        dy=[5 + 2 * header_row_height], class_='LegendText'))
    table_group.add(dwg.text(
        'LCTracer parameters state:', dx=[2],
        dy=[5 + 3 * header_row_height], class_='Title_stat'))
    table_group.add(dwg.text(
        'buffer size: ', dx=[2], dy=[5 + 4 * header_row_height],
        class_='LegendText'))
    table_group.add(dwg.text(
        '%s' % BUFFER_SIZE, dx=[header_max_width],
        dy=[5 + 4 * header_row_height], class_='LegendText'))
    table_group.add(dwg.text(
        'thread shold: ', dx=[2], dy=[5 + 5 * header_row_height],
        class_='LegendText'))
    table_group.add(dwg.text(
        '%s' % (THRESHOLD), dx=[header_max_width],
        dy=[5 + 5 * header_row_height], class_='LegendText'))
    table_group.add(dwg.text(
        'autostart: ', dx=[2], dy=[5 + 6 * header_row_height],
        class_='LegendText'))
    table_group.add(dwg.text(
        '%s' % AUTOSTART, dx=[header_max_width],
        dy=[5 + 6 * header_row_height], class_='LegendText'))
    table_group.add(dwg.text(
        'store to filesystem: ',
        dx=[2], dy=[5 + 7 * header_row_height], class_='LegendText'))
    table_group.add(dwg.text('%s' % STORE_TO_FILESYSTEM, dx=[header_max_width],
                    dy=[5 + 7 * header_row_height], class_='LegendText'))
    table_group.add(dwg.text('io wait sample interval: ', dx=[2],
                    dy=[5 + 8 * header_row_height], class_='LegendText'))
    table_group.add(dwg.text(
        '%s' % IOWAIT_SAMPLE_INTERVAL, dx=[header_max_width],
        dy=[5 + 8 * header_row_height], class_='LegendText'))
    table_group.add(dwg.text(
        'saving path: ', dx=[2], dy=[5 + 9 * header_row_height],
        class_='LegendText'))
    table_group.add(dwg.text(
        '%s' % SAVE_PATH, dx=[header_max_width],
        dy=[5 + 9 * header_row_height], class_='LegendText'))
    table_group.add(dwg.text(
        'buffer address: ', dx=[2],
        dy=[5 + 10 * header_row_height], class_='LegendText'))
    table_group.add(dwg.text(
        '%s' % BUFFER_ADDRESS, dx=[header_max_width],
        dy=[5 + 10 * header_row_height], class_='LegendText'))

    BASE_HEIGHT = header_row_height * 11 + ROW_HEIGHT

    table_group.add(dwg.rect(
        insert=(2, BASE_HEIGHT - ROW_HEIGHT), size=(10, 10), class_="IO"))
    table_group.add(dwg.text(
        "I/O (wait)", dx=[18],  dy=[BASE_HEIGHT - 10], class_='LegendText'))
    text_y = BASE_HEIGHT + IOWAIT_HEIGHT + 10
    rect_y = BASE_HEIGHT + IOWAIT_HEIGHT

    if OUTPUT_FILE_FLAG == 1:
        table_group.add(dwg.rect(
            insert=(10, rect_y), size=(10, 10), class_="IRQ"))
        table_group.add(dwg.text(
            "Thread running", dx=[25], dy=[text_y], class_='LegendText'))
        table_group.add(dwg.rect(
            insert=(120, rect_y), size=(10, 10), class_="USER"))
        table_group.add(dwg.text(
            "User log", dx=[135],  dy=[text_y], class_='LegendText'))
    else:
        table_group.add(dwg.rect(
            insert=(10, rect_y), size=(10, 10), class_="THRD"))
        table_group.add(dwg.text(
            "Thread running", dx=[25],  dy=[text_y], class_='LegendText'))
        table_group.add(dwg.rect(
            insert=(150, rect_y), size=(10, 10), class_="THRD1"))
        table_group.add(dwg.text(
            "Thread running", dx=[163], dy=[text_y], class_='LegendText'))
        table_group.add(dwg.rect(
            insert=(300, rect_y), size=(10, 10), class_="IRQ"))
        table_group.add(dwg.text(
            "ISR running", dx=[313], dy=[text_y], class_='LegendText'))
        table_group.add(dwg.rect(
            insert=(450, rect_y), size=(10, 10), class_="IRQ1"))
        table_group.add(dwg.text(
            "ISR running", dx=[463], dy=[text_y], class_='LegendText'))
        table_group.add(dwg.rect(
            insert=(600, rect_y), size=(10, 10), class_="USER"))
        table_group.add(dwg.text(
            "User-defined", dx=[613], dy=[text_y], class_='LegendText'))
    return table_group


def draw_io_wait(dwg, table):
    """This functuion is used to draw io wait information to svg"""

    global MS_PER_JIFFIES

    items = IOWAIT_TIME_DATA.items()
    items.sort()
    iowait_data = []
    for item in items:
        tmp_iowait_list = []
        try:
            time = item[0] * 1000
            start = (time - START_TIME) * RATE + BASE_WIDTH
        except ValueError:
            print ValueError
            return
        if time >= START_TIME and time < END_TIME:
            tmp_iowait_list.append(start)
            tmp_iowait_list.append(item[1])
            iowait_data.append(tmp_iowait_list)
        elif time > END_TIME:
            break

    i = 1
    iowait_points = []
    iowait_data_len = len(iowait_data)
    iowait_hight = []
    iowait_low = []
    for i in range(i, iowait_data_len):
        tmp_hight = []
        tmp_low = []
        y_size_two = int(iowait_data[i][1]) * MS_PER_JIFFIES
        y_size = int(iowait_data[i - 1][1]) * MS_PER_JIFFIES
        y_line = ((
            BASE_HEIGHT + IOWAIT_HEIGHT - ROW_HEIGHT)) -\
            (y_size_two - y_size)

        x_line = iowait_data[i][0]
        if i == 1:
            iowait_hight.append(
                [x_line, (BASE_HEIGHT + IOWAIT_HEIGHT - ROW_HEIGHT)])
        tmp_hight.append(x_line)
        tmp_hight.append(y_line)
        tmp_low.append(x_line)
        tmp_low.append((BASE_HEIGHT + IOWAIT_HEIGHT - ROW_HEIGHT))
        iowait_hight.append(tmp_hight)
        iowait_low.append(tmp_low)

    i = 0
    line = table.add(dwg.g(class_='Tick', transform='translate(0.5, 0.5)'))
    y_size = BASE_HEIGHT - ROW_HEIGHT
    y_size_two = BASE_HEIGHT + IOWAIT_HEIGHT - ROW_HEIGHT
    for i in range(i, TOTAL + 1):
        if OUTPUT_FILE_FLAG == 1:
            line.add(dwg.line(
                start=(BASE_WIDTH + i * SCALE, y_size),
                end=(BASE_WIDTH + i * SCALE, y_size_two), class_='Bold'))
        elif OUTPUT_FILE_FLAG == 2:
            line.add(dwg.line(
                start=(NAME_WIDTH + i * SCALE, y_size),
                end=(NAME_WIDTH + i * SCALE, y_size_two), class_='Bold'))
    i = 0
    for i in range(i, 11):
        if OUTPUT_FILE_FLAG == 1:
            line.add(dwg.line(
                start=(BASE_WIDTH, BASE_HEIGHT + (i - 1) * ROW_HEIGHT),
                end=(SVG_WIDTH + 8, BASE_HEIGHT + (i - 1) * ROW_HEIGHT),
                class_='Bold'))
        elif OUTPUT_FILE_FLAG == 2:
            line.add(dwg.line(
                start=(NAME_WIDTH, BASE_HEIGHT + (i - 1) * ROW_HEIGHT),
                end=(SVG_WIDTH, BASE_HEIGHT + (i - 1) * ROW_HEIGHT),
                class_='Bold'))
        table.add(dwg.text(
            '%sms' % i, class_='AxisLabel', dx=[BASE_WIDTH - 35],
            dy=[BASE_HEIGHT + IOWAIT_HEIGHT - (i + 1) * ROW_HEIGHT]))

    if len(iowait_hight) > 0:
        iowait_points = iowait_hight + sorted(iowait_low, reverse=True)
        pyline = dwg.polyline(class_='IO')

        pyline.points.extend(iowait_points)
        table.add(pyline)

    return


def draw_coordinate(dwg, table):
    """This function is used to draw coordinate to svg"""

    line = table.add(dwg.g(class_='Tick', transform='translate(0.5, 0.5)'))
    line.add(dwg.line(
        start=(BASE_WIDTH, BASE_HEIGHT + IOWAIT_HEIGHT),
        end=(BASE_WIDTH, BASE_HEIGHT + IOWAIT_HEIGHT - 3), class_='Bold'))

    i = 1
    for i in range(i, TOTAL + 1):
        if OUTPUT_FILE_FLAG == 1:
            line.add(dwg.line(
                start=(BASE_WIDTH + i * SCALE, BASE_HEIGHT + IOWAIT_HEIGHT),
                end=(BASE_WIDTH + i * SCALE, BASE_HEIGHT + IOWAIT_HEIGHT - 3),
                class_='Bold'))
        elif OUTPUT_FILE_FLAG == 2:
            line.add(dwg.line(
                start=(BASE_WIDTH + i * SCALE, BASE_HEIGHT + IOWAIT_HEIGHT),
                end=(NAME_WIDTH + i * SCALE, BASE_HEIGHT + IOWAIT_HEIGHT - 3),
                class_='Bold'))

    group = table.add(dwg.g(transform='translate(0, 10)'))
    i = 0
    for i in (range(i, (TOTAL + 1))):
        if i % 5 == 0:
            time = '%0.6f' % (i * UNIT + START_TIME)
            time_len = (len(time) - 1) * 4
            group.add(dwg.text(
                '%sms' % time, class_='AxisLabel',
                dx=[i * SCALE + BASE_WIDTH - time_len],
                dy=[BASE_HEIGHT + IOWAIT_HEIGHT - 15]))

    return


def draw_thread_body(
    dwg, group, key, i, thread_name_begin, tgid_begin,
        pid_begin, prio_begin, cpu_ratio_begin, thread):
    """This function is used to draw thread body to svg"""

    tgid = -1
    thread_name = ''
    prio = -1
    total_time = END_TIME - START_TIME
    draw_data_count = 0
    x_two_size = BASE_HEIGHT + IOWAIT_HEIGHT + i * ROW_HEIGHT
    line_start = x_two_size - 15
    line_end = line_start + ROW_HEIGHT
    table_group = group.add(dwg.g())

    j = 1
    running_time = 0
    thrd_lenth = len(thread)
    for j in range(j, thrd_lenth):
        if key != thread[j - 1][DATA_FMAT_NPID] or\
                key != thread[j][DATA_FMAT_PPID]:
            continue

        start = 0
        end = 0
        start = float(thread[j - 1][DATA_FMAT_TIME]) / TIME_UNIT

        if key == thread[j][DATA_FMAT_NPID] and\
                ((thread[j][DATA_FMAT_OFFSET] >> 30) & 0x3) == IRQ_INFO:
            end = float(thread[j][DATA_FMAT_TIME]) / TIME_UNIT\
                - float(thread[j][DATA_FMAT_DATA] & 0xffff) / IRQ_DATA_UNIT
        else:
            end = float(thread[j][DATA_FMAT_TIME]) / TIME_UNIT

        if end == 0 or end < start:
            continue
        elif end >= END_TIME:
            if start < END_TIME:
                end = END_TIME
            else:
                break
        if start <= START_TIME:
            if end > START_TIME:
                start = START_TIME
            else:
                continue

        width = end - start
        running_time += width
        table_group.add(dwg.rect(
            class_='THREAD',
            insert=((start - START_TIME) * RATE + BASE_WIDTH, x_two_size - 15),
            size=(width * RATE, ROW_HEIGHT)))
        # user defined flag
        if USERINFO_LIST is not None and key in USERINFO_LIST.keys():
            userinfo = USERINFO_LIST[key]
            user_index = 0
            userinfo_len = len(userinfo)
            for user_index in range(user_index, userinfo_len):
                usr_start = float(
                    userinfo[user_index][DATA_FMAT_TIME]) / TIME_UNIT
                if usr_start >= START_TIME and usr_start <= END_TIME:
                    table_group.add(dwg.rect(class_='USER', insert=((
                        usr_start - START_TIME) * RATE + BASE_WIDTH,
                        x_two_size - 15), size=(1, ROW_HEIGHT)))
                    if SHOW_FUNC_INFO_FLAG == 1:
                        thread_function = (
                            '%s' % userinfo[user_index][DATA_FMAT_NAME]
                            .split('\x00')[0])
                        table_group.add(dwg.text(
                            thread_function,
                            dx=[(usr_start - START_TIME) * RATE + BASE_WIDTH],
                            dy=[x_two_size], class_='ProcessLabel'))

        draw_data_count += 1
        if SHOW_FUNC_INFO_FLAG == 1 and \
                ((thread[j][DATA_FMAT_OFFSET] >> 30) & 0x3) == THREAD_INFO:
            if (thread[j][DATA_FMAT_OFFSET] & 0xff) != 0 and\
                    thread[j][DATA_FMAT_PPID] != thread[j][DATA_FMAT_NPID]:
                thread_function = '%s+%x(%x)' % (
                    thread[j][DATA_FMAT_NAME].split('\x00')[0],
                    thread[j][DATA_FMAT_DATA],
                    ((thread[j][DATA_FMAT_OFFSET]) >> 16) & 0x7ff)
                table_group.add(dwg.text(thread_function, dx=[(
                    start - START_TIME) * RATE + BASE_WIDTH],
                    dy=[x_two_size], class_='ProcessLabel'))

        if SHOW_TIME_INFO_FLAG == 1 and \
                ((thread[j][DATA_FMAT_OFFSET] >> 30) & 0x3) == THREAD_INFO:
            table_group.add(dwg.text(
                '%0.6f' % start,
                dx=[(start - START_TIME) * RATE + BASE_WIDTH],
                dy=[x_two_size], class_='ProcessLabel'))
            table_group.add(dwg.text(
                '%0.6f' % end,
                dx=[(start - START_TIME) * RATE + BASE_WIDTH + width * RATE],
                y=[x_two_size], class_='ProcessLabel'))
        thread_name = thread[j][DATA_FMAT_TNAME].split('\x00')[0]

    if draw_data_count > 0:
        if (i % 2) != 0:
            table_group.add(dwg.rect(
                class_='Sleeping', insert=(8, line_start),
                size=(SVG_WIDTH, ROW_HEIGHT)))
        if key in PID_TID_FUNC_LIST.keys():
            thread_name = PID_TID_FUNC_LIST[key]
        table_group.add(dwg.text(
            thread_name, dx=[10], dy=[x_two_size], class_='ProcessLabel'))
        table_group.add(dwg.line(
            class_='Bold', start=(thread_name_begin, line_start),
            end=(thread_name_begin, line_end)))
        # get and tgid
        if key in PID_TGID_LIST.keys():
            tgid = PID_TGID_LIST[key]
        else:
            tgid = thread[j][DATA_FMAT_TGID]
        table_group.add(dwg.text(
            '%d' % tgid, dx=[2 + thread_name_begin],
            dy=[x_two_size], class_='ProcessLabel'))
        table_group.add(dwg.line(
            class_='Bold', start=(tgid_begin, line_start),
            end=(tgid_begin, line_end)))
        table_group.add(dwg.text(
            '%d' % key, dx=[4 + tgid_begin], dy=[x_two_size],
            class_='ProcessLabel'))
        table_group.add(dwg.line(
            class_='Bold', start=(pid_begin, line_start),
            end=(pid_begin, line_end)))
        # get and  set priority
        prio = (thread[0][DATA_FMAT_OFFSET] >> 8) & 0xff
        table_group.add(dwg.text(
            '%d' % prio, dx=[6 + pid_begin], dy=[x_two_size],
            class_='ProcessLabel'))
        table_group.add(dwg.line(
            class_='Bold', start=(prio_begin, line_start),
            end=(prio_begin, line_end)))
        table_group.add(dwg.rect(
            class_='Process', insert=(8, line_start),
            size=(SVG_WIDTH, ROW_HEIGHT)))
        table_group.add(dwg.text(
            '%f' % (float(running_time) / total_time * 100),
            dx=[8 + prio_begin], dy=[x_two_size], class_='ProcessLabel'))
        table_group.add(dwg.line(
            class_='Bold', start=(cpu_ratio_begin, line_start),
            end=(cpu_ratio_begin, line_end)))
    return draw_data_count


def draw_thread(dwg, group, i, final_thd_list):
    """This function is used to get thread information and draw thread, irq"""

    thread_name_begin = 10 + NAME_WIDTH
    tgid_begin = thread_name_begin + TGID_WIDTH
    pid_begin = tgid_begin + PID_WIDTH
    prio_begin = pid_begin + PRIO_WIDTH
    cpu_ratio_begin = prio_begin + CPU_RATIO_WIDTH
    line_end = IOWAIT_HEIGHT + BASE_HEIGHT
    line_start = line_end - 15

    list_len = len(final_thd_list)
    j = 0
    for j in range(j, list_len):
        count = draw_thread_body(
            dwg, group, final_thd_list[j][0], i, thread_name_begin,
            tgid_begin, pid_begin, prio_begin,
            cpu_ratio_begin, final_thd_list[j][1])
        if count > 0:
            i += 1
    if i > 1:
        group.add(dwg.text(
            'name', dx=[10], dy=[line_end], class_='LegendTextBold'))
        group.add(dwg.line(
            class_='Bold', start=(thread_name_begin, line_start),
            end=(thread_name_begin, line_end + 5)))
        group.add(dwg.text(
            'tgid', dx=[2 + thread_name_begin], dy=[line_end],
            class_='LegendTextBold'))
        group.add(dwg.line(
            class_='Bold', start=(tgid_begin, line_start),
            end=(tgid_begin, line_end + 5)))
        group.add(dwg.text(
            'pid', dx=[4 + tgid_begin], dy=[line_end],
            class_='LegendTextBold'))
        group.add(dwg.line(
            class_='Bold', start=(pid_begin, line_start),
            end=(pid_begin, line_end + 5)))
        group.add(dwg.text(
            'prio', dx=[6 + pid_begin], dy=[line_end],
            class_='LegendTextBold'))
        group.add(dwg.line(
            class_='Bold', start=(prio_begin, line_start),
            end=(prio_begin, line_end + 5)))
        group.add(dwg.text(
            'CPU ratio(%)', dx=[8 + prio_begin], dy=[line_end],
            class_='LegendTextBold'))
        group.add(dwg.line(
            class_='Bold', start=(cpu_ratio_begin, line_start),
            end=(cpu_ratio_begin, line_end + 5)))
        group.add(dwg.rect(
            class_='Process', insert=(8, line_start),
            size=(SVG_WIDTH, ROW_HEIGHT)))
    return i


def get_finally_irqs_list(input_list):
    """This function is used get finally irq information to list"""

    finally_list = {}
    if SHOW_ALL_IRQ_MODE == 2:
        for key in IRQS_LIST.keys():
            if key in input_list:
                finally_list.update({key: IRQS_LIST[key]})
    else:
        finally_list = IRQS_LIST

    return finally_list


def draw_irqs(dwg, group, i, irq_final_list):
    """This function is used to draw irq information to svg"""

    tmp_i = i
    irq_name = 'IRQ'
    irq_name_begin = 10 + NAME_WIDTH
    irq_id_begin = irq_name_begin + TGID_WIDTH
    irq_id_end = irq_id_begin + PID_WIDTH + PRIO_WIDTH + CPU_RATIO_WIDTH

    i = i + 1
    draw_irq_flag = 0
    irqs_finally_list = get_finally_irqs_list(irq_final_list)
    if irqs_finally_list is None:
        return

    j = 0
    for key in irqs_finally_list.keys():

        irqs = irqs_finally_list[key]

        x_two_size = BASE_HEIGHT + IOWAIT_HEIGHT + i * ROW_HEIGHT
        line_start = x_two_size - 15
        line_end = line_start + ROW_HEIGHT

        table_group = group.add(dwg.g())

        is_draw_flag = 0
        for irq in irqs:
            end = (float(irq[DATA_FMAT_TIME]) / TIME_UNIT)
            time = float((irq[DATA_FMAT_DATA] >> 16) & 0xffff) / IRQ_DATA_UNIT
            start = end - time
            if start <= START_TIME:
                if end > START_TIME:
                    start = START_TIME
                else:
                    continue
            if end >= END_TIME:
                if start < END_TIME:
                    end = END_TIME
                else:
                    break
            table_group.add(dwg.rect(class_='IRQ', insert=((
                start - START_TIME) * RATE + BASE_WIDTH, x_two_size - 15),
                size=((end - start) * RATE, ROW_HEIGHT)))
            is_draw_flag += 1
            if SHOW_TIME_INFO_FLAG == 1:
                table_group.add(dwg.text(
                    '%d' % (time * IRQ_DATA_UNIT),
                    dx=[(start - START_TIME) * RATE + BASE_WIDTH],
                    dy=[x_two_size], class_='ProcessLabel'))
        if is_draw_flag > 0:
            draw_irq_flag = 1
            if (j % 2) == 0:
                table_group.add(dwg.rect(
                    class_='Sleeping', insert=(8, line_start),
                    size=(SVG_WIDTH, ROW_HEIGHT)))
            j += 1
            x_two_size = BASE_HEIGHT + IOWAIT_HEIGHT + i * ROW_HEIGHT
            table_group.add(dwg.text(
                irq_name, dx=[10], dy=[x_two_size], class_='ProcessLabel'))
            table_group.add(dwg.line(
                class_='Bold', start=(irq_name_begin, line_start),
                end=(irq_name_begin, line_end)))
            table_group.add(dwg.text(
                '%d' % key, dx=[2 + irq_name_begin], dy=[x_two_size],
                class_='ProcessLabel'))
            table_group.add(dwg.line(
                class_='Bold', start=(irq_id_begin, line_start),
                end=(irq_id_begin, line_end)))
            table_group.add(dwg.line(
                class_='Bold', start=(irq_id_end, line_start),
                end=(irq_id_end, line_end)))
            table_group.add(dwg.rect(
                class_='Process', insert=(8, line_start),
                size=(SVG_WIDTH, ROW_HEIGHT)))
        else:
            i -= 1
        i += 1
    if draw_irq_flag == 1:
        x_two_size = BASE_HEIGHT + IOWAIT_HEIGHT + tmp_i * ROW_HEIGHT
        group.add(dwg.text(
            'IRQ', dx=[10], dy=[x_two_size], class_='LegendTextBold'))
        group.add(dwg.line(
            class_='Bold', start=(irq_name_begin, x_two_size - 15),
            end=(irq_name_begin, x_two_size - 15 + ROW_HEIGHT)))
        group.add(dwg.text(
            'irq_id', dx=[12 + NAME_WIDTH], dy=[x_two_size],
            class_='LegendTextBold'))
        group.add(dwg.line(
            class_='Bold', start=(irq_id_begin, x_two_size - 15),
            end=(irq_id_begin, x_two_size - 15 + ROW_HEIGHT)))
        group.add(dwg.rect(
            class_='Process', insert=(8, x_two_size - 15),
            size=(SVG_WIDTH, ROW_HEIGHT)))
        group.add(dwg.line(
            class_='Bold', start=(irq_id_end, x_two_size - 15),
            end=(irq_id_end, x_two_size - 15 + ROW_HEIGHT)))

    return i


def draw_horizontal(dwg, table, final_list):
    """This function is used to draw svg with horizontal"""

    i = 1
    j = 0
    group = table.add(dwg.g(
        class_='ProcessStatus', transform='translate(0, 16)'))
    if len(final_list) > 0:
        i = draw_thread(dwg, group, i, final_list)
    x_size = BASE_HEIGHT + IOWAIT_HEIGHT + i * ROW_HEIGHT - 15
    if SHOW_ALL_IRQ_MODE > 0 or SHOW_TID_IRQ_INFO_FLAG == 0:
        if i == 1:
            j = draw_irqs(dwg, group, 0, IRQ_INPUT_LIST)
        else:
            j = draw_irqs(dwg, group, i + 1, IRQ_INPUT_LIST)
    if i != 1 and j != i + 2:
        group.add(dwg.rect(
            class_='THRD1', insert=(8, x_size), size=(SVG_WIDTH, ROW_HEIGHT)))

    return


def get_tid_through_tgid(tgid_list):
    """This function is used to get tig through tgid"""

    tid_list = []
    if len(tgid_list) == 0:
        return tid_list

    # get the thread id from lctracer.dat
    for key in THREADS_LIST.keys():
        thread_lenth = len(THREADS_LIST[key])
        for i in range(0, thread_lenth):
            if key == THREADS_LIST[key][i][DATA_FMAT_PPID]:
                if THREADS_LIST[key][i][DATA_FMAT_TGID] in tgid_list:
                    tid_list.append(key)
                    break
    # just get the thread id from thread table
    for tgid in tgid_list:
        for ppid in PID_TGID_LIST.keys():
            if tgid == PID_TGID_LIST[ppid]:
                tid_list.append(ppid)
    if len(tid_list) == 0:
        print 'Warning: input tgid does not exist!'
    return tid_list


def draw_in_2_lines(dwg, table, final_thread_list):
    """This function is used to draw in two line in SVG"""

    i = 0
    clour = 'THRD1'
    thread_clour_flag = 0

    tid_x = 8
    tid_start_height = IOWAIT_HEIGHT + BASE_HEIGHT
    irq_start_height = IOWAIT_HEIGHT + BASE_HEIGHT + ROW_HEIGHT
    table.add(dwg.text(
        'Thread', class_='ProcessLabel',
        dx=[tid_x + 2], dy=[tid_start_height + 15]))
    table.add(dwg.text(
        'IRQ', class_='ProcessLabel',
        dx=[tid_x + 2], dy=[irq_start_height + 15]))
    table.add(dwg.line(
        class_='Bold', start=(NAME_WIDTH, tid_start_height),
        end=(NAME_WIDTH, irq_start_height + ROW_HEIGHT)))

    # Draw thread thread
    list_len = len(final_thread_list)
    j = 0
    for j in range(j, list_len):
        key = final_thread_list[j][0]
        thread = final_thread_list[j][1]
        thread_len = len(thread)
        i = 0
        for i in range(i, thread_len):
            if i + 1 == thread_len:
                break

            data = thread[i]
            next_data = thread[i + 1]
            if data[DATA_FMAT_NPID] != key:
                continue
            if key != next_data[DATA_FMAT_PPID]:
                continue

            start = float(data[DATA_FMAT_TIME]) / TIME_UNIT
            end = float(next_data[DATA_FMAT_TIME]) / TIME_UNIT
            if start <= START_TIME:
                if end > START_TIME:
                    start = START_TIME
                else:
                    continue
            if end >= END_TIME:
                if start < END_TIME:
                    end = END_TIME
                else:
                    break

            table_group = table.add(dwg.g(style='fill:none'))
            tid_running_time = 0
            x_size = BASE_WIDTH
            y_size = 0
            if (SHOW_ALL_TGID_MODE > 0 or SHOW_ALL_TID_MODE > 0)\
                    or SHOW_TID_IRQ_INFO_FLAG == 0:
                y_size = tid_start_height
                tid_running_time = (end - start) * RATE
                x_size = (start - START_TIME) * RATE + BASE_WIDTH
                if thread_clour_flag == 1:
                    clour = 'THRD'
                    thread_clour_flag = 0
                else:
                    clour = 'THRD1'
                    thread_clour_flag = 1
                table_group.add(dwg.rect(
                    class_=clour, insert=(x_size, y_size),
                    size=(tid_running_time, ROW_HEIGHT)))
            irq_running_time = 0
            if ((thread[i][DATA_FMAT_OFFSET] >> 30) & 0x3) == USER_INFO:
                start = float(thread[i][DATA_FMAT_TIME]) / TIME_UNIT
                x_size = (start - START_TIME) * RATE + BASE_WIDTH
                y_size = tid_start_height
                table_group.add(dwg.rect(
                    class_='USER', insert=(x_size, y_size),
                    size=(1, ROW_HEIGHT)))
                if SHOW_TIME_INFO_FLAG == 1:
                    table_group.add(dwg.text(
                        '%2.6f' % start, class_='AxisLabel',
                        insert=(x_size, y_size + 15)))
                if SHOW_FUNC_INFO_FLAG == 1:
                    thread_function = '%s+%x(%x)' % (
                        data[DATA_FMAT_NAME].split('\x00')[0],
                        data[DATA_FMAT_DATA],
                        ((data[DATA_FMAT_OFFSET] >> 16) & 0x7ff))
                    table_group.add(dwg.text(
                        thread_function, class_='AxisLabel',
                        insert=(x_size, y_size + 10), dx='1', dy='3'))

            # Show the time information on chart
            if SHOW_TIME_INFO_FLAG == 1:
                if data[DATA_FMAT_NPID] != data[DATA_FMAT_PPID]:
                    table_group.add(dwg.text(
                        '%2.6f' % start, class_='AxisLabel',
                        insert=(x_size, y_size + 15)))
                    table_group.add(dwg.text(
                        '%2.6f' % end, class_='AxisLabel',
                        insert=(x_size + irq_running_time, y_size + 15)))
            # Show the function information on chart
            if SHOW_FUNC_INFO_FLAG == 1 and\
                    (data[DATA_FMAT_OFFSET] >> 30) & 0x3 == THREAD_INFO:
                if data[DATA_FMAT_NPID] != data[DATA_FMAT_PPID]:
                    if 0 != data[DATA_FMAT_OFFSET] & 0xff:
                        thread_function = '%s+%x(%x)' % (
                            data[DATA_FMAT_NAME].split('\x00')[0],
                            data[DATA_FMAT_DATA],
                            ((data[DATA_FMAT_OFFSET] >> 16) & 0x7ff))
                        table_group.add(dwg.text(
                            thread_function, class_='AxisLabel',
                            insert=(x_size, y_size + 10), dx='1', dy='3'))
    if SHOW_TID_IRQ_INFO_FLAG == 0 or SHOW_ALL_IRQ_MODE > 0:
        draw_in_2_line_irq(dwg, table)

    table.add(dwg.rect(
        class_='Process', insert=(tid_x, tid_start_height),
        size=(SVG_WIDTH - 8, ROW_HEIGHT)))
    table.add(dwg.rect(
        class_='Process', insert=(tid_x, irq_start_height),
        size=(SVG_WIDTH - 8, ROW_HEIGHT)))
    return


def draw_in_2_line_irq(dwg, table_group):
    """This function is used to draw in two line of irq information to SVG"""

    irq_clour_flag = 0
    irqs_finally_list = get_finally_irqs_list(IRQ_INPUT_LIST)
    if IRQS_LIST is None:
        return

    for key in irqs_finally_list.keys():
        irq = irqs_finally_list[key]
        irq_len = len(irq)
        i = 0
        x_size = 0
        y_size = 0
        for i in range(i, irq_len):
            if i + 1 == irq_len:
                break
            data = irq[i]
            end = float(data[DATA_FMAT_TIME]) / TIME_UNIT
            irq_running_time = float((
                data[DATA_FMAT_DATA] >> 16) & 0xffff) / IRQ_DATA_UNIT
            start = end - irq_running_time
            if start <= START_TIME:
                if end > START_TIME:
                    start = START_TIME
                else:
                    continue
            if end >= END_TIME:
                if start < END_TIME:
                    end = END_TIME
                else:
                    break
            if irq_clour_flag == 1:
                clour = 'IRQ'
                irq_clour_flag = 0
            else:
                clour = 'IRQ1'
                irq_clour_flag = 1
            x_size = (start - START_TIME) * RATE + BASE_WIDTH
            y_size = IOWAIT_HEIGHT + BASE_HEIGHT + ROW_HEIGHT
            table_group.add(dwg.rect(
                class_=clour, insert=(x_size, y_size),
                size=(irq_running_time, ROW_HEIGHT)))
            if SHOW_TIME_INFO_FLAG == 1:
                table_group.add(dwg.text(
                    '%d' % (irq_running_time * IRQ_DATA_UNIT),
                    class_='AxisLabel', insert=(x_size, y_size + 15)))
    return


def update_table_to_thread(threads, input_list):
    """This function is used to update thread table information to
    threads_list, which include priority, tgid, function and so on"""

    is_input_list_exits = 0
    if len(input_list) > 0:
        is_input_list_exits = 1
    new_threads = {}
    for key in threads.keys():
        if is_input_list_exits == 1:
            if key not in input_list:
                continue

        data = threads[key]
        if key in PID_PRIO_LIST.keys():
            if len(data) < 1:
                break

            if re.search('FIFO', PID_PRIO_LIST[key][0]):
                thread_table_policy = 1
            elif re.search('R.R.', PID_PRIO_LIST[key][0]):
                thread_table_policy = 2
            else:
                thread_table_policy = 3

            data[0][DATA_FMAT_OFFSET] = (
                (int(PID_PRIO_LIST[key][1]) << 8) |
                (threads[key][0][DATA_FMAT_OFFSET] & 0xffff00ff))
            # update policy
            data[0][DATA_FMAT_OFFSET] = thread_table_policy << 27 |\
                (threads[key][0][DATA_FMAT_OFFSET] & 0xc7ffffff)

            data[0][DATA_FMAT_TGID] = PID_TGID_LIST[key]

        new_threads.update({key: data})

    return new_threads


def get_thread_info():
    """This function is used to get thread information"""

    final_thread_list = {}
    input_list = []
    if (SHOW_ALL_TGID_MODE == 1 or SHOW_ALL_TID_MODE == 1) or\
            SHOW_TID_IRQ_INFO_FLAG == 0:
        if THREADS_LIST is None:
            return

        final_thread_list = update_table_to_thread(
            THREADS_LIST, input_list)
    else:
        if SHOW_ALL_TGID_MODE == 2:
            input_list += get_tid_through_tgid(TGID_INPUT_LIST)
        if SHOW_ALL_TID_MODE == 2:
            input_list += TID_INPUT_LIST
        if len(input_list) == 0:
            return final_thread_list

        final_thread_list = update_table_to_thread(
            THREADS_LIST, input_list)

    return thread_sort(final_thread_list)


def thread_sort(thread_dict):
    """This function is used to sort thread by priority"""

    thread_len = len(thread_dict)
    i = 0
    items = thread_dict.items()
    for i in range(i, thread_len):
        priority_i = (items[i][1][0][DATA_FMAT_OFFSET] >> 8) & 0xff
        minimum = i
        for j in range(i + 1, thread_len):
            priority_j = (items[j][1][0][DATA_FMAT_OFFSET] >> 8) & 0xff
            if priority_i > priority_j:
                minimum = j
                priority_i = (
                    items[minimum][1][0][DATA_FMAT_OFFSET] >> 8) & 0xff
        if minimum != i:
            tmp_item = items[minimum]
            items[minimum] = items[i]
            items[i] = tmp_item

    for i in range(0, thread_len):
        priority_i = (items[i][1][0][DATA_FMAT_OFFSET] >> 8) & 0xff
        policy_i = (items[i][1][0][DATA_FMAT_OFFSET] >> 27) & 0x7
        minimum = i
        for j in range(i + 1, thread_len):
            priority_j = (items[j][1][0][DATA_FMAT_OFFSET] >> 8) & 0xff
            policy_j = (items[j][1][0][DATA_FMAT_OFFSET] >> 27) & 0x7
            if priority_i == priority_j:
                if policy_i > policy_j:
                    minimum = j
                    policy_i = (
                        items[minimum][1][0][DATA_FMAT_OFFSET] >> 27) & 0x7
        if minimum != i:
            tmp_item = items[minimum]
            items[minimum] = items[i]
            items[i] = tmp_item
    return items


def basic_shapes(name, css):
    """This function is used to draw basic shapes"""

    dwg = svgwrite.Drawing(filename=name, debug=True, size=(
        SVG_WIDTH + BASE_WIDTH + SCALE, SVG_HEIGHT + IOWAIT_HEIGHT + SCALE))
    dwg.defs.add(dwg.style(css))
    dwg.add(dwg.rect(class_='Back', rx=None, ry=None, size=(
        SVG_WIDTH + BASE_WIDTH + SCALE, SVG_HEIGHT + IOWAIT_HEIGHT + SCALE)))
    table = dwg.add(dwg.g(transform="translate(10, 50)", style="fill: none"))
    draw_header(dwg, table)
    draw_io_wait(dwg, table)
    draw_coordinate(dwg, table)

    final_thread_list = []
    if SHOW_ALL_TID_MODE != 0 or SHOW_ALL_TGID_MODE != 0 \
            or SHOW_TID_IRQ_INFO_FLAG == 0:
        final_thread_list = get_thread_info()
    if OUTPUT_FILE_FLAG == 1:
        draw_horizontal(dwg, table, final_thread_list)
    else:
        draw_in_2_lines(dwg, table, final_thread_list)

    dwg.save()


def output_txt_file(output_file):
    """This function uses to output format txt"""

    i = 0
    data_buff = ''
    file_obj = open(output_file, 'w')

    input_list = []
    global SHOW_ALL_TID_MODE
    global SHOW_ALL_TGID_MODE
    global SHOW_ALL_IRQ_MODE

    header_title = 'LCTracer chart(V%s.%s):%s\n' %\
        (VERSION[0], VERSION[1], TITLE_NAME)
    header_uname = 'system uname: %s\n' % SYSTEM_UNAME
    header_cpu = 'cpu:         %s' % SYSTEM_CPU
    header_lctracer = 'LCTracer parameters state:\n'
    header_buffer_size = 'buffer size:             %s' % BUFFER_SIZE
    header_threshold = 'thread shold:            %s' % THRESHOLD
    header_autostart = 'aurostart:               %s' % AUTOSTART
    header_filesystem = 'store to filesystem:     %s' % STORE_TO_FILESYSTEM
    header_interval = 'io wait sample interval: %s' % IOWAIT_SAMPLE_INTERVAL
    header_save_path = 'saving path:             %s' % SAVE_PATH
    header_buffer_address = 'buffer address:          %s\n\n' % BUFFER_ADDRESS

    header_info = header_title + header_uname + header_cpu + header_lctracer +\
        header_buffer_size + header_threshold + header_autostart +\
        header_filesystem + header_interval + header_save_path +\
        header_buffer_address
    file_obj.write(header_info)

    if SHOW_TID_IRQ_INFO_FLAG == 1:
        if SHOW_ALL_TGID_MODE == 2:
            input_list += get_tid_through_tgid(TGID_INPUT_LIST)
        if SHOW_ALL_TID_MODE == 2:
            input_list = input_list + TID_INPUT_LIST
    else:
        SHOW_ALL_IRQ_MODE = 3
        SHOW_ALL_TGID_MODE = 3
        SHOW_ALL_TID_MODE = 3

    for i in range(i, BINARY_DATA_LEN):
        data = BINARY_DATA[i]
        if (data[DATA_FMAT_OFFSET] >> 30) & 0x3 == IRQ_INFO and\
            SHOW_ALL_IRQ_MODE == 2 and ((
                data[DATA_FMAT_DATA] & 0xffff) not in IRQ_INPUT_LIST):
            continue
        if ((data[DATA_FMAT_OFFSET] >> 30) & 0x3 != IRQ_INFO) and\
                (SHOW_ALL_TGID_MODE == 2 or SHOW_ALL_TID_MODE == 2):
            if (data[DATA_FMAT_NPID] not in input_list and
                    data[DATA_FMAT_PPID] not in input_list):
                continue

        time = float(data[DATA_FMAT_TIME]) / TIME_UNIT
        if time < START_TIME:
            continue
        if time > END_TIME:
            break
        line = ''
        time = '%0.6f' % (float(data[DATA_FMAT_TIME]) / TIME_UNIT / 1000)
        if SHOW_ALL_IRQ_MODE > 0 and\
                ((data[DATA_FMAT_OFFSET] >> 30) & 0x3) == IRQ_INFO:
            line = '[ %13s ] ' % time
            line += '-int '
            irq_number = data[DATA_FMAT_DATA] & 0xffff
            line += 'irq:%d ' % irq_number
            executing_time = (data[DATA_FMAT_DATA] >> 16) & 0xffff
            line += 'exe:%d\n' % executing_time
            data_buff += line
        elif (SHOW_ALL_TID_MODE > 0 or SHOW_ALL_TGID_MODE > 0) and\
                (data[DATA_FMAT_PPID] != data[DATA_FMAT_NPID]):
            ppid = data[DATA_FMAT_PPID]
            line = '[ %13s ] ' % time
            line += '-ctx '
            tgid = 0
            task_state = ''
            if ppid in PID_TGID_LIST.keys():
                tgid = PID_TGID_LIST[ppid]
            else:
                tgid = data[DATA_FMAT_TGID]
                if data[DATA_FMAT_OFFSET] & 0xff in TASK_STATE_BITMASK.keys():
                    task_state = TASK_STATE_BITMASK[
                        data[DATA_FMAT_OFFSET] & 0xff]
                else:
                    task_state = 'UNKNOWN_TYPE'
            line += 'prev:%d:%d -> next:%d pstate:%s '\
                    % (ppid, tgid, data[DATA_FMAT_NPID], task_state)
            if 0 == data[DATA_FMAT_OFFSET] & 0xff:
                line += 'pwchan:0(0) '
            else:
                line += 'pwchan:%s+%x(%x) '\
                        % (data[DATA_FMAT_NAME].split('\x00')[0],
                        data[DATA_FMAT_DATA], (data[DATA_FMAT_OFFSET] >> 16)
                        & 0x7ff)
            func_name = ''
            if ppid in PID_TID_FUNC_LIST.keys():
                func_name = PID_TID_FUNC_LIST[ppid].rstrip()
            else:
                func_name = data[DATA_FMAT_TNAME].replace('\x00', '')
            line += 'ptask:%s ' % func_name
            pprio = ''
            if ppid in PID_PRIO_LIST.keys():
                pprio = PID_PRIO_LIST[ppid][1]
            else:
                pprio = str((data[DATA_FMAT_OFFSET] >> 8) & 0xff)
            line += 'pprio:%s\n' % pprio

            data_buff += line
        elif ((data[DATA_FMAT_OFFSET] >> 30) & 0x3) == USER_INFO:
            line = '[ %13s ] ' % time
            line += '-usr current:%d log:' % data[DATA_FMAT_NPID]
            line += '%s\n' % (data[DATA_FMAT_NAME].split('\x00')[0])
            data_buff += line
    file_obj.write(data_buff)
    file_obj.close()

    return


def get_thread_name(thread_table_name):
    """This function get tgid, function name, priority through thread table"""

    tgid_index = 4
    pid_index = 5
    prio_index = 8

    if not os.path.isfile(thread_table_name):
        return

    thread_table_fd = open(thread_table_name, 'r')
    global PRIO_PID_LIST

    PRIO_PID_LIST = []
    for line in thread_table_fd.readlines():
        data_list = line.split('|')
        if len(data_list) > 1:
            PID_TGID_LIST.update(
                {int(data_list[pid_index]): int(data_list[tgid_index])})
            PID_TID_FUNC_LIST.update({int(
                data_list[pid_index]): str(data_list[TID_FUNC_INDEX])})
            PID_PRIO_LIST.update({int(
                data_list[pid_index]):
                data_list[prio_index].split('/')[0].split()})
    thread_table_fd.close()
    for key in sorted(
            PID_PRIO_LIST.iteritems(), key=lambda (k, v): (v, k)):
        PRIO_PID_LIST.append(key)

    return


if __name__ == "__main__":

    ARG_VALUE = sys.argv
    ARGC = len(sys.argv)
    parse_parameter(ARGC, ARG_VALUE)

    OUTPUT_FILE = ARG_VALUE[ARGC - 1]

    parse_log(ARG_VALUE[ARGC - 2])

    # If the input is *.svg now output file is *.svg.svg
    if OUTPUT_FILE_FLAG == 1:
        OUTPUT_FILE += '.svg'
        basic_shapes(OUTPUT_FILE, CSS)
    elif OUTPUT_FILE_FLAG == 2:
        BASE_WIDTH = NAME_WIDTH
        OUTPUT_FILE += '.SVG'
        basic_shapes(OUTPUT_FILE, CSS)
    else:
        OUTPUT_FILE += '.txt'
        output_txt_file(OUTPUT_FILE)
